if (!Object.keys) {
	Object.keys = (function() {
		var hasOwnProperty = Object.prototype.hasOwnProperty,
			hasDontEnumBug = !({
				toString: null
			}).propertyIsEnumerable('toString'),
			dontEnums = ['toString', 'toLocaleString', 'valueOf', 'hasOwnProperty', 'isPrototypeOf', 'propertyIsEnumerable', 'constructor'];

		return function(obj) {
			if (typeof obj !== 'object' && typeof obj !== 'function' || obj === null) throw new TypeError('Object.keys called on non-object');

			var result = [];

			for (var prop in obj) {
				if (hasOwnProperty.call(obj, prop)) result.push(prop);
			}

			if (hasDontEnumBug) {
				for (var i = 0; i < dontEnums.length; i++) {
					if (hasOwnProperty.call(obj, dontEnums[i])) result.push(dontEnums[i]);
				}
			}

			return result;
		};
	})();
}

// Taken from underscore.js
if (!Array.range) {
	Array.range = function(start, stop, step) {
		if (arguments.length <= 1) {
			stop = start || 0;
			start = 0;
		}
		step = arguments[2] || 1;
		
		var range = new Array(Math.max(Math.ceil(Math.abs(stop - start) / step), 0));
		
		for(var i = 0; i < range.length; i++) {
			range[i] = start;
			start += step;
		}
		
		return range;
	};
}

// fix for IE missing window.location.origin
if (!window.location.origin) {
  window.location.origin = window.location.protocol + "//" + window.location.hostname + (window.location.port ? ':' + window.location.port: '');
}

//	this file is the entry point for the application. 
//	The Gruntfile loads start.js, then all js files in src/javascript/, then end.js

// 'app' is the top level - all views, models, routers, etc are contained in 'app'.
var app = app || { view: {}, collection: {}, model: {}, router: {}, event: {}, config: {}, graphs: {}, utils: {}};

// load the config.js file
/**
 * Intercept clicks of Download buttons so we can see if they have agreed to the User Agreement
 * @param event
 */
app.utils.interceptDownload = function(event) {
    console.log('download intercepted');
    console.log(event);
    // if the project has not added a User Agreement, then we won't prompt them, so this function exits here
    if (!app.config.userAgreement) {
        return this;
    }

    // a User Agreement exists, so prevent the file from automatically downloading until we have them ack the form
    event.preventDefault();

    // if they haven't accepted the User Agreement, there won't be a cookie
    if (!Cookies.get(app.config.title + '_agreement')) {
        // the template ID is the script ID set in main.scala.html for the user_agreement.scala.html form
        var template = $("#user-agreement-template").html();

        // the user-agreement ID is set in the bottom of index.scala.html. It is the hidden modal that we will
        // activate when we want to prompt for the agreement
        // we will pass in the original URL that the user wanted to download
        $("#user-agreement").html(_.template(template, {url: event.currentTarget.href}));

        // the user-agreement-modal ID is found in user_agreement.scala.html.
        // We must call .modal to register the modal, and we are calling show at the same time
        $("#user-agreement-modal").modal('show');

    } else {
        // if the user has already agreed to the user agreement, then we will trigger a click
        // of the link to start the download. We don't use the original element, since that
        // has the class of 'download' on it and it would recursively call interceptDownload since
        // the event bus is setup to listen for clicks of 'a.download' links.
        // We set the target attribute to open in a new window, and the download attribute
        // so it will suppress the new tab and just directly download the file.
        // (the download 'attribute' is not the same as the download 'class')
        var passThroughLink = document.createElement('a');
        passThroughLink.href = event.currentTarget.href;
        passThroughLink.target = "_blank";
        passThroughLink.download = true;
        passThroughLink.click();
    }
};

app.utils.setCookie = function() {
    // this gets called by an 'onclick' attribute in user_agreement.scala.html
    Cookies.set(app.config.title + '_agreement', Date.now());
    app.utils.closeModals();
};

app.utils.closeModals = function() {
    // this is called by app.utils.setCookie
    $(".modal").modal('hide');
};
// customize this file to suit your branch
app.config = {
    features: {
        boxWhiskerRaw: true
    },
    landingPage:"welcome",
    clowderDevUrl: "https://ilnlrs-dev.ncsa.illinois.edu/clowder",
    alert: false,
    disclaimer_head:"GLGVO Disclaimer",
    disclaimer_text: (
        "<p class='about_text03' style='width:100%;'>" +
            "<b>Illinois Nutrient Loss Reduction Strategy Data Portal</b> is an interactive geospatially referenced " +
            "database that aggregates water quality data from several sources." +
        "</p>"
    ),
    title: "Illinois Nutrient Loss Reduction Strategy",
    googleAnalytics: "",
    metadata: {
        title: "Illinois Nutrient Loss Reduction Strategy",
        keywords: "greon, ngrrec, great rivers, uiuc, ncsa, water quality, environment, university of illinois at urbana-champaign, ltrm, noaa, usgs, lewis and clark community college, monitoring, virtual observatory, mississippi, river restoration, Continuous water quality monitoring, USACE",
        description: "Illinois Nutrient Loss Reduction Strategy",
        url: "#"
    },
    binning: {
        time: "smart",
        depth: 1
    },
    areas: {
        title: "Areas",
        noneFound: "No areas have been specified.",
        shapeOptions: {
          color: '#20201F'
        },
        zoomInOnClick: true,
        items: [
            // override this array in ./config/areas.js
        ]
    },
    layers: {
        title: "Layers",
        noneFound: "No layers have been specified.",
        items: [
            // override this array in ./config/layers.js
        ]
    },
    basins: {
        title: "Basins",
        noneFound: "No basins have been specified.",
        items: [
            // override this array in ./config/basins.js
        ]
    },
    parameters: {
        title: "Parameters",
        noneFound: "No parameters have been specified.",
        defaults: {
            detailView: 'ignore',
            searchView: 'ignore',
            exploreView: 'ignore'
        },
        items: [
            // override this array in ./config/parameters.js
        ]
    },
    sources: {
        title: "Data Sources",
        noneFound: "No data sources have been specified.",
        items: [
            // override this array in ./config/sources.js
        ]
    },
    sensors: {
        title: "Sensors",
        noneFound: "No sensors have been specified.",
        items: [
            // override this array in ./config/sensors.js
        ]
    },
    graphs: {
        timeSeries: {
            showUnits: {
                sideBar: false,
                graphTitle: true,
                graphXAxis: true,
                graphYAxis: true,
                graphTooltip: true
            },
            showSource: true,
            showLine: true
        },
        depthProfile: {
            showUnits: {
                sideBar: false,
                graphXAxis: true,
                graphYAxis: true

            }
        },
        compareGraphs: {
            chartType: 'line',
            pointSize: '5',
            showPointsOnLineGraphs: true
        }
    },
    search_link: {
        url: "/gd3/#/search",
        title: "Download"
    },
    useV3: true,
    useExploreParams: false
};
// load the env file from /src/javascript/envs/{env}.js
// @formatter:off
app.config.areas.groups = [{
    title: "HUC4 Watersheds",
    items: [
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.31891874210174,40.18302008866954],[-88.25855151511212,40.12547344396722],[-88.27502343800319,39.93799841717487],[-88.34691274830828,39.76021516640918],[-88.25280321303768,39.633149627023045],[-88.31331161398543,39.53701672300559],[-88.6159236208074,39.40228137633977],[-88.60065317916445,39.252361449489115],[-88.80602122155398,39.019533159225546],[-88.77178517785711,38.82885575535482],[-88.87917695269044,38.74271410861354],[-88.86083842146888,38.602133693206724],[-88.9015054484891,38.51305383605336],[-88.82998988818343,38.410030276838256],[-88.76413185182736,38.17806929490666],[-88.6311108843255,38.106202603351505],[-88.68731457694656,37.947723806722536],[-88.74519971748174,37.92405936196758],[-88.78878659762239,37.72892454352046],[-88.88162270997833,37.70375600501785],[-89.01835522851604,37.55726853024527],[-89.19531089907474,37.561581284405236],[-89.26539070209094,37.50932446156969],[-89.0277833722514,37.399335264865385],[-88.93569315260271,37.327148586852445],[-89.0608809076167,37.21684640681531],[-89.24339617087509,37.14429101317796],[-89.2564015177299,37.092140316383905],[-89.13808670124689,36.9846176478008],[-89.137152292915,36.97285348636069],[-89.24350959170823,37.00706797589095],[-89.38224246440956,36.95114938535272],[-89.4476456747247,37.093535698673406],[-89.72870423574682,37.231458576584316],[-90.00175275303133,37.10438889969822],[-90.20550897146501,37.123371518418764],[-90.26142619637824,37.09449112992189],[-90.38782985555702,37.14508657463506],[-90.4176705378024,37.22225245680693],[-90.27962770572498,37.42093068566521],[-90.23664980787504,37.54929174900764],[-90.27769504114468,37.777009996570825],[-90.3930853680489,37.848822717292705],[-90.63383905100852,37.758256297641594],[-90.72346957482773,37.64073566969904],[-90.88204604333157,37.70802084980289],[-91.07099855137164,37.67257018839962],[-91.18383913452982,37.57514366355082],[-91.28722996249434,37.58097607291677],[-91.37624482798117,37.498735998044424],[-91.54465240480306,37.57653715104868],[-91.59726915680477,37.518600737596955],[-91.78950481171466,37.57076745001598],[-91.74738402011343,37.68972790295629],[-91.75958397321949,38.09706370232402],[-91.79152658254486,38.173703632413435],[-91.74049693054076,38.28229215099486],[-91.6057836474165,38.293577970769036],[-91.31034640933342,38.41749181328504],[-91.09623516174912,38.42158833827864],[-91.04564862641098,38.480335291312485],[-90.85967746107463,38.47088652674381],[-90.68290991030733,38.581944124488075],[-90.61202852708402,38.56600670472113],[-90.35059350769819,38.700206432637856],[-90.29509687445096,38.780519040846514],[-90.12205688513626,38.80940667726003],[-90.04881400608326,38.860105335514675],[-89.9393162822949,39.084645869541134],[-89.72115016909186,39.20301944227407],[-89.62008363904039,39.34829831392352],[-89.33971939364227,39.317901077512374],[-89.0367051149459,39.405627134667895],[-88.99052355460094,39.49103138141032],[-88.88199929226937,39.50085789285339],[-88.87451652457264,39.708348456073],[-88.76938543515251,39.765068207026616],[-88.7690367778614,39.83344456942052],[-88.68258494882889,39.92645338490115],[-88.46708527937176,40.01928222538203],[-88.31891874210174,40.18302008866954]]]},"properties":{"id":"0714","title":"Upper Mississippi-Kaskaskia-Meramec"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.67947653633377,36.966526060328874],[-88.79172766949284,36.99947935298604],[-88.90387473286046,36.978749095726585],[-89.08604294820265,37.12184029758777],[-89.16576830328728,37.035775869596364],[-89.13808670124689,36.9846176478008],[-89.2564015177299,37.092140316383905],[-89.24339617087509,37.14429101317796],[-89.0608809076167,37.21684640681531],[-88.93569315260271,37.327148586852445],[-89.0277833722514,37.399335264865385],[-89.26539070209094,37.50932446156969],[-89.19531089907474,37.561581284405236],[-89.01835522851604,37.55726853024527],[-88.88162270997833,37.70375600501785],[-88.78878659762239,37.72892454352046],[-88.74519971748174,37.92405936196758],[-88.68731457694656,37.947723806722536],[-88.6311108843255,38.106202603351505],[-88.49352005849738,38.125482471029954],[-88.32998714520954,38.048659922190836],[-88.30218363483607,37.99054420040608],[-88.16730035067042,37.939160169235834],[-88.15281576631787,37.81329879234784],[-88.0038593082158,37.80197398924042],[-87.92529640208772,37.95247033275683],[-87.66267845666204,37.973954976473465],[-87.6244237785964,38.082361311721854],[-87.52695765374767,38.16311298138817],[-87.66770804519587,38.23614063960815],[-87.56928638180699,38.28542444265668],[-87.31967361552773,38.277781626001854],[-87.17299383138044,38.21345942714339],[-87.0711074221636,38.278537901000675],[-86.93637323903937,38.172535867831925],[-86.70944536230832,38.3207994415601],[-86.60329986038971,38.314479441569915],[-86.3998270971639,38.3861330758337],[-86.28518060255016,38.54910943078903],[-86.11066309032105,38.671564372265664],[-85.99893916028611,38.663202377486925],[-85.87941101567998,38.57095232763015],[-85.58469927655415,38.6558476702067],[-85.50305316105585,38.66017758999163],[-85.38195422582714,38.81381356891984],[-85.31652748842873,38.951264144748166],[-85.31482336238969,38.95129341141478],[-85.31294796135091,38.95384109266081],[-85.31242183114341,38.9570956551558],[-85.31247124364336,38.95896011869456],[-85.2636984020524,38.998663279049595],[-85.20664018651593,38.95604193432405],[-85.15849912409067,38.73838100028695],[-85.20710376151521,38.66796878164621],[-85.15563625326178,38.47084149861888],[-85.20267951777208,38.39722949456649],[-85.1114942158303,38.344648005064755],[-85.05260917738002,38.0752233877746],[-84.88372768285052,38.08859811275386],[-84.89865035053566,38.01607123161642],[-84.80967658921543,37.86231094643841],[-84.79543337465424,37.74938801015537],[-84.89995897032532,37.44515687208593],[-85.01891041076567,37.40183600756984],[-85.17152154698715,37.46409529497322],[-85.2459343104133,37.420722902332216],[-85.35026571441801,37.47126263350373],[-85.63518822543409,37.472039729335904],[-85.68965322118288,37.67991948213819],[-85.82604316263786,37.74501228412049],[-85.97271162595183,37.731720626849494],[-86.09324095076477,37.78695002259707],[-86.22592395993382,37.73541089976038],[-86.37655833782497,37.78330507885272],[-86.48195922828637,37.71386557896051],[-86.84536454543064,37.773676604909326],[-87.19187373447613,37.78092351635644],[-87.40641385914313,37.9353933640333],[-87.49741328296022,37.903178454708325],[-87.49215549859338,37.71233207062954],[-87.60831565257973,37.66497451132807],[-87.69392124411354,37.56415453544287],[-87.6218811338087,37.434676468977216],[-87.50259590691047,37.35971373367687],[-87.49276066421743,37.25242486613507],[-87.54899255058848,37.1605322954444],[-87.4857597704783,37.00008961548508],[-87.60736045883124,36.97558960302314],[-87.65311456292687,37.03190267064406],[-87.8283718886965,37.08659139243417],[-88.00852180195852,37.1931255453938],[-88.03796709253783,37.27002990256608],[-88.240781701598,37.30236175668256],[-88.39333254823623,37.19178196727091],[-88.47171104707292,37.038212477925924],[-88.59282682084324,37.08771342159912],[-88.67947653633377,36.966526060328874]]]},"properties":{"id":"0514","title":"Lower Ohio"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-89.01623291601936,41.67520180927005],[-89.0738982044715,41.60168520209248],[-88.92716006199095,41.52408460325461],[-88.95635414944564,41.443901127337426],[-88.69047594777498,41.26654983073769],[-88.63452366765353,41.058445518560745],[-88.58224568544301,40.99347424991157],[-88.42168847423386,40.934859430210906],[-88.2610231411499,40.93497610625235],[-88.11697427262351,40.972574179110666],[-88.10642216326488,40.75425222840789],[-88.20267621311552,40.66100141917764],[-88.33613843374167,40.64241824003983],[-88.35597041912757,40.54902828185146],[-88.29366848589092,40.44657689763545],[-88.15274373402633,40.36632100192668],[-88.16028524130633,40.308488074933166],[-88.31891874210174,40.18302008866954],[-88.46708527937176,40.01928222538203],[-88.68258494882889,39.92645338490115],[-88.7690367778614,39.83344456942052],[-88.76938543515251,39.765068207026616],[-88.87451652457264,39.708348456073],[-88.88199929226937,39.50085789285339],[-88.99052355460094,39.49103138141032],[-89.0367051149459,39.405627134667895],[-89.33971939364227,39.317901077512374],[-89.62008363904039,39.34829831392352],[-89.72115016909186,39.20301944227407],[-89.9393162822949,39.084645869541134],[-90.21666182040605,39.122532280940675],[-90.58456800420998,38.89394155212881],[-90.6299031176813,38.98195624365883],[-90.6774504884408,39.3285847629125],[-90.71586720192283,39.349247905588754],[-90.66938177803667,39.570900770869684],[-90.77151174350314,39.62424582599522],[-90.8249594673785,39.71210551440049],[-91.01738433166315,39.80471161321509],[-91.08027676802391,39.88664818183793],[-91.1980742032577,39.944582019248],[-91.18089957099272,40.03335068056856],[-91.03284169309751,40.0445138847179],[-91.09775171799674,40.32592735094772],[-91.20856795844975,40.43530086119466],[-91.2161045782297,40.541536187071415],[-91.12479818982979,40.54829362456093],[-91.07637272323831,40.62557247756598],[-90.90337638079848,40.66885060562379],[-90.75367669353085,40.66163474834332],[-90.65386326139412,40.847661559512915],[-90.36875919516996,40.90895787400109],[-90.20704541521263,41.16755986734967],[-89.99087513429816,41.15899921007133],[-89.96655602183591,41.22852242558838],[-89.66888017333963,41.32301527856674],[-89.57414261411174,41.52950603345454],[-89.30957678639737,41.641351675989256],[-89.0726498148901,41.70521069151516],[-89.01623291601936,41.67520180927005]]]},"properties":{"id":"0713","title":"Lower Illinois"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.67947653633377,36.966526060328874],[-88.58951304064004,36.7586193075266],[-88.43605724712825,36.59490412340568],[-88.4392420721233,36.50578531521069],[-88.3439788816462,36.47909927462712],[-88.28940582235589,36.38292694144309],[-88.35154724100943,36.32349777278529],[-88.30661010670417,36.09025955231402],[-88.32561492854967,35.94173431400287],[-88.44008636483034,35.86473846412241],[-88.48643493663337,35.59889602599338],[-88.43900299087369,35.56954574478891],[-88.5427627563376,35.255138398401925],[-88.50013732932047,35.20673370472707],[-88.403791257595,34.748900891896085],[-88.54923872820257,34.64027051810632],[-88.66811152176803,34.70466412633971],[-88.72317449980756,34.61646300355994],[-88.8342370433852,34.62129451501079],[-88.96546123172317,34.80133366993965],[-89.10589032525519,34.849866967781],[-89.2624383354289,34.83540148863682],[-89.4264587695493,34.89566156666825],[-89.44310338723176,34.946865989505454],[-89.6461556827499,34.975549250919244],[-89.75761286903526,34.951322572831884],[-89.81980895956372,34.99670410192812],[-89.93212547918102,34.92155973433637],[-90.03705345089315,34.91629974996954],[-90.19890366001698,35.06740459452669],[-90.10080749766922,35.223362264076286],[-90.1901504496139,35.391030279441],[-90.04797459566788,35.48631417512644],[-89.92397422502705,35.64689457175223],[-89.96295673538322,35.73202618724508],[-89.87881381259712,35.77546283405263],[-89.74377758155674,35.91950192862072],[-89.73897726385587,36.00678320244356],[-89.61690065362865,36.15938989283171],[-89.70195438682998,36.287765006174084],[-89.55884571309377,36.374345014373034],[-89.57936716722861,36.56543225678479],[-89.41888308206103,36.55496011721772],[-89.36524022276927,36.63496135771851],[-89.2420449115022,36.581380489051696],[-89.17305732619263,36.68749926701196],[-89.18711012825412,36.816281398062074],[-89.137152292915,36.97285348636069],[-89.13808670124689,36.9846176478008],[-89.16576830328728,37.035775869596364],[-89.08604294820265,37.12184029758777],[-88.90387473286046,36.978749095726585],[-88.79172766949284,36.99947935298604],[-88.67947653633377,36.966526060328874]]]},"properties":{"id":"0801","title":"Lower Mississippi-Hatchie"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-85.2636984020524,38.998663279049595],[-85.31247124364336,38.95896011869456],[-85.31242183114341,38.9570956551558],[-85.31294796135091,38.95384109266081],[-85.31482336238969,38.95129341141478],[-85.31652748842873,38.951264144748166],[-85.38195422582714,38.81381356891984],[-85.50305316105585,38.66017758999163],[-85.58469927655415,38.6558476702067],[-85.87941101567998,38.57095232763015],[-85.99893916028611,38.663202377486925],[-86.11066309032105,38.671564372265664],[-86.28518060255016,38.54910943078903],[-86.3998270971639,38.3861330758337],[-86.60329986038971,38.314479441569915],[-86.70944536230832,38.3207994415601],[-86.93637323903937,38.172535867831925],[-87.0711074221636,38.278537901000675],[-87.17299383138044,38.21345942714339],[-87.31967361552773,38.277781626001854],[-87.56928638180699,38.28542444265668],[-87.66770804519587,38.23614063960815],[-87.52695765374767,38.16311298138817],[-87.6244237785964,38.082361311721854],[-87.66267845666204,37.973954976473465],[-87.92529640208772,37.95247033275683],[-88.0038593082158,37.80197398924042],[-88.15281576631787,37.81329879234784],[-88.16730035067042,37.939160169235834],[-88.30218363483607,37.99054420040608],[-88.32998714520954,38.048659922190836],[-88.49352005849738,38.125482471029954],[-88.6311108843255,38.106202603351505],[-88.76413185182736,38.17806929490666],[-88.82998988818343,38.410030276838256],[-88.9015054484891,38.51305383605336],[-88.86083842146888,38.602133693206724],[-88.87917695269044,38.74271410861354],[-88.77178517785711,38.82885575535482],[-88.80602122155398,39.019533159225546],[-88.60065317916445,39.252361449489115],[-88.6159236208074,39.40228137633977],[-88.31331161398543,39.53701672300559],[-88.25280321303768,39.633149627023045],[-88.34691274830828,39.76021516640918],[-88.27502343800319,39.93799841717487],[-88.25855151511212,40.12547344396722],[-88.31891874210174,40.18302008866954],[-88.16028524130633,40.308488074933166],[-88.15274373402633,40.36632100192668],[-88.29366848589092,40.44657689763545],[-88.35597041912757,40.54902828185146],[-88.33613843374167,40.64241824003983],[-88.20267621311552,40.66100141917764],[-88.1837822860615,40.543729458943005],[-87.79889417311728,40.39869738104312],[-87.73635233050601,40.469227944475335],[-87.40763128414125,40.5696018380695],[-87.25841114478953,40.64079127129236],[-86.9715068691932,40.860802387617525],[-86.94212378382213,40.94391621144683],[-86.95911866608742,41.10268046953371],[-86.78716454343765,41.21133736311509],[-86.63633680096342,41.1460944340497],[-86.51457912198578,41.269061928650444],[-86.23088303596779,41.18609914752926],[-86.23375738075498,41.274787609891575],[-86.09243879139103,41.34127334312171],[-85.91456276562542,41.30683234838352],[-85.71087661698328,41.354499018101194],[-85.51702618915914,41.28931853695235],[-85.34792140400498,41.32470810981408],[-85.18422720113409,41.18687505065304],[-85.21518802816934,41.1392640246853],[-85.20058742402534,40.915432845866064],[-85.06737628777375,40.83643761786368],[-85.09735203043556,40.73060307427795],[-84.98005098061765,40.65058718794381],[-84.75085634243175,40.57559785681019],[-84.42882532001494,40.55535281204993],[-84.42756247626687,40.41056663206638],[-84.56438467084621,40.39031097688945],[-84.62812519262224,40.29584634682777],[-84.74683666431298,40.28788370309013],[-84.91614180988353,40.07586861071087],[-84.97109123688159,40.10211357733681],[-85.19909814173599,40.031135581613626],[-85.2988931540811,39.81414668507546],[-85.26959985725154,39.7290080820826],[-85.35067206858406,39.43610066482893],[-85.36661153522596,39.24509779325041],[-85.30266198011691,39.161779520463085],[-85.2636984020524,38.998663279049595]]]},"properties":{"id":"0512","title":"Wabash"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-91.42204775082672,40.38066964877942],[-91.29426325519177,40.337929461345766],[-91.20856795844975,40.43530086119466],[-91.09775171799674,40.32592735094772],[-91.03284169309751,40.0445138847179],[-91.18089957099272,40.03335068056856],[-91.1980742032577,39.944582019248],[-91.08027676802391,39.88664818183793],[-91.01738433166315,39.80471161321509],[-90.8249594673785,39.71210551440049],[-90.77151174350314,39.62424582599522],[-90.66938177803667,39.570900770869684],[-90.71586720192283,39.349247905588754],[-90.6774504884408,39.3285847629125],[-90.6299031176813,38.98195624365883],[-90.58456800420998,38.89394155212881],[-90.21666182040605,39.122532280940675],[-89.9393162822949,39.084645869541134],[-90.04881400608326,38.860105335514675],[-90.12205688513626,38.80940667726003],[-90.30912985359583,38.904257673987786],[-90.57813116463666,38.72263355968636],[-90.7617047049767,38.68988727640385],[-90.91029662245438,38.6995601472222],[-91.00089441710543,38.77156713981873],[-91.41530388000388,38.88554924797518],[-91.54458937771983,39.002475432168694],[-91.55135478291766,39.06567731748726],[-91.73764380867021,39.11991855906973],[-91.87853413865986,39.021490970680816],[-91.98766088224045,39.086109316413854],[-92.14649546845226,39.118806644488075],[-92.16553669342267,39.202614756858],[-92.29096729114463,39.20400995060584],[-92.32091691088982,39.27020045987808],[-92.44059307320407,39.340540286852274],[-92.42866597634759,39.480274415802],[-92.47744555752189,39.59545807499825],[-92.46469151379165,39.70952124461286],[-92.49290497416456,40.07803756591585],[-92.60622159378033,40.15434750017238],[-92.62519180937585,40.243414903159135],[-92.56906248446302,40.30537747597964],[-92.56848053654721,40.40779993519567],[-92.74634740710445,40.78133868357418],[-92.51489866475538,40.80608817103575],[-92.11295434246261,40.70394874098599],[-91.80882718876802,40.59712298490177],[-91.60396015679436,40.44038375910344],[-91.42204775082672,40.38066964877942]]]},"properties":{"id":"0711","title":"Upper Mississippi-Salt"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-92.63651785414993,43.61298883855392],[-92.66569236868798,43.564421403212634],[-92.50273779706595,43.413804112821424],[-92.33713881294801,43.193517644413305],[-92.18371987881113,43.03479232486802],[-92.03561701549933,42.92196350733485],[-91.9961821301439,42.83779512413213],[-91.85638826265256,42.72286983264388],[-91.78597128047016,42.716762302445034],[-91.6861029868752,42.50323919756812],[-91.50339081736712,42.3946785789866],[-91.3879886060879,42.227673604245865],[-91.30098387809795,42.19797903762526],[-91.17912321891214,42.08283131280399],[-91.07333809095132,42.03289359308985],[-90.96247602237338,42.036931816000276],[-90.67012999678553,41.93384235053526],[-90.505953435582,41.90489215162188],[-90.41952381904952,41.957134696332446],[-90.12132681326239,41.90160284121032],[-90.0364082248525,42.048592782648825],[-89.88861993862355,42.078122860728],[-90.0236278863307,41.789394652842816],[-90.20075396626407,41.74238333729079],[-90.41589350343014,41.4910674158059],[-90.59171346878225,41.49021450226553],[-90.55331696050848,41.30890527963027],[-90.24190316620019,41.35918668476057],[-90.17787126629958,41.29701948694043],[-90.0078463946885,41.2866037244566],[-89.96655602183591,41.22852242558838],[-89.99087513429816,41.15899921007133],[-90.20704541521263,41.16755986734967],[-90.36875919516996,40.90895787400109],[-90.65386326139412,40.847661559512915],[-90.75367669353085,40.66163474834332],[-90.90337638079848,40.66885060562379],[-91.07637272323831,40.62557247756598],[-91.12479818982979,40.54829362456093],[-91.2161045782297,40.541536187071415],[-91.20856795844975,40.43530086119466],[-91.29426325519177,40.337929461345766],[-91.42204775082672,40.38066964877942],[-91.44212741433722,40.51523052565392],[-91.51858081526024,40.565997122450085],[-91.59556666409907,40.724200074287864],[-91.82786409290514,40.80616466791065],[-92.07654992585248,40.96605536870413],[-92.23410002664957,40.95058534268645],[-92.58243470110892,41.188836279816655],[-92.56444825947017,41.24924024326458],[-92.71624711548452,41.34049220562292],[-92.96012884635599,41.41021676176473],[-93.18855116370975,41.58849734273798],[-93.26187931880429,41.59617449897604],[-93.46427790182344,41.74174858104175],[-93.59931667453048,41.86932404230208],[-93.6786981869073,41.885060892277636],[-93.95168491044188,42.20600775427948],[-93.88343360117284,42.315470512442914],[-93.79184546277332,42.33945109678069],[-93.7038878889515,42.438811275793114],[-93.63688349530554,42.6082539390718],[-93.75941286907369,43.03899733006983],[-93.86922813973655,43.148820758024385],[-93.75659425345304,43.293279460925135],[-93.79367088881213,43.340672665018246],[-93.71886524309491,43.46531112940812],[-93.65957625881197,43.47514683668453],[-93.5919020412087,43.631340113525425],[-93.47892698721739,43.7489165925096],[-93.26158817713804,43.823213364269236],[-93.08798605865752,43.84117460069973],[-93.00041476296013,43.95008125469735],[-92.81899865178337,43.909655712051745],[-92.80003736014612,43.761975827905985],[-92.69776849676322,43.71205979881677],[-92.63651785414993,43.61298883855392]]]},"properties":{"id":"0708","title":"Upper Mississippi-Iowa-Skunk-Wapsipinicon"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.46680650853887,43.65650347806968],[-88.32890397958624,43.55022991573463],[-88.26066131094217,43.42835746071546],[-88.29179228068551,43.358117409782835],[-88.19896245687124,43.20293188919038],[-88.3299788004179,43.117001630990444],[-88.36941710869002,43.01356080927599],[-88.47455320436018,42.88866610113655],[-88.67436406759168,42.75259981280607],[-88.58600903335383,42.67435917959415],[-88.48160958976587,42.6633358077363],[-88.52723834594502,42.53596035376734],[-88.4698357356175,42.42848739768414],[-88.53222653968731,42.36148006445484],[-88.36389888057357,42.279969549997986],[-88.35553366600323,42.11270616171595],[-88.4866842105913,42.08672852008965],[-88.47175819394784,41.902162528709425],[-88.76315934245383,41.85935804856757],[-89.01623291601936,41.67520180927005],[-89.0726498148901,41.70521069151516],[-89.30957678639737,41.641351675989256],[-89.57414261411174,41.52950603345454],[-89.66888017333963,41.32301527856674],[-89.96655602183591,41.22852242558838],[-90.0078463946885,41.2866037244566],[-90.17787126629958,41.29701948694043],[-90.24190316620019,41.35918668476057],[-90.55331696050848,41.30890527963027],[-90.59171346878225,41.49021450226553],[-90.41589350343014,41.4910674158059],[-90.20075396626407,41.74238333729079],[-90.0236278863307,41.789394652842816],[-89.88861993862355,42.078122860728],[-89.73294661386524,42.13948563459104],[-89.87760190114068,42.29322589164411],[-90.00831411239614,42.339703388446935],[-89.92409060315185,42.43524334559032],[-90.03181804152632,42.52446094857686],[-90.10736677995072,42.5208921506657],[-90.27659715156301,42.63086871091167],[-90.26930207969934,42.688350774364096],[-90.3848342993117,42.7607999221683],[-90.43379269090235,42.90505043444443],[-90.41296128468468,42.96284283018804],[-90.24664322556788,42.967875906221934],[-89.85100758972362,43.02851770091945],[-89.76172010027886,43.00206426554382],[-89.42110441018258,43.33619553690022],[-89.29717961766659,43.362459996234406],[-89.21878285007995,43.45453987421649],[-89.13656232520759,43.45347122213485],[-89.04386434722647,43.60490409377479],[-88.9454992494625,43.621819028123525],[-88.76569429765823,43.779069760171126],[-88.65120858741926,43.75729698832993],[-88.58475498127245,43.65523056036335],[-88.46680650853887,43.65650347806968]]]},"properties":{"id":"0709","title":"Rock"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-90.41296128468468,42.96284283018804],[-90.43379269090235,42.90505043444443],[-90.3848342993117,42.7607999221683],[-90.26930207969934,42.688350774364096],[-90.27659715156301,42.63086871091167],[-90.10736677995072,42.5208921506657],[-90.03181804152632,42.52446094857686],[-89.92409060315185,42.43524334559032],[-90.00831411239614,42.339703388446935],[-89.87760190114068,42.29322589164411],[-89.73294661386524,42.13948563459104],[-89.88861993862355,42.078122860728],[-90.0364082248525,42.048592782648825],[-90.12132681326239,41.90160284121032],[-90.41952381904952,41.957134696332446],[-90.505953435582,41.90489215162188],[-90.67012999678553,41.93384235053526],[-90.96247602237338,42.036931816000276],[-91.07333809095132,42.03289359308985],[-91.17912321891214,42.08283131280399],[-91.30098387809795,42.19797903762526],[-91.3879886060879,42.227673604245865],[-91.50339081736712,42.3946785789866],[-91.6861029868752,42.50323919756812],[-91.78597128047016,42.716762302445034],[-91.85638826265256,42.72286983264388],[-91.9961821301439,42.83779512413213],[-92.03561701549933,42.92196350733485],[-92.18371987881113,43.03479232486802],[-92.33713881294801,43.193517644413305],[-92.50273779706595,43.413804112821424],[-92.66569236868798,43.564421403212634],[-92.63651785414993,43.61298883855392],[-92.49977573561222,43.62637460519977],[-92.24995103391666,43.51759344599361],[-92.00165576555207,43.55837920009702],[-91.81180906897174,43.48583004916793],[-91.57430552559038,43.5809783406869],[-91.49728543508496,43.66902638013357],[-91.37165828423826,43.683552909277694],[-91.30392627080175,43.75556579979093],[-91.1346015439812,43.795559772645504],[-90.90453706204664,43.80003435284692],[-90.79538692054945,43.74194646752039],[-90.8847614329107,43.546052762616114],[-90.95263821926369,43.460074999207905],[-90.92102129327111,43.321090049423674],[-91.1313417064863,43.00317860408376],[-90.97975025672162,42.95062796041532],[-90.85205988191979,42.98076380099354],[-90.47797242833377,42.979938097869876],[-90.41296128468468,42.96284283018804]]]},"properties":{"id":"0706","title":"Upper Mississippi-Maquoketa-Plum"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-87.68505374204398,42.075592256565244],[-87.68457288162801,42.075359798232284],[-87.61261568069807,41.88920257143786],[-87.61285159632268,41.88791531935658],[-87.62444711297138,41.77894959452567],[-87.51858702876069,41.62849233121756],[-87.3579288185934,41.6128319864502],[-87.4275980351519,41.52048558555185],[-87.43611288618035,41.38900572638096],[-87.27175749685216,41.364258831627694],[-87.19643313238572,41.43785140443015],[-87.02236783994755,41.41988313779137],[-86.98478302854755,41.55631584487128],[-86.82203644963352,41.6016913416758],[-86.7870187215629,41.64884058222765],[-86.48197576891135,41.749741828946014],[-86.45346948978897,41.78611866222292],[-86.34179786079562,41.756291881019195],[-86.30128974106685,41.5922135771072],[-86.10868917053244,41.55550454799754],[-85.97793960927709,41.474793111664496],[-85.91456276562542,41.30683234838352],[-86.09243879139103,41.34127334312171],[-86.23375738075498,41.274787609891575],[-86.23088303596779,41.18609914752926],[-86.51457912198578,41.269061928650444],[-86.63633680096342,41.1460944340497],[-86.78716454343765,41.21133736311509],[-86.95911866608742,41.10268046953371],[-86.94212378382213,40.94391621144683],[-86.9715068691932,40.860802387617525],[-87.25841114478953,40.64079127129236],[-87.40763128414125,40.5696018380695],[-87.73635233050601,40.469227944475335],[-87.79889417311728,40.39869738104312],[-88.1837822860615,40.543729458943005],[-88.20267621311552,40.66100141917764],[-88.10642216326488,40.75425222840789],[-88.11697427262351,40.972574179110666],[-88.2610231411499,40.93497610625235],[-88.42168847423386,40.934859430210906],[-88.58224568544301,40.99347424991157],[-88.63452366765353,41.058445518560745],[-88.69047594777498,41.26654983073769],[-88.95635414944564,41.443901127337426],[-88.92716006199095,41.52408460325461],[-89.0738982044715,41.60168520209248],[-89.01623291601936,41.67520180927005],[-88.76315934245383,41.85935804856757],[-88.47175819394784,41.902162528709425],[-88.4866842105913,42.08672852008965],[-88.35553366600323,42.11270616171595],[-88.36389888057357,42.279969549997986],[-88.53222653968731,42.36148006445484],[-88.4698357356175,42.42848739768414],[-88.52723834594502,42.53596035376734],[-88.48160958976587,42.6633358077363],[-88.58600903335383,42.67435917959415],[-88.67436406759168,42.75259981280607],[-88.47455320436018,42.88866610113655],[-88.36941710869002,43.01356080927599],[-88.3299788004179,43.117001630990444],[-88.19896245687124,43.20293188919038],[-88.10956574138504,43.15486697155666],[-88.12714589344108,43.071226872728175],[-88.0607454539608,42.859375454306985],[-88.08510730288134,42.68256986916475],[-87.92684406666865,42.69905266913918],[-87.9387065833169,42.60025737450087],[-87.86687207822007,42.545849630835335],[-87.88439396673454,42.38743508524789],[-87.82934696161163,42.21977878342477],[-87.68505374204398,42.075592256565244]]]},"properties":{"id":"0712","title":"Upper Illinois"}}
    ]
},
{
    title: "HUC6 Watersheds",
    items: [
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-90.41296128468468,42.96284283018804],[-90.3848342993117,42.7607999221683],[-90.27659715156301,42.63086871091167],[-89.92409060315185,42.43524334559032],[-90.00831411239614,42.339703388446935],[-89.87760190114068,42.29322589164411],[-89.73294661386524,42.13948563459104],[-89.88861993862355,42.078122860728],[-90.0364082248525,42.048592782648825],[-90.12132681326239,41.90160284121032],[-90.41952381904952,41.957134696332446],[-90.505953435582,41.90489215162188],[-90.67012999678553,41.93384235053526],[-90.96247602237338,42.036931816000276],[-91.17912321891214,42.08283131280399],[-91.30098387809795,42.19797903762526],[-91.3879886060879,42.227673604245865],[-91.50339081736712,42.3946785789866],[-91.6861029868752,42.50323919756812],[-91.78597128047016,42.716762302445034],[-91.85638826265256,42.72286983264388],[-91.9961821301439,42.83779512413213],[-92.03561701549933,42.92196350733485],[-92.18371987881113,43.03479232486802],[-92.33713881294801,43.193517644413305],[-92.50273779706595,43.413804112821424],[-92.66037962807127,43.55368555843762],[-92.62670151354018,43.62060619479206],[-92.49977573561222,43.62637460519977],[-92.24995103391666,43.51759344599361],[-92.00165576555207,43.55837920009702],[-91.81180906897174,43.48583004916793],[-91.57430552559038,43.5809783406869],[-91.49728543508496,43.66902638013357],[-91.37165828423826,43.683552909277694],[-91.30392627080175,43.75556579979093],[-91.1346015439812,43.795559772645504],[-90.90453706204664,43.80003435284692],[-90.79538692054945,43.74194646752039],[-90.95263821926369,43.460074999207905],[-90.92102129327111,43.321090049423674],[-91.1313417064863,43.00317860408376],[-90.97975025672162,42.95062796041532],[-90.85205988191979,42.98076380099354],[-90.41296128468468,42.96284283018804]]]},"properties":{"id":"070600","title":"Upper Mississippi-Maquoketa-Plum"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-87.68505374204398,42.0755922576069],[-87.68457288162801,42.075359798232284],[-87.61261568069807,41.88920257143786],[-87.61285159632268,41.88791531935658],[-87.62444711297138,41.77894959452567],[-87.51858702876069,41.62849233121756],[-87.3579288185934,41.6128319864502],[-87.4275980351519,41.52048558555185],[-87.43611288618035,41.38900572638096],[-87.27175749685216,41.364258831627694],[-87.19643313238572,41.43785140443015],[-87.02236783994755,41.41988313779137],[-86.98478302854755,41.55631584487128],[-86.7870187215629,41.64884058222765],[-86.48197576891135,41.749741828946014],[-86.45346948978897,41.78611866222292],[-86.34179786079562,41.756291881019195],[-86.30128974106685,41.5922135771072],[-86.10868917053244,41.55550454799754],[-85.97793960927709,41.474793111664496],[-85.91456276562542,41.30683234838352],[-86.09243879139103,41.34127334312171],[-86.23375738075498,41.274787609891575],[-86.23088303596779,41.18609914752926],[-86.51457912198578,41.269061928650444],[-86.63633680096342,41.1460944340497],[-86.78716454343765,41.21133736311509],[-86.95911866608742,41.10268046953371],[-86.9715068691932,40.860802387617525],[-87.25841114478953,40.64079127129236],[-87.40763128414125,40.5696018380695],[-87.73635233050601,40.469227944475335],[-87.79889417311728,40.39869738104312],[-88.1837822860615,40.543729458943005],[-88.20267621311552,40.66100141917764],[-88.10642216326488,40.75425222840789],[-88.11697427262351,40.972574179110666],[-88.42168847423386,40.934859430210906],[-88.58224568544301,40.99347424991157],[-88.63452366765353,41.058445518560745],[-88.69047594777498,41.26654983073769],[-88.95635414944564,41.443901127337426],[-88.92716006199095,41.52408460325461],[-89.0738982044715,41.60168520209248],[-89.01623291601936,41.67520180927005],[-88.76315934245383,41.85935804856757],[-88.47175819394784,41.902162528709425],[-88.4866842105913,42.08672852008965],[-88.35553366600323,42.11270616171595],[-88.36389888057357,42.279969549997986],[-88.53222653968731,42.36148006445484],[-88.4698357356175,42.42848739768414],[-88.52723834594502,42.53596035376734],[-88.48160958976587,42.6633358077363],[-88.58600903335383,42.67435917959415],[-88.67436406759168,42.75259981280607],[-88.47455320436018,42.88866610113655],[-88.3299788004179,43.117001630990444],[-88.19896245687124,43.20293188919038],[-88.10956574138504,43.15486697155666],[-88.12714589344108,43.071226872728175],[-88.0607454539608,42.859375454306985],[-88.08510730288134,42.68256986916475],[-87.92684406666865,42.69905266913918],[-87.9387065833169,42.60025737450087],[-87.86687207822007,42.545849630835335],[-87.88439396673454,42.38743508524789],[-87.82934696161163,42.21977878342477],[-87.68505374204398,42.0755922576069]]]},"properties":{"id":"071200","title":"Upper Illinois"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-89.70329827953623,39.20198384227564],[-89.76193023673687,39.02214831338813],[-89.76172400444557,39.021558018597375],[-89.80807609291526,38.97463983533686],[-89.80984948041254,38.97178277596629],[-89.87566597197701,38.87302950216127],[-89.87577591260185,38.872498812578726],[-89.8816546886344,38.8432170469992],[-89.88167194488437,38.84268611887501],[-89.88135907925982,38.836224558468416],[-89.88143111155142,38.83538753034469],[-89.89341465528281,38.806436302264615],[-89.89408020111512,38.80564442518255],[-89.94946987394582,38.64867577125949],[-89.94979111248693,38.64826086396846],[-90.09525285080286,38.45696908405705],[-90.16691766006659,38.433227121593916],[-90.14239687677133,38.321012438434764],[-90.19760862981065,38.23802021252192],[-90.08524701956838,38.14544540745726],[-90.01775587488146,38.02664464826671],[-89.88289967925743,37.98031738271362],[-89.79642395126666,38.08931139191941],[-89.52963117668082,38.22276887400392],[-89.44689376014253,38.29518801139153],[-89.31167972910242,38.32671508321761],[-89.08885676486494,38.47640061527693],[-88.90157202661402,38.49419772774928],[-88.82998988818343,38.410030276838256],[-88.76413185182736,38.17806929490666],[-88.6311108843255,38.106202603351505],[-88.68731457694656,37.947723806722536],[-88.74519971748174,37.92405936196758],[-88.78878659762239,37.72892454352046],[-88.88162270997833,37.70375600501785],[-89.01835522851604,37.55726853024527],[-89.19531089907474,37.561581284405236],[-89.26539070209094,37.50932446156969],[-89.0277833722514,37.399335264865385],[-88.93569315260271,37.327148586852445],[-89.0608809076167,37.21684640681531],[-89.24339617087509,37.14429101317796],[-89.2564015177299,37.092140316383905],[-89.13808670124689,36.9846176478008],[-89.137152292915,36.97285348636069],[-89.24350959170823,37.00706797589095],[-89.38224246440956,36.95114938535272],[-89.4476456747247,37.093535698673406],[-89.72870423574682,37.231458576584316],[-90.00175275303133,37.10438889969822],[-90.38782985555702,37.14508657463506],[-90.4176705378024,37.22225245680693],[-90.27962770572498,37.42093068566521],[-90.23664980787504,37.54929174900764],[-90.27769504114468,37.777009996570825],[-90.3930853680489,37.848822717292705],[-90.63383905100852,37.758256297641594],[-90.72346957482773,37.64073566969904],[-90.88204604333157,37.70802084980289],[-91.07099855137164,37.67257018839962],[-91.18383913452982,37.57514366355082],[-91.28722996249434,37.58097607291677],[-91.37624482798117,37.498735998044424],[-91.54465240480306,37.57653715104868],[-91.59726915680477,37.518600737596955],[-91.78950481171466,37.57076745001598],[-91.74738402011343,37.68972790295629],[-91.75958397321949,38.09706370232402],[-91.79152658254486,38.173703632413435],[-91.74049693054076,38.28229215099486],[-91.6057836474165,38.293577970769036],[-91.31034640933342,38.41749181328504],[-91.09623516174912,38.42158833827864],[-91.04564862641098,38.480335291312485],[-90.85967746107463,38.47088652674381],[-90.68290991030733,38.581944124488075],[-90.61202852708402,38.56600670472113],[-90.35059350769819,38.700206432637856],[-90.29509687445096,38.780519040846514],[-90.12205688513626,38.80940667726003],[-90.04881400608326,38.860105335514675],[-89.9393162822949,39.084645869541134],[-89.70329827953623,39.20198384227564]]]},"properties":{"id":"071401","title":"Upper Mississippi-Meramec"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-91.42204775082672,40.38066964877942],[-91.29426325519177,40.337929461345766],[-91.20856795844975,40.43530086119466],[-91.09775171799674,40.32592735094772],[-91.03284169309751,40.0445138847179],[-91.18089957099272,40.03335068056856],[-91.1980742032577,39.944582019248],[-91.01738433166315,39.80471161321509],[-90.8249594673785,39.71210551440049],[-90.66938177803667,39.570900770869684],[-90.71586720192283,39.349247905588754],[-90.6774504884408,39.3285847629125],[-90.6299031176813,38.98195624365883],[-90.58456800420998,38.89394155212881],[-90.21666182040605,39.122532280940675],[-89.9393162822949,39.084645869541134],[-90.04881400608326,38.860105335514675],[-90.12205688513626,38.80940667726003],[-90.30912985359583,38.904257673987786],[-90.57813116463666,38.72263355968636],[-90.7617047049767,38.68988727640385],[-90.91029662245438,38.6995601472222],[-91.00089441710543,38.77156713981873],[-91.41530388000388,38.88554924797518],[-91.54458937771983,39.002475432168694],[-91.55135478291766,39.06567731748726],[-91.73764380867021,39.11991855906973],[-91.87853413865986,39.021490970680816],[-91.98766088224045,39.086109316413854],[-92.14649546845226,39.118806644488075],[-92.16553669342267,39.202614756858],[-92.29096729114463,39.20400995060584],[-92.44059307320407,39.340540286852274],[-92.42866597634759,39.480274415802],[-92.47744555752189,39.59545807499825],[-92.46469151379165,39.70952124461286],[-92.49290497416456,40.07803756591585],[-92.60622159378033,40.15434750017238],[-92.62519180937585,40.243414903159135],[-92.56848053654721,40.40779993519567],[-92.74634740710445,40.78133868357418],[-92.51489866475538,40.80608817103575],[-92.11295434246261,40.70394874098599],[-91.80882718876802,40.59712298490177],[-91.60396015679436,40.44038375910344],[-91.42204775082672,40.38066964877942]]]},"properties":{"id":"071100","title":"Upper Mississippi-Salt"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-89.01623291601936,41.67520180927005],[-89.0738982044715,41.60168520209248],[-88.92716006199095,41.52408460325461],[-88.95635414944564,41.443901127337426],[-88.69047594777498,41.26654983073769],[-88.63452366765353,41.058445518560745],[-88.58224568544301,40.99347424991157],[-88.42168847423386,40.934859430210906],[-88.11697427262351,40.972574179110666],[-88.10642216326488,40.75425222840789],[-88.20267621311552,40.66100141917764],[-88.33613843374167,40.64241824003983],[-88.35597041912757,40.54902828185146],[-88.29366848589092,40.44657689763545],[-88.15274373402633,40.36632100192668],[-88.16028524130633,40.308488074933166],[-88.31891874210174,40.18302008866954],[-88.46708527937176,40.01928222538203],[-88.68258494882889,39.92645338490115],[-88.76938543515251,39.765068207026616],[-88.87451652457264,39.708348456073],[-88.88199929226937,39.50085789285339],[-88.99052355460094,39.49103138141032],[-89.0367051149459,39.405627134667895],[-89.33971939364227,39.317901077512374],[-89.62008363904039,39.34829831392352],[-89.70329827953623,39.20198384227564],[-89.9393162822949,39.084645869541134],[-90.21666182040605,39.122532280940675],[-90.58456800420998,38.89394155212881],[-90.6299031176813,38.98195624365883],[-90.6774504884408,39.3285847629125],[-90.71586720192283,39.349247905588754],[-90.66938177803667,39.570900770869684],[-90.8249594673785,39.71210551440049],[-91.01738433166315,39.80471161321509],[-91.1980742032577,39.944582019248],[-91.18089957099272,40.03335068056856],[-91.03284169309751,40.0445138847179],[-91.09775171799674,40.32592735094772],[-91.20856795844975,40.43530086119466],[-91.2161045782297,40.541536187071415],[-91.07637272323831,40.62557247756598],[-90.90337638079848,40.66885060562379],[-90.75367669353085,40.66163474834332],[-90.65386326139412,40.847661559512915],[-90.36875919516996,40.90895787400109],[-90.20704541521263,41.16755986734967],[-89.99087513429816,41.15899921007133],[-89.96655602183591,41.22852242558838],[-89.66888017333963,41.32301527856674],[-89.57414261411174,41.52950603345454],[-89.30957678639737,41.641351675989256],[-89.0726498148901,41.70521069151516],[-89.01623291601936,41.67520180927005]]]},"properties":{"id":"071300","title":"Lower Illinois"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.94519741092131,36.98950532070984],[-89.06173677011537,36.96305245095925],[-89.06469468052745,36.842748909479326],[-89.19210464595471,36.57183655989985],[-89.37608004462749,36.57206720052449],[-89.45460509658892,36.432811736157305],[-89.44033049765272,36.297271726992676],[-89.53926085791585,36.242504118744364],[-89.69448089204991,36.01106262222862],[-89.63722991922214,35.913963318212666],[-89.4776000965532,35.84318160998919],[-89.6620254514753,35.766029358025605],[-89.8913650511193,35.612083434306214],[-89.86392974803692,35.54328536878802],[-90.05080413837186,35.31642532434847],[-90.06406402585128,35.234134674476195],[-90.00807952906314,35.122318268399795],[-90.16129302986701,35.07365036118364],[-90.27742133697842,34.954860720743056],[-90.32712397960961,35.02256646438792],[-90.19890366001698,35.06740459452669],[-90.10080749766922,35.223362264076286],[-90.1901504496139,35.391030279441],[-90.04797459566788,35.48631417512644],[-89.92397422502705,35.64689457175223],[-89.96295673538322,35.73202618724508],[-89.87881381259712,35.77546283405263],[-89.74377758155674,35.91950192862072],[-89.73897726385587,36.00678320244356],[-89.61690065362865,36.15938989283171],[-89.70195438682998,36.287765006174084],[-89.55884571309377,36.374345014373034],[-89.57936716722861,36.56543225678479],[-89.41888308206103,36.55496011721772],[-89.36524022276927,36.63496135771851],[-89.2420449115022,36.581380489051696],[-89.17305732619263,36.68749926701196],[-89.18711012825412,36.816281398062074],[-89.137152292915,36.97285348636069],[-89.13808670124689,36.9846176478008],[-89.08604294820265,37.12184029758777],[-89.01535496914573,37.10098677887015],[-88.94519741092131,36.98950532070984]]]},"properties":{"id":"080101","title":"Lower Mississippi-Memphis"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-87.55028251412813,38.35726696442015],[-87.66770804519587,38.23614063960815],[-87.52695765374767,38.16311298138817],[-87.6244237785964,38.082361311721854],[-87.66267845666204,37.973954976473465],[-87.92529640208772,37.95247033275683],[-88.0038593082158,37.80197398924042],[-88.15281576631787,37.81329879234784],[-88.16730035067042,37.939160169235834],[-88.49352005849738,38.125482471029954],[-88.6311108843255,38.106202603351505],[-88.76413185182736,38.17806929490666],[-88.82998988818343,38.410030276838256],[-88.90157202661402,38.49419772774928],[-88.86083842146888,38.602133693206724],[-88.87917695269044,38.74271410861354],[-88.77178517785711,38.82885575535482],[-88.80602122155398,39.019533159225546],[-88.60065317916445,39.252361449489115],[-88.6159236208074,39.40228137633977],[-88.31331161398543,39.53701672300559],[-88.25280321303768,39.633149627023045],[-88.34691274830828,39.76021516640918],[-88.27502343800319,39.93799841717487],[-88.25855151511212,40.12547344396722],[-88.31891874210174,40.18302008866954],[-88.16028524130633,40.308488074933166],[-88.15274373402633,40.36632100192668],[-88.29366848589092,40.44657689763545],[-88.35597041912757,40.54902828185146],[-88.33613843374167,40.64241824003983],[-88.20267621311552,40.66100141917764],[-88.1837822860615,40.543729458943005],[-87.79889417311728,40.39869738104312],[-87.73635233050601,40.469227944475335],[-87.40763128414125,40.5696018380695],[-87.25841114478953,40.64079127129236],[-86.9715068691932,40.860802387617525],[-86.95911866608742,41.10268046953371],[-86.78716454343765,41.21133736311509],[-86.63633680096342,41.1460944340497],[-86.51457912198578,41.269061928650444],[-86.23088303596779,41.18609914752926],[-86.23375738075498,41.274787609891575],[-86.09243879139103,41.34127334312171],[-85.91456276562542,41.30683234838352],[-85.71087661698328,41.354499018101194],[-85.51702618915914,41.28931853695235],[-85.34792140400498,41.32470810981408],[-85.18422720113409,41.18687505065304],[-85.21518802816934,41.1392640246853],[-85.20058742402534,40.915432845866064],[-85.06737628777375,40.83643761786368],[-85.09735203043556,40.73060307427795],[-84.98005098061765,40.65058718794381],[-84.75085634243175,40.57559785681019],[-84.42882532001494,40.55535281204993],[-84.42756247626687,40.41056663206638],[-84.56438467084621,40.39031097688945],[-84.62812519262224,40.29584634682777],[-84.74683666431298,40.28788370309013],[-84.76576277782527,40.20254862718093],[-85.15027716993677,40.18885684386885],[-85.32382762695903,40.23309889796684],[-85.33064288215678,40.28484100517818],[-85.49994239543571,40.313786868674924],[-85.60138389736153,40.37515461857964],[-85.7904486689431,40.373774770665136],[-85.9174978468709,40.30010402598782],[-86.19794544956056,40.30204067077648],[-86.3011242494004,40.10982087628315],[-86.42371166171012,39.991193149384],[-86.53098524071032,40.01457074622272],[-86.81069469965115,39.77022138618531],[-86.93582657445688,39.769654814311195],[-87.2010985073785,39.41314378673957],[-87.29214954890381,39.384807202408524],[-87.19705594175974,39.155558887139364],[-87.28272361141848,39.040404888359774],[-87.3044833301347,38.84159621783505],[-87.41410834142283,38.725971014889524],[-87.53820899748024,38.52111020895751],[-87.72136995761258,38.47833508402391],[-87.7458868679912,38.39646700185932],[-87.55028251412813,38.35726696442015]]]},"properties":{"id":"051201","title":"Wabash"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-89.88861993862355,42.078122860728],[-90.0236278863307,41.789394652842816],[-90.20075396626407,41.74238333729079],[-90.41589350343014,41.4910674158059],[-90.59171346878225,41.49021450226553],[-90.55331696050848,41.30890527963027],[-90.24190316620019,41.35918668476057],[-90.17787126629958,41.29701948694043],[-90.0078463946885,41.2866037244566],[-89.96655602183591,41.22852242558838],[-89.99087513429816,41.15899921007133],[-90.20704541521263,41.16755986734967],[-90.36875919516996,40.90895787400109],[-90.65386326139412,40.847661559512915],[-90.75367669353085,40.66163474834332],[-90.90337638079848,40.66885060562379],[-91.07637272323831,40.62557247756598],[-91.2161045782297,40.541536187071415],[-91.20856795844975,40.43530086119466],[-91.29426325519177,40.337929461345766],[-91.42204775082672,40.38066964877942],[-91.44212741433722,40.51523052565392],[-91.51858081526024,40.565997122450085],[-91.59556666409907,40.724200074287864],[-91.82786409290514,40.80616466791065],[-92.07654992585248,40.96605536870413],[-92.23410002664957,40.95058534268645],[-92.58243470110892,41.188836279816655],[-92.56444825947017,41.24924024326458],[-92.71624711548452,41.34049220562292],[-92.96012884635599,41.41021676176473],[-93.18855116370975,41.58849734273798],[-93.26187931880429,41.59617449897604],[-93.46427790182344,41.74174858104175],[-93.59931667453048,41.86932404230208],[-93.6786981869073,41.885060892277636],[-93.95168491044188,42.20600775427948],[-93.88343360117284,42.315470512442914],[-93.79184546277332,42.33945109678069],[-93.61331823388377,42.511558269430225],[-93.48134503096361,42.34899557176584],[-93.39919353734115,42.16845727412942],[-93.29974712291215,42.12023206587094],[-93.27236504170469,42.01342807020342],[-93.03609885873806,41.94245945885524],[-92.82910281426769,41.85195799441237],[-92.70793450820577,41.76309265809198],[-92.73871719253299,41.69285827695097],[-92.51816535329198,41.598255863556176],[-92.39476328160856,41.48000102311471],[-92.16859967362626,41.43779341693022],[-91.89549010738352,41.434123266935956],[-91.79186776900269,41.40758692322714],[-91.68554350458442,41.29154283382388],[-91.41689026958471,41.16067307256873],[-91.28206933333564,41.05223831544538],[-91.1318814523188,41.01959859570434],[-91.04313239412323,41.1191220767999],[-91.18976486472894,41.27942970571769],[-91.15977071581716,41.432140391939015],[-90.95227595780591,41.55228881779419],[-90.7805459007808,41.52406373450464],[-90.72159771545563,41.59936583959609],[-90.91149774120254,41.64664303327271],[-90.93760923387032,41.742445183124005],[-91.05485684722998,41.81319802363919],[-91.13228950231814,41.92175018492901],[-91.23674199173934,41.896909649550935],[-91.3585560728003,41.95687994633283],[-91.36301943529332,42.06984841699085],[-91.61888785468784,42.20125004595354],[-91.65084512338825,42.2582511281567],[-91.85559179494544,42.31693962494063],[-91.87027044075597,42.4315391758044],[-92.01122071657886,42.475860785110626],[-92.24443120996688,42.5932455078451],[-92.3975242430626,42.855387077229864],[-92.39934102430976,42.9387491885588],[-92.56601706155107,43.20600441210229],[-92.66037962807127,43.55368555843762],[-92.50273779706595,43.413804112821424],[-92.33713881294801,43.193517644413305],[-92.18371987881113,43.03479232486802],[-92.03561701549933,42.92196350733485],[-91.9961821301439,42.83779512413213],[-91.85638826265256,42.72286983264388],[-91.78597128047016,42.716762302445034],[-91.6861029868752,42.50323919756812],[-91.50339081736712,42.3946785789866],[-91.3879886060879,42.227673604245865],[-91.30098387809795,42.19797903762526],[-91.17912321891214,42.08283131280399],[-90.96247602237338,42.036931816000276],[-90.67012999678553,41.93384235053526],[-90.505953435582,41.90489215162188],[-90.41952381904952,41.957134696332446],[-90.12132681326239,41.90160284121032],[-90.0364082248525,42.048592782648825],[-89.88861993862355,42.078122860728]]]},"properties":{"id":"070801","title":"Upper Mississippi-Skunk-Wapsipinicon"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.46680650853887,43.65650347806968],[-88.32890397958624,43.55022991573463],[-88.26066131094217,43.42835746071546],[-88.29179228068551,43.358117409782835],[-88.19896245687124,43.20293188919038],[-88.3299788004179,43.117001630990444],[-88.47455320436018,42.88866610113655],[-88.67436406759168,42.75259981280607],[-88.58600903335383,42.67435917959415],[-88.48160958976587,42.6633358077363],[-88.52723834594502,42.53596035376734],[-88.4698357356175,42.42848739768414],[-88.53222653968731,42.36148006445484],[-88.36389888057357,42.279969549997986],[-88.35553366600323,42.11270616171595],[-88.4866842105913,42.08672852008965],[-88.47175819394784,41.902162528709425],[-88.76315934245383,41.85935804856757],[-89.01623291601936,41.67520180927005],[-89.0726498148901,41.70521069151516],[-89.30957678639737,41.641351675989256],[-89.57414261411174,41.52950603345454],[-89.66888017333963,41.32301527856674],[-89.96655602183591,41.22852242558838],[-90.0078463946885,41.2866037244566],[-90.17787126629958,41.29701948694043],[-90.24190316620019,41.35918668476057],[-90.55331696050848,41.30890527963027],[-90.59171346878225,41.49021450226553],[-90.41589350343014,41.4910674158059],[-90.20075396626407,41.74238333729079],[-90.0236278863307,41.789394652842816],[-89.88861993862355,42.078122860728],[-89.73294661386524,42.13948563459104],[-89.87760190114068,42.29322589164411],[-90.00831411239614,42.339703388446935],[-89.92409060315185,42.43524334559032],[-90.27659715156301,42.63086871091167],[-90.3848342993117,42.7607999221683],[-90.41296128468468,42.96284283018804],[-90.24664322556788,42.967875906221934],[-89.85100758972362,43.02851770091945],[-89.76172010027886,43.00206426554382],[-89.42110441018258,43.33619553690022],[-89.29717961766659,43.362459996234406],[-89.13656232520759,43.45347122213485],[-89.04386434722647,43.60490409377479],[-88.9454992494625,43.621819028123525],[-88.76569429765823,43.779069760171126],[-88.65120858741926,43.75729698832993],[-88.58475498127245,43.65523056036335],[-88.46680650853887,43.65650347806968]]]},"properties":{"id":"070900","title":"Rock"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.94519741092131,36.98950532070984],[-89.01535496914573,37.10098677887015],[-89.08604294820265,37.12184029758777],[-89.13808670124689,36.9846176478008],[-89.2564015177299,37.092140316383905],[-89.24339617087509,37.14429101317796],[-89.0608809076167,37.21684640681531],[-88.93569315260271,37.327148586852445],[-89.0277833722514,37.399335264865385],[-89.26539070209094,37.50932446156969],[-89.19531089907474,37.561581284405236],[-89.01835522851604,37.55726853024527],[-88.88162270997833,37.70375600501785],[-88.78878659762239,37.72892454352046],[-88.74519971748174,37.92405936196758],[-88.68731457694656,37.947723806722536],[-88.6311108843255,38.106202603351505],[-88.49352005849738,38.125482471029954],[-88.16730035067042,37.939160169235834],[-88.15281576631787,37.81329879234784],[-88.0038593082158,37.80197398924042],[-87.92529640208772,37.95247033275683],[-87.66267845666204,37.973954976473465],[-87.6244237785964,38.082361311721854],[-87.52695765374767,38.16311298138817],[-87.66770804519587,38.23614063960815],[-87.55028251412813,38.35726696442015],[-87.42137873724488,38.29090565618981],[-87.17299383138044,38.21345942714339],[-87.0711074221636,38.278537901000675],[-86.93637323903937,38.172535867831925],[-86.70944536230832,38.3207994415601],[-86.58001587292586,38.281599135370925],[-86.61526808641281,38.097481638781744],[-86.48195922828637,37.71386557896051],[-86.84536454543064,37.773676604909326],[-87.19187373447613,37.78092351635644],[-87.40641385914313,37.9353933640333],[-87.49741328296022,37.903178454708325],[-87.49215549859338,37.71233207062954],[-87.60831565257973,37.66497451132807],[-87.69392124411354,37.56415453544287],[-87.6218811338087,37.434676468977216],[-87.50259590691047,37.35971373367687],[-87.49276066421743,37.25242486613507],[-87.54899255058848,37.1605322954444],[-87.4857597704783,37.00008961548508],[-87.60736045883124,36.97558960302314],[-87.65311456292687,37.03190267064406],[-87.8283718886965,37.08659139243417],[-88.00852180195852,37.1931255453938],[-88.03796709253783,37.27002990256608],[-88.240781701598,37.30236175668256],[-88.39333254823623,37.19178196727091],[-88.47171104707292,37.038212477925924],[-88.59282682084324,37.08771342159912],[-88.67364371550946,36.97550421864827],[-88.94519741092131,36.98950532070984]]]},"properties":{"id":"051402","title":"Lower Ohio"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.31891874210174,40.18302008866954],[-88.25855151511212,40.12547344396722],[-88.27502343800319,39.93799841717487],[-88.34691274830828,39.76021516640918],[-88.25280321303768,39.633149627023045],[-88.31331161398543,39.53701672300559],[-88.6159236208074,39.40228137633977],[-88.60065317916445,39.252361449489115],[-88.80602122155398,39.019533159225546],[-88.77178517785711,38.82885575535482],[-88.87917695269044,38.74271410861354],[-88.86083842146888,38.602133693206724],[-88.90157202661402,38.49419772774928],[-89.08885676486494,38.47640061527693],[-89.31167972910242,38.32671508321761],[-89.44689376014253,38.29518801139153],[-89.52963117668082,38.22276887400392],[-89.79642395126666,38.08931139191941],[-89.88289967925743,37.98031738271362],[-90.01775587488146,38.02664464826671],[-90.08524701956838,38.14544540745726],[-90.19760862981065,38.23802021252192],[-90.14239687677133,38.321012438434764],[-90.16691766006659,38.433227121593916],[-90.09525285080286,38.45696908405705],[-89.94979111248693,38.64826086396846],[-89.94946987394582,38.64867577125949],[-89.89408020111512,38.80564442518255],[-89.89341465528281,38.806436302264615],[-89.88143111155142,38.83538753034469],[-89.88135907925982,38.836224558468416],[-89.88167194488437,38.84268611887501],[-89.8816546886344,38.8432170469992],[-89.87577591260185,38.872498812578726],[-89.87566597197701,38.87302950216127],[-89.80984948041254,38.97178277596629],[-89.80807609291526,38.97463983533686],[-89.76172400444557,39.021558018597375],[-89.76193023673687,39.02214831338813],[-89.70329827953623,39.20198384227564],[-89.62008363904039,39.34829831392352],[-89.33971939364227,39.317901077512374],[-89.0367051149459,39.405627134667895],[-88.99052355460094,39.49103138141032],[-88.88199929226937,39.50085789285339],[-88.87451652457264,39.708348456073],[-88.76938543515251,39.765068207026616],[-88.68258494882889,39.92645338490115],[-88.46708527937176,40.01928222538203],[-88.31891874210174,40.18302008866954]]]},"properties":{"id":"071402","title":"Kaskaskia"}}
    ]
},
{
    title: "HUC8 Watersheds",
    items: [
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-89.70329827953623,39.20198384227564],[-89.71228502118896,39.09849006222794],[-89.76193023673687,39.02214831338813],[-89.76172400444557,39.021558018597375],[-89.80807609291526,38.97463983533686],[-89.80984948041254,38.97178277596629],[-89.87566597197701,38.87302950216127],[-89.87577591260185,38.872498812578726],[-89.8816546886344,38.8432170469992],[-89.88167194488437,38.84268611887501],[-89.88135907925982,38.836224558468416],[-89.88143111155142,38.83538753034469],[-89.89341465528281,38.806436302264615],[-89.89408020111512,38.80564442518255],[-89.94946987394582,38.64867577125949],[-89.94979111248693,38.64826086396846],[-90.09525285080286,38.45696908405705],[-90.16691766006659,38.433227121593916],[-90.14239687677133,38.321012438434764],[-90.19760862981065,38.23802021252192],[-90.08524701956838,38.14544540745726],[-89.93928316979492,37.97415906084814],[-90.14954227051027,37.9379045119461],[-90.3194951025381,37.86629027559894],[-90.40480106803068,37.97273451501701],[-90.56485687611558,38.0415936159518],[-90.62018390102969,38.20323577299257],[-90.5084606355781,38.437263579920966],[-90.34617205770502,38.39052288416019],[-90.30471951401938,38.47161959653431],[-90.3988572253316,38.54033877663602],[-90.446271509633,38.67070072330864],[-90.29509687445096,38.780519040846514],[-90.12205688513626,38.80940667726003],[-90.04881400608326,38.860105335514675],[-89.9393162822949,39.084645869541134],[-89.70329827953623,39.20198384227564]]]},"properties":{"id":"07140101","title":"Cahokia-Joachim"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-89.59184665366757,41.279592639050804],[-89.58876744742236,41.19046047877248],[-89.66438064834665,41.0983650737071],[-89.58125082555898,41.04736915711959],[-89.63225893068818,40.958130185383084],[-89.73841623364837,40.902273958386445],[-89.84491319389974,40.917822246903995],[-90.01733908946545,40.77637526899855],[-89.96894995308224,40.687823955594354],[-90.02840093736495,40.5287683058412],[-90.11803565389249,40.408336345611474],[-90.15201030175643,40.29120377079329],[-90.43928350026886,40.38786405710158],[-90.5174575251475,40.54097921103062],[-90.51488761160982,40.64977387023674],[-90.75743893831668,40.66131940980216],[-90.65386326139412,40.847661559512915],[-90.36875919516996,40.90895787400109],[-90.20704541521263,41.16755986734967],[-89.99087513429816,41.15899921007133],[-89.96655602183591,41.22852242558838],[-89.75404767112411,41.3113663546265],[-89.59184665366757,41.279592639050804]]]},"properties":{"id":"07130005","title":"Spoon"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.81570915174728,42.476853646567406],[-88.85923425167971,42.377988882137515],[-89.03600610140535,42.22671470737231],[-89.13635847833291,42.18327624285644],[-89.03784558994414,41.94537139635071],[-88.92709831199107,41.79445613095993],[-89.09719172943534,41.86311042981174],[-89.27265165103802,41.77451187994927],[-89.60386590052389,41.70257026547756],[-89.77096260338953,41.58740336253135],[-89.87786097197363,41.581526702123824],[-89.99654541345603,41.645396446816335],[-90.08705342789892,41.532719081366224],[-90.39569933158651,41.468384390841095],[-90.39091379409393,41.33838400770952],[-90.68815312800751,41.321013873361494],[-90.56777013340275,41.392825638875024],[-90.59171346878225,41.49021450226553],[-90.41589350343014,41.4910674158059],[-90.20075396626407,41.74238333729079],[-90.0236278863307,41.789394652842816],[-89.88861993862355,42.078122860728],[-89.7421479013509,42.08208460238848],[-89.70532856286644,42.143711633542864],[-89.46182567782773,42.23524448965077],[-89.24090186567065,42.25163511150032],[-89.06993494197769,42.457576520555676],[-88.92687485365809,42.52925343086105],[-88.81570915174728,42.476853646567406]]]},"properties":{"id":"07090005","title":"Lower Rock"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.35264880142438,40.591030631786225],[-88.35732043475048,40.56084812141643],[-88.51470319179782,40.46791661218566],[-88.74688918206243,40.468857710100906],[-89.06524191073498,40.566613380782485],[-89.17895007930844,40.56042746204207],[-89.32145205929561,40.43741995494133],[-89.46583727886315,40.36348662172276],[-89.59981960886353,40.43970248514614],[-89.79218734189828,40.36108208110153],[-89.75569794507987,40.5240701777235],[-89.56635049849882,40.495685462142546],[-89.53207818501033,40.61143559633791],[-89.33826718531117,40.6988146055773],[-89.42368770497023,40.77606553358237],[-89.30541435723717,40.82955682724935],[-89.05793703366294,40.856109713666456],[-89.03845626910987,40.891569112569755],[-88.86615467875231,40.76396287005946],[-88.56638572400931,40.62250975257069],[-88.35264880142438,40.591030631786225]]]},"properties":{"id":"07130004","title":"Mackinaw"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.35732043475048,40.56084812141643],[-88.29366848589092,40.44657689763545],[-88.15274373402633,40.36632100192668],[-88.16028524130633,40.308488074933166],[-88.31891874210174,40.18302008866954],[-88.46708527937176,40.01928222538203],[-88.68258494882889,39.92645338490115],[-88.76938543515251,39.765068207026616],[-88.87451652457264,39.708348456073],[-88.8870560005949,39.620233747876455],[-89.02313149725865,39.726224813336955],[-89.15743795850852,39.620124044751606],[-89.43540848203537,39.691488596724184],[-89.5405207954139,39.81408557674223],[-89.41245133519601,39.89641222765607],[-89.23875463859065,39.918052378664186],[-89.07571543051034,39.901260171398576],[-88.89451280995831,40.111078265864535],[-88.69484017068487,40.19963977093545],[-88.57158220525122,40.19519707927566],[-88.50554204702041,40.26601183854075],[-88.52910912719216,40.41190940602263],[-88.69382382172813,40.40944567790143],[-88.74688918206243,40.468857710100906],[-88.51470319179782,40.46791661218566],[-88.35732043475048,40.56084812141643]]]},"properties":{"id":"07130006","title":"Upper Sangamon"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.20267621311552,40.66100141917764],[-88.1837822860615,40.543729458943005],[-87.79889417311728,40.39869738104312],[-87.73635233050601,40.469227944475335],[-87.41693506225181,40.56934443598658],[-87.39439368832842,40.36117414047635],[-87.54689962246675,40.18356183554374],[-87.45175484448941,39.9526195567355],[-87.5900538921914,40.02420521183274],[-87.76454511900391,40.02408367745795],[-87.79251552208547,39.96670955775534],[-88.06707126540931,39.974869761909304],[-88.14536870174612,39.98739064105655],[-88.25565764844993,40.099098984633144],[-88.31891874210174,40.18302008866954],[-88.16028524130633,40.308488074933166],[-88.15274373402633,40.36632100192668],[-88.29366848589092,40.44657689763545],[-88.35732043475048,40.56084812141643],[-88.35264880142438,40.591030631786225],[-88.20267621311552,40.66100141917764]]]},"properties":{"id":"05120109","title":"Vermilion"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-89.56120870371512,43.08028075917241],[-89.43206764141559,42.992430129100455],[-89.45553381117082,42.938175358351316],[-89.25605393231376,42.776497937768966],[-89.2938505197551,42.676817067090326],[-89.22668636673438,42.60251922137235],[-89.19780883969588,42.435697294547936],[-89.40495915083267,42.41554728832921],[-89.72192449304902,42.72822941076055],[-89.71335257014562,42.89272350842191],[-89.76172010027886,43.00206426554382],[-89.67203111291809,43.07716060501059],[-89.56120870371512,43.08028075917241]]]},"properties":{"id":"07090004","title":"Sugar"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-89.96655602183591,41.22852242558838],[-89.99087513429816,41.15899921007133],[-90.20704541521263,41.16755986734967],[-90.36875919516996,40.90895787400109],[-90.65386326139412,40.847661559512915],[-90.75743893831668,40.66131940980216],[-90.90337638079848,40.66885060562379],[-91.07637272323831,40.62557247756598],[-91.2161045782297,40.541536187071415],[-91.20856795844975,40.43530086119466],[-91.29426325519177,40.337929461345766],[-91.42204775082672,40.38066964877942],[-91.44212741433722,40.51523052565392],[-91.51858081526024,40.565997122450085],[-91.59556666409907,40.724200074287864],[-91.4629175538883,40.762267389853776],[-91.19714111992585,40.715255438885094],[-91.29000936978167,40.87432374176319],[-91.40804525709848,40.941578219783764],[-91.28206933333564,41.05223831544538],[-91.1318814523188,41.01959859570434],[-91.01218068792127,41.161834479858555],[-90.91941268285689,41.27185085989612],[-90.68815312800751,41.321013873361494],[-90.39091379409393,41.33838400770952],[-90.24190316620019,41.35918668476057],[-90.17787126629958,41.29701948694043],[-90.0078463946885,41.2866037244566],[-89.96655602183591,41.22852242558838]]]},"properties":{"id":"07080104","title":"Flint-Henderson"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-89.9393162822949,39.084645869541134],[-90.04881400608326,38.860105335514675],[-90.12205688513626,38.80940667726003],[-90.30912985359583,38.904257673987786],[-90.57813116463666,38.72263355968636],[-90.7617047049767,38.68988727640385],[-90.99597326294639,38.73586340237415],[-90.89871600580568,38.80400728247673],[-90.69080913737844,38.84879274282389],[-90.68697195821773,38.93310996977635],[-90.65080420202383,38.94943359579264],[-90.51985965222713,38.91080483856098],[-90.2952039036175,39.07315534560064],[-90.1527350048803,39.13972079966396],[-90.09126648518401,39.093934045568346],[-89.9393162822949,39.084645869541134]]]},"properties":{"id":"07110009","title":"Peruque-Piasa"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-89.5405207954139,39.81408557674223],[-89.43540848203537,39.691488596724184],[-89.15743795850852,39.620124044751606],[-89.02313149725865,39.726224813336955],[-88.8870560005949,39.620233747876455],[-88.88199929226937,39.50085789285339],[-88.99052355460094,39.49103138141032],[-89.0367051149459,39.405627134667895],[-89.16089088558647,39.368337694100774],[-89.22447756777945,39.30997500148305],[-89.46459723407344,39.313722702518874],[-89.57495383286044,39.40835869612198],[-89.59655416928524,39.453460517927],[-89.82185766997719,39.46983513560991],[-89.9415537802081,39.517727080327234],[-89.8988843052743,39.57671544273569],[-90.02604455716028,39.67554256654057],[-89.5405207954139,39.81408557674223]]]},"properties":{"id":"07130007","title":"South Fork Sangamon"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.60458902290833,42.54742375791619],[-88.52723834594502,42.53596035376734],[-88.4698357356175,42.42848739768414],[-88.53222653968731,42.36148006445484],[-88.36389888057357,42.279969549997986],[-88.35553366600323,42.11270616171595],[-88.48375017205421,42.023498988937774],[-88.47175819394784,41.902162528709425],[-88.76315934245383,41.85935804856757],[-88.92246950470656,41.77671111223748],[-88.92709831199107,41.79445613095993],[-89.03784558994414,41.94537139635071],[-89.13635847833291,42.18327624285644],[-89.03600610140535,42.22671470737231],[-88.85923425167971,42.377988882137515],[-88.81570915174728,42.476853646567406],[-88.60458902290833,42.54742375791619]]]},"properties":{"id":"07090006","title":"Kishwaukee"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.19479913187774,42.007701582712286],[-88.27492497029505,41.665659612409854],[-88.44270453565957,41.61248675624239],[-88.55366019173738,41.53821975531599],[-88.64404589680538,41.41647899717162],[-88.84708002982359,41.34795466602799],[-88.96448669630803,41.46869579500725],[-88.92716006199095,41.52408460325461],[-89.0738982044715,41.60168520209248],[-89.01623291601936,41.67520180927005],[-88.92246950470656,41.77671111223748],[-88.76315934245383,41.85935804856757],[-88.47175819394784,41.902162528709425],[-88.48375017205421,42.023498988937774],[-88.19479913187774,42.007701582712286]]]},"properties":{"id":"07120007","title":"Lower Fox"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-89.88861993862355,42.078122860728],[-90.0236278863307,41.789394652842816],[-90.20075396626407,41.74238333729079],[-90.41589350343014,41.4910674158059],[-90.59171346878225,41.49021450226553],[-90.56777013340275,41.392825638875024],[-90.68815312800751,41.321013873361494],[-90.91941268285689,41.27185085989612],[-91.01218068792127,41.161834479858555],[-91.08027765864887,41.15271696216439],[-91.18976486472894,41.27942970571769],[-91.15977071581716,41.432140391939015],[-90.95227595780591,41.55228881779419],[-90.7805459007808,41.52406373450464],[-90.56823452090202,41.653020438471174],[-90.41230537426907,41.6368605239129],[-90.35022999624039,41.723780042528006],[-90.38516289410285,41.80862212572964],[-90.30767621297309,41.92723942242054],[-90.12132681326239,41.90160284121032],[-90.0364082248525,42.048592782648825],[-89.88861993862355,42.078122860728]]]},"properties":{"id":"07080101","title":"Copperas-Duck"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-89.69348118580149,38.995568247804385],[-89.6925844337195,38.99578237801239],[-89.5682017891209,38.89315107713003],[-89.59360934220649,38.74709469298176],[-89.5396141797903,38.71046403262193],[-89.5812677526423,38.48466810693077],[-89.44079233098535,38.32927358321359],[-89.60539109427151,38.180954040735514],[-89.79642395126666,38.08931139191941],[-89.8065793012509,38.00041034934907],[-89.93928316979492,37.97415906084814],[-90.08524701956838,38.14544540745726],[-90.19760862981065,38.23802021252192],[-90.14239687677133,38.321012438434764],[-90.16691766006659,38.433227121593916],[-90.09525285080286,38.45696908405705],[-89.94979111248693,38.64826086396846],[-89.94946987394582,38.64867577125949],[-89.89408020111512,38.80564442518255],[-89.89341465528281,38.806436302264615],[-89.88143111155142,38.83538753034469],[-89.88135907925982,38.836224558468416],[-89.88167194488437,38.84268611887501],[-89.8816546886344,38.8432170469992],[-89.87577591260185,38.872498812578726],[-89.87566597197701,38.87302950216127],[-89.80984948041254,38.97178277596629],[-89.80807609291526,38.97463983533686],[-89.76172400444557,39.021558018597375],[-89.76193023673687,39.02214831338813],[-89.71228502118896,39.09849006222794],[-89.69348118580149,38.995568247804385]]]},"properties":{"id":"07140204","title":"Lower Kaskaskia"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.6311108843255,38.106202603351505],[-88.68731457694656,37.947723806722536],[-88.74519971748174,37.92405936196758],[-88.78878659762239,37.72892454352046],[-88.88162270997833,37.70375600501785],[-89.01835522851604,37.55726853024527],[-89.26637197396445,37.54079273339585],[-89.39137584772874,37.620873303063206],[-89.49196434861426,37.61571729265455],[-89.49123442986541,37.73730102267416],[-89.60747067030997,37.88164660370006],[-89.56602246412433,37.93959397131846],[-89.60539109427151,38.180954040735514],[-89.44079233098535,38.32927358321359],[-89.31167972910242,38.32671508321761],[-89.08885676486494,38.47640061527693],[-88.90157202661402,38.49419772774928],[-88.82998988818343,38.410030276838256],[-88.76413185182736,38.17806929490666],[-88.6311108843255,38.106202603351505]]]},"properties":{"id":"07140106","title":"Big Muddy"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.3135655587767,38.02659711389174],[-88.49352005849738,38.125482471029954],[-88.6311108843255,38.106202603351505],[-88.76413185182736,38.17806929490666],[-88.82998988818343,38.410030276838256],[-88.90157202661402,38.49419772774928],[-88.86083842146888,38.602133693206724],[-88.87917695269044,38.74271410861354],[-88.78058219763511,38.800401212690645],[-88.62088701871636,38.78058606584642],[-88.62995664057729,38.69541096910365],[-88.42836309193183,38.40082094247754],[-88.24162591201338,38.22725340837195],[-88.17111331941447,38.128658712691674],[-88.3135655587767,38.02659711389174]]]},"properties":{"id":"05120115","title":"Skillet"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-87.6170996483994,38.64281073376861],[-87.48113051631884,38.63576030877954],[-87.53820899748024,38.52111020895751],[-87.72136995761258,38.47833508402391],[-87.7458868679912,38.39646700185932],[-87.55028251412813,38.35726696442015],[-87.66770804519587,38.23614063960815],[-87.52695765374767,38.16311298138817],[-87.6244237785964,38.082361311721854],[-87.66267845666204,37.973954976473465],[-87.92529640208772,37.95247033275683],[-88.0038593082158,37.80197398924042],[-88.14614764549492,37.787890553845614],[-88.15436015173213,37.90944889219861],[-88.0608705352106,37.934193760910205],[-88.07534111227147,38.11856582833235],[-88.02534145818242,38.63447189523987],[-87.99021000302861,38.75958573983735],[-87.89545623130067,38.665780813941296],[-87.81565130434126,38.706508817003055],[-87.6170996483994,38.64281073376861]]]},"properties":{"id":"05120113","title":"Lower Wabash"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-87.87314651050201,42.34921569364053],[-87.91323659481475,42.225881791748634],[-87.82654631786602,42.05482498472247],[-87.81716904704723,41.9547705713361],[-87.74095768362383,41.896388954760084],[-87.73717889821302,41.771152532037775],[-87.84384975429748,41.69804952173462],[-87.85825536260842,41.576040571923954],[-87.7479559544463,41.566854155271585],[-87.73630282946442,41.43528473985077],[-87.83342758452198,41.46966100333913],[-88.17859101940286,41.350328480607686],[-88.257327086989,41.38952339304683],[-88.2434634338855,41.543022430308554],[-88.27492497029505,41.665659612409854],[-88.19479913187774,42.007701582712286],[-88.12464556740326,42.09808245965536],[-88.03926763628579,42.29797891455337],[-88.05521640813606,42.45209718514752],[-88.11615429033316,42.55611605165268],[-88.08692312892015,42.686506785825316],[-87.92684406666865,42.69905266913918],[-87.9387065833169,42.60025737450087],[-87.86687207822007,42.545849630835335],[-87.87314651050201,42.34921569364053]]]},"properties":{"id":"07120004","title":"Des Plaines"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.06707126540931,39.974869761909304],[-88.03894490711963,39.9291919505219],[-87.80494130331618,39.868822965198945],[-87.86486632301484,39.6749161811249],[-87.76942235857967,39.60731733122981],[-87.84000316888677,39.547259542781376],[-87.84367162617275,39.320088121259005],[-87.79838303353472,39.28857934734958],[-87.76456532525384,39.03811460919667],[-87.71371800345781,38.948000612461556],[-87.61470750569481,38.92640763749506],[-87.51957327667583,38.75818431588118],[-87.6170996483994,38.64281073376861],[-87.81565130434126,38.706508817003055],[-87.89545623130067,38.665780813941296],[-87.99021000302861,38.75958573983735],[-88.09433282057535,38.85883856051663],[-88.10697463826403,38.95704344161419],[-88.36488360869703,39.10307049972084],[-88.45518318564024,39.22458178078227],[-88.37273160347655,39.35026419412884],[-88.38767818366165,39.48673016162536],[-88.20910847873051,39.58912366979973],[-88.33095053583304,39.70565742586888],[-88.32497621292566,39.88860273704324],[-88.27502343800319,39.93799841717487],[-88.25565764844993,40.099098984633144],[-88.14536870174612,39.98739064105655],[-88.06707126540931,39.974869761909304]]]},"properties":{"id":"05120112","title":"Embarras"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-86.45346948978897,41.78611866222292],[-86.48197576891135,41.749741828946014],[-86.7870187215629,41.64884058222765],[-86.98478302854755,41.55631584487128],[-87.02236783994755,41.41988313779137],[-87.19643313238572,41.43785140443015],[-87.27175749685216,41.364258831627694],[-87.43611288618035,41.38900572638096],[-87.48562208402018,41.45430386690458],[-87.35852462588412,41.53125882720184],[-87.3579288185934,41.6128319864502],[-87.51858702876069,41.62849233121756],[-87.53954479643647,41.73392406751225],[-87.52899525270283,41.733856296678994],[-87.43288192681035,41.64816907702033],[-87.18630758448478,41.629914707257],[-86.92230242968623,41.71385786546006],[-86.62177607911104,41.891830033933786],[-86.46441218664694,41.882859057906046],[-86.45346948978897,41.78611866222292]]]},"properties":{"id":"04040001","title":"Little Calumet-Galien"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-89.22447756777945,39.30997500148305],[-89.30045086974485,39.24040803596603],[-89.33096800094751,38.90804937710692],[-89.31516958118033,38.778062852308665],[-89.38341809149108,38.61517799110317],[-89.5812677526423,38.48466810693077],[-89.5396141797903,38.71046403262193],[-89.59360934220649,38.74709469298176],[-89.5682017891209,38.89315107713003],[-89.6925844337195,38.99578237801239],[-89.69348118580149,38.995568247804385],[-89.71228502118896,39.09849006222794],[-89.70329827953623,39.20198384227564],[-89.70145789724745,39.280122908821056],[-89.57495383286044,39.40835869612198],[-89.46459723407344,39.313722702518874],[-89.22447756777945,39.30997500148305]]]},"properties":{"id":"07140203","title":"Shoal"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-89.60539109427151,38.180954040735514],[-89.56602246412433,37.93959397131846],[-89.60747067030997,37.88164660370006],[-89.49123442986541,37.73730102267416],[-89.49196434861426,37.61571729265455],[-89.39137584772874,37.620873303063206],[-89.26637197396445,37.54079273339585],[-89.24595157087111,37.44673639500013],[-89.41911420289404,37.16012909752834],[-89.36475670297841,37.08332881118923],[-89.2528576844021,37.08674621326725],[-89.13808670124689,36.9846176478008],[-89.137152292915,36.97285348636069],[-89.24350959170823,37.00706797589095],[-89.38224246440956,36.95114938535272],[-89.44671097472616,37.153825238163165],[-89.5474707183198,37.244129707814636],[-89.64021006400918,37.23446660991294],[-89.58211592868264,37.41439755130034],[-89.64419957233628,37.488047886602715],[-89.76421703152499,37.49073185222352],[-90.13745438511233,37.688870376915986],[-90.241604671409,37.701160728980255],[-90.3194951025381,37.86629027559894],[-90.14954227051027,37.9379045119461],[-89.93928316979492,37.97415906084814],[-89.8065793012509,38.00041034934907],[-89.79642395126666,38.08931139191941],[-89.60539109427151,38.180954040735514]]]},"properties":{"id":"07140105","title":"Upper Mississippi-Cape Girardeau"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.94519741092131,36.98950532070984],[-89.06173677011537,36.96305245095925],[-89.06469468052745,36.842748909479326],[-89.19210464595471,36.57183655989985],[-89.37608004462749,36.57206720052449],[-89.45460509658892,36.432811736157305],[-89.44033049765272,36.297271726992676],[-89.53926085791585,36.242504118744364],[-89.69448089204991,36.01106262222862],[-89.63722991922214,35.913963318212666],[-89.55487060476662,35.91884613070505],[-89.4776000965532,35.84318160998919],[-89.6620254514753,35.766029358025605],[-89.8913650511193,35.612083434306214],[-89.86392974803692,35.54328536878802],[-90.05080413837186,35.31642532434847],[-90.06406402585128,35.234134674476195],[-90.00807952906314,35.122318268399795],[-90.16129302986701,35.07365036118364],[-90.27742133697842,34.954860720743056],[-90.32712397960961,35.02256646438792],[-90.19890366001698,35.06740459452669],[-90.10080749766922,35.223362264076286],[-90.1901504496139,35.391030279441],[-90.04797459566788,35.48631417512644],[-89.92397422502705,35.64689457175223],[-89.96295673538322,35.73202618724508],[-89.87881381259712,35.77546283405263],[-89.74377758155674,35.91950192862072],[-89.73897726385587,36.00678320244356],[-89.61690065362865,36.15938989283171],[-89.70195438682998,36.287765006174084],[-89.55884571309377,36.374345014373034],[-89.57936716722861,36.56543225678479],[-89.41888308206103,36.55496011721772],[-89.36524022276927,36.63496135771851],[-89.2420449115022,36.581380489051696],[-89.17305732619263,36.68749926701196],[-89.18711012825412,36.816281398062074],[-89.137152292915,36.97285348636069],[-89.13808670124689,36.9846176478008],[-89.08604294820265,37.12184029758777],[-89.01535496914573,37.10098677887015],[-88.94519741092131,36.98950532070984]]]},"properties":{"id":"08010100","title":"Lower Mississippi-Memphis"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-89.2528576844021,37.08674621326725],[-89.36475670297841,37.08332881118923],[-89.41911420289404,37.16012909752834],[-89.24595157087111,37.44673639500013],[-89.16167641475192,37.4645852741391],[-88.93569315260271,37.327148586852445],[-89.0608809076167,37.21684640681531],[-89.24339617087509,37.14429101317796],[-89.2528576844021,37.08674621326725]]]},"properties":{"id":"07140108","title":"Cache"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-89.01835522851604,37.55726853024527],[-88.83051828297431,37.51379933968775],[-88.83064756526574,37.38370418572299],[-88.64756370304997,37.32010367540505],[-88.51726830741887,37.152270733998876],[-88.40798178988018,37.14442530276108],[-88.47171104707292,37.038212477925924],[-88.59282682084324,37.08771342159912],[-88.67364371550946,36.97550421864827],[-88.94519741092131,36.98950532070984],[-89.01535496914573,37.10098677887015],[-89.08604294820265,37.12184029758777],[-89.13808670124689,36.9846176478008],[-89.2528576844021,37.08674621326725],[-89.24339617087509,37.14429101317796],[-89.0608809076167,37.21684640681531],[-88.93569315260271,37.327148586852445],[-89.16167641475192,37.4645852741391],[-89.24595157087111,37.44673639500013],[-89.26637197396445,37.54079273339585],[-89.01835522851604,37.55726853024527]]]},"properties":{"id":"05140206","title":"Lower Ohio"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.257327086989,41.38952339304683],[-88.22663137974496,41.2526215963843],[-88.14417729445631,41.14856571529583],[-88.15297836735931,41.013701891546816],[-88.20400831832177,40.9556846666369],[-88.42168847423386,40.934859430210906],[-88.58224568544301,40.99347424991157],[-88.68769700090434,41.2045212735423],[-88.74350405498438,41.31561261920319],[-88.84708002982359,41.34795466602799],[-88.64404589680538,41.41647899717162],[-88.55366019173738,41.53821975531599],[-88.44270453565957,41.61248675624239],[-88.27492497029505,41.665659612409854],[-88.2434634338855,41.543022430308554],[-88.257327086989,41.38952339304683]]]},"properties":{"id":"07120005","title":"Upper Illinois"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-87.20858377715854,40.68613800872197],[-87.41693506225181,40.56934443598658],[-87.73635233050601,40.469227944475335],[-87.79889417311728,40.39869738104312],[-88.1837822860615,40.543729458943005],[-88.20267621311552,40.66100141917764],[-88.10642216326488,40.75425222840789],[-88.11006630180094,40.97093654161324],[-87.95499426766662,41.06380168105238],[-87.74738670028052,41.04451558524903],[-87.56304792139997,41.088762557055304],[-87.51521579022426,41.04565433212224],[-87.31459430407733,41.02448431132177],[-87.1509217376647,41.13563266948256],[-87.07845977423551,41.11717904138624],[-86.90372264846508,41.18155742253629],[-86.95911866608742,41.10268046953371],[-86.94212378382213,40.94391621144683],[-86.9715068691932,40.860802387617525],[-87.20858377715854,40.68613800872197]]]},"properties":{"id":"07120002","title":"Iroquois"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-87.86259994593502,37.60848552079074],[-87.99377167906476,37.72917130602008],[-88.14963925590615,37.74486655078738],[-88.14614764549492,37.787890553845614],[-88.0038593082158,37.80197398924042],[-87.92529640208772,37.95247033275683],[-87.66267845666204,37.973954976473465],[-87.6244237785964,38.082361311721854],[-87.52695765374767,38.16311298138817],[-87.66770804519587,38.23614063960815],[-87.55028251412813,38.35726696442015],[-87.2431476771049,38.21767282609517],[-87.32094702698413,38.15006157828344],[-87.3430788571581,38.047981888858544],[-87.50499412982344,37.88481989536183],[-87.49215549859338,37.71233207062954],[-87.60831565257973,37.66497451132807],[-87.6924045441159,37.569037169810315],[-87.86259994593502,37.60848552079074]]]},"properties":{"id":"05140202","title":"Highland-Pigeon"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.74688918206243,40.468857710100906],[-88.69382382172813,40.40944567790143],[-88.52910912719216,40.41190940602263],[-88.50554204702041,40.26601183854075],[-88.57158220525122,40.19519707927566],[-88.69484017068487,40.19963977093545],[-88.89451280995831,40.111078265864535],[-89.07571543051034,39.901260171398576],[-89.23875463859065,39.918052378664186],[-89.41245133519601,39.89641222765607],[-89.47444788509978,40.002217147283545],[-89.78025073670847,40.02202085037777],[-89.82837704705042,40.16813168348432],[-89.7185475315959,40.242857877118354],[-89.46583727886315,40.36348662172276],[-89.32145205929561,40.43741995494133],[-89.17895007930844,40.56042746204207],[-89.06524191073498,40.566613380782485],[-88.74688918206243,40.468857710100906]]]},"properties":{"id":"07130009","title":"Salt"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.31891874210174,40.18302008866954],[-88.25565764844993,40.099098984633144],[-88.27502343800319,39.93799841717487],[-88.32497621292566,39.88860273704324],[-88.33095053583304,39.70565742586888],[-88.20910847873051,39.58912366979973],[-88.38767818366165,39.48673016162536],[-88.6159236208074,39.40228137633977],[-88.61943871559362,39.285303058812985],[-88.77992988409449,39.21219653184312],[-88.87760273394287,39.12640238093462],[-89.10568121900553,39.19597876311832],[-89.16089088558647,39.368337694100774],[-89.0367051149459,39.405627134667895],[-88.99052355460094,39.49103138141032],[-88.88199929226937,39.50085789285339],[-88.8870560005949,39.620233747876455],[-88.87451652457264,39.708348456073],[-88.76938543515251,39.765068207026616],[-88.68258494882889,39.92645338490115],[-88.46708527937176,40.01928222538203],[-88.31891874210174,40.18302008866954]]]},"properties":{"id":"07140201","title":"Upper Kaskaskia"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.10247827264601,43.011951215528484],[-88.0607454539608,42.859375454306985],[-88.08692312892015,42.686506785825316],[-88.11615429033316,42.55611605165268],[-88.05521640813606,42.45209718514752],[-88.03926763628579,42.29797891455337],[-88.12464556740326,42.09808245965536],[-88.19479913187774,42.007701582712286],[-88.48375017205421,42.023498988937774],[-88.35553366600323,42.11270616171595],[-88.36389888057357,42.279969549997986],[-88.53222653968731,42.36148006445484],[-88.4698357356175,42.42848739768414],[-88.52723834594502,42.53596035376734],[-88.60458902290833,42.54742375791619],[-88.58600903335383,42.67435917959415],[-88.67436406759168,42.75259981280607],[-88.47455320436018,42.88866610113655],[-88.3299788004179,43.117001630990444],[-88.19896245687124,43.20293188919038],[-88.10956574138504,43.15486697155666],[-88.10247827264601,43.011951215528484]]]},"properties":{"id":"07120006","title":"Upper Fox"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-87.20858377715854,40.68613800872197],[-87.00010261498215,40.64965373690359],[-86.8099870725689,40.52400368918194],[-86.87329483497064,40.46007708198954],[-86.77720964449475,40.352127019657075],[-86.73431340289471,40.2365395760865],[-86.88759214224012,40.18474784908352],[-87.05671558260258,40.04534161492495],[-87.14769402725307,40.05835098365475],[-87.24599087814215,39.97156340253946],[-87.25658670625069,39.86528367145439],[-87.14026386059788,39.857260767300204],[-87.07806408881947,39.929810378645925],[-86.90197101721776,39.885550741214615],[-86.83337423711595,39.96748215358747],[-86.63453039992459,39.93517321717928],[-86.81069469965115,39.77022138618531],[-86.93582657445688,39.769654814311195],[-87.0485671211569,39.623604837454536],[-87.22368980005177,39.595858130205954],[-87.38497198209308,39.62071711141738],[-87.50420779544964,39.75538546224999],[-87.47589171528529,39.80047285905499],[-87.58613464844751,39.90178248598107],[-87.80494130331618,39.868822965198945],[-88.03894490711963,39.9291919505219],[-88.06707126540931,39.974869761909304],[-87.79251552208547,39.96670955775534],[-87.76454511900391,40.02408367745795],[-87.5900538921914,40.02420521183274],[-87.45175484448941,39.9526195567355],[-87.54689962246675,40.18356183554374],[-87.39439368832842,40.36117414047635],[-87.41693506225181,40.56934443598658],[-87.20858377715854,40.68613800872197]]]},"properties":{"id":"05120108","title":"Middle Wabash-Little Vermilion"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-89.76172010027886,43.00206426554382],[-89.71335257014562,42.89272350842191],[-89.72192449304902,42.72822941076055],[-89.40495915083267,42.41554728832921],[-89.19780883969588,42.435697294547936],[-89.22668636673438,42.60251922137235],[-89.08099047112717,42.55844799435744],[-89.06993494197769,42.457576520555676],[-89.24090186567065,42.25163511150032],[-89.46182567782773,42.23524448965077],[-89.70532856286644,42.143711633542864],[-89.85247568242966,42.22927043549339],[-89.87760190114068,42.29322589164411],[-90.00831411239614,42.339703388446935],[-89.92409060315185,42.43524334559032],[-90.03181804152632,42.52446094857686],[-90.10736677995072,42.5208921506657],[-90.27659715156301,42.63086871091167],[-90.38391594097976,42.75747508571516],[-90.43379269090235,42.90505043444443],[-90.40474421803077,42.96634275518261],[-90.24664322556788,42.967875906221934],[-89.85100758972362,43.02851770091945],[-89.76172010027886,43.00206426554382]]]},"properties":{"id":"07090003","title":"Pecatonica"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-91.42204775082672,40.38066964877942],[-91.29426325519177,40.337929461345766],[-91.20856795844975,40.43530086119466],[-91.09775171799674,40.32592735094772],[-91.02561313998376,40.04764938783802],[-91.20788512720083,40.002051953533794],[-91.45445981119309,39.887776136002856],[-91.59836896930301,40.01717755559366],[-91.71583161182906,40.1684237178589],[-91.98040565933502,40.4178371883051],[-92.09260379249423,40.43879937993921],[-92.3620091597844,40.63810864212985],[-92.58942402505642,40.720058725335946],[-92.66651793222837,40.68051832227235],[-92.74634740710445,40.78133868357418],[-92.51489866475538,40.80608817103575],[-92.11295434246261,40.70394874098599],[-91.80882718876802,40.59712298490177],[-91.60396015679436,40.44038375910344],[-91.42204775082672,40.38066964877942]]]},"properties":{"id":"07110001","title":"Bear-Wyaconda"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-89.57495383286044,39.40835869612198],[-89.70145789724745,39.280122908821056],[-89.70329827953623,39.20198384227564],[-89.9393162822949,39.084645869541134],[-90.09126648518401,39.093934045568346],[-90.1527350048803,39.13972079966396],[-90.2952039036175,39.07315534560064],[-90.32588897127818,39.11683103719952],[-90.50645811370623,39.10764828617209],[-90.58894647191153,39.186846526674174],[-90.50109143038122,39.26386279113797],[-90.29910353798641,39.320297213967024],[-90.07664261229007,39.464042905410565],[-89.9415537802081,39.517727080327234],[-89.82185766997719,39.46983513560991],[-89.59655416928524,39.453460517927],[-89.57495383286044,39.40835869612198]]]},"properties":{"id":"07130012","title":"Macoupin"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.3135655587767,38.02659711389174],[-88.15436015173213,37.90944889219861],[-88.14614764549492,37.787890553845614],[-88.14963925590615,37.74486655078738],[-88.22622386307893,37.7084355060523],[-88.12587455594303,37.57121834168191],[-88.46936980645154,37.616013885362406],[-88.69581706130839,37.5319271979929],[-88.83051828297431,37.51379933968775],[-89.01835522851604,37.55726853024527],[-88.88162270997833,37.70375600501785],[-88.78878659762239,37.72892454352046],[-88.74519971748174,37.92405936196758],[-88.68731457694656,37.947723806722536],[-88.6311108843255,38.106202603351505],[-88.49352005849738,38.125482471029954],[-88.3135655587767,38.02659711389174]]]},"properties":{"id":"05140204","title":"Saline"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-89.7185475315959,40.242857877118354],[-89.82837704705042,40.16813168348432],[-89.78025073670847,40.02202085037777],[-89.47444788509978,40.002217147283545],[-89.41245133519601,39.89641222765607],[-89.5405207954139,39.81408557674223],[-90.02604455716028,39.67554256654057],[-89.97875566244198,39.861947181876246],[-90.11109317265323,39.937529001550615],[-90.41793433780197,39.981233987941096],[-90.3649797243425,40.0772170169588],[-90.16572821944345,40.062107089898916],[-89.95939363018039,40.172932509518546],[-89.92961672918494,40.272238080197724],[-89.7185475315959,40.242857877118354]]]},"properties":{"id":"07130008","title":"Lower Sangamon"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-86.90372264846508,41.18155742253629],[-87.07845977423551,41.11717904138624],[-87.1509217376647,41.13563266948256],[-87.31459430407733,41.02448431132177],[-87.51521579022426,41.04565433212224],[-87.56304792139997,41.088762557055304],[-87.74738670028052,41.04451558524903],[-87.95499426766662,41.06380168105238],[-88.11006630180094,40.97093654161324],[-88.20400831832177,40.9556846666369],[-88.15297836735931,41.013701891546816],[-88.14417729445631,41.14856571529583],[-88.22663137974496,41.2526215963843],[-88.257327086989,41.38952339304683],[-88.17859101940286,41.350328480607686],[-87.83342758452198,41.46966100333913],[-87.73630282946442,41.43528473985077],[-87.64623895877088,41.35043055352418],[-87.48562208402018,41.45430386690458],[-87.43611288618035,41.38900572638096],[-87.27175749685216,41.364258831627694],[-87.19643313238572,41.43785140443015],[-87.02236783994755,41.41988313779137],[-86.98478302854755,41.55631584487128],[-86.7870187215629,41.64884058222765],[-86.48197576891135,41.749741828946014],[-86.45346948978897,41.78611866222292],[-86.34179786079562,41.756291881019195],[-86.30128974106685,41.5922135771072],[-86.10868917053244,41.55550454799754],[-85.97793960927709,41.474793111664496],[-85.92969286768528,41.3056152515104],[-86.09243879139103,41.34127334312171],[-86.23375738075498,41.274787609891575],[-86.23088303596779,41.18609914752926],[-86.51457912198578,41.269061928650444],[-86.63633680096342,41.1460944340497],[-86.78716454343765,41.21133736311509],[-86.90372264846508,41.18155742253629]]]},"properties":{"id":"07120001","title":"Kankakee"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-90.75743893831668,40.66131940980216],[-90.51488761160982,40.64977387023674],[-90.5174575251475,40.54097921103062],[-90.43928350026886,40.38786405710158],[-90.6366015176709,40.27551114998431],[-90.55750695841868,40.09949666692421],[-90.54366947198179,39.9574072994364],[-90.729714046693,39.946326945286955],[-90.945777754691,40.05178302949827],[-91.02561313998376,40.04764938783802],[-91.09775171799674,40.32592735094772],[-91.20856795844975,40.43530086119466],[-91.2161045782297,40.541536187071415],[-91.07637272323831,40.62557247756598],[-90.90337638079848,40.66885060562379],[-90.75743893831668,40.66131940980216]]]},"properties":{"id":"07130010","title":"La Moine"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-89.70532856286644,42.143711633542864],[-89.7421479013509,42.08208460238848],[-89.88861993862355,42.078122860728],[-90.0364082248525,42.048592782648825],[-90.12132681326239,41.90160284121032],[-90.30767621297309,41.92723942242054],[-90.34827178061846,42.177251945990804],[-90.52092787722546,42.20508073553094],[-90.66653841970776,42.39665047273354],[-90.79465280075891,42.368372457152475],[-90.84954305484035,42.46442543096168],[-90.4425973690137,42.61026070261033],[-90.38391594097976,42.75747508571516],[-90.27659715156301,42.63086871091167],[-90.10736677995072,42.5208921506657],[-90.03181804152632,42.52446094857686],[-89.92409060315185,42.43524334559032],[-90.00831411239614,42.339703388446935],[-89.87760190114068,42.29322589164411],[-89.85247568242966,42.22927043549339],[-89.70532856286644,42.143711633542864]]]},"properties":{"id":"07060005","title":"Apple-Plum"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-89.63225893068818,40.958130185383084],[-89.58121399014237,40.63354897963694],[-89.53207818501033,40.61143559633791],[-89.56635049849882,40.495685462142546],[-89.75569794507987,40.5240701777235],[-89.79218734189828,40.36108208110153],[-89.59981960886353,40.43970248514614],[-89.46583727886315,40.36348662172276],[-89.7185475315959,40.242857877118354],[-89.92961672918494,40.272238080197724],[-89.95939363018039,40.172932509518546],[-90.16572821944345,40.062107089898916],[-90.3649797243425,40.0772170169588],[-90.41793433780197,39.981233987941096],[-90.54366947198179,39.9574072994364],[-90.55750695841868,40.09949666692421],[-90.6366015176709,40.27551114998431],[-90.43928350026886,40.38786405710158],[-90.15201030175643,40.29120377079329],[-90.11803565389249,40.408336345611474],[-90.02840093736495,40.5287683058412],[-89.96894995308224,40.687823955594354],[-90.01733908946545,40.77637526899855],[-89.84491319389974,40.917822246903995],[-89.73841623364837,40.902273958386445],[-89.63225893068818,40.958130185383084]]]},"properties":{"id":"07130003","title":"Lower Illinois-Lake Chautauqua"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.61943871559362,39.285303058812985],[-88.60065317916445,39.252361449489115],[-88.80602122155398,39.019533159225546],[-88.78058219763511,38.800401212690645],[-88.87917695269044,38.74271410861354],[-88.86083842146888,38.602133693206724],[-88.90157202661402,38.49419772774928],[-89.08885676486494,38.47640061527693],[-89.31167972910242,38.32671508321761],[-89.44079233098535,38.32927358321359],[-89.5812677526423,38.48466810693077],[-89.38341809149108,38.61517799110317],[-89.31516958118033,38.778062852308665],[-89.33096800094751,38.90804937710692],[-89.30045086974485,39.24040803596603],[-89.22447756777945,39.30997500148305],[-89.16089088558647,39.368337694100774],[-89.10568121900553,39.19597876311832],[-88.87760273394287,39.12640238093462],[-88.77992988409449,39.21219653184312],[-88.61943871559362,39.285303058812985]]]},"properties":{"id":"07140202","title":"Middle Kaskaskia"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.38767818366165,39.48673016162536],[-88.37273160347655,39.35026419412884],[-88.45518318564024,39.22458178078227],[-88.36488360869703,39.10307049972084],[-88.10697463826403,38.95704344161419],[-88.09433282057535,38.85883856051663],[-87.99021000302861,38.75958573983735],[-88.02534145818242,38.63447189523987],[-88.07534111227147,38.11856582833235],[-88.0608705352106,37.934193760910205],[-88.15436015173213,37.90944889219861],[-88.3135655587767,38.02659711389174],[-88.17111331941447,38.128658712691674],[-88.24162591201338,38.22725340837195],[-88.42836309193183,38.40082094247754],[-88.62995664057729,38.69541096910365],[-88.62088701871636,38.78058606584642],[-88.78058219763511,38.800401212690645],[-88.80602122155398,39.019533159225546],[-88.60065317916445,39.252361449489115],[-88.61943871559362,39.285303058812985],[-88.6159236208074,39.40228137633977],[-88.38767818366165,39.48673016162536]]]},"properties":{"id":"05120114","title":"Little Wabash"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-87.86259994593502,37.60848552079074],[-88.04655573419114,37.59515661039478],[-88.04824129356354,37.47594773766315],[-87.98860178011444,37.39920923153227],[-88.08923698099989,37.28056988067476],[-88.240781701598,37.30236175668256],[-88.40798178988018,37.14442530276108],[-88.51726830741887,37.152270733998876],[-88.64756370304997,37.32010367540505],[-88.83064756526574,37.38370418572299],[-88.83051828297431,37.51379933968775],[-88.69581706130839,37.5319271979929],[-88.46936980645154,37.616013885362406],[-88.12587455594303,37.57121834168191],[-88.22622386307893,37.7084355060523],[-88.14963925590615,37.74486655078738],[-87.99377167906476,37.72917130602008],[-87.86259994593502,37.60848552079074]]]},"properties":{"id":"05140203","title":"Lower Ohio-Bay"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.92709831199107,41.79445613095993],[-88.92246950470656,41.77671111223748],[-89.01623291601936,41.67520180927005],[-89.0726498148901,41.70521069151516],[-89.30957678639737,41.641351675989256],[-89.57414261411174,41.52950603345454],[-89.66888017333963,41.32301527856674],[-89.59184665366757,41.279592639050804],[-89.75404767112411,41.3113663546265],[-89.96655602183591,41.22852242558838],[-90.0078463946885,41.2866037244566],[-90.17787126629958,41.29701948694043],[-90.24190316620019,41.35918668476057],[-90.39091379409393,41.33838400770952],[-90.39569933158651,41.468384390841095],[-90.08705342789892,41.532719081366224],[-89.99654541345603,41.645396446816335],[-89.87786097197363,41.581526702123824],[-89.77096260338953,41.58740336253135],[-89.60386590052389,41.70257026547756],[-89.27265165103802,41.77451187994927],[-89.09719172943534,41.86311042981174],[-88.92709831199107,41.79445613095993]]]},"properties":{"id":"07090007","title":"Green"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-89.01623291601936,41.67520180927005],[-89.0738982044715,41.60168520209248],[-88.92716006199095,41.52408460325461],[-88.96448669630803,41.46869579500725],[-88.84708002982359,41.34795466602799],[-88.74350405498438,41.31561261920319],[-88.68769700090434,41.2045212735423],[-88.82814324651963,41.20207363917109],[-89.0672594982318,41.31326211087355],[-89.06410256802837,41.16350424860599],[-88.99097687751691,41.05023634982348],[-89.07457474717881,40.99488854886772],[-89.03845626910987,40.891569112569755],[-89.05793703366294,40.856109713666456],[-89.30541435723717,40.82955682724935],[-89.42368770497023,40.77606553358237],[-89.33826718531117,40.6988146055773],[-89.53207818501033,40.61143559633791],[-89.58121399014237,40.63354897963694],[-89.63225893068818,40.958130185383084],[-89.58125082555898,41.04736915711959],[-89.66438064834665,41.0983650737071],[-89.58876744742236,41.19046047877248],[-89.59184665366757,41.279592639050804],[-89.66888017333963,41.32301527856674],[-89.57414261411174,41.52950603345454],[-89.30957678639737,41.641351675989256],[-89.0726498148901,41.70521069151516],[-89.01623291601936,41.67520180927005]]]},"properties":{"id":"07130001","title":"Lower Illinois-Senachwine Lake"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.68769700090434,41.2045212735423],[-88.58224568544301,40.99347424991157],[-88.42168847423386,40.934859430210906],[-88.20400831832177,40.9556846666369],[-88.11006630180094,40.97093654161324],[-88.10642216326488,40.75425222840789],[-88.20267621311552,40.66100141917764],[-88.35264880142438,40.591030631786225],[-88.56638572400931,40.62250975257069],[-88.86615467875231,40.76396287005946],[-89.03845626910987,40.891569112569755],[-89.07457474717881,40.99488854886772],[-88.99097687751691,41.05023634982348],[-89.06410256802837,41.16350424860599],[-89.0672594982318,41.31326211087355],[-88.82814324651963,41.20207363917109],[-88.68769700090434,41.2045212735423]]]},"properties":{"id":"07130002","title":"Vermilion"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-88.19896245687124,43.20293188919038],[-88.3299788004179,43.117001630990444],[-88.47455320436018,42.88866610113655],[-88.67436406759168,42.75259981280607],[-88.58600903335383,42.67435917959415],[-88.60458902290833,42.54742375791619],[-88.81570915174728,42.476853646567406],[-88.92687485365809,42.52925343086105],[-89.06993494197769,42.457576520555676],[-89.08099047112717,42.55844799435744],[-89.22668636673438,42.60251922137235],[-89.2938505197551,42.676817067090326],[-89.25605393231376,42.776497937768966],[-89.45553381117082,42.938175358351316],[-89.43206764141559,42.992430129100455],[-89.56120870371512,43.08028075917241],[-89.60211817344327,43.21376071209022],[-89.48676461841399,43.25008789536719],[-89.42110441018258,43.33619553690022],[-89.29717961766659,43.362459996234406],[-89.18005014284842,43.16209156112876],[-89.02487493371427,43.0873451612448],[-88.82163239340474,42.93437490627389],[-88.72051284460338,42.93776930210197],[-88.6343368364038,43.07290706855889],[-88.49822486161509,43.043373502979705],[-88.35951859412205,43.10240081747145],[-88.28071228382771,43.220911362079164],[-88.19896245687124,43.20293188919038]]]},"properties":{"id":"07090002","title":"Middle Rock"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-91.45445981119309,39.887776136002856],[-91.20788512720083,40.002051953533794],[-91.01738433166315,39.80471161321509],[-90.8249594673785,39.71210551440049],[-90.77151174350314,39.62424582599522],[-90.66938177803667,39.570900770869684],[-90.71586720192283,39.349247905588754],[-90.6774504884408,39.3285847629125],[-90.63459555309066,39.100372880975044],[-90.65080420202383,38.94943359579264],[-90.68697195821773,38.93310996977635],[-90.8925536735236,38.98895469573131],[-90.90414982246392,39.121780079900134],[-91.05069801911145,39.27506376716224],[-91.20594372720382,39.345562162886154],[-91.19305859493215,39.40430170029498],[-91.06408848367403,39.46876453769488],[-91.40361743835535,39.65937304677402],[-91.55020240583616,39.68005688215857],[-91.73182779617923,39.63737862701652],[-91.86082258347898,39.65871876448335],[-91.88354841156871,39.77086781639264],[-92.04740852798102,39.83528857254265],[-92.17123176424718,39.9255428932359],[-91.95897583749331,39.95147295465398],[-91.84300741475664,39.85922764646381],[-91.64100048173685,39.826327418389894],[-91.45445981119309,39.887776136002856]]]},"properties":{"id":"07110004","title":"The Sny"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-91.02561313998376,40.04764938783802],[-90.945777754691,40.05178302949827],[-90.729714046693,39.946326945286955],[-90.54366947198179,39.9574072994364],[-90.41793433780197,39.981233987941096],[-90.11109317265323,39.937529001550615],[-89.97875566244198,39.861947181876246],[-90.02604455716028,39.67554256654057],[-89.8988843052743,39.57671544273569],[-89.9415537802081,39.517727080327234],[-90.07664261229007,39.464042905410565],[-90.29910353798641,39.320297213967024],[-90.50109143038122,39.26386279113797],[-90.58894647191153,39.186846526674174],[-90.50645811370623,39.10764828617209],[-90.32588897127818,39.11683103719952],[-90.2952039036175,39.07315534560064],[-90.51985965222713,38.91080483856098],[-90.65080420202383,38.94943359579264],[-90.63459555309066,39.100372880975044],[-90.6774504884408,39.3285847629125],[-90.71586720192283,39.349247905588754],[-90.66938177803667,39.570900770869684],[-90.77151174350314,39.62424582599522],[-90.8249594673785,39.71210551440049],[-91.01738433166315,39.80471161321509],[-91.20788512720083,40.002051953533794],[-91.02561313998376,40.04764938783802]]]},"properties":{"id":"07130011","title":"Lower Illinois"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-87.87314651050201,42.34921569364053],[-87.82934696161163,42.21977878342477],[-87.68505374204398,42.075592256565244],[-87.68457288162801,42.075359798232284],[-87.61261568069807,41.88920257143786],[-87.61285159632268,41.88791531935658],[-87.62444711297138,41.77894959452567],[-87.53954479643647,41.73392406751225],[-87.51858702876069,41.62849233121756],[-87.3579288185934,41.6128319864502],[-87.35852462588412,41.53125882720184],[-87.48562208402018,41.45430386690458],[-87.64623895877088,41.35043055352418],[-87.73630282946442,41.43528473985077],[-87.7479559544463,41.566854155271585],[-87.85825536260842,41.576040571923954],[-87.84384975429748,41.69804952173462],[-87.73717889821302,41.771152532037775],[-87.74095768362383,41.896388954760084],[-87.81716904704723,41.9547705713361],[-87.82654631786602,42.05482498472247],[-87.91323659481475,42.225881791748634],[-87.87314651050201,42.34921569364053]]]},"properties":{"id":"07120003","title":"Chicago"}},
        {"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[-87.0485671211569,39.623604837454536],[-87.2010985073785,39.41314378673957],[-87.29214954890381,39.384807202408524],[-87.19705594175974,39.155558887139364],[-87.28272361141848,39.040404888359774],[-87.3044833301347,38.84159621783505],[-87.48113051631884,38.63576030877954],[-87.6170996483994,38.64281073376861],[-87.51957327667583,38.75818431588118],[-87.61470750569481,38.92640763749506],[-87.71371800345781,38.948000612461556],[-87.76456532525384,39.03811460919667],[-87.79838303353472,39.28857934734958],[-87.84367162617275,39.320088121259005],[-87.84000316888677,39.547259542781376],[-87.76942235857967,39.60731733122981],[-87.86486632301484,39.6749161811249],[-87.80494130331618,39.868822965198945],[-87.58613464844751,39.90178248598107],[-87.47589171528529,39.80047285905499],[-87.50420779544964,39.75538546224999],[-87.38497198209308,39.62071711141738],[-87.22368980005177,39.595858130205954],[-87.0485671211569,39.623604837454536]]]},"properties":{"id":"05120111","title":"Middle Wabash-Busseron"}}
    ]
}
];


app.config.glossary = {
    items: [
        {
            "id": "contaminants",
            "title": "Contaminants",
            "description":
                "Established by the EPA to protect public health by limiting the levels of contaminants " +
                "in drinking water, National Primary Drinking Water Regulations (NPDWRs or primary standards) " +
                "are legally enforceable standards that apply to public water systems. Drinking water, " +
                "including bottled water, may reasonably be expected to contain at least small amounts of " +
                "some contaminants. The presence of contaminants does not necessarily indicate that water " +
                "poses a health risk. EPA sets standards for approximately 90 contaminants and indicators " +
                "in drinking water.",
            "descriptionSources": [
                {
                    "title": "EPA",
                    "link": "http://water.epa.gov/drink/contaminants/basicinformation/index.cfm"
                }
            ]
        },
        {
            "id": "delta",
            "title": "Delta",
            "description":
                "Deltas are wetlands that form as rivers empty their water and sediment into another body of " +
                "water, such as an ocean, lake, or another river. Deltas can also empty into land, although " +
                "this is less common. A river moves more slowly as it nears its mouth, or end. This causes " +
                "sediment, solid material carried downstream by currents, to fall to the river bottom.\\nThe " +
                "slowing velocity of the river and the build-up of sediment allow the river to break from its " +
                "single channel as it nears its mouth. Under the right conditions, a river forms a deltaic " +
                "lobe. A mature deltaic lobe includes a distributary network—a series of smaller, shallower " +
                "channels, called distributaries, which branch off from the main stem of the river.\\nIn a " +
                "deltaic lobe, heavier, coarser material settles first. Smaller, finer sediment is carried " +
                "farther downstream. The finest material is deposited beyond the river's mouth. This material " +
                "is called alluvium or silt. As silt builds up the delta forms, extending a river's mouth into " +
                "the body of water into which it is emptying.\\nA delta is sometimes divided into two parts: " +
                "subaqueous and subaerial. The subaqueous part of a delta is underwater. This is the most steeply " +
                "sloping part of the delta, and contains the finest silt. The newest part of the subaqueous delta, " +
                "furthest from the mouth of the river, is called the prodelta. The subaerial part of a delta is " +
                "above water. The subaerial region most influenced by waves and tides is called the lower delta. " +
                "The region most influenced by the river's flow is called the upper delta.\\nPast decades of river " +
                "management has prevented the Mississippi River from naturally flowing through its delta wetlands, " +
                "causing erosion. Between 1990 and 2000, the Mississippi River Delta lost 62 square kilometers " +
                "(24 square miles) of wetlands per year.",
            "descriptionSources": [
                {
                    "title": "National Geographic",
                    "link": "http://education.nationalgeographic.com/education/encyclopedia/delta/?ar_a=1"
                }
            ]},
        {
            "id": "flood",
            "title": "Flood",
            "description":
                "Floods are one of the most common hazards in the United States.  Anywhere it rains, it can flood. " +
                "A flood is a general and temporary condition where two or more acres of normally dry land or two " +
                "or more properties are inundated by water or mudflow. However not all floods are alike with some " +
                "floods developing slowly, while others such as flash floods, develop in just a few minutes and " +
                "without visible signs of rain. Additionally, floods can be local, impacting a neighborhood or " +
                "community, or very large, affecting entire river basins and multiple states.\\nMany conditions " +
                "can result in a flood: hurricanes, overtopped levees, outdated or clogged drainage systems and " +
                "rapid accumulation of rainfall.  Overland flooding, the most common type of flooding event " +
                "typically occurs when waterways such as rivers or streams overflow their banks as a result of " +
                "rainwater or a possible levee breach and cause flooding in surrounding areas.",
            "descriptionSources": [
                {
                    "title": "NFIP",
                    "link": "https://www.floodsmart.gov/floodsmart/pages/flooding_flood_risks/ffr_overview.jsp"
                },
                {
                    "title": "Ready.gov",
                    "link": "http://ready.gov/floods"
                }
            ]
        },
        {
            "id": "flood-risk",
            "title": "Flood Risk",
            "description":
                "Flooding can happen anywhere, but certain areas are especially prone to serious flooding. Flood " +
                "risk can be informed by history, but risk is also based on a number of factors: rainfall, " +
                "river-flow and tidal-surge data, topography, flood-control measures, climate change, and " +
                "changes due to building and development. To help communities understand their risk, flood " +
                "maps (Flood Insurance Rate Maps, FIRMs) have been created to show the locations of high-risk, " +
                "moderate-to-low risk and undetermined-risk areas.",
            "descriptionSources": [
                {
                    "title": "NFIP",
                    "link": "https://www.floodsmart.gov/floodsmart/pages/flooding_flood_risks/defining_flood_risks.jsp"
                }
            ]
        },
        {
            "id": "special-flood-hazard-areas",
            "title": "Special Flood Hazard Areas",
            "description":
                "In high-risk areas, there is at least a 1 in 4 chance of flooding during a 30-year mortgage. " +
                "All home and business owners in these areas with mortgages from federally regulated or insured " +
                "lenders are required to buy flood insurance. They are shown on the flood maps as zones labeled " +
                "with the letters A or V.",
            "descriptionSources": [
                {
                    "title": "NFIP",
                    "link": "https://www.floodsmart.gov/floodsmart/pages/flooding_flood_risks/defining_flood_risks.jsp"
                }
            ]
        },
        {
            "id": "non-special-flood-hazard-areas",
            "title": "Non-Special Flood Hazard Areas",
            "description":
                "In moderate-to-low risk areas, the risk of being flooded is reduced but not completely removed. " +
                "These areas submit over 20% of NFIP claims and receive one-third of disaster assistance for " +
                "flooding. Flood insurance isn't federally required in moderate-to-low areas, but it is recommended " +
                "for all property owners and renters. They are shown on flood maps as zones labeled with the " +
                "letters B, C or X (or a shaded X).",
            "descriptionSources": [
                {
                    "title": "NFIP",
                    "link": "https://www.floodsmart.gov/floodsmart/pages/flooding_flood_risks/defining_flood_risks.jsp"
                }
            ]
        },

        {
            "id": "undetermined-risk-areas",
            "title": "Undetermined Risk Areas",
            "description":
                "No flood-hazard analysis has been conducted in these areas, but a flood risk still exists. " +
                "Flood insurance rates reflect the uncertainty of the flood risk. These areas are labeled with " +
                "the letter D on the flood maps.",
            "descriptionSources": [
                {
                    "title": "NFIP",
                    "link": "https://www.floodsmart.gov/floodsmart/pages/flooding_flood_risks/defining_flood_risks.jsp"
                }
            ]
        },
        {
            "id": "geospatial",
            "title": "Geospatial",
            "description":
                "Geospatial is a term used to refer to data derived from or related to the earth’s surface, " +
                "including both geographic and spatial characteristics."
        },
        {
            "id": "greon",
            "title": "GREON℠",
            "description": "The Great Rivers Ecological Observatory Network (GREON℠) consists of a series of " +
            "water quality monitoring platforms on the Mississippi River collecting near real-time, continuous " +
            "water quality data. Data collected from the network will lead to a greater understanding of water " +
            "quality and aquatic ecology in the Mississippi River system and the factors the drive and influence " +
            "Gulf hypoxia. The platforms use state-of-the-art sensor technology mounted on a floating platform " +
            "for in-stream measurement of a suite of water quality parameters.",
        },
        {
            "id": "gulf-of-mexico-hypoxic-zone",
            "title": "Gulf of Mexico Hypoxic Zone",
            "description":
                "The hypoxic zone in the northern Gulf of Mexico is an area along the Louisiana-Texas coast, " +
                "where water near the bottom of the Gulf contains less than 2 parts per million of dissolved " +
                "oxygen, causing a condition referred to as hypoxia.\\nEach summer, the size of the hypoxic zone " +
                "is measured. The size of the zone is an important indicator of how much progress is being made " +
                "to reduce nutrient inputs into the Gulf of Mexico. Sometimes the size of the zone is influenced " +
                "by other factors, such as droughts or hurricanes that can reduce the size of the zone, or " +
                "floods that can increase the size.",
            "descriptionSources": [
                {
                    "title": "EPA",
                    "link": "http://water.epa.gov/type/watersheds/named/msbasin/zone.cfm"
                }
            ]
        },
        {
            "id": "huc",
            "title": "HUC",
            "description":
                "Hydrologic Unit Code (HUC) – Just as a phone is assigned a numeric area code, watersheds are " +
                "assigned a numeric locator, called a Hydrologic Unit Code (HUC).  A HUC can have anywhere from " +
                "2 to 12 digits.  The more digits, the smaller the land area of the watershed.\\nThe United States " +
                "is divided and sub-divided into successively smaller hydrologic units which are classified into " +
                "four levels: regions, sub-regions, accounting units, and cataloging units. The hydrologic units " +
                "are arranged or nested within each other, from the largest geographic area (regions) to the " +
                "smallest geographic area (cataloging units). Each hydrologic unit is identified by a unique " +
                "hydrologic unit code (HUC) based on the four levels of classification in the hydrologic unit system.",
            "descriptionSources": [
                {
                    "title": "Arkansas Water",
                    "link": "http://arkansaswater.org/index.php?option=com_content&task=view&id=312&Itemid=86"
                },
                {
                    "title": "USGS",
                    "link": "http://water.usgs.gov/GIS/huc.html"
                }
            ]
        },
        {
            "id": "hydrology",
            "title": "Hydrology",
            "description":
                "Hydrology is the science that encompasses the occurrence, distribution, movement and " +
                "properties of the waters of the earth and their relationship with the environment within each " +
                "phase of the hydrologic cycle. The water cycle, or hydrologic cycle, is a continuous process by " +
                "which water is purified by evaporation and transported from the earth's surface (including the " +
                "oceans) to the atmosphere and back to the land and oceans. All of the physical, chemical and " +
                "biological processes involving water as it travels its various paths in the atmosphere, over and " +
                "beneath the earth's surface and through growing plants, are of interest to those who study the " +
                "hydrologic cycle.",
            "descriptionSources": [
                {
                    "title": "USGS",
                    "link": "http://water.usgs.gov/edu/hydrology.html#HDR2"
                }
            ]
        },
        {
            "id": "hypoxia",
            "title": "Hypoxia",
            "description":
                "Hypoxia, meaning low oxygen, occurs when dissolved oxygen levels are too low to support aquatic " +
                "organisms creating “dead zones.” The causes of hypoxia include excess nutrients in the water " +
                "column, which facilitate algal blooms and phytoplankton growth.\\nThe phytoplankton soon die " +
                "and decompose, consuming oxygen in the process, which causes the death of aquatic organisms that " +
                "cannot escape to areas with higher oxygen levels. Although the Gulf’s waters are rich in oxygen, " +
                "a process called stratification, caused by the natural layering of fresh water from the river and " +
                "salt water from the Gulf, prevents the two from mixing and keeps the dead zone in place for months " +
                "at a time.\\nDirect effects of hypoxia include fish kills, which deplete valuable fisheries and " +
                "disrupt ecosystems. The Gulf of Mexico hypoxic zone is located in the northern Gulf of Mexico off " +
                "the coast of Louisiana and Texas. The zone was first documented in 1972.",
            "descriptionSources": [
                {
                    "title": "EPA",
                    "link": "http://water.epa.gov/type/watersheds/named/msbasin/hypoxia101.cfm"
                }
            ]
        },
        {
            "id": "infrastructure",
            "title": "Infrastructure",
            "description":
                "River infrastructure refers to the various levees, locks, dams, flood walls, and other man-made " +
                "structures used to support and maintain transportation on the Mississippi.  The existing 9-foot " +
                "Channel Navigation Project was largely constructed in the 1930s and extends down the Upper " +
                "Mississippi River from Minneapolis-St. Paul to its confluence with the Ohio River and up the " +
                "Illinois Waterway to the Thomas J. O’Brien Lock in Chicago. It includes 37 Locks and approximately " +
                "1,200 miles of navigable waterway in Illinois, Iowa, Minnesota, Missouri and Wisconsin.",
            "descriptionSources": [
                {
                    "title": "USACE",
                    "link": "http://www.mvr.usace.army.mil/Portals/48/docs/CC/FactSheets/Miss/Mississippi River Locks and Dams (2012).pdf"
                }
            ]
        },
        {
            "id": "macroinvertebrates",
            "title": "Macroinvertebrates",
            "description":
                "Macroinvertebrates are organisms that are large (macro) enough to be seen with the naked eye and " +
                "lack a backbone (invertebrate). They inhabit all types of running waters, from fastflowing " +
                "mountain streams to slowmoving muddy rivers. Examples of aquatic macroinvertebrates include " +
                "insects in their larval or nymph form, crayfish, clams, snails, and worms (Fig. 4.1). Most live " +
                "part or most of their life cycle attached to submerged rocks, logs, and vegetation.\\nBeing " +
                "relatively easy to sample and identify, aquatic macroinvertebrates are good indicators of stream " +
                "quality because they are affected by the physical, chemical, and biological conditions of the " +
                "stream. The basic principle behind the study of macroinvertebrates is that some are more " +
                "sensitive to pollution than others. Therefore, if a stream site is inhabited by organisms " +
                "that can tolerate pollution and the more pollution sensitive organisms are missing a pollution " +
                "problem is likely.",
            "descriptionSources": [
                {
                    "title": "EPA",
                    "link": "http://water.epa.gov/type/rsl/monitoring/vms40.cfm"
                }
            ]
        },
        {
            "id": "navigation-channels",
            "title": "Navigation Channels",
            "description":
                "The navigation system on the Mississippi River refers to 1,200 miles of a 9-foot navigation " +
                "channel, 37 lock and dam sites, and thousands of channel training structures, all maintained " +
                "by the US Army Corps of Engineers. The 1,200 miles of 9-foot channel created by the 37 locks " +
                "and dams allow waterway traffic to move from one pool to another providing an integral regional, " +
                "national, and international transportation network.",
        },
        {
            "id": "nutrients",
            "title": "Nutrients",
            "description":
                "Nitrogen and phosphorus are nutrients that are natural parts of aquatic ecosystems. " +
                "Nitrogen is also the most abundant element in the air we breathe. Nitrogen and phosphorus " +
                "support the growth of algae and aquatic plants, which provide food and habitat for fish, " +
                "shellfish and smaller organisms that live in water.  Excess nitrogen and phosphorus (nutrients) " +
                "are among the most prevalent cause of water quality impairment in the United States.",
            "descriptionSources": [
                {
                    "title": "EPA",
                    "link": "http://www2.epa.gov/nutrientpollution/problem"
                }
            ]
        },
        {
            "id": "nutrient-pollution",
            "title": "Nutrient Pollution",
            "description":
                "Nutrient pollution is one of America's most widespread, costly and challenging environmental " +
                "problems, and is caused by excess nitrogen and phosphorus in the air and water. Nitrogen and " +
                "phosphorus are nutrients that are natural parts of aquatic ecosystems. Nitrogen and phosphorus " +
                "support the growth of algae and aquatic plants, which provide food and habitat for fish, " +
                "shellfish and smaller organisms that live in water. But when too much nitrogen and phosphorus " +
                "enter the environment - usually from a wide range of human activities - the air and water can " +
                "become polluted. Nutrient pollution in ground water - which millions of people in the United " +
                "States use as their drinking water source - can be harmful, even at low levels.\\nNutrient " +
                "pollution has impacted many streams, rivers, lakes, bays and coastal waters for the past " +
                "several decades, resulting in serious environmental and human health issues and negative " +
                "economic impacts. Too much nitrogen and phosphorus in the water causes excessive algal growth " +
                "or algal blooms that harm water quality, food resources and habitats, and decrease the oxygen " +
                "that aquatic life need to survive. Some algal blooms are harmful to humans because they produce " +
                "elevated toxins and bacterial growth that can make people sick if they come into contact with " +
                "polluted water, consume tainted fish or shellfish, or drink contaminated water.",
            "descriptionSources": [
                {
                    "title": "EPA",
                    "link": "http://www2.epa.gov/nutrientpollution/problem"
                }
            ]
        },
        {
            "id": "phosphorus",
            "title": "Phosphorus",
            "description":
                "Both phosphorus and nitrogen are essential nutrients for the plants and animals that make up " +
                "the aquatic food web. Since phosphorus is the nutrient in short supply in most fresh waters, " +
                "even a modest increase in phosphorus can, under the right conditions, set off a whole chain of " +
                "undesirable events in a stream including accelerated plant growth, algae blooms, low dissolved " +
                "oxygen, and the death of certain fish, invertebrates, and other aquatic animals.\\nThere are " +
                "many sources of phosphorus, both natural and human. These include soil and rocks, wastewater " +
                "treatment plants, runoff from fertilized lawns and cropland, failing septic systems, runoff " +
                "from animal manure storage areas, disturbed land areas, drained wetlands, water treatment, and " +
                "commercial cleaning preparations. In a stream system, the phosphorus cycle tends to move " +
                "phosphorus downstream as the current carries decomposing plant and animal tissue and dissolved " +
                "phosphorus. It becomes stationary only when it is taken up by plants or is bound to particles that " +
                "settle to the bottom of pools.\\nMonitoring phosphorus is challenging because it involves " +
                "measuring very low concentrations down to 0.01 milligram per liter (mg/L) or even lower. Even " +
                "such very low concentrations of phosphorus can have a dramatic impact on streams. Less sensitive " +
                "methods should be used only to identify serious problem areas.",
            "descriptionSources": [
                {
                    "title": "EPA",
                    "link": "http://water.epa.gov/type/rsl/monitoring/vms56.cfm"
                }
            ]},
        {
            "id": "phytoplankton",
            "title": "Phytoplankton",
            "description":
                "Phytoplankton, also known as microalgae, are similar to terrestrial plants in that they " +
                "contain chlorophyll and require sunlight in order to live and grow. Most phytoplankton are " +
                "buoyant and float in the upper part of the ocean, where sunlight penetrates the water. " +
                "Phytoplankton also require inorganic nutrients such as nitrates, phosphates, and sulfur which " +
                "they convert into proteins, fats, and carbohydrates.\\nIn a balanced ecosystem, phytoplankton " +
                "provide food for a wide range of sea creatures including whales, shrimp, snails, and jellyfish. " +
                "When too many nutrients are available, phytoplankton may grow out of control and form harmful " +
                "algal blooms.",
            "descriptionSources": [
                {
                    "title": "NOAA",
                    "link": "http://oceanservice.noaa.gov/facts/phyto.html"
                }
            ]},
        {
            "id": "reaches",
            "title": "Reaches",
            "description":
                "On the Mississippi River, the term reach is used to describe areas of similar geomorphology, " +
                "vegetation cover, and land use practices.  On the Mississippi River, the term Pool is used to " +
                "describe the area between navigation dams. For example, Pool 13 is the area upstream of Lock " +
                "and Dam 13, to Lock and Dam 12.  There are 26 navigation pools in the Upper Mississippi River.",
            "descriptionSources": [
                {
                    "title": "USGS",
                    "link": "http://www.umesc.usgs.gov/rivers/upper_mississippi/reach_1/sel_a_pool_r1.html"
                }
            ]},
        {
            "id": "river-mile",
            "title": "River Mile",
            "description":
                "The term river mile is the distance from the mouth of the creek or river to the given location, " +
                "to the nearest tenth of a mile.  The USACE uses river miles in its navigation charts.",
            "descriptionSources": [
                {
                    "title": "USACE",
                    "link": "http://www.mvr.usace.army.mil/Missions/Navigation/NavigationCharts.aspx"
                }
            ]},

        {
            "id": "salinity",
            "title": "Salinity",
            "description":
                "Conductivity measurements, along with temperature, also allow for salinity values to be " +
                "calculated through algorithms.",
            "descriptionSources": [
                {
                    "title": "YSI",
                    "link": "http://www.ysi.com/parametersdetail.php?Conductivity-7"
                }
            ]},
        {
            "id": "sediment",
            "title": "Sediment",
            "description":
                "Sediments are fragmented materials that originate from weathering and erosion of rocks or " +
                "unconsolidated deposits and are transported by, suspended in, or deposited by water. Many of " +
                "the sediments in our rivers, lakes, and oceans have been contaminated by pollutants. Many of " +
                "the contaminants were released years ago while other contaminants enter our water every day. " +
                "Some contaminants flow directly from industrial and municipal waste dischargers, while others " +
                "come from polluted runoff in urban and agricultural areas.\\nSuspended and bedded sediments " +
                "(SABS) are defined by EPA as particulate organic and inorganic matter that suspend in or are " +
                "carried by the water, and/or accumulate in a loose, unconsolidated form on the bottom of natural " +
                "water bodies. This includes the frequently used terms of clean sediment, suspended sediment, " +
                "total suspended solids, bedload, turbidity, or in common terms, dirt, soils or eroded materials.",
            "descriptionSources": [
                {
                    "title": "EPA",
                    "link": "http://water.epa.gov/polwaste/sediments/"
                }
            ]},

        {
            "id": "stratification",
            "title": "Stratification",
            "description":
                "Stratification occurs when water masses with different properties form layers that prevent " +
                "the mixing of oxygen-rich surface water with oxygen-poor water on the bottom of the Gulf.  " +
                "Nutrient-laden freshwater from the Mississippi River flows into the Gulf of Mexico. This " +
                "freshwater is less dense and remains above the more dense saline Gulf water. In addition " +
                "to the saline gradient caused where the freshwater and saline water meet, the freshwater is " +
                "warmer than the deeper ocean water, further contributing to the stratification. Without mixing, " +
                "oxygen in the bottom water is limited and the hypoxic condition remains.",
            "descriptionSources": [
                {
                    "title": "EPA",
                    "link": "http://water.epa.gov/type/watersheds/named/msbasin/hypoxia101.cfm"
                }
            ]},
        {
            "id": "total-suspended-solids",
            "title": "Total Suspended Solids",
            "description":
                "Total solids are dissolved solids plus suspended and settleable solids in water. " +
                "In stream water, dissolved solids consist of calcium, chlorides, nitrate, phosphorus, iron, " +
                "sulfur, and other ions particles that will pass through a filter with pores of around 2 microns " +
                "(0.002 cm) in size. Suspended solids include silt and clay particles, plankton, algae, fine " +
                "organic debris, and other particulate matter. These are particles that will not pass through a " +
                "2-micron filter. Sources of total solids include industrial discharges, sewage, fertilizers, " +
                "road runoff, and soil erosion. Total solids are measured in milligrams per liter (mg/L).\\nThe " +
                "concentration of total dissolved solids affects the water balance in the cells of aquatic " +
                "organisms. Higher concentrations of suspended solids can serve as carriers of toxics, which " +
                "readily cling to suspended particles. A high concentration of total solids will make drinking " +
                "water unpalatable and might have an adverse effect on people who are not used to drinking such " +
                "water. Higher solids decrease the passage of light through water, thereby slowing photosynthesis " +
                "by aquatic plants. Levels of total solids that are too high or too low can also reduce the " +
                "efficiency of wastewater treatment plants, as well as the operation of industrial processes " +
                "that use raw water.\\nTotal solids are important to measure in areas where there are discharges " +
                "from sewage treatment plants, industrial plants, or extensive crop irrigation. In particular, " +
                "streams and rivers in arid regions where water is scarce and evaporation is high tend to have " +
                "higher concentrations of solids and are more readily affected by human introduction of solids " +
                "from land use activities.",
            "descriptionSources": [
                {
                    "title": "EPA",
                    "link": "http://water.epa.gov/type/rsl/monitoring/vms58.cfm"
                }
            ]},
        {
            "id": "tributary",
            "title": "Tributary",
            "description":
                "A tributary is a freshwater stream that feeds into a larger stream or river. The larger, or " +
                "parent, river is called the mainstem. The point where a tributary meets the mainstem is " +
                "called the confluence. Tributaries, also called affluents, do not flow directly into the " +
                "ocean.\\nMost large rivers are formed from many tributaries. Each tributary drains a different " +
                "watershed, carrying runoff and snowmelt from that area. Each tributary's watershed makes up " +
                "the larger watershed of the mainstem.",
            "descriptionSources": [
                {
                    "title": "National Geographic",
                    "link": "http://education.nationalgeographic.com/education/encyclopedia/tributary/?ar_a=1"
                }
            ]},

        {
            "id": "wastewater",
            "title": "Wastewater",
            "description":
                "Wastewater is used water. Much of the water used by homes, industries, and businesses must be " +
                "treated before it is released back to the environment. Wastewater includes substances such as " +
                "human waste, food scraps, oils, soaps and chemicals. In homes, this includes water from sinks, " +
                "showers, bathtubs, toilets, washing machines and dishwashers. Businesses and industries also " +
                "contribute their share of used water that must be cleaned. Wastewater also includes storm " +
                "runoff. Harmful substances that wash off roads, parking lots, and rooftops can harm our " +
                "rivers and lakes.\\nNature has an amazing ability to cope with small amounts of water wastes " +
                "and pollution, but it would be overwhelmed if we didn't treat the billions of gallons of " +
                "wastewater and sewage produced every day before releasing it back to the environment. " +
                "Treatment plants reduce pollutants in wastewater to a level nature can handle.",
            "descriptionSources": [
                {
                    "title": "USGS",
                    "link": "http://water.usgs.gov/edu/wuww.html"
                }
            ]},
        {
            "id": "water-quality",
            "title": "Water Quality",
            "description":
                "Water quality can be thought of as a measure of the suitability of water for a particular " +
                "use based on selected physical, chemical, and biological characteristics. Water quality can " +
                "be thought of as a measure of the suitability of water for a particular use based on selected " +
                "physical, chemical, and biological characteristics. To determine water quality, scientists " +
                "first measure and analyze characteristics of the water such as temperature, dissolved mineral " +
                "content, and number of bacteria.\\nSelected characteristics are then compared to numeric " +
                "standards and guidelines to decide if the water is suitable for a particular use. Some aspects " +
                "of water quality can be determined right in the stream or at the well. These include " +
                "temperature, acidity (pH), dissolved oxygen, and electrical conductance (an indirect " +
                "indicator of dissolved minerals in the water). Analyses of individual chemicals generally " +
                "are done at a laboratory.",
            "descriptionSources": [
                {
                    "title": "USGS",
                    "link": "http://pubs.usgs.gov/fs/fs-027-01/"
                }
            ]},

        {
            "id": "watershed",
            "title": "Watershed",
            "description":
                "A watershed is the area of land where all of the water that is under it or drains off of it " +
                "goes into the same place. Watersheds range from a few acres draining to a neighborhood " +
                "stream, to hundreds of thousands of square miles draining to a major river, such as the " +
                "Mississippi. The Mississippi River watershed is the fourth largest in the world, including " +
                "all or parts of 31 states and 2 Canadian Provinces. The watershed measures approximately " +
                "1.2 million square miles, covering about 40% of the lower 48 states.",
            "descriptionSources": [
                {
                    "title": "EPA",
                    "link": "http://water.epa.gov/type/watersheds/whatis.cfm"
                },
                {
                    "title": "NPS",
                    "link": "http://www.nps.gov/miss/riverfacts.htm"
                }
            ]}
    ]
};

app.config.layers.items = [
{
  "type": "Feature",
  "geometry": {
    "type": "Point",
    "coordinates": [-88.84902, 40.09147]
  },
  "properties": {
    "title": "Large Rivers",
    "id": "gltg:us-rivers",
    "wms": "https://gltg.ncsa.illinois.edu/geoserver/wms",
    "opacity": 0.25,
    "checked": true,
    "showLayerLegend": false,
    "layerLegendTitle": "",
    "layerLegendContents": ""
  }
}, {
  "type": "Feature",
  "geometry": {
    "type": "Point",
    "coordinates": [-88.84902, 40.09147]
  },
  "properties": {
    "title": "HUC 8",
    "id": "gltg:huc250k",
    "wms": "https://gltg.ncsa.illinois.edu/geoserver/wms",
    "opacity": 0.05,
    "checked": true,
    "showLayerLegend": false,
    "layerLegendTitle": "",
    "layerLegendContents": ""
  }
},  {
  "type": "Feature",
  "geometry": {
    "type": "Point",
    "coordinates": [-88.84902, 40.09147]
  },
  "properties": {
    "title": "HUC 4",
    "id": "gltg:huc4",
    "wms": "https://gltg.ncsa.illinois.edu/geoserver/wms",
    "opacity": 0.05,
    "checked": false,
    "showLayerLegend": false,
    "layerLegendTitle": "",
    "layerLegendContents": ""
  }
}, {
  "type": "Feature",
  "geometry": {
    "type": "Point",
    "coordinates": [-88.84902, 40.09147]
  },
  "properties": {
    "title": "HUC 2",
    "id": "gltg:huc2",
    "wms": "https://gltg.ncsa.illinois.edu/geoserver/wms",
    "opacity": 0.05,
    "checked": false,
    "showLayerLegend": false,
    "layerLegendTitle": "",
    "layerLegendContents": ""
  }
}, {
  "type": "Feature",
  "geometry": {
    "type": "Point",
    "coordinates": [-88.84902, 40.09147]
  },
  "properties": {
    "title": "US States",
    "id": "gltg:us-states",
    "wms": "https://gltg.ncsa.illinois.edu/geoserver/wms",
    "opacity": 0.5,
    "checked": true,
    "showLayerLegend": false,
    "layerLegendTitle": "",
    "layerLegendContents": ""
  }
}, {
  "type": "Feature",
  "geometry": {
    "type": "Point",
    "coordinates": [-88.84902, 40.09147]
  },
  "properties": {
    "title": "Total annual nitrogen from point sources by HUC8 watershed (avg. 2007-2014)",
    "id": "gltg:agg-pt-source",
    "wms": "https://gltg.ncsa.illinois.edu/geoserver/wms",
    "opacity": 0.5,
    "checked": false,
    "showLayerLegend": false,
    "layerLegendTitle": "",
    "layerLegendContents": ""
  }
}, {
  "type": "Feature",
  "geometry": {
    "type": "Point",
    "coordinates": [-88.84902, 40.09147]
  },
  "properties": {
    "title": "Average Annual Nitrogen Fertilizer Inputs for 1997 to 2006",
    "id": "gltg:fertilizer_n",
    "wms": "https://gltg.ncsa.illinois.edu/geoserver/wms",
    "opacity": 0.50,
    "checked": false,
    "showLayerLegend": false,
    "layerLegendTitle": "",
    "layerLegendContents": ""
  }
}, {
    "type": "Feature",
    "geometry": {
        "type": "Point",
        "coordinates": [-88.84902, 40.09147]
    },
    "properties": {
      "title": "EPA Impaired Stream Segments",
      "id": "gltg:impaired-epa",
      "wms": "https://gltg.ncsa.illinois.edu/geoserver/wms",
      "opacity": 0.75,
      "checked": false,
      "showLayerLegend": false,
      "layerLegendTitle": "",
      "layerLegendContents": ""
    }
}
];

app.config.parameters.items = [
    {
        "id": "discharge-ft3s",
        "title": "Discharge",
        "unit": "ft3/s",
        "categories": ["Flow"],
        "description":
            "Discharge, cubic feet per second - Stream flow, or discharge, is the volume of water that moves " +
            "over a designated point over a fixed period of time. It is often expressed as cubic feet per second " +
            "(ft3/sec). The flow of a stream is directly related to the amount of water moving off the watershed " +
            "into the stream channel. It is affected by weather, increasing during rainstorms and decreasing during " +
            "dry periods. It also changes during different seasons of the year, decreasing during the summer months " +
            "when evaporation rates are high and shoreline vegetation is actively growing and removing water from " +
            "the ground.\nFlow is a function of water volume and velocity. It is important because of its impact on " +
            "water quality and on the living organisms and habitats in the stream. Large, swiftly flowing rivers can " +
            "receive pollution discharges and be little affected, whereas small streams have less capacity to dilute " +
            "and degrade wastes.\nStream velocity, which increases as the volume of the water in the stream " +
            "increases, determines the kinds of organisms that can live in the stream (some need fast-flowing areas; " +
            "others need quiet pools). It also affects the amount of silt and sediment carried by the stream. " +
            "Sediment introduced to quiet, slow-flowing streams will settle quickly to the stream bottom. Fast moving " +
            "streams will keep sediment suspended longer in the water column. Lastly, fast-moving streams generally " +
            "have higher levels of dissolved oxygen than slow streams because they are better aerated.",
        "descriptionSources" : [
            {
                "title": "EPA",
                "link": "http://water.epa.gov/type/rsl/monitoring/vms51.cfm"
            }
        ],
        "glossary": true,
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "nitrate-nitrite-as-n-mgl",
        "title": "Nitrate and Nitrite as N",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "Nitrate plus nitrite, water, in situ, mg/L as N",
        "alternates": ["nitrate-nitrite-as-n"],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },{
        "id": "load-Mg",
        "title": "Nitrate Load",
        "unit": "Mg",
        "categories": ["Calculated"],
        "description":
            "Nitrate Load - Excess nitrogen and phosphorus loading impacts not only local waters, but also " +
            "downstream water bodies and coastal systems.  Nutrient and cumulative (total) load refers to " +
            "the amount of excess nitrogen and phosphorous that is being exported from watersheds and delivered " +
            "to downstream water bodies.",
        "descriptionSources": [
            {
                "title": "EPA",
                "link": "http://www2.epa.gov/nutrient-policy-data/estimated-total-nitrogen-and-total-phosphorus-loads-and-yields-generated-within"
            }
        ],
        "glossary": true,
        "alternates": [ ],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "cumulative-load-Mg",
        "title": "Nitrate Cumulative Load",
        "unit": "Mg",
        "categories": ["Calculated"],
        "description":
            "Nitrate Cumulative Load - Excess nitrogen and phosphorus loading impacts not only local waters, " +
            "but also downstream water bodies and coastal systems.  Nutrient and cumulative (total) load refers " +
            "to the amount of excess nitrogen and phosphorous that is being exported from watersheds and delivered " +
            "to downstream water bodies.",
        "descriptionSources": [
            {
                "title": "EPA",
                "link": "http://www2.epa.gov/nutrient-policy-data/estimated-total-nitrogen-and-total-phosphorus-loads-and-yields-generated-within"
            }
        ],
        "alternates": [],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "ammonia-ammonium-mgl",
        "title": "Ammonia / Ammonium",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description":
            "Ammonium (NH4+) — or its uncharged form ammonia (NH3) — is a form of nitrogen which aquatic plants " +
            "can absorb and incorporate into proteins, amino acids, and other molecules. High concentrations of " +
            "ammonium can enhance the growth of algae and aquatic plants. Bacteria can also convert high ammonium " +
            "to nitrate (NO3-) by the process of nitrification.",
        "descriptionSources": {
            "title": "YSI",
            "link": "http://www.ysi.com/parametersdetail.php?Ammonia---Ammonium-16"
        },
        "glossary": true,
        "alternates": [],
        "detailView" : "ignore",
        "searchView": "ignore",
        "exploreView" : "ignore"
    },
    {
        "id": "ignored",
        "title": "Ignored Parameters",
        "unit": "",
        "categories": [],
        "description": "",
        "alternates": [
            "YSI_Batt_V",
            "YSI_CablePwr_V",
            "YSI_WiperPos_V",
            "ammonia-mgl",
            "barometric-pressure-24hr-avg-hg",
            "chlorophyll-ugl-qc",
            "colored-dissolved-organic-matter-ppbqse-qc",
            "cumulative-load-Mg-qc",
            "discharge-ft3s-qc",
            "dissolved-oxygen-mgl-qc",
            "dissolved-oxygen-percent-saturation-qc",
            "hail-hits-total-24hr",
            "load-Mg-qc",
            "nitrate-as-n-mgl-qc",
            "owner",
            "procedures",
            "rain-accumulation-24hr-total-inch",
            "record",
            "relative-humidity-24hr-avg",
            "site",
            "source",
            "specific-conductivity-uscm-qc",
            "suspended-solids-mgl",
            "unit_code",
            "water-acidity-ph-qc",
            "water-temperature-c-qc",
            "water-turbidity-fnu-qc",
            "wind-speed-24hr-max-mph",
            "acidity-ph-qc",
            "specific-conductance-uScm-qc",
            "turbidity-fnu-qc",
            "water-dissolved-oxygen-mgl-qc",
            "water-dissolved-oxygen-percent-saturation-qc",
            "water-specific-conductance-uScm-qc",
            "dissolved-oxygen-saturation-pct-qc",
            "pH-qc"
        ],
        "detailView" : "ignore",
        "searchView": "ignore",
        "exploreView" : "ignore"
    },
    {
        "id": "total-nitrogen-mgl",
        "title": "Total Nitrogen",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "Total Nitrogen",
        "alternates": [],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "salinity-psu",
        "title": "Salinity",
        "unit": "PSU",
        "categories": ["Water Quality"],
        "description": "Salinity in Practical Salinity Units",
        "alternates": [ ],
        "detailView" : "ignore",
        "searchView": "ignore",
        "exploreView" : "ignore"
    },
    {
        "id": "specific-conductance-uScm",
        "title": "Specific Conductance",
        "unit": "uS/cm",
        "categories": ["Water Quality"],
        "description":
            "Conductivity is a measure of the ability of water to pass an electrical current. Conductivity in " +
            "water is affected by the presence of inorganic dissolved solids such as chloride, nitrate, sulfate, " +
            "and phosphate anions (ions that carry a negative charge) or sodium, magnesium, calcium, iron, and " +
            "aluminum cations (ions that carry a positive charge). Organic compounds like oil, phenol, alcohol, " +
            "and sugar do not conduct electrical current very well and therefore have a low conductivity when in " +
            "water.\nConductivity is also affected by temperature: the warmer the water, the higher the " +
            "conductivity. For this reason, conductivity is reported as conductivity at 25 degrees Celsius (25 C). " +
            "Conductivity in streams and rivers is affected primarily by the geology of the area through which the " +
            "water flows.\nConductivity data can determine concentration of solutions, detect contaminants and " +
            "determine the purity of water. GREON unit conductivity sensor measures conductivity by AC voltage " +
            "applied to nickel electrodes. These electrodes are placed in a water sample (or other liquid), where " +
            "the current flows through the electrodes and the sample. Current level has a direct relationship with " +
            "the conductivity of the solution.\nThe basic unit of conductivity is the siemens (S), sometimes " +
            "referred to as mho. Since cell geometry affects conductivity values, standardized measurements are " +
            "expressed in specific conductivity units (S/cm) to compensate for variations in electrode dimensions.",
        "descriptionSources" : [
            {
                "title": "EPA",
                "link": "http://water.epa.gov/type/rsl/monitoring/vms59.cfm"
            },
            {
                "title": "YSI",
                "link": "http://www.ysi.com/parametersdetail.php?Conductivity-7"
            }
        ],
        "glossary": true,
        "alternates": ["specific-conductance-uScm", "water-specific-conductance-uScm"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "turbidity-fnu",
        "title": "Turbidity",
        "unit": "FNU",
        "categories": ["Water Quality"],
        "description":
            "Turbidity is the measure of relative clarity of a liquid. It is an optical characteristic of water " +
            "and is an expression of the amount of light that is scattered by material in the water when a light " +
            "is shined through the water sample. The higher the intensity of scattered light, the higher the " +
            "turbidity. Material that causes water to be turbid include clay, silt, finely divided inorganic and " +
            "organic matter, algae, soluble colored organic compounds, and plankton and other microscopic " +
            "organisms.\nHigh concentrations of particulate matter affect light penetration and productivity, " +
            "recreational values, and habitat quality, and cause lakes to fill in faster. In streams, increased " +
            "sedimentation and siltation can occur, which can result in harm to habitat areas for fish and other " +
            "aquatic life. Particles also provide attachment places for other pollutants, notably metals and " +
            "bacteria. For this reason, turbidity readings can be used as an indicator of potential pollution in " +
            "a water body.\nTurbidity can be useful as an indicator of the effects of runoff from construction, " +
            "agricultural practices, logging activity, discharges, and other sources. Turbidity often increases " +
            "sharply during a rainfall, especially in developed watersheds, which typically have relatively high " +
            "proportions of impervious surfaces. The flow of stormwater runoff from impervious surfaces rapidly " +
            "increases stream velocity, which increases the erosion rates of streambanks and channels.",
        "descriptionSources": [
            {
                "title": "USGS",
                "link": "http://water.usgs.gov/edu/turbidity.html"
            },
            {
                "title": "EPA",
                "link": "http://water.epa.gov/type/rsl/monitoring/vms55.cfm"
            }
        ],
        "glossary": true,
        "alternates": [ ],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "nitrate-as-n-mgl",
        "title": "Nitrate as N",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description":
            "Nitrates and nitrite are forms of nitrogen, which is found in several different forms in terrestrial " +
            "and aquatic ecosystems. These forms of nitrogen include nitrates (NO3), and nitrites (NO2). Nitrate " +
            "forms in water when bacteria use dissolved oxygen to oxidize ammonium. Nitrates are essential plant " +
            "nutrients, but in excess amounts they can cause significant water quality problems. Together with " +
            "phosphorus, nitrates in excess amounts can accelerate eutrophication, causing dramatic increases in " +
            "aquatic plant growth and changes in the types of plants and animals that live in the stream. This, in " +
            "turn, affects dissolved oxygen, temperature, and other indicators.\nExcess nitrates can cause hypoxia " +
            "(low levels of dissolved oxygen) and can become toxic to warm-blooded animals at higher concentrations " +
            "(10 mg/L) or higher) under certain conditions. The natural level of nitrate in surface water is " +
            "typically low (less than 1 mg/L); in the effluent of wastewater treatment plants, it can range up to " +
            "30 mg/L.\nSources of nitrates include wastewater treatment plants, runoff from fertilized lawns and " +
            "cropland, failing on-site septic systems, runoff from animal manure storage areas, and industrial " +
            "discharges that contain corrosion inhibitors.",
        "descriptionSources": [
            {
                "title": "EPA",
                "link": "http://water.epa.gov/type/rsl/monitoring/vms57.cfm"
            },
            {
                "title": "YSI",
                "link": "http://www.ysi.com/parametersdetail.php?Nitrate-11"
            }
        ],
        "glossary": true,
        "alternates": [],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "par-avg-mv",
        "title": "PAR",
        "unit": "mV",
        "categories": ["Meteorological"],
        "description":
            "Photosynthetically Active Radiation (PAR) measures irradiance--or the amount of sunlight or ambient " +
            "light that diffuses through water compared to surface light. PAR focuses on the dynamics of the photic " +
            "zone, typically 1-5 meters below the surface, and leads to an understanding of photosynthesis, toxic " +
            "algae blooms, and eutrophication (nutrient loading).  Scientists studying phytoplankton and " +
            "photosynthesis and measuring chlorophyll need to measure PAR in order to resolve the absorption rate. " +
            "High levels of PAR can indicate photoinhibition (limiting photosynthesis in shallow waters), which " +
            "affects submerged aquatic vegetation and certain aquatic species.",
        "descriptionSources": [
            {
                "title": "YSI",
                "link": "http://www.ysi.com/parametersdetail.php?Photosynthetically-Active-Radiation---PAR-18"
            }
        ],
        "glossary": true,
        "alternates": [],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "par-density-avg",
        "title": "PAR Density",
        "unit": "",
        "categories": ["Meteorological"],
        "description":
            "PAR Density Average - Photosynthetically Active Radiation (PAR) measures irradiance--or the amount of " +
            "sunlight or ambient light that diffuses through water compared to surface light. PAR focuses on the " +
            "dynamics of the photic zone, typically 1-5 meters below the surface, and leads to an understanding of " +
            "photosynthesis, toxic algae blooms, and eutrophication (nutrient loading).  Scientists studying " +
            "phytoplankton and photosynthesis and measuring chlorophyll need to measure PAR in order to resolve the " +
            "absorption rate. High levels of PAR can indicate photoinhibition (limiting photosynthesis in shallow " +
            "waters), which affects submerged aquatic vegetation and certain aquatic species.",
        "descriptionSources": [
            {
                "title": "YSI",
                "link": "http://www.ysi.com/parametersdetail.php?Photosynthetically-Active-Radiation---PAR-18"
            }
        ],
        "alternates": [],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "blue-green-algae-pct-rfu",
        "title": "Blue Green Algae RFU",
        "unit": "% RFU",
        "categories": ["Biological"],
        "description":
            "Blue Green Algae (BGA) is actually cyanobacteria, and can range in colors from blues, greens, reds, " +
            "and black. BGA can reduce nitrogen and carbon in water, but can also deplete dissolved oxygen when " +
            "overabundant. Monitoring BGA is important because they pose a serious threat to water quality, " +
            "ecosystem stability, surface drinking water supplies, and public health through toxin production " +
            "and the large biomass produced in algal blooms.",
        "descriptionSources": [
            {
                "title": "YSI",
                "link": "http://www.ysi.com/parametersdetail.php?Blue-Green-Algae-3"
            }
        ],
        "glossary": true,
        "alternates": [],
        "detailView" : "ignore",
        "searchView": "ignore",
        "exploreView" : "ignore"
    },
    {
        "id": "blue-green-algae-pct-ugl",
        "title": "Blue Green Algae ug/L",
        "unit": "ug/L",
        "categories": ["Biological"],
        "description":
            "Blue Green Algae in Micrograms per Liter - (BGA) is actually cyanobacteria, and can range in colors " +
            "from blues, greens, reds, and black. BGA can reduce nitrogen and carbon in water, but can also " +
            "deplete dissolved oxygen when overabundant. Monitoring BGA is important because they pose a serious " +
            "threat to water quality, ecosystem stability, surface drinking water supplies, and public health " +
            "through toxin production and the large biomass produced in algal blooms.",
        "descriptionSources": [
            {
                "title": "YSI",
                "link": "http://www.ysi.com/parametersdetail.php?Blue-Green-Algae-3"
            }
        ],
        "alternates": [],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "chlorophyll-mgl",
        "title": "Chlorophyll mg/L",
        "unit": "mg/L",
        "categories": ["Biological"],
        "description":
            "Chlorophyll in various forms is bound within living cells of photosynthetic organisms, such as " +
            "phytoplankton and cyanobacteria (blue-green algae). The amount of chlorophyll found in a water " +
            "sample is used as a measure of the concentration of phytoplankton. These measurements contribute " +
            "to the understanding of the general biological 'health' of the system, such as its trophic status " +
            "or primary production. Chlorophyll measurements can also identify algal bloom events and their " +
            "effects on water quality and anticipate toxic algal blooms.\nChlorophyll fluoresces when irradiated " +
            "with light of a particular wavelength (435-470 nm). For field measurements, in-situ fluorometers " +
            "induce chlorophyll to fluoresce by shining a beam of light of the proper wavelength into the water " +
            "and then measuring the emitted, higher wavelength light. These real-time chlorophyll measurements " +
            "complement extractive lab analysis.",
        "descriptionSources": [
            {
                "title": "YSI",
                "link": "http://www.ysi.com/parametersdetail.php?Chlorophyll-6"
            }
        ],
        "glossary": true,
        "alternates": [],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "chlorophyll-ugl",
        "title": "Chlorophyll ug/L",
        "unit": "ug/L",
        "categories": ["Biological"],
        "description":
            "Chlorophyll in various forms is bound within living cells of photosynthetic organisms, such as " +
            "phytoplankton and cyanobacteria (blue-green algae). The amount of chlorophyll found in a water " +
            "sample is used as a measure of the concentration of phytoplankton. These measurements contribute " +
            "to the understanding of the general biological 'health' of the system, such as its trophic status " +
            "or primary production. Chlorophyll measurements can also identify algal bloom events and their " +
            "effects on water quality and anticipate toxic algal blooms.\nChlorophyll fluoresces when irradiated " +
            "with light of a particular wavelength (435-470 nm). For field measurements, in-situ fluorometers induce " +
            "chlorophyll to fluoresce by shining a beam of light of the proper wavelength into the water and then " +
            "measuring the emitted, higher wavelength light. These real-time chlorophyll measurements complement " +
            "extractive lab analysis.",
        "descriptionSources": [
            {
                "title": "YSI",
                "link": "http://www.ysi.com/parametersdetail.php?Chlorophyll-6"
            }
        ],
        "glossary": true,
        "alternates": [],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "chlorophyll-rfu",
        "title": "Chlorophyll RFU",
        "unit": "RFU",
        "categories": ["Biological"],
        "description":
            "Chlorophyll in Relative Flourescent Units - Chlorophyll in various forms is bound within living cells " +
            "of photosynthetic organisms, such as phytoplankton and cyanobacteria (blue-green algae). The amount of " +
            "chlorophyll found in a water sample is used as a measure of the concentration of phytoplankton. These " +
            "measurements contribute to the understanding of the general biological 'health' of the system, such as " +
            "its trophic status or primary production. Chlorophyll measurements can also identify algal bloom events " +
            "and their effects on water quality and anticipate toxic algal blooms.\nChlorophyll fluoresces when " +
            "irradiated with light of a particular wavelength (435-470 nm). For field measurements, in-situ f" +
            "luorometers induce chlorophyll to fluoresce by shining a beam of light of the proper wavelength into " +
            "the water and then measuring the emitted, higher wavelength light. These real-time chlorophyll " +
            "measurements complement extractive lab analysis.",
        "descriptionSources": [
            {
                "title": "YSI",
                "link": "http://www.ysi.com/parametersdetail.php?Chlorophyll-6"
            }
        ],
        "alternates": [],
        "detailView" : "ignore",
        "searchView": "ignore",
        "exploreView" : "ignore"
    },
    {
        "id": "conductivity-uscm",
        "title": "Conductivity",
        "unit": "uS/cm",
        "categories": ["Water Quality"],
        "description":
            "Conductivity is a measure of the ability of water to pass an electrical current. Conductivity in water " +
            "is affected by the presence of inorganic dissolved solids such as chloride, nitrate, sulfate, and " +
            "phosphate anions (ions that carry a negative charge) or sodium, magnesium, calcium, iron, and aluminum " +
            "cations (ions that carry a positive charge). Organic compounds like oil, phenol, alcohol, and sugar do " +
            "not conduct electrical current very well and therefore have a low conductivity when in " +
            "water.\nConductivity is also affected by temperature: the warmer the water, the higher the " +
            "conductivity. For this reason, conductivity is reported as conductivity at 25 degrees Celsius (25 C). " +
            "Conductivity in streams and rivers is affected primarily by the geology of the area through which the " +
            "water flows.\nConductivity data can determine concentration of solutions, detect contaminants and " +
            "determine the purity of water. GREON unit conductivity sensor measures conductivity by AC voltage " +
            "applied to nickel electrodes. These electrodes are placed in a water sample (or other liquid), where " +
            "the current flows through the electrodes and the sample. Current level has a direct relationship with " +
            "the conductivity of the solution.\nThe basic unit of conductivity is the siemens (S), sometimes " +
            "referred to as mho. Since cell geometry affects conductivity values, standardized measurements are " +
            "expressed in specific conductivity units (S/cm) to compensate for variations in electrode dimensions.",
        "descriptionSources" : [
            {
                "title": "EPA",
                "link": "http://water.epa.gov/type/rsl/monitoring/vms59.cfm"
            },
            {
                "title": "YSI",
                "link": "http://www.ysi.com/parametersdetail.php?Conductivity-7"
            }
        ],
        "glossary": true,
        "alternates": [],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "dissolved-oxygen-mgl",
        "title": "Dissolved Oxygen",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description":
            "Oxygen is measured in its dissolved form as dissolved oxygen (DO). The stream system both produces " +
            "and consumes oxygen. It gains oxygen from the atmosphere and from plants as a result of " +
            "photosynthesis. Running water, because of its churning, dissolves more oxygen than still water, " +
            "such as that in a reservoir behind a dam. Respiration by aquatic animals, decomposition, and " +
            "various chemical reactions consume oxygen. If more oxygen is consumed than is produced, dissolved " +
            "oxygen levels decline and some sensitive animals may move away, weaken, or die.\nSince dissolved " +
            "oxygen levels are directly related to good water quality, the two are highly interdependent. Many " +
            "factors can affect DO levels, and an understanding of these levels in order to make informed " +
            "decisions concerning wastewater treatment operations, hypoxic zones, aquaculture facilities or " +
            "large-scale ecosystems is essential.",
        "descriptionSources": [
            {
                "title": "EPA",
                "link": "http://water.epa.gov/type/rsl/monitoring/vms52.cfm"
            },
            {
                "title": "YSI",
                "link": "http://www.ysi.com/parametersdetail.php?Dissolved-Oxygen-1"
            }
        ],
        "glossary": true,
        "alternates": ["water-dissolved-oxygen-mgl"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "dissolved-oxygen-saturation-pct",
        "title": "Dissolved Oxygen Saturation",
        "unit": "%",
        "categories": ["Water Quality"],
        "description":
            "Dissolved Oxygen Saturation in Percentage - Oxygen is measured in its dissolved form as dissolved " +
            "oxygen (DO). The stream system both produces and consumes oxygen. It gains oxygen from the atmosphere " +
            "and from plants as a result of photosynthesis. Running water, because of its churning, dissolves " +
            "more oxygen than still water, such as that in a reservoir behind a dam. Respiration by aquatic " +
            "animals, decomposition, and various chemical reactions consume oxygen. If more oxygen is consumed " +
            "than is produced, dissolved oxygen levels decline and some sensitive animals may move away, weaken, " +
            "or die.\nSince dissolved oxygen levels are directly related to good water quality, the two are highly " +
            "interdependent. Many factors can affect DO levels, and an understanding of these levels in order to " +
            "make informed decisions concerning wastewater treatment operations, hypoxic zones, aquaculture " +
            "facilities or large-scale ecosystems is essential.",
        "descriptionSources": [
            {
                "title": "EPA",
                "link": "http://water.epa.gov/type/rsl/monitoring/vms52.cfm"
            },
            {
                "title": "YSI",
                "link": "http://www.ysi.com/parametersdetail.php?Dissolved-Oxygen-1"
            }
        ],
        "alternates": ["dissolved-oxygen-percent-saturation", "water-dissolved-oxygen-percent-saturation"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "fluorescent-dissolved-matter-rfu",
        "title": "Fluorescent Dissolved Organic Matter",
        "unit": "RFU",
        "categories": ["Water Quality"],
        "description":
            "Fluorescent Dissolved Organic Matter is found in suspended and bedded sediments (SABS) in the water " +
            "column of the Mississippi River.  SABS include particulate organic and inorganic matter that suspend " +
            "in or are carried by the water, and/or accumulate in a loose, unconsolidated form on the bottom of " +
            "natural water bodies. This includes the frequently used terms of clean sediment, suspended sediment, " +
            "total suspended solids, bedload, turbidity, or in common terms, dirt, soils or eroded materials.  " +
            "EPA's definition of SABS also includes organic solids such as algal material, particulate leaf " +
            "detritus and other organic material.",
        "descriptionSources": [
            {
                "title": "EPA",
                "link": "http://water.epa.gov/scitech/datait/tools/warsss/sabs.cfm"
            }
        ],
        "glossary": true,
        "alternates": [],
        "detailView" : "ignore",
        "searchView": "ignore",
        "exploreView" : "ignore"
    },
    {
        "id": "fluorescent-dissolved-organic-matter-qsu",
        "title": "Fluorescent Dissolved Organic Matter",
        "unit": "QSU",
        "categories": ["Water Quality"],
        "description":
            "Fluorescent Dissolved Organic Matter in Quinine Sulfate Units - Fine Particulate Organic Matter is " +
            "found in suspended and bedded sediments (SABS) in the water column of the Mississippi River.  " +
            "SABS include particulate organic and inorganic matter that suspend in or are carried by the water, " +
            "and/or accumulate in a loose, unconsolidated form on the bottom of natural water bodies. This " +
            "includes the frequently used terms of clean sediment, suspended sediment, total suspended solids, " +
            "bedload, turbidity, or in common terms, dirt, soils or eroded materials.  EPA's definition of SABS " +
            "also includes organic solids such as algal material, particulate leaf detritus and other " +
            "organic material.",
        "descriptionSources": [
            {
                "title": "EPA",
                "link": "http://water.epa.gov/scitech/datait/tools/warsss/sabs.cfm"
            }
        ],
        "alternates": [],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "soluble-reactive-phosphorus-mgl",
        "title": "Soluble Reactive Phosphorus",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "phosphorus-insitu-orthophosphate-as-p-mgl",
        "title": "Phosphorous in Situ Orthophosphate as P",
        "unit": "mg/L",
        "alternates": [],
        "categories": ["Water Quality"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "total-phosphorus-mgl",
        "title": "Total Phosphorus",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "total-soluble-nitrogen-mgl",
        "title": "Total Soluble Nitrogen",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "total-soluble-phosphorus-mgl",
        "title": "Total Soluble Phosphorus",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "water-temperature-c",
        "title": "Water Temperature",
        "unit": "C",
        "categories": ["Meteorological"],
        "description":
            "The rates of biological and chemical processes depend on temperature. Aquatic organisms from " +
            "microbes to fish are dependent on certain temperature ranges for their optimal health. Optimal " +
            "temperatures for fish depend on the species: some survive best in colder water, whereas others " +
            "prefer warmer water. Benthic macroinvertebrates are also sensitive to temperature and will move " +
            "in the stream to find their optimal temperature. If temperatures are outside this optimal range " +
            "for a prolonged period of time, organisms are stressed and can die. Temperature is measured in " +
            "degrees Fahrenheit (F) or degrees Celsius (C).\nTemperature affects the oxygen content of the " +
            "water (oxygen levels become lower as temperature increases); the rate of photosynthesis by aquatic " +
            "plants; the metabolic rates of aquatic organisms; and the sensitivity of organisms to toxic wastes, " +
            "parasites, and diseases.\nCauses of temperature change include weather, removal of shading streambank " +
            "vegetation, impoundments (a body of water confined by a barrier, such as a dam), dis-charge of cooling " +
            "water, urban storm water, and groundwater inflows to the stream.",
        "glossary": true,
        "alternates": ["temperature-c"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id":"pH",
        "title": "Water Acidity",
        "unit":"pH",
        "alternates": ["acidity-ph"],
        "categories": ["Water Quality"],
        "detailView" : "visible",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id":"water-turbidity-fnu",
        "title": "Water Turbidity",
        "unit":"FNU",
        "categories": ["Water Quality"],
        "alternates": [],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id":"water-turbidity-ntu",
        "title": "Water Turbidity",
        "unit":"NTU",
        "categories": ["Water Quality"],
        "alternates": ["turbidity-ntu"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id":"colored-dissolved-organic-matter-ppbqse",
        "title": "Colored Dissolved Organic Matter",
        "unit":"ppb QSE",
        "alternates": [],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id":"total-nitrogen-lbs",
        "title": "Total Annual Nitrogen",
        "unit":"lbs",
        "alternates": [],
        "categories": ["Water Quality"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id":"total-phosphorus-lbs",
        "title": "Total Annual Phosphorous",
        "unit":"lbs",
        "alternates": [],
        "categories": ["Water Quality"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "ammonia-unionized-as-n-mgl",
        "title": "Ammonia Unionized as N",
        "unit": "mg/L",
        "alternates": [],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "ammonia-unionized-temp-ph-nh4-mgl",
        "title": "Ammonia Unionized Temp pH NH4",
        "unit": "mg/L",
        "alternates": [],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "nitrate-nitrite-total-as-n-mgl",
        "title": "Nitrate Nitrite Total as N",
        "unit": "mg/L",
        "alternates": [],
        "categories": ["Water Quality"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "nitrate-nitrogen-total-as-n-mgl",
        "title": "Nitrate Nitrogen Total as N",
        "unit": "mg/L",
        "alternates": [],
        "categories": ["Water Quality"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "nitrite-nitrogen-dissolved-as-n-mgl",
        "title": "Nitrite Nitrogen Dissolved as N",
        "unit": "mg/L",
        "alternates": [],
        "categories": ["Water Quality"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "nitrite-nitrogen-total-as-n-mgl",
        "title": "Nitrite Nitrogen Total as N",
        "unit": "mg/L",
        "alternates": [],
        "categories": ["Water Quality"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "nitrite-plus-nitrate-diss-1det-as-n-mgl",
        "title": "Nitrite + Nitrate Diss 1det as N",
        "unit": "mg/L",
        "alternates": [],
        "categories": ["Water Quality"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "nitrogen-ammonia-total-as-n-mgl",
        "title": "Nitrogen Ammonia Total as N",
        "unit": "mg/L",
        "alternates": [],
        "categories": ["Water Quality"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "nitrogen-ammonia-dissolved-as-n-mgl",
        "title": "Nitrogen Ammonia Dissolved as N",
        "unit": "mg/L",
        "alternates": [],
        "categories": ["Water Quality"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "nitrogen-kjeldahl-total-as-n-mgl",
        "title": "Nitrogen Kjeldahl Total as N",
        "unit": "mg/L",
        "alternates": [],
        "categories": ["Water Quality"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "nitrogen-kjeldahl-total-bottom-dep-dry-wt-Mgkg",
        "title": "Nitrogen Kjehdahl Total Bottom Dep Dry Weight",
        "unit": "Mg/kg",
        "alternates": [],
        "categories": ["Water Quality"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "nitrogen-kjeldahl-dissolved-as-n-mgl",
        "title": "Nitrogen Kjeldahl Dissolved as N",
        "unit": "mg/L",
        "alternates": [],
        "categories": ["Water Quality"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "nitrogen-organic-total-as-n-mgl",
        "title": "Nitrogen Organic Total as N",
        "unit": "mg/L",
        "alternates": [],
        "categories": ["Water Quality"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "nitrogen-total-as-n-mgl",
        "title": "Nitrogen Total as N",
        "unit": "mg/L",
        "alternates": [],
        "categories": ["Water Quality"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "nitrogen-total-as-no3-mgl",
        "title": "Nitrogen Total as NO3",
        "unit": "mg/L",
        "alternates": [],
        "categories": ["Water Quality"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "phosphorus-total-as-p-mgl",
        "title": "Phosphorous Total as P",
        "unit": "mg/L",
        "alternates": [],
        "categories": ["Water Quality"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "phosphorus-dissolved-as-p-mgl",
        "title": "Phosphorous Dissolved as P",
        "unit": "mg/L",
        "alternates": [],
        "categories": ["Water Quality"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "phosphorus-dissolved-orthophosphate-as-p-mgl",
        "title": "Phosphorous Dissolved Orthophosphate as P",
        "unit": "mg/L",
        "alternates": [],
        "categories": ["Water Quality"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "phosphorus-in-total-orthophosphate-as-p-mgl",
        "title": "Phosphorous in Total Orthophosphate as P",
        "unit": "mg/L",
        "alternates": [],
        "categories": ["Water Quality"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },
    {
        "id": "phosphorus-total-bottom-deposit-dry-wgt-Mgkg",
        "title": "Phosphorous Total Bottom Deposit Dry Weight",
        "unit": "Mg/kg",
        "alternates": [],
        "categories": ["Water Quality"],
        "detailView" : "time",
        "searchView": "visible",
        "exploreView" : "visible"
    },{
        "id": "nitrate-nitrite-inorganic-total-as-n-mgl",
        "title": "Inorganic Nitrogen (Nitrate and Nitrite) Total",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },{
        "id": "suspended-solids-total-mgl",
        "title": "Total Suspended Solids",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },{
        "id": "volatile-solids-suspended-mgl",
        "title": "Volatile Suspended Solids",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },{
        "id": "volatile-solids-total-mgl",
        "title": "Volatile Solids Total",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },{
        "id": "chlorophyll-b-ugl",
        "title": "Chlorophyll B Total",
        "unit": "ug/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },{
        "id": "chlorophyll-c-ugl",
        "title": "Chlorophyll C Total",
        "unit": "ug/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },{
        "id": "chlorophyll-a-corrected-for-pheophytin-ugl",
        "title": "Chlorophyll A Corrected for Pheophytin Total",
        "unit": "ug/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },{
        "id": "chlorophyll-a-uncorrected-for-pheophytin-ugl",
        "title": "Chlorophyll A Uncorrected for Pheophytin Total",
        "unit": "ug/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },{
        "id": "alkalinity-total-mgl",
        "title": "Alkalinity Total",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },{
        "id": "nitrate-mgl",
        "title": "Nitrate Total",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },{
        "id": "total-nitrite-mgl",
        "title": "Nitrite Total",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "nitrogen-dissolved-as-n-mgl",
        "title": "Nitrogen Dissolved (MG/L AS N)",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "phosphorus-sed-bot-<63-wet-sieve-field-total-pct",
        "title": "Phosphorus, SED, BOT, <63, Wet Sieve, Field, Total",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "nitrate-nitrogen-dissolved-as-no3-mgl",
        "title": "Nitrate Nitrogen, Dissolved (MG/L AS NO3)",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "nitrogen-ammonia-dissolved-as-nh4-mgl",
        "title": "Nitrogen Ammonia, Dissolved (MG/L AS NH4)",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "phosphate-ortho-as-po4-mgl",
        "title": "Phosphate Ortho(MG/L AS PO4)",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "nitrogen-ammonia-bottom-deposits-MgKg",
        "title": "Nitrogen Ammonia, Bottom Deposits (MG/KG-N)",
        "unit": "MgKg",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "phosphate-total-as-po4-mgl",
        "title": "Phosphate Total (MG/L AS PO4",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "ortho-phosphate-nes-algal-assay-mgl",
        "title": "Ortho-Phosphate NES Algal Assay",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "phosphate-total-soluble-mgl",
        "title": "Phosphate Total Soluble",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "phosphate-poly-as-po4-mgl",
        "title": "Phosphate Poly (MG/L AS PO4)",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "phosphorus-total-as-po4-mgl",
        "title": "Phosphorous Total (MG/L AS PO4)",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "phosphorus-sed-bot-<63-dry-sieve-lab-totals-pct",
        "title": "Phosphorus SED BOT <63 Dry Sieve Lab Total ",
        "unit": "pct",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "nitrate+nitrite-mgl",
        "title": "Nitrate Plus Nitrite",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "chlorophyll-a-corrected-for-pheophytin-ugl",
        "title": "Chlorophyll A Corrected for Pheophytin Total",
        "unit": "ug/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": ["Chlorophyll a, corrected for pheophytin"],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "chlorophyll-a-uncorrected-for-pheophytin-ugl",
        "title": "Chlorophyll A Uncorrected for Pheophytin Total",
        "unit": "ug/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": ["Chlorophyll a, uncorrected for pheophytin"],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "flourescent-dissolved-organic-matter-qsu",
        "title": "Flourescent Dissolved Organic Matter QSU",
        "unit": "QSU",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "flourescent-dissolved-organic-matter-rfu",
        "title": "Flourescent Dissolved Organic Matter RFU",
        "unit": "RFU",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "specific-conductivity-uscm",
        "title": "Specific Conductivity",
        "unit": "USCM",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "ammonium-mgl",
        "title": "Ammonium",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "ignore", // depth, or ignore
        "searchView": "ignore", // only checking if it is != ignore as of 2014-06
        "exploreView" : "ignore" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "compass-heading-24hr-avg",
        "title": "Compass Heading 24hr Avg",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "ignore", // depth, or ignore
        "searchView": "ignore", // only checking if it is != ignore as of 2014-06
        "exploreView" : "ignore" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "air-temperature-24hr-avg-f",
        "title": "Air Temperature 24hr Avg",
        "unit": "F",
        "categories": ["Meteorological"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "wind-speed-mph",
        "title": "Wind Speed",
        "unit": "F",
        "categories": ["Meteorological"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "wind-direction",
        "title": "Wind Direction",
        "unit": "Deg",
        "categories": ["Meteorological"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "chloride-mgl",
        "title": "Chloride",
        "unit": "mg/L",
        "categories": ["Water Quality"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "precipitation-in",
        "title": "Precipitation",
        "unit": "in",
        "categories": ["Meteorological"],
        "description": "",
        "alternates": [],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    }
];


app.config.sensors.items = [];

app.config.sources.items = [
    {
        "id": "usgs",
        "title": "United States Geological Survey",
        "prettyName": "United States Geological Survey",
        "description": "The USGS investigates the occurrence, quantity, quality, distribution, and movement of surface and " +
            "underground waters and disseminates the data to the public, State and local governments, public and " +
            "private utilities, and other Federal agencies involved with managing our water resources. U.S. " +
            "Geological Survey conducts long-term monitoring of lakes and rivers nationwide. They have gauges " +
            "throughout many rivers in the Mississippi River basin. We've chosen to focus on gauges on the " +
            "Mississippi River and its tributaries that are equipped with continuous nitrate sensors for this website.",
        "color": "#39B54A",
        "parameters":{
            "description": "The following are a subset of parameters that are available from USGS:",
            "ids":["discharge-ft3s","nitrate-mgl","load-Mg","cumulative-load-Mg"]
        },
        "moreinfo" : {
            "text" : "For more information and for access to all USGS water quantity and quality data.",
            "link" : "http://waterdata.usgs.gov/nwis"
        },
        "sop" : {
            "text" :
                "For more information on collection methods and analysis, " +
                "take a look at the Standard of Operating Procedures.",
            "link" : "http://pubs.usgs.gov/fs/2007/3043/FS2007-3043.pdf"
        }
    },
    {
        "id": "usgs-sg",
        "title": "United States Geological Survey (Supergauge)",
        "prettyName": "United States Geological Survey (Supergauge)",
        "description": "The USGS investigates the occurrence, quantity, quality, distribution, and movement of surface and " +
            "underground waters and disseminates the data to the public, State and local governments, public and " +
            "private utilities, and other Federal agencies involved with managing our water resources. U.S. " +
            "Geological Survey conducts long-term monitoring of lakes and rivers nationwide. They have gauges " +
            "throughout many rivers in the Mississippi River basin. We've chosen to focus on gauges on the " +
            "Mississippi River and its tributaries that are equipped with continuous nitrate sensors for this website.",
        "color": "#228e31",
        "parameters":{
            "description": "The following are a subset of parameters that are available from USGS:",
            "ids":["discharge-ft3s","nitrate-mgl","load-Mg","cumulative-load-Mg"]
        },
        "moreinfo" : {
            "text" : "For more information and for access to all USGS water quantity and quality data.",
            "link" : "http://waterdata.usgs.gov/nwis"
        },
        "sop" : {
            "text" :
                "For more information on collection methods and analysis, " +
                "take a look at the Standard of Operating Procedures.",
            "link" : "http://pubs.usgs.gov/fs/2007/3043/FS2007-3043.pdf"
        }
    },
    {
        "id": "greon",
        "title": "Great Rivers Ecological Observation Network ",
        "prettyName": "Great Rivers Ecological Observation Network ",
        "description": "The GREON℠ program seeks to establish a network of real-time water quality monitoring platforms " +
            "on great rivers around the world. NGRREC℠ partnered with YSI Inc. to design and launch a monitoring " +
            "buoy capable of real-time, continuous collection of water quality and phytoplankton data. The first " +
            "YSI PISCES (Pontoon for In-situ Characterization of Environmental Systems) buoy launched in May 2013 " +
            "on the Upper Mississippi River System, but the program aims to expand to deploy platforms across the " +
            "globe on other international great river systems.",
        "color": "#BDBFA6",
        "parameters":{
            "description": "The following parameters are available from GREON:",
            "ids":[
                "blue-green-algae-pct-rfu","blue-green-algae-pct-ugl","chlorophyll-mgl","chlorophyll-rfu",
                "conductivity-uscm","dissolved-oxygen-mgl","dissolved-oxygen-saturation-pct","par-avg-mv",
                "par-density-avg","fine-dissolved-organic-matter-qsu","fine-dissolved-organic-matter-rfu",
                "SUNA_LightAve","SUNA_Nitrate_mgL","SUNA_Nitrate_uM"
            ]
        },
        "moreinfo" : {
            "text" : "More information about GREON",
            "link" : "http://www.ngrrec.org/GREON/"
        }
    },
    {
        "id": "umrr-ltrm",
        "title": "Upper Mississippi River Restoration",
        "prettyName": "Upper Mississippi River Restoration",
        "acronym": "UMRR LTRM",
        "description": "The UMRR LTRM water quality sampling design focuses on a subset of limnological variables related " +
            "to habitat quality and ecosystem function that includes physicochemical features, suspended sediment, " +
            "and major plant nutrients known to be significant to aquatic habitat in this system. We've chosen to " +
            "focus on long term nitrate data sets available on the Mississippi River for this website.",
        "color": "#5F99C1",
        "parameters": {
            "description": "The following are a subset of parameters that are available from UMRR LTRM:",
            "ids":["nitrate-nitrite-as-n","total-nitrogen-mgl"]
        },
        "moreinfo" : {
            "text" : "Click here for more information and for access to all original UMRR LTRM data.",
            "link" : "http://www.umesc.usgs.gov/data_library/water_quality/water1_query.shtml"
        },
        "sop" :{
            "text": "Click here for more information on collection methods and analysis in the " +
            "UMRR LTRM Procedures Manual.",
            "link": "http://www.umesc.usgs.gov/documents/reports/2004/04t00201.pdf"
        }
    },
    {
        "id": "sierra-club",
        "title": "Fox River Study Group",
        "prettyName": "Fox River Study Group",
        "description": "These locations contain data related to the Fox River Study Group.",
        "color": "#9C2C1A",
        "moreinfo" : {
            "text" : "Click here for more information about the Fox River Study Group",
            "link" : "http://foxriverstudygroup.org/"
        },
        "parameters":{
            "description": "The following parameters are available from the Sierra Club Fox River data:",
            "ids":[
                "nitrate-nitrite-total-as-n-mgl","nitrate-nitrogen-total-as-n-mgl","nitrite-nitrogen-dissolved-as-n-mgl",
                "nitrite-nitrogen-total-as-n-mgl","nitrite-plus-nitrate-diss-1det-as-n-mgl","nitrogen-ammonia-dissolved-as-n-mgl",
                "nitrogen-ammonia-total-as-n-mgl","nitrogen-dissolved-as-n-mgl","nitrogen-kjeldahl-dissolved-as-n-mgl",
                "nitrogen-kjeldahl-total-as-n-mgl","phosphorus-dissolved-as-p-mgl","phosphorus-dissolved-orthophosphate-as-p-mgl",
                "phosphorus-sed-bot-<63-wet-sieve-field-total-pct","phosphorus-total-as-p-mgl"
            ]
        }
    },
    {
        "id": "illinois-epa",
        "title": "IEPA Ambient Water Quality Monitoring Network",
        "prettyName": "IEPA Ambient Water Quality Monitoring Network",
        "color": "#d89b0d",
        "description":
            "These locations contain data related to the " +
            "Iepa Ambient Water Quality Monitoring Network.",
        "moreinfo" : {
            "text" : "Click here for more information about the IEPA Ambient Water Quality Monitoring Network (AWQMN)",
            "link" : "http://www.epa.illinois.gov/topics/water-quality/monitoring/river-and-stream/"
        },
        "parameters":{
            "description": "The following parameters are available from IEPA AWQMN:",
            "ids":[
                "nitrogen-kjeldahl-total-as-n-mgl","phosphorus-dissolved-as-p-mgl","phosphorus-total-as-p-mgl","pH"
            ]
        }
    }
];

app.config.views = {
    welcome: {
        title: "Welcome",
        link: "#welcome",
        "template": "#welcome-template"
    },
    home: {
        title: "Explore",
        link: "#explore",
        attributes: {
            id: 'home-view'
        },
        templateSelector: "#explore-template",
        explore: {
            attributes: {
                class: "accordion"
            },
            map: {
                maxBounds: [
                    [17, -107],
                    [59, -71]
                ],
                minZoom: 7,
                maxZoom: 18,
                scrollWheelZoom: false,
                id: "#explore-map-view",
                mapTileSource: '//{s}.tile.openstreetmap.org/{z}/{x}/{y}.png',
                mapAttribution: '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors',
                clusterSize: 45,
                cluster: true,
                center: [38,-89]
            },
            accordion: {
                containerID: "#explore-accordion-container",
                // set the defaults for an accordion item
                // can be overridden in an item object
                template: '#explore-accordion-item-template',
                class: 'accordion-group',
                hidePills: true,
                maxPills: 20,
                maxPillsCustom: 24,
                items: [
                    {
                        attributes: {
                            id: "layers",
                            class: 'accordion-group'
                        },
                        title: "Explore Layers",
                        defaultOpen: false,
                        showStationLegend: false,
                        contents: {
                            viewType: "LayersCollectionView",
                            containerID: "explore-layers-accordion-contents",
                            viewConfig: {
                                templateID: '#explore-layers-template'
                            }
                        }
                    }, {
                        attributes: {
                            id: "sources"
                        },
                        title: "Explore Data by Source",
                        defaultOpen: true,
                        showStationLegend: true,
                        contents: {
                            viewType: "ExploreSourcesView",
                            containerID: "explore-sources-accordion-contents",
                            viewConfig: {
                                templateID: '#explore-sources-template',
                                map: {
                                    id: "#explore-map-view",
                                    scrollWheelZoom: false
                                },
                                modalsID: "#source-modals-template",
                                pills: {
                                    className: ".explore-pill",
                                    tooltipOptions: {
                                        animation: true,
                                        placement: "right",
                                        container: "body"
                                    }
                                },
                                collapse: {
                                    className: '.collapse',
                                    inputSelector: 'input[name="source"]'
                                }
                            }
                        }
                    },
                    {
                        attributes: {
                            id: "explore-accordion-group-watershed",
                            class: 'accordion-group'
                        },
                        title: "Explore Data by Watershed",
                        defaultOpen: false,
                        showStationLegend: true,
                        contents: {
                            viewType: "ExploreByCustomView",
                            containerID: "explore-custom-accordion-contents",
                            viewConfig: {
                                groupingOrder: ["huc4", "sources", "sensors"],
                                primaryCollection: "sensors",
                                collectionGroupingFunction: "groupByHUCcodeSourcesSensors",
                                collectionGroupingFunctionArguments: {
                                    hucCode: 4
                                },
                                templateID: '#explore-custom-template',
                                map: {
                                    id: "#explore-map-view",
                                    scrollWheelZoom: false
                                },
                                modalsID: false
                            }
                        }
                    },
                    {
                        attributes: {
                            id: "explore-accordion-group-featured-watershed",
                            class: 'accordion-group'
                        },
                        title: "Featured Watersheds",
                        defaultOpen: true,
                        showStationLegend: true,
                        contents: {
                            viewType: "ExploreByCustomView",
                            containerID: "explore-custom-accordion-contents",
                            viewConfig: {
                                groupingOrder: ["huc8", "sources", "sensors"],
                                primaryCollection: "sensors",
                                collectionGroupingFunction: "groupByHUCcodeSourcesSensors",
                                collectionGroupingFunctionArguments: {
                                    hucCode: 8,
                                    includedCodes: [
                                        '05120109', '05120112', '05120114',
                                        '07080104', '07080209',
                                        '07090005',
                                        '07110009',
                                        '07120004', '07120006',
                                        '07130001', '07130002', '07130006',
                                        '07140106'
                                    ]
                                },
                                templateID: '#explore-custom-template',
                                map: {
                                    id: "#explore-map-view",
                                    scrollWheelZoom: false
                                },
                                modalsID: false
                            }
                        }
                    }
                ]
            },
            "moreinfo": {
                "text": "More information about EPA/GLMPO",
                "link": "http://www.epa.gov/greatlakes/"
            }
        }
    },
    compare: {
        title: "Compare",
        link: "#compare",
        removeBinDropdown: true,
        miniMap: {
            id: "compare-sidebar-minimap",
            coordinates: [
                [28, -123],
                [48, -68]
            ],
            config: {
                tagName: 'div',
                mapOptions: {
                    zoomControl: false,
                    attributionControl: false,
                    dragging: false,
                    touchZoom: false,
                    scrollWheelZoom: false,
                    doubleClickZoom: false,
                    boxZoom: false,
                    mapTileSource: '//{s}.tile.openstreetmap.org/{z}/{x}/{y}.png',
                    mapAttribution: '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors',
                    center: [40, -88],
                    zoom: 4

                },
                geoJsonFeature: []
            }
        },
        subviews: {
            sensors: {
                label: "Stations",
                sidebar: {
                    template: "#compare-sidebar-stations-template",
                    clickableDOMItems: 'button.compare-pill'
                },
                graph: {
                    template: "#compare-graph-stations-template"
                }
            },
            areas: {
                label: "Lakes",
                sidebar: {
                    template: "#compare-sidebar-areas-template"
                },
                graph: {
                    template: "#compare-graph-areas-template"
                }
            }
        }
    },
    detail: {
        attributes: {
            id: "detail-view",
            class: 'row-fluid'
        },
        templateSelector: "#detail-template",
        timeSeriesFullExtent: true,
        miniMap: [
            {
                id: "#mini-map",
                config: {
                    tagName: 'div',
                    mapOptions: {
                        zoomControl: false,
                        attributionControl: false,
                        dragging: false,
                        touchZoom: false,
                        scrollWheelZoom: false,
                        doubleClickZoom: false,
                        boxZoom: false,
                        mapTileSource: '//{s}.tile.openstreetmap.org/{z}/{x}/{y}.png',
                        mapAttribution: '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors'
                    },
                    geoJsonFeature: []
                }
            }
        ],
        controlsViews: [{
            name: "controlsView",
            config: {
                name: "time",
                el: "#modal-content",
                parametersID: "parameters-time",
                dateRangeID: "date-range-time",
                sliderID: "slider-time",
                attributes: {
                    tagName: 'div',
                    id: 'controls'
                }
            }

        }, {
            name: "controlsDepthView",
            config: {
                name: "depth",
                el: "#modal-content",
                yearFormID: "year-depth",
                yearSelectID: "year-depth-select",
                parametersID: "parameters-depth",
                attributes: {
                    tagName: 'div',
                    id: 'controls'
                }
            }

        }],
        timeSeriesViews: [],
        limnologyViews: []
    },
    search: {
        sites_count_search_from_sources: false,
        title: app.config.search_link.title,
        link: app.config.search_link.url,
        attributes: {
            id: "search-body",
            class: 'row-fluid'
        },
        templateSelector: "#search-template",
        map: {
            id: '#search-map',
            maxBounds: [
                [20, -106],
                [56, -72]
            ],
            minZoom: 6,
            maxZoom: 12,
            scrollWheelZoom: false,
            mapTileSource: '//{s}.tile.openstreetmap.org/{z}/{x}/{y}.png',
            mapAttribution: '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors',
            markerFillColor: "#C0C0C0",
            markerColor: "#000000",
            radius: 3,
            weight: 1,
            opacity: 1,
            fillOpacity: 1
        },
        shapeOptions: {
            stroke: true,
            color: '#20201F',
            weight: 4,
            opacity: 0.5,
            fill: true,
            fillColor: null, //same as color by default
            fillOpacity: 0.2,
            clickable: true
        },
        drawOptions: {
            position: 'topright',
            draw: {
                polyline: false,
                marker: false
            },
            edit: {
                remove: true
            }
        },
        resetMapButtonOptions: {
            text: 'Reset Search', // string
            iconUrl: '', // string
            glyph: '', // bootstrap glyph icons
            hideText: true, // bool
            maxWidth: 30, // number
            doToggle: false, // bool
            toggleStatus: false // bool
        },
        layers: {
            layerWeight: 1,
            layerOpacity: 1,
            bindPopupClass: "search-popup",
            items: [{
                name: "basins",
                featureProperties: "LAKEBASIN"
            }]
        },
        dateRangeSelector: {
            id: '#searchtime',
            format: 'YYYY-MM-DD h:mm:ss',
            opens: 'right'
        },
        downloadButtons: {
            permalink: {
                id: "#permalink",
                downloadID: "#download-permalink",
                template: "#permalink-template",
                modalID: "#permalink-modal"
            },
            json: {
                id: '#download-json'
            },
            csv: {
                id: "#download-csv"
            },
            visualize: {
                id: "#download-visualize"
            }

        }
    },
    footer: {
        templateID: "#footer-template"
    },
    about: {
        title: "About",
        link: "#about",
        "template": "#about-template",
        "children": [
            {
                "title": "Partners",
                "link": "#partners",
                "template": "#partners-template"
            },
            {
                "title": "Glossary",
                "link": "#glossary",
                "template": "#glossary-template"
            },
            {
                "title": "Help",
                "link": "#help",
                "template": "#help-template"
            },
            {
                "title": "FAQ",
                "link": "#faq",
                "template": "#faq-template"
            }
        ]
    }
};

// your public api url. Prod and Dev use this read-only, so it is staticly set here.
app.config.clowderUrl = window.location.origin + "/clowder";

// loads jQuery and Backbone
(function($, Backbone, window, undefined) {
    "use strict";

app.graphs.boxwhisker = function() {
	var width = 185, height = 500, vMargin = 40, hMargin = 50;

	function boxwhisker(selection) {
		selection.each(function(d) {
			d = d.values[0];
      if (app.config.features.boxWhiskerRaw) {
        d = _.flatten(d.values.map(function(d) { return d.raw; })).sort(d3.ascending);
      } else {
        d = _.flatten(d.values.map(function(d) { return d.y; })).sort(d3.ascending);
      }

			var x = d3.scale.quantize().domain([0, 100]).range([0, hMargin, width / 2, width - hMargin, width]),
				y = d3.scale.linear().domain([Math.min(0, d[0]), d[d.length - 1]]).range([height - vMargin, vMargin]);

			var median = d3.quantile(d, 0.5),
				quartileRange = [d3.quantile(d, 0.25), d3.quantile(d, 0.75)],
				IQR = (quartileRange[1] - quartileRange[0]),
				extremefn = function(d) { return d < quartileRange[0] - 3 * IQR || d > quartileRange[1] + 3 * IQR; },
				outlierfn = function(d) { return !extremefn(d) && (d < quartileRange[0] - 1.5 * IQR || d > quartileRange[1] + 1.5 * IQR); },
				extremes = d.filter(function(d) { return extremefn(d); }),
				outliers = d.filter(function(d) { return outlierfn(d); }),
				mean = d3.mean(d),
				extent = d3.extent(d.filter(function(d) { return !(outlierfn(d) || extremefn(d)); }));

			var svg = d3.select(this).selectAll(".graphs-bw").data([0]);
			svg.enter().append("svg").attr("xmlns", "http://www.w3.org/2000/svg").attr("version", "1.1")
				.attr("preserveAspectRatio", "none").attr("class", "graphs-bw");
			svg.attr("viewBox", "0 0 " + width + " " + height);
			
			// Create lines which represent the upper and lower quartiles.
			var line = svg.selectAll(".graphs-quartile-range").data([x(25), x(75)]);
			line.enter().append("line").attr("class", "graphs-quartile-range");
			line.transition().attr("x1", function(d) { return d; }).attr("x2", function(d) { return d; })
				.attr("y1", y(quartileRange[0])).attr("y2", y(quartileRange[1]));
				
			// Create the circle that represents the mean.	
			var circle = svg.selectAll(".graphs-mean").data([mean]);
			circle.enter().append("circle").attr("class", "graphs-mean");
			circle.transition().attr("cy", function(d) { return y(d); })
				.attr("r", ((x(75) - x(25))/16)).attr("cx", x(50));
			
			
			// Create the lines that connect the quartiles to their extrema	
			var connection = svg.selectAll(".graphs-connection").data(extent);
			connection.enter().append("line").attr("class", "graphs-connection");
			connection.transition().attr("x1", x(50)).attr("y1", function(d) { return y(d); })
				.attr("x2", x(50)).attr("y2", function(d, i) { return y(quartileRange[i]); });
			
			// Create the circles for the outliers	
			var outlier = svg.selectAll(".graphs-outlier").data(outliers);
			outlier.enter().append("circle").attr("class", "graphs-outlier")
				.attr("cy", function(d) { return d < extent[0] ? height + 20 : -10; });
			outlier.exit().transition().style("opactiy", "0")
				.attr("cy", function(d) { return d3.select(this).attr("cy") >= height / 2 ? height + 20 : -10; }).remove();
			outlier.transition().attr("cy", function(d) { return y(d); })
				.attr("r", ((x(75) - x(25))/32)).attr("cx", x(50));

			// create the circles for the extreme outliers
			var extreme = svg.selectAll(".graphs-extreme").data(extremes);
			extreme.enter().append("circle").attr("class", "graphs-extreme")
				.attr("cy", function(d) { return d < extent[0] ? height + 20 : -10; });
			extreme.exit().transition().style("opactiy", "0")
				.attr("cy", function(d) { return d3.select(this).attr("cy") >= height / 2 ? height + 20 : -10; }).remove();
			extreme.transition().attr("cy", function(d) { return y(d); })
				.attr("r", ((x(75) - x(25))/32)).attr("cx", x(50));

			
			// Create the labels for all the interesting features, min, max, quartiles, median
			extent.splice(1, 0, quartileRange[0], median, quartileRange[1]);
			var feature = svg.selectAll(".graphs-feature").data(extent),
				featureEnter = feature.enter().append("g").attr("class", "graphs-feature");

			featureEnter.append("line");
			featureEnter.append("text");

			feature.select("line").transition().attr("x1", x(25)).attr("x2", x(75))
				.attr("y1", function(d) { return y(d); })
				.attr("y2", function(d) { return y(d); });

			feature.select("text").transition().attr("dy", ".32em")
				.attr("x", function(d, i)  { return i % 2 ? x(25) -5 : x(75) + 5; })
				.attr("y", function(d)  { return y(d); }).text(function(d) { return Math.round(d*100)/100; }) // use d3.format
				.style("text-anchor", function(d, i) { return i % 2 ? "end" : "start"; });
		});
	}
	
	boxwhisker.width = function(value) {
		if(!arguments.length) return width;
		width = value;
		return boxwhisker;
	};
	
	boxwhisker.height = function(value) {
		if(!arguments.length) return height;
		height = value;
		return boxwhisker;
	};
	
	boxwhisker.margin = function(vertical, horizontal) {
		if(!arguments.length) return [vMargin, hMargin];
		
		vMargin = vertical;
		
		if(horizontal === undefined) 
			hMargin = vertical;
		else hMargin = horizontal;
		
		return boxwhisker;
	};
	
	return boxwhisker;
};

app.graphs.depth = function() {
    var width = 510, height = 500, margin = 40,
        min = 0, max = 10, interpolate = "linear",
        dispatch = null;
    
    function depth(selection) {
		selection.each(function(d) {
			d = mapper(d);
			
			var ys = yScale(d.values);
			
			var y = d3.scale.linear().range([margin, height]).domain(ys);
            	
            var range = Array.range(+ys[0], +ys[1]);
            
            var graph = d3.select(this).selectAll("svg").data([0]);
            graph.enter()
                .append("svg")
                .attr("class", "graphs-depth-chart");
            graph.attr("width", width).attr("height", height + d.values.length * margin);
            
            // Y-Axis Label
            if(app.config.graphs.depthProfile.showUnits.graphYAxis)
            {
                var yAxisLabel = graph.selectAll(".graphs-yAxisLabel").data([0]);
                
                yAxisLabel.enter()
                    .append("text")
                    .attr("class", "graphs-yAxisLabel");
                
                yAxisLabel.transition()
                    .attr("transform", "rotate(-90)")
                    .style("text-anchor", "middle")
                    .attr("x", -215) // Moving up/down. + (up), - (down)
                    .attr("y", 10) // Moving left/right. + (right), - (left)
                    .text("Meters");
                
                yAxisLabel.exit().remove();
            }
                
            /* Y Axis */
            var yAxis = graph.selectAll(".graphs-y-axis").data([d.values]);
            yAxis.enter().append("g").attr("class", "graphs-y-axis")
                .attr("transform", "translate(" + margin + ", 0)");
            yAxis.exit().remove();
            yAxis.transition().call(d3.svg.axis().scale(y).orient("left"));
            
            if (false) {
              var depthBins = graph.selectAll(".graphs-depth-bin").data(range);
              depthBins.enter().append("rect").attr("class", "graphs-depth-bin").style("color", "white")
                  .style("opacity", "0").style("cursor", "pointer")
                  .on("mouseover.graphs", function(d, i) {
                      d3.select(this).transition().style("color", "grey").style("opacity", "0.6");
                  }).on("mouseout.graphs", function(d, i) {
                      d3.select(this).transition().style("color", "white").style("opacity", "0");
                  }).on("click.graphs", function(d, i) {
                      d3.selectAll(".graphs-selected-bin").classed("graphs-selected-bin", false);
                      d3.select(this).classed("graphs-selected-bin", true);
                      dispatch.changeDepth.call(this, i);
                  });
              depthBins.exit().remove();
              depthBins.transition().attr("x", margin).attr("y", function(d) { return y(d); }).attr("width", width - 2 * margin).attr("height", (height) / range.length);
            }            
            
            var chart = graph.selectAll(".graphs-chart").data(d.values, function(d) { return d.key; });
            chart.enter().append("g").attr("class", function(d) { return "graphs-chart " + d.key; })
                .style("color", function(f) { return f.color; })
                .style("stroke", function(f) { return f.color; })
                .style("fill", function(f) { return f.color; });
            chart.exit().transition().style("opacity", 0).remove();
            
            /* X Axis */
            var xAxis = chart.selectAll(".graphs-x-axis").data(function(d) { return [d]; }, function(d) { return d.key; });
            xAxis.enter().append("g").attr("class", "graphs-x-axis").attr("transform", function(d) { return "translate(0, " + (height + 5) + ")"; });
            xAxis.transition().call(function(selection) { selection.each(function(d) { d3.select(this).call(d3.svg.axis().scale(d.xScale).orient("bottom")); }); })
				.attr("transform", function(d, i, j) { return "translate(0, " + (height + margin * (j)) + ")"; });
            
            // Parameter + Unit
            if(app.config.graphs.depthProfile.showUnits.graphXAxis)
            {
                xAxis.append("text")
                    .attr("x", 40)
                    .attr("y", 27)
                    .text(function(d) {
                        // show the parameter's unit if it exists, otherwise show just the parameter
                        return (d.unit) ? (d.key).toUpperCase() + " ( " + d.unit + " )" : (d.key).toUpperCase()
                    });
            }
            
            var line = chart.selectAll(".graphs-line").data(function(d) { return [d]; }, function(d) { return d.key; });
            line.enter().append("path").attr("class", "graphs-line");
            line.transition().attr("d", function(d) { return d3line(d.xScale, y)(d.values); });
			
			var area = chart.selectAll(".graphs-area").data(function(d) { return [d]; }, function(d) { return d.key; });
			area.enter().append("path").attr("class", "graphs-area");
			area.transition().attr("d", function(d) { return d3area(d.xScale, y)(d.values); });
		});
    }
    
    function d3line(x, y) {
        return d3.svg.line()
			.x(function(d) { return x(d.x); })
			.y(function(d) { return y(d.y); })
			.interpolate(interpolate);
    }

    function d3area(x, y) {
        return d3.svg.area()
            .x0(function(d) { return x(d.x0); })
            .x1(function(d) { return x(d.x1); })
            .y(function(d) { return y(d.y); })
            .interpolate(interpolate);
    }

    function yScale(values) {
    	var yValues = values.reduce(function(a, b) { // Flatten the outer most array
    		return a.concat(b.values.reduce(function(a, b) { // Flatten the value objects
                return a.concat([b.y]); 
    		}, [])); 
    	}, []); // We should now have an array of all the y values in the heirarchy
        
        return d3.extent(yValues);
        // return d3.scale.linear().range([margin, height - values.length * margin])
        //     .domain(d3.extent(yValues));
    }

    function xScale(values) {
        var xValues = values.reduce(function(a, b) { return a.concat([b.x]); }, []);
        return d3.scale.linear().range([margin, width - margin])
            .domain([d3.min(xValues), d3.max(xValues)]);
    }
    
    // Precomputation on data, should happen before (or during) call to mapper
    function depthBin(values) {
        var depthSums = {}, minBoundSum = {}, maxBoundSum = {};
        var depthCounts = {};
        values.forEach(function(d) {
            var val = Math.floor(d.y);
            if(!depthCounts[val]) {
                depthCounts[val] = depthSums[val] = minBoundSum[val] = maxBoundSum[val] = 0;
            }
            
            depthCounts[val]++;
            depthSums[val] += d.x;
            minBoundSum[val] += d.x0;
            maxBoundSum[val] += d.x1;
        });
        
        return Object.keys(depthCounts).map(function(d) {
			return {
				x : depthSums[d] / depthCounts[d],
				y : +d,
				x0 : minBoundSum[d] / depthCounts[d],
				x1 : maxBoundSum[d] / depthCounts[d]
			};
        });
    }
        
    function mapper(data) {
        data.values.map(function(d) {
            d.values = depthBin(d.values);
            d.xScale = xScale(d.values);
            return d;
        });
        
        return data;
    }

    depth.remove = function(selection) {
        selection.each(function(d) {
            d3.select(this).selectAll("svg").transition()
                .attr("height", 0).attr("width", 0).remove();
        });
    };

    depth.width = function(value) {
        if (!arguments.length) return width;
        width = value;
        return depth;
    };

    depth.height = function(value) {
        if (!arguments.length) return height;
        height = value;
        return depth;
    };

    depth.margin = function(value) {
        if (!arguments.length) return margin;
        margin = value;
        return depth;
    };

    depth.min = function(value) {
        if (!arguments.length) return min;
        min = value;
        return depth;
    };

    depth.max = function(value) {
        if (!arguments.length) return max;
        max = value;
        return depth;
    };

    depth.interpolate = function(value) {
        if (!arguments.length) return interpolate;
        interpolate = value;
        return depth;
    };
    
    depth.dispatch = function(value) {
		if(!arguments.length) return dispatch;
		dispatch = value;
		return depth;
    };

    return depth;
};
// this doesn't assume multiple lines in a chart
app.graphs.sparkline = function() {
	var height = 30,
		width = 800,
		max = 100,
		min = 0,
		margin = 2,
		dateFormat = d3.time.format("%x"),
		xScale = null,
		yScale = null,
		interpolate = "linear";

	function sparkline(selection) {
		selection.each(function(d) {
			d = d.values;

			var x = xScale ? xScale : d3.time.scale().domain(d.map(function(d) {
					return d.x;
				})),
				y = yScale ? yScale : d3.scale.linear().domain(d3.extent(d, function(d) {
					return d.y;
				}));

			x.range([margin, width - margin]);
			y.range([height - margin, margin]);

            // only one line per sparkline graph
			var line = d3.svg.line().x(function(d) {
				return x(d.x);
			}).y(function(d) {
				return y(d.y);
			}).interpolate(interpolate);

			var container = d3.select(this);

			var svg = container.selectAll("svg").data([0]);
			svg.enter().append("svg").attr("width", width).attr("height", height);

			var g = svg.selectAll("g").data([0]);
			g.enter().append("g");

			var path = g.selectAll(".graphs-line").data([d]);
			path.enter().append("path").attr("class", "graphs-line").attr("fill", "none").attr("stroke-width", "1").attr("stroke", "#08C");
			path.transition().attr("d", function(d) {
				return line(d);
			});
			
			// nick had wanted to change to add circles at each datapoint
			// he's added to the timeseries to do the reverse lookup in geodashboard
			// in timeseries, ther eis a separate function that could be extended to this if we want it later
			var dataPoints = g.selectAll(".graphs-dataPoint").data(d);
			dataPoints.enter().append("circle").attr("class", "graphs-dataPoint")
				.attr("r", 5).style("opacity", 0);
			dataPoints.attr("cx", function(d) { return x(d.x); }).attr("cy", function(d) { return y(d.y); });
			
			var clearText = g.selectAll(".graphs-clearText").data([0]);
			clearText.enter().append("text").attr("class", "graphs-clearText").attr("x", margin).attr("y", margin + 12);
			
			dataPoints
				.on("mouseover", function(d) {
					d3.select(this).style("opacity", 1);
					g.selectAll(".graphs-clearText").text(dateFormat(d.x) + ", " + d.y);
				})
				.on("mouseout", function(d) {
					d3.select(this).style("opacity", 0);
					g.selectAll(".graphs-clearText").text("");
				});

			// sparkline.exts.forEach(function(d) {
			// 	d.call(sparkline, svg);
			// });
		});
	}

	sparkline.height = function(value) {
		if (!arguments.length) return height;
		height = value;
		return sparkline;
	};

	sparkline.width = function(value) {
		if (!arguments.length) return width;
		width = value;
		return sparkline;
	};

	sparkline.interpolate = function(value) {
		if (!arguments.length) return interpolate;
		interpolate = value;
		return sparkline;
	};

	sparkline.margin = function(value) {
		if (!arguments.length) return margin;
		margin = value;
		return sparkline;
	};

	sparkline.max = function(value) {
		if (!arguments.length) return max;
		max = value;
		return sparkline;
	};

	sparkline.min = function(value) {
		if (!arguments.length) return min;
		min = value;
		return sparkline;
	};

	sparkline.x = function(value) {
		if (!arguments.length) return xScale;
		xScale = value;
		return sparkline;
	};

	sparkline.y = function(value) {
		if (!arguments.length) return yScale;
		yScale = value;
		return sparkline;
	};

	sparkline.ext = function(ext) {
		if (!sparkline.exts) sparkline.exts = [];

		sparkline.exts.push(ext);
	};

	if(sparkline.extensions) {
		sparkline.extensions.forEach(function(d) {
			d.call(sparkline);
		});
	}

	return sparkline;
};

app.graphs.sparkline.extend = function(extension) {
	if (!graphs.sparkline.extensions) graphs.sparkline.extensions = [];

	graphs.sparkline.extensions.push(extension);
};
// similar to the depth graph
// this can do multiple lines on a single graph

// testing  

app.graphs.timeSeries = function() {
	var width = 100, height = 100, // default values overwritten by functions below
		margin = { top: 0, right: 0, bottom: 0, left: 0 }, // default values
		padding = { top: 30, right: 30, bottom: 30, left: 60 }, // default values
		x = null, y = null, interpolate = "linear",
		min = 0, max = 10, xFormat = function(x) { return x; },
		yFormat = function(y) { return y; },
		sharedXAxis = true, sharedYAxis = true;
	
	function timeSeries(selection) {
		selection.each(function(data) {
		    // sanity checks to make sure we are getting real Dates
			data.values = data.values.map(function(e) {
				e.values = e.values.map(function(f) { f.x = new Date(f.x); f.y = +f.y; return f;})
					.sort(function(a, b) { return a.x - b.x; });
				return e;	
			});

			x = d3.time.scale().range([padding.left, width - padding.right]);
			y = d3.scale.linear().range([height - padding.bottom, padding.top]);
			
			var viz = d3.select(this);
			
			var svg = viz.selectAll("svg").data([data], function(d) { return d.key; });
			svg.enter().append("svg")
			  .attr("xmlns", "http://www.w3.org/2000/svg").attr("version", "1.1")
			  .attr("preserveAspectRatio", "none")
			  .attr("class", function(e) { return "graphs-time-series " + e.key; })
			  .attr("viewBox", "0 0 " + width + " " + height);
			  

			/* Plot */
			// creates a different plot for each line you might have in a datapoint
			// allows you to hvae different lines on the same chart
			// different from depth in that it allows point highlighting for each graphed element
			var arr = data.key.split(" ");
			
			var plotG = svg.selectAll(".graphs-plotG").data(function(e) { return e.values; });
			plotG.enter().append("g");
			  
			plotG.transition()
				.attr("class", function(f) { return "graphs-plotG " + f.key; })
                .style("color", function(f) { return f.color; })
                .style("stroke", function(f) { return f.color; })
                .style("fill", function(f) { return f.color; })
				.call(plots, arr[arr.length - 1]);

			plotG.exit().remove();
		});
	}
	
	function plots(selection, units) {
		selection.each(function(data, i) {
			var bisectDate = d3.bisector(function(d) { return d.x; }).left;
			// only show two digits for the point on the graph
			var valueFormat = d3.format(".2f");
			var dateFormat = d3.time.format("%Y-%m-%d");
			
			var plot = d3.select(this);

			// main title 
			var title = plot.selectAll(".graphs-title").data([data]);
			
			title.enter()
				.append("text")
				.attr("class", "graphs-title");

			title.transition()
				.attr("x", 70)
				.attr("y", 15)
				.style("text-anchor", "left")
				.text(function(d) { return _.isUndefined(d.title) ? d.key : d.title; });

			title.exit().remove();
			
			// X-Axis Label
			if(app.config.graphs.timeSeries.showUnits.graphXAxis)
			{
				var xAxisLabel = plot.selectAll(".graphs-xAxisLabel").data([data]);
				
				xAxisLabel.enter()
					.append("text")
					.attr("class", "graphs-xAxisLabel");
				
				xAxisLabel.transition()
					.style("text-anchor", "middle")
					.attr("x", 275)
					.attr("y", 247)
					.text("Date");
				
				xAxisLabel.exit().remove();
			}
			
			// source Label
			if(app.config.graphs.timeSeries.showSource && data.sources && data.sources.length > 0)
			{
				var sourceLabel = plot.selectAll(".graphs-sourceLabel").data([data]);

				sourceLabel.enter()
					.append("text")
					.text("Sources: ")
					.attr("class", "graphs-sourceLabel");
				sourceLabel.transition()
					.style("text-anchor", "end")
					.attr("x", 510)
					.attr("y", 25);
				sourceLabel.exit().remove();

				// TODO make sure with many sources it doesn't fall off the graph. Anchor to right.
				var sources;
				if (data.sources.length < 5) {
					sources = data.sources;
				} else {
					sources = data.sources.splice(0, 5);
					sources.push("...");
				}
				var sourceLabelIndex = sourceLabel.selectAll(".graphs-sourceLabelIndex").data(sources, function(d) { return(d); });
				sourceLabelIndex.enter()
					.append("a")
					.attr("xlink:href", function(d) { return d; })
					.attr("target", "clowder")
					.attr("class", "graphs-sourceLabelIndex")
					.text(function(d, i) { if (d == "...") { return d; } else { return "[" + (i + 1) + "]"; } })
					.append("title")
					.text("Original data source");
				sourceLabelIndex.exit().remove();
			}

			// Y-Axis Label
			if(app.config.graphs.timeSeries.showUnits.graphYAxis)
			{
				var yAxisLabel = plot.selectAll(".graphs-yAxisLabel").data([data]);
				
				yAxisLabel.enter()
					.append("text")
					.attr("class", "graphs-yAxisLabel");
				
				yAxisLabel.transition()
					.attr("transform", "rotate(-90)")
					.style("text-anchor", "middle")
					.attr("x", -110) // Moving up/down. + (up), - (down)
					.attr("y", 25) // Moving left/right. + (right), - (left)
					.text(function(d) { return d.unit; }); // Catch the undefined text
				
				yAxisLabel.exit().remove();
			}
			
			var x = d3.time.scale()
				.domain(data.xextent)
				.range([padding.left, width - padding.right]);
					
			var y = d3.scale.linear()
				.domain(data.yrange)
				.range([height - padding.bottom, padding.top]);
								
		    // background color
		    // TODO make sure to use enter/exit/remove to clean up when redrawing
		    var background = plot.selectAll(".graphs-background-rect").data([data.values]);
			background.enter()
			  .append("g")
			  .attr("class", "graphs-background")
			  .append("rect")
			  .attr("class", "graphs-background-rect")
			  .attr("transform", "translate(" + padding.left + ", " + padding.top + ")")
			  .attr("width", width - padding.left - padding.right)
			  .attr("height", height - padding.top - padding.bottom);

			// y axis before x axis so that y tick lines don't cover x axis
			// TODO there must be a better way
			var yAxisG = plot.selectAll(".graphs-y-axis").data([data.values]);
			
			yAxisG.enter()
				.append("g")
				.attr("class", "graphs-y-axis")
				.attr("transform", "translate(" + padding.left + ", 0)");
				
			yAxisG
				.transition()
				.call(d3.svg.axis()
					.scale(y)
					.orient("left")
					.ticks(5)
                    .tickSize(-width + padding.right + padding.left, 0, 5)
					.tickFormat(valueFormat));

			yAxisG.selectAll("text")
	          .attr("transform", function(d) {
	             return "translate(" + -5 + "," + 0 + ")";
	         });

			var xAxisG = plot.selectAll(".graphs-x-axis").data([data.values]);
			
			xAxisG.enter()
				.append("g")
				.attr("class", "graphs-x-axis");
			
			xAxisG
				.transition()
				.attr("transform", function(d) { return "translate(0, " + (height - padding.bottom ) + ")"; })
				.call(d3.svg.axis()
					.scale(x)
					.orient("bottom")
					.ticks(10)
					);

			// rotate labels at a 45 degree angle so that they don't overlap
			xAxisG.selectAll("text")
	          .attr("transform", function(d) {
	             return "translate(" + (this.getBBox().height*-1.5) + "," + this.getBBox().height + ")rotate(-45)";
	         });
				
			if (_.isUndefined(app.config.graphs.timeSeries['showLine']) || app.config.graphs.timeSeries['showLine'] == true) {
				var linePath = plot.selectAll(".graphs-line").data([data.values]);

				linePath.enter()
					.append("path")
					.attr("class", "graphs-line");

				linePath
					.transition()
					.attr("d",
					function(g) {
						return (d3.svg.line()
							.x(function(d) { return x(d.x); })
							.y(function(d) { return y(d.y); })
							.interpolate(interpolate))(g);
					}
				);

			}

			// Vertical Line on mouseOver
			//if(app.config.graphs.timeSeries.mouseOver.verticalLine)
			var mouseOverVerticalLine = plot.selectAll("graphs-mouseOverVerticalLine").data([data.values]);
			
			mouseOverVerticalLine.enter()
				.append("line")
				.attr("class", "graphs-mouseOverVerticalLine")
				.attr({
					"x1": 0,
					"y1": padding.top,
					"x2": 0,
					"y2": height - padding.bottom
				});
			
			mouseOverVerticalLine.exit().remove();
			
			// div shown with popup when mouse hovers over dot
			var popupDiv = d3.select('.graphs-tooltip').data([0]);
			popupDiv.enter()
				.append("div")
				.attr("class", "graphs-tooltip")
				.style("opacity", "0");
			popupDiv.exit().remove();

			// add little dots to the graph
			var dots = plot.selectAll('.graphs-dots').data(data.values);
			dots.enter()
				.append("path")
				.attr("class", "graphs-dots")
			dots
				.transition()
				.attr("transform", function(d) { return "translate(" + x(d.x) + "," + y(d.y) + ")"; })
				.attr("d", d3.svg.symbol().size(16));
			dots.exit().remove(); 

			var focusDot = plot.selectAll(".graphs-focus").data([0]);
			focusDot.enter()
				.append("circle")
				.attr("class", "graphs-focus");
			focusDot.append("circle");
			focusDot.exit().remove();

			// Mouse hover handling
			var rect = plot.selectAll(".graphs-listener-rect").data([data.values]);
			
			rect.enter()
				.append("rect")
				.attr("class", "graphs-listener-rect")
				.attr("transform", "translate(" + padding.left + ", " + padding.top + ")")
				.attr("width", width - padding.left - padding.right)
				.attr("height", height - padding.top - padding.bottom)
				.style("opacity", 0);
			rect.on("mouseout", function() {
				focusDot.transition().attr("r", "0");
				popupDiv.transition().style("opacity", 0);
				mouseOverVerticalLine.transition().style("opacity", 0);
			});
			rect.on("mouseover", function() {
				focusDot.transition().attr("r", "4.5");
				popupDiv.transition().style("opacity", 0.9);
				mouseOverVerticalLine.transition().style("opacity", 0.9);
			});
			rect.on("mousemove", mousemove);
				
			// handles the hovering over the graph and switching which datapoint is drawn on the map
			// give me the mouse's position, invert it
			// padding.left handles issue where rectangle is offset from the left padding of the svg
			// mouse position + graph padding + adjustments to accommodate
			// bisectDate function defined above: for data values, give me the index of nearest datapoint
			function mousemove() {
				var x0 = x.invert(d3.mouse(this)[0] + padding.left);
                var i = bisectDate(data.values, x0, 1);
                var d;
                if (i >= data.values.length) {
                	d = data.values[i - 1];
                } else {
	                var d0 = data.values[i - 1];
	                var d1 = data.values[i];
	                d = x0 - d0.x > d1.x - x0 ? d1 : d0;
                }

				var label = (d.label === undefined) ? dateFormat(d.x) : d.label;
				var value = valueFormat(d.y);
				
				var unit = "";
				if(app.config.graphs.timeSeries.showUnits.graphTooltip && data.unit)
				{
					unit = " " + data.unit;
				}
        
				mouseOverVerticalLine.attr("transform", function () { return "translate(" + x(d.x) + ",0)"; });

			    popupDiv.html("<b>Date</b>  : " + label + "<br/>" +
												"<b>Value</b> : "  + value + unit + " Avg.<br />")
			        .style("left", (this.getBoundingClientRect().left - padding.left + x(d.x)) + "px")     
			        .style("top", (this.getBoundingClientRect().top - padding.top + y(d.y)) + "px");
			    focusDot.attr("cx", x(d.x)).attr("cy", y(d.y));
			}
		});
	}

	timeSeries.width = function(value) {
		if(!arguments.length) return width;
		width = value;
		return timeSeries;
	};
	
	timeSeries.height = function(value) {
		if(!arguments.length) return height;
		height = value;
		return timeSeries;
	};
	
	timeSeries.margin = function(value) {
		if(!arguments.length) return margin;
		applyPadding(value.split(" "), margin);
		
		return timeSeries;
	};
	
	timeSeries.padding = function(value) {
		if(!arguments.length) return padding;
		applyPadding(value.split(" "), padding);
		
		return timeSeries;
	};
	
	timeSeries.min = function(value) {
		if(!arguments.length) return min;
		min = value;
		return timeSeries;
	};
	
	timeSeries.max = function(value) {
		if(!arguments.length) return max;
		max = value;
		return timeSeries;
	};
	
	timeSeries.x = function(value) {
		if(!arguments.length) return x;
		x = value;
		return timeSeries;
	};
	
	timeSeries.xFormat = function(value) {
		if(!arguments.length) return xFormat;
		xFormat = value;
		return timeSeries;
	};
	
	timeSeries.y = function(value) {
		if(!arguments.length) return y;
		y = value;
		return timeSeries;
	};
	
	timeSeries.yFormat = function(value) {
		if(!arguments.length) return yFormat;
		yFormat = value;
		return timeSeries;
	};
	
	timeSeries.sharedXAxis = function(value) {
		if(!arguments.length) return sharedXAxis;
		sharedXAxis = value;
		return timeSeries;
	};
	
	timeSeries.sharedYAxis = function(value) {
		if(!arguments.length) return sharedYAxis;
		sharedYAxis = value;
		return timeSeries;
	};
	
	timeSeries.interpolate = function(value) {
		if(!arguments.length) return interpolate;
		interpolate = value;
		return timeSeries;
	};
	
	function applyPadding(values, obj) {
		if(values[0]) {
			obj.top = obj.right = obj.bottom = obj.left = +values[0];
		}
		
		if(values[1]) {
			obj.right = obj.left = +values[1];
		}
		
		if(values[2]) {
			obj.bottom = +values[2];
		}
		
		if(values[3]) {
			obj.left = +values[3];
		}
	}

	return timeSeries;
};

// attach the event object to the app to ensure the object exists during the build process
app.event = {};
app.event.EventBus = _({}).extend(Backbone.Events);
app.router = {};
app.router.AppRouter = Backbone.Router.extend({
    
    routes : {
        "" : app.config.landingPage,
        "explore" : "home",
        "search" : "search",
        "compare" : "compare",
        "welcome": "welcome",
        "detail/location/:location(/startTime/:startTime/endTime/:endTime)(/parameters/:parameters)(/bin/:bin)(/depth/:depth)(/depthYear/:depthYear)(/depthBin/:depthBin)(/biologyStartTime/:biologyStartTime)(/biologyEndTime/:biologyEndTime)(/biologyBin/:biologyBin)(/)" : "populateDetailView",
        "about" : "about"
    },

    initialize: function() {
        var self = this;
        // on route changes update google analytics
        this.bind('route', this._pageView);

        // dynamically create a route listener for any "child views" specified in app.config
        _.each(app.config.views, function(view) {
          if (view.children) {
            _.each(view.children, function(childView) {
              if (childView.title && childView.link && childView.template) {
                var routeString = new RegExp('^' + view.link.replace('#', '') + '\/' + childView.link.replace('#', ''));
                self.route(routeString, childView.title.toLowerCase());
              }
            });
          }
        });
    },
    
    // split backbone and send information
    _pageView: function() {
        if (app.config.googleAnalytics != "") {
            var path = Backbone.history.getFragment();
            if (path.match(/^detail/)) {
                path = path.replace(/\/([^/]*)\/([^/]*)/g, '&$1=$2').replace(/&/, "?").replace(/\/$/, "");
            }
            ga('send', 'pageview', {page: "/" + path});
        }
    }
});

// attach the model object to the app to ensure the object exists during the build process
app.model = {};
app.model.Sensor = Backbone.Model.extend({
    keepId: true,
    defaults: {
        selectable: true,
        selected: false,
        checked: false,
        sourceSelected: false,
        sourceChecked: false,
        parameterSelected: false,
        parameterChecked: false,
        areaChecked: false,
        areaSelected: false
    },

    initialize: function() {
        this.on("change:sourceSelected", this.isSelected, this);
        this.on("change:sourceChecked", this.isChecked, this);
        this.on("change:areaChecked", this.isChecked, this);
        this.on("change:areaSelected", this.isSelected, this);
        this.on("change:parameterSelected", this.isSelected, this);
        this.on("change:parameterChecked", this.isChecked, this);
        this.on("change:checked", this.isSelected, this);
        this.on("change:selected", this.isSelected, this);
        this.on("change:parameters[*]", this.isParamSelected, this);
        this.on("change:parameters[*].checked", this.isParamChecked);
    },

    isParamSelected: function() {
        if (this.isParamSelectedProcessing) return;
        this.isParamSelectedProcessing = true;
        var params = this.get("parameters");
        var paramSelected = _.findWhere(params, {selected: true});
        if (paramSelected) {
            this.set("parameterSelected", true);
        }
        this.isParamSelectedProcessing = false;
    },

    isParamChecked: function(changed) {
        if (this.isParamCheckedProcessing) return;
        this.isParamCheckedProcessing = true;
        var params = this.get("parameters");
        var paramChecked = _.findWhere(params, {checked: true});
        if (paramChecked) {
            this.set("parameterChecked", true);
        }
        this.isParamCheckedProcessing = false;
    },

    isSelected: function() {
        if (this.isSelectedProcessing) return;
        this.isSelectedProcessing = true;
        if (this.get("sourceSelected") || this.get("parameterSelected") || this.get("areaSelected") || this.get("checked")) {
            this.set("selected", true);
        } else {
            this.set("selected", false);
        }
        this.isSelectedProcessing = false;
    },

    isChecked: function(changed) {
        if (this.isCheckedProcessing) return;
        this.isCheckedProcessing = true;
        // if a parameter gets checked, and a source is checked, but this sensor's source is not checked, uncheck it
        if ( (this.get("parameterChecked")) && ( (app.collection.sources.getChecked().length == 1) && (!this.get("sourceChecked")) )  ) {
            this.isCheckedProcessing = false;
            return this.set("checked", false);
        }
        if ( (app.collection.sources.getChecked().length == 1) && (this.get("sourceChecked")) ) {
            this.set("checked", true);
        } else if (this.get("sourceChecked") || this.get("parameterChecked") || this.get("areaChecked")) {
            this.set("checked", true);
        } else {
            this.set("checked", false);
        }
        this.isCheckedProcessing = false;
    },

    url: function() {
        if (_.isUndefined(this.id)) {
            return app.config.clowderUrl + '/api/geostreams/sensors';
        } else {
            return app.config.clowderUrl + '/api/geostreams/sensors/' + encodeURIComponent(this.id);
        }
    },

    // set is called after app.collection.sensors calls fetch, then calls parse.
    // set is given JSON objects for each of the sensors, and creates a Backbone model for each sensor.
    // We are checking the JSON object's "type" property (which is the Data Source, like "epa") and
    // ensuring that the Data Source already exists in app.collection.sources.
    set: function ( key, val, options ) {
        var self = this;
        var args, attrs, _attrs, geometry;
        args = arguments;

        // Handle both `"key", value` and `{key: value}` -style arguments.
        if ( typeof key === 'object' ) {
            attrs = key;
            options = val;
        }
        // Handle GeoJSON argument.
        if ( attrs && attrs['type'] && attrs['type'] === 'Feature' ) {
            _attrs = _.clone( attrs['properties'] ) || {};

            if (attrs.created) _attrs['created'] = attrs.created;
            if (attrs.name) _attrs['name'] = attrs.name;
            if (attrs.min_start_time) _attrs['min_start_time'] = attrs.min_start_time;
            if (attrs.max_end_time) _attrs['max_end_time'] = attrs.max_end_time;
            _attrs['selectable'] = true;
            _attrs['selected'] = false;
            _attrs['checked'] = false;
            _attrs['sourceSelected'] = false;
            _attrs['sourceChecked'] = false;
            _attrs['parameterSelected'] = false;
            _attrs['parameterChecked'] = false;
            _attrs['areaChecked'] = false;
            _attrs['areaSelected'] = false;

            if (_attrs['type'] && _attrs['type']['id']) {
                _attrs['type']['id'] = _attrs['type']['id'].toLowerCase().replace(/ /g, '-');
            }

            if (attrs.parameters) {

                // we create a new instance of app.collection.Parameters, inheriting the ability to be custom sorted
                // by defining a comparator in app.config.parameters.comparator in the project's 0config.js file
                _attrs['parameters'] = new app.collection.Parameters();
                _.each(attrs.parameters, function(parameterID) {
                    if (parameterID == null) return;
                    // if this parameterID matches a known parameter, we'll link them, otherwise we'll create a new parameter and link this sensor to it
                    // "alternates" is an array of possible spellings of that parameter

                    // find a parameter model for this parameterID, or see if it is listed as an alternate in any other parameter
                    var parameterModel = app.collection.parameters.find(function(model) {
                        return model.id == parameterID || _.contains(model.get("alternates"), parameterID);
                    });

                    // Only enable the next if/else when set in config
                    if (app.config.useExploreParams == true) {
                        if (parameterModel !== undefined) {
                            // here we're adding this sensor to an existing parameter model
                            // by telling it "merge: true" we'll add this sensor to any other sensors it already has
                            app.collection.parameters.add({id: parameterModel.id, sensors: self}, {merge: true});
                        } else {
                            console.log(_attrs['name'] + " has a new parameter that is not yet listed in config.js : " + parameterID);
                            /*RemoveLogging:skip*/
                            // here we're creating a new parameter model since the parameterID didn't match the ID of an existing parameter model
                            // and wasn't in any other parameters list of "alternates"
                            // we'll sanitize the ID, and if the sanitized version is different, we'll store that as it's alternate value so it can be found
                            parameterModel = app.collection.parameters.add({
                                id: parameterID.replace(/[^a-zA-Z0-9-_]/g, ''),
                                title: parameterID,
                                alternates: (parameterID == parameterID.replace(/[^a-zA-Z0-9-_]/g, '')) ? [] : [parameterID],
                                sensors: self
                            }, {merge: true});
                        }
                    }
                    // by pushing this parameterModel back onto "parameters", our sensor will have a reference to each parameter's model
                    _attrs.parameters.add(parameterModel);

                });
            }
            // Clone the geometry attribute.
            geometry = _.clone( attrs['geometry'] ) || null;
            if ( geometry ) {
                geometry.coordinates = geometry['coordinates'].slice();
            }
            _attrs['geometry'] = geometry;

            if ( attrs[this.idAttribute] ) {
                _attrs[this.idAttribute] = attrs[this.idAttribute];
            }
            args = [_attrs, options];
        }

        // this "finalizes" the Backbone model for this sensor
        var result = Backbone.Model.prototype.set.apply( this, args );

        // This next if statement associates a sensor's "source" (ie. "epa" for Environmental Protection Agency)
        // to a Backbone Model called a Source. We already use Source Models,
        // so we'll add this sensor to the source model to make the connection

        if (_attrs && _attrs.type && _attrs.type.id) {
            // ensure that a "source" model exists
            // since _attrs.type includes id and name, the add method can be used. "add" won't duplicate for the same id
            // 'source' is a single Data Source
            var source = app.collection.sources.add(_attrs.type);

            // assign the source's color to this sensor
            if (source.get('color')) {
                this.set('color', source.get('color'));
            }

            // Get all the sensorIDs for this source
            if (source.get('sensors')) {
                var sensorIDs = source.get('sensors');
                if (source.id == _attrs.type.id) {
                    sensorIDs.push(this.id);
                    source.set({'sensors': sensorIDs});
                }
            }
        }

        // This next if statement associates a sensor's "region" (ie. "Illinois")
        // to a Backbone Model called a Region. We already use Region Models,
        // so we'll add this sensor to the region model to make the connection

        if (_attrs && _attrs.region) {
            // ensure that a "region" model exists
            // add the region to the regions collection (or get the model if it already exists)
            var region = app.collection.regions.add({id: _attrs.region});

            // each region model maintains a Backbone Collection of sources
            if (! region.get('sources')) {
                region.set('sources', new Backbone.Collection);
            }

            region.get("sources").add(this.get('type'));
        }

        return result;

    },

    // The GeoJSON representation of a `Feature`.
    // http://www.geojson.org/geojson-spec.html#feature-objects
    // whenever the collection or model calls the toJSON() function, we manipulate the output
    // to suit our purposes. This allows us to completely alter what is stored in the "model" for a sensor
    // while still ensuring that a true GeoJSON object can be created from the model when needed
    toJSON: function ( options ) {
        var attrs, props, geometry;
        options = options || {};
        attrs = _.clone( this.attributes );
        props = _.omit( attrs, ['geometry', 'created', 'id']);
        // Add model cid to internal use.
        if ( options.cid ) {
            props.cid = this.cid;
        }
        // Clone the geometry attribute.
        geometry = _.clone( attrs['geometry'] ) || null;
        if ( geometry ) {
            geometry.coordinates = geometry['coordinates'].slice();
        }
        var json = {
            type: 'Feature',
            name: attrs.name,
            created: attrs.created,
            geometry: geometry,
            properties: props
        };
        if ( this.keepId ) {
            json[ this.idAttribute ] = this.id;
        }
        return json;
    }
});
// the GeoStreamService model interfaces with '/datapoints' on the API

// if we have already collected the datapoints from the API, the 'geostream' argument will not be empty, and subsequent page loads will not hit the API.
app.model.GeoStreamService = function(geostream, options) {
    this.geostream = geostream ? geostream : {};
    this.options = {};
    // set the API endpoint for this model
    this.options.url = app.config.clowderUrl + "/api/geostreams/datapoints?";
    this.options.urlTrend = app.config.clowderUrl + "/api/geostreams/datapoints/trends?";
    this.options.urlBins = app.config.clowderUrl + "/api/geostreams/datapoints/bin/";

    // lookup a datapoint by location
    this.getByLocation = function(location, callback) {
        // if we have this location in our cached results, return it
        if(this.geostream[location.id]) {
            callback(this.geostream[location.id]);
            return;
        } else {
          // this location is not in our cached results, so fetch from the API using id
          var query = this.options.url + $.param({sensor_id: location.id });
          $.ajax(query, {
              dataType: "jsonp",
              success: (function(geostream) {
                  return function(data) {
                      var cleanedData = cleanupData(data, location.id);

                      // keep a cached copy of the cleaned data
                      geostream[location.id] = cleanedData;
                      callback(cleanedData);
                  };
              })(this.geostream)
          });
        }
    };

    this.getTrend = function(parameter, season, callback){
        //We might need to add cached results here to help site responsiveness

        var binString;
        switch(app.config.binning.time) {
            case "decade":
                binString = "&decade=";
                break;
            case "lustrum":
                binString = "&lustrum=";
                break;
            case "year":
                binString = "&year=";
                break;
            case "semi":
            case "season":
                binString = "&semi=";
                break;
            case "month":
                binString = "&month=";
                break;
            default:
                binString = "";
        }

        season = season || '';

        var query = this.options.urlTrend + parameter  + binString + season;
        console.log(query);
        $.ajax(query, {
            dataType: "jsonp",
            success: function(geostream) {
                // return function(data) {
                var sensorResults = [];
                var parameterTitle = "";
                _.each(geostream, function(trend) {
                    var trendsObject = {};
                    var statsObject = {};
                    _.each(trend.properties, function(value, param) {                      
                      if (param == 'source') { return }
                      if (param.split(/_(.+)?/)[1]) statsObject[param.split(/_(.+)?/)[1]] = value;
                      var parameterModel = app.collection.parameters.find(function(model) {
                        return model.get("id") == param.split('_')[0];
                      });
                      if (parameterModel) {
                        parameterTitle = parameterModel.get('title').replace(/<(.*?)>/g, '');
                        if (parameterTitle) trendsObject[parameterTitle] = {'last_time': trend.last_time};
                      }
                    });
                    var sensor = app.collection.sensors.findWhere({id: +trend.sensor_id});
                    if (!_.isEmpty(statsObject)) {
                        $.extend( trendsObject[parameterTitle], statsObject );
                        var trends = sensor.get('trends');
                        if (trends) {
                            _.extend(trends, trendsObject);
                        } else {
                            sensor.set("trends", trendsObject);
                        }
                    }
                    sensorResults.push(sensor);

                });

                // keep a cached copy of the cleaned data
                //geostream[location.properties.name] = cleanedData;
                //callback(cleanedData);
                // };
                //app.event.eventBus.trigger("change:trends", sensorResults, parameterTitle);
                callback(sensorResults, parameterTitle);

            }

        });


    };
  // retrive binned datapoints
  this.getBinnedData = function(time, depth, sensor_id, cache, callback) {
    var query = this.options.urlBins + time + "/" + depth + "?" + $.param({sensor_id: sensor_id});
    console.log("Query for binned datapoints");
    // Check the cache for results for a sensor / bin
    if(this.geostream[sensor_id + time]) {
      callback(this.geostream[sensor_id + time]);
      return;
    } else {
      $.ajax(query, {
        dataType: "jsonp",
        success: (function(geostream) {
          return function(data) {
            if(cache) {
              geostream[sensor_id + time] = data;
            }
            callback(data);
          };
        })(this.geostream)
      });
    }
  };
};

// Given the data returned by clowder geostreams api this will
// reorganize the data. This will create a single object that
// represents a single reading.
// This can also bin multiple readings time wise, for example for
// SeaGrant this will combine multiple readings per year/season.
// [ {
//      year: 1999,
//      season: 'summer',
//      parameter: 'Chlorophyll a ug/l',
//      values: [ {
//         raw: [ 19.5, 20.1, 18.0, 19.8 ]
//         value: 77.4,
//         count: 4,
//         average: 19.35
//         depth: 0,
//         source: [ 'http://localhost:9001/clowder/XYZ' ]
//      }],
//      raw:
// } ]
//
// value is the sum of data
//
// This function asssumes that start/end date is the same and will
// only use the startTime returned!!!
function cleanupData(data, sensorID) {
    var cleanedData = [];

    // this.binningTime will be something like 'smart', 'semi', 'yearly', etc. It is how the datapoints
    // are averaged before being drawn in the graphs.
    // Projects can setup a filterFunction that accepts the current sensor (the Backbone Sensor Model)
    // as an argument. From there, you can return 'smart', 'semi', or some other binning value
    // based on a condition. For example: return (sensor.get('type')['id'] == 'epa') ? 'semi' : 'smart';
    // would return 'semi' as the binningTime for epa sensors, all others would use 'smart' binning
    var binningTime = "";
    if (app.config.binning.filterFunction) {
      binningTime = app.config.binning.filterFunction(app.collection.sensors.findWhere({name: sensorID}));
    } else if (app.config.binning.time) {
      binningTime = app.config.binning.time;
    } else {
      binningTime = 'smart';
    }

    data.forEach(function(d) {
        // convert returned date to javascript
        var start_time = new Date(d.start_time);

        // check if there is a source in the parameter list
        var source = d.properties.source;

        // check if there is a depth code in parameter list
        var depthcode = d.properties.DEPTH_CODE;
        if (_.isUndefined(depthcode)) depthcode="NA";
        
        // map all these to EP
        if (["SRF", "LEP", "TRM"].indexOf(depthcode) > -1) {
          depthcode = "EP";
        }

        // find the depth binning requirements
        var depth = app.config.binning.depth * Math.ceil(d.geometry.coordinates[2] / app.config.binning.depth);

        // extract the information for each parameter
        for (var k in d.properties) {
            // source is handled differently
            if (k == 'source') continue;

            // depth code is handled differently
            if (k == 'DEPTH_CODE') continue;

            // check the value and make sure it is a number
            var v = parseFloat(d.properties[k]);
            if (_.isNaN(v)) continue;

            // filter to be used to find entry
            var filter = {};

            // find the parameter
            var parr = app.collection.parameters.filter(function(x) {
                if (_.isUndefined(x.get("alternates"))) {
                    
                }
                return (x.id == k) || x.get("alternates").indexOf(k) >= 0;
            });
            if (parr.length == 0) {
                // Format the id of this parameter: "Chlorophyll-a, ug/l" becomes "chlorophyll-a-ug-l"
                var p = k.replace(/[^a-zA-Z0-9-]{1,2}/g, '-').toLowerCase();
                app.collection.parameters.add({
                    id: p,
                    title: k,
                    alternates: [k],
                    detailView: app.config.parameters.defaults.detailView
                });
                filter.parameter = k;
                filter.detailView = app.config.parameters.defaults.detailView;
            } else {
                filter.parameter = parr[0].id;
                filter.detailView = parr[0].get('detailView');
            }

            // find the time binning requirements
            filter.year = start_time.getUTCFullYear();
            filter.label = filter.year.toString();
            var date = new Date(filter.year, 7, 1, 0, 0, 0);
            if (binningTime == 'semi') {
                if (start_time.getUTCMonth() < 6) {
                    filter.bin = "spring";
                    date.setUTCMonth(4);
                } else {
                    filter.bin = "summer";
                    date.setUTCMonth(7);
                }
                filter.label = filter.label + " " + filter.bin;
            } else if (binningTime == 'season') {
                filter.bin = getSeason(start_time);
                filter.label = filter.label + " " + filter.bin;
                if (filter.bin == "winter") {
                    date.setUTCMonth(1);
                } else if (filter.bin == "spring") {
                    date.setUTCMonth(4);
                } else if (filter.bin == "summer") {
                    date.setUTCMonth(7);
                } else if (filter.bin == "fall") {
                    date.setUTCMonth(10);
                }
            } else {
                if (_.contains(['month', 'day', 'hour', 'minute', 'none'], binningTime)) {
                    filter.month = start_time.getUTCMonth();
                    filter.label = filter.label + "-" + (filter.month + 1).toString();
                    date.setUTCMonth(start_time.getUTCMonth());
                }
                // if binningTime is "none" we won't bin beyond the day level
                if (_.contains(['day', 'hour', 'minute', 'none'], binningTime)) {
                    filter.day = start_time.getUTCDate();
                    filter.label = filter.label + "-" + filter.day.toString();
                    date.setUTCDate(filter.day);

                }
                if (_.contains(['hour', 'minute', 'none'], binningTime)) {
                    filter.hour = start_time.getUTCHours();
                    filter.label = filter.label + " " + filter.hour.toString();
                    date.setUTCHours(filter.hour);
                }
                if (_.contains(['minute', 'none'], binningTime)) {
                    filter.minute = start_time.getUTCMinutes();
                    filter.label = filter.label + ":" + filter.minute.toString();
                    date.setUTCMinutes(filter.minute);
                }

            }

            // find the entry in cleanedData, if not found add it.
            var entry;
            if (filter.parameter in cleanedData) {
                entry = _.findWhere(cleanedData[filter.parameter], filter);
                if (_.isUndefined(entry)) {
                    entry = filter;
                    entry.values = [];
                    entry.date = date;
                    cleanedData[filter.parameter].push(entry);
                }
            } else {
                entry = filter;
                entry.values = [];
                entry.date = date;
                cleanedData[filter.parameter] = [entry];
            }

            // find the appropriate depth value, if not found add it.
            var depthval = _.findWhere(entry.values, {depth: depth, depthcode: depthcode});
            if (_.isUndefined(depthval)) {
                depthval = {
                    average: 0,
                    depth: depth,
                    depthcode: depthcode,
                    count: 0,
                    value: 0,
                    raw: [ ],
                    source: [ ]
                };
                entry.values.push(depthval);
            }

            // update the depth value and compute average
            depthval.count++;
            depthval.value += v;
            depthval.raw.push(v);
            depthval.average = depthval.value / depthval.count;

            // add source to depth value if not there already
            if (!_.isUndefined(source) && depthval.source.indexOf(source) == -1) {
                depthval.source.push(source);
            }
        }
    });

    return _.flatten(_.values(cleanedData));
}

// given a date this will return the associated season.
function getSeason(date) {
    if ((date.getUTCMonth() < 2) || (date.getUTCMonth() == 2 && date.getUTCDate() < 21)) {
        return 'winter';
    }
    if ((date.getUTCMonth() < 5) || (date.getUTCMonth() == 5 && date.getUTCDate() < 21)) {
        return 'spring';
    }
    if ((date.getUTCMonth() < 8) || (date.getUTCMonth() == 8 && date.getUTCDate() < 21)) {
        return 'summer';
    }
    if ((date.getUTCMonth() < 11) || (date.getUTCMonth() == 11 && date.getUTCDate() < 21)) {
        return 'fall';
    }
    return 'winter';
}



app.model.Area = Backbone.Model.extend({
    defaults: {
        sensors: new Backbone.Collection,
        markers: {},
        checked: false,
        selected: false
    },

    set: function ( key, val, options ) {
        var args, attrs, _attrs, geometry;
        args = arguments;

        // Handle both `"key", value` and `{key: value}` -style arguments.
        if ( typeof key === 'object' ) {
            attrs = key;
            options = val;
        }
        // Handle GeoJSON argument.
        if ( attrs && attrs['type'] && attrs['type'] === 'Feature' ) {
            _attrs = _.clone( attrs['properties'] ) || {};

            if (attrs.created) _attrs['created'] = attrs.created;
            if (attrs.name) _attrs['name'] = attrs.name;
            if (attrs.checked) _attrs['checked'] = attrs.checked;

            _attrs['selected'] = false;
            _attrs['sensors'] = new Backbone.Collection;

            // Clone the geometry attribute.
            geometry = _.clone( attrs['geometry'] ) || null;
            if ( geometry ) {
                geometry.coordinates = geometry['coordinates'].slice();
            }
            _attrs['geometry'] = geometry;

            if ( attrs[this.idAttribute] ) {
                _attrs[this.idAttribute] = attrs[this.idAttribute];
            }
            args = [_attrs, options];
        }

        return Backbone.Model.prototype.set.apply( this, args );
    },

    // since we are not using the api, we must override any api-type functions
    // or else backbone will try to hit the api and complain because no url exists for this model
    sync : function() { return null; },
    fetch : function() { return null; }

});
app.model.HucItem = Backbone.Model.extend({
    defaults: {
        title: false,
        checked: false,
        selected: false
    },

    initialize: function() {
        this.set('sensors', new Backbone.Collection);

    },

    // since we are not using the api, we must override any api-type functions
    // or else backbone will try to hit the api and complain because no url exists for this model
    sync : function() { return null; },
    fetch : function() { return null; }


});
app.model.Huc = Backbone.Model.extend({
    defaults: {
        title: false, // should be in the format of "huc2", "huc4", etc
        checked: false,
        selected: false
    },

    initialize: function() {
        this.set('items', new app.collection.HucItems);

    },

    // app.collection.hucs.findWhere({id: "huc2"}).groupByHUCSourcesSensors()
    // this creates a response that groups all huc regions in a group, ie, all the HUC2 regions
    // then in each HUC2 region (like Ohio 05), groups by source, then sensors
    groupByHUCSourcesSensors: function() {
        var response = new Backbone.Collection;
        this.get('items').each(function(huc) {
          var responseHucModel = response.add({id: huc.get('id'), title: huc.get('title')});
          responseHucModel.set('sources', new Backbone.Collection);
          huc.get('sensors').each(function(sensor) {
            var responseSource = responseHucModel.get('sources').add(sensor.get('type'));
            if (!responseSource.get('sensors')) {
              responseSource.set('sensors', new Backbone.Collection);
            }
            responseSource.get('sensors').add(sensor);
          });
        });
        return response;
    },

    // since we are not using the api, we must override any api-type functions
    // or else backbone will try to hit the api and complain because no url exists for this model
    sync : function() { return null; },
    fetch : function() { return null; }


});
app.model.Region = Backbone.Model.extend({
  defaults: {
    checked: false,
    selected: false,
    title: null
  },

  // since we are not using the api, we must override any api-type functions
  // or else backbone will try to hit the api and complain because no url exists for this model
  sync : function() { return null; },
  fetch : function() { return null; }


});
app.model.Parameter = Backbone.Model.extend({
    defaults: {
        sensors: [],
        markers: {},
        checked: false,
        selected: false,
        enabled: true,
        sourceChecked: false,
        detailView: app.config.parameters.defaults.detailView,
        searchView: app.config.parameters.defaults.searchView,
        exploreView: app.config.parameters.defaults.exploreView
    },
    // since we are not using the api, we must override any api-type functions
    // or else backbone will try to hit the api and complain because no url exists for this model
    sync : function() { return null; },
    fetch : function() { return null; },

    initialize: function() {
        var self = this;
        if (this.get('categories')) {
            _.each(this.get('categories'), function(cat) {
                var categoryModel = app.collection.parameterCategories.add({id: cat});
                if (!categoryModel.get('parameters')) {
                    categoryModel.set('parameters', new Backbone.Collection);
                }
                categoryModel.get('parameters').add(self);
            });
        }

        this.on("change:sensors[*].selected", this.isSelected, this);
        this.on("change:sensors[*].sourceChecked", this.isSourceChecked, this);
    },

    isSelected: function() {
        var sensors = this.get("sensors");

        var isSensorSelected = _.any(sensors.models, function(sensor) {
            return sensor.get("selected");
        });
        this.set("selected", isSensorSelected);

    },

    isSourceChecked: function() {
        // The scenario: on the search page, click an area, then click a source
        // The problem this fixes: the parameters that show up in the ResultsView should update to only show the parameters for the checked source
        // How we solve this:
        // 1. get all the sensors for this parameter
        // 2. ask the sensor if it's source is checked - because we set this value on the model
        // 3. Using underscore's "some" function, we will return true if any of this parameter's sensors have a checked source
        // 4. If true, we will set "sourceChecked" true on this parameter. The ResultsView will then update the list of parameters
        var sensors = this.get('sensors');
        var sensorsSourceChecked = _.some(sensors.models, function(sensor) {
            return sensor.get("sourceChecked");
        });

        if (sensorsSourceChecked) {
            this.set("sourceChecked", true);
        } else {
            this.set("sourceChecked", false);
        }
    },

    getSelectedSensors: function() {
        var that = this;
        var theSelectedSensors = app.collection.sensors.filter(function(e){
            if (e.attributes.type.id == that.id && e.attributes.selected == true) {
                return e;
            }
        });
        return theSelectedSensors;
    },

    getSelectableSensors: function() {
        var that = this;
        var theSelectableSensors = app.collection.sensors.filter(function(e){
            if ( e.attributes.selectable == true) {
                return e;
            }
        });
        return theSelectableSensors;
    },

    getCheckedSensors: function() {
        var that = this;
        var theCheckedSensors = app.collection.sensors.filter(function(e){
            if (e.attributes.type.id == that.id && e.attributes.checked == true) {
                return e;
            }
        });
        return theCheckedSensors;
    }

});
app.model.Source = Backbone.Model.extend({
    defaults: {
        sensors: [],
        regions: new Backbone.Collection,
        areas: [],
        markers: {},
        checked: false,
        selected: false
    },

    initialize: function(options) {
        if (this.id) {
            this.set('id', this.id.toLowerCase().replace(' ', '-'));
        }

        // this is defined in app.config.sources.parameters.description
        // and is used in source_modals.scala.html (called by ExploreSourcesView)
        if (options.parameters && options.parameters.description) {
            this.set('aboutParametersDescription', options.parameters.description);
        }

        // this is defined in app.config.sources.parameters.ids, and is used in ResultsView
        if (options.parameters && options.parameters.ids) {
            this.set('ParametersIDs', options.parameters.ids);
        }

        this.on("change:sensors[*].selected", this.isSelected, this);

        // if prettyName isn't defined, this is a new Data Source and it isn't listed in sources.json
        // convert the source's title to Proper Case, and set it as the prettyName.
        if (this.get('prettyName') == undefined) {
            var prettyName = this.get('title');
            prettyName = prettyName.replace(/\w\S*/g, function(txt){return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase();});
            this.set("prettyName", prettyName);
        }
    },

    isSelected: function() {
        var sensors = this.get("sensors");

        var isSensorSelected = _.any(sensors.models, function(sensor) {
            return sensor.get("selected");
        });
        this.set("selected", isSensorSelected);

    },

    // since we are not using the api, we must override any api-type functions
    // or else backbone will try to hit the api and complain because no url exists for this model
    sync : function() { return null; },
    fetch : function() { return null; },

    // returns "selected" sensors for this source
    getSelectedSensors: function() {
        var that = this;
        var theSelectedSensors = app.collection.sensors.filter(function(e){
            if (e.attributes.type.id == that.id && e.attributes.selected == true) {
                return e;
            }
        });
        return theSelectedSensors;
    },

    // returns "selectable" sensors for this source
    getSelectableSensors: function() {
        var that = this;
        var theSelectableSensors = app.collection.sensors.filter(function(e){
            if (e.attributes.type.id == that.id && e.attributes.selectable == true) {
                return e;
            }
        });
        return theSelectableSensors;
    },

    getCheckedSensors: function() {
        var that = this;
        var theCheckedSensors = app.collection.sensors.filter(function(e){
            if (e.attributes.type.id == that.id && e.attributes.checked == true) {
                return e;
            }
        });
        return theCheckedSensors;
    },

    getColor: function() {
        if (!this.get('color')) {
            return "black";
        } else {
            return this.get('color');
        }
    }
});
app.model.Layer = Backbone.Model.extend({
    defaults: {
        markers: {},
        checked: false,
        initialized: false
    },

    initialize: function() {
      var self = this;
      _.each(this.get('properties'), function(value, key) {
          self.set(key, value);
      });
      this.set("layer", new L.TileLayer.WMS(this.get("wms"), {
          layers: this.get("id"),
          format: 'image/png',
          transparent: true,
          opacity: this.get("opacity")
      }));
      if (this.get("default") == "true") {
          this.set("checked", true);
      }
    }

});

app.collection.Sensors = Backbone.Collection.extend({
	url: app.config.clowderUrl + "/api/geostreams/sensors",
    model: app.model.Sensor,
    
    visible: function() {
        return this.where({ visible: true });
    },
    
    selected: function() {
        return this.where({ selected: true });
    },
    
    getSelectableSensors: function() {
        return this.where({ selectable: true });
    },

    getChecked: function() {
        return this.where({ checked: true });
    },

    getOrFetchStations: function(callback) {
        if (!this.models.length > 0) {
            this.fetch({
                success: function(collection) {
                    callback(collection);
                }
            });
        } else {
            // return "this" which refers to app.collection.sensors
            callback(this);
        }
    },
    
    getStationByName: function(name, callback) {
        this.getOrFetchStations((function(self) {
            return function(sensorsCollection) {
                // we could also use the following command, because we know that app.collection.sensors is populated now:
                // var station = app.collection.sensors.findWhere({name: name})

                var station = sensorsCollection.findWhere({name: name});
                if (station) {
                    callback(station.toJSON());
                } else {
                    callback(undefined);
                }
            }
        })(this));
    },

    uncheckAll: function() {
        _.each(this.models, function(sensor) {
            sensor.set({
                "sourceSelected" : false,
                "sourceChecked" : false,
                "areaChecked" : false,
                "areaSelected" : false,
                "parameterSelected" : false,
                "parameterChecked" : false,
                "checked" : false,
                "selected" : false,
                "selectable" : true
            });
        });
    },

    // parse isn't necessary here at all
    // parse happens after a collection fetch, and before saving each JSON object as a sensor model
    // but I left it in as a reminder in case we would ever want to "filter" our sensors
    // before they become sensor "models". ie: manipulate their structure, etc.
    parse: function(response) {
        return response;
    },
    
    // "select" is used by the draw tools to select sensors on the map.
    // if we use "fetch", all the sensors in the collection get reset, so "select" uses ajax 
    // and doesn't alter the collection
    select: function(params, callback) {
        $.ajax({
            url: this.url,
            dataType: "jsonp",
            data: $.param(params),
            success: function (sensors) {
                
                // anytime someone selects something on the map, we want to designate the sensors as "selectable" and all others as "unselectable"
                // here we are simply getting all known id's...
                var all = _.pluck(app.collection.sensors.toJSON(), 'id');
                // here we get the id's for the sensors selected on the map...
                var found = _.pluck(sensors, 'id');
                // and here we indicate which sensors were NOT selected on the map
                var disabledSensors = _.difference(all, found);
                
                // set "selectable" to false for all the sensors we didn't just select (since they are true by default)
                _.each(disabledSensors, function(disabledSensor) {
                    app.collection.sensors.findWhere({id: disabledSensor}).set({selectable: false, selected: false, areaChecked: false, sourceChecked: false, parameterChecked: false });
                });

                // at this point, the variable 'sensors' is the raw JSON of all our sensors returned from the API
                // however, when fetching this selection from the API, we only specified the coordinates.
                // If the user has already checked a Data Source or a Parameter, we should filter those out of what was returned from the API

                // let's check to see if a Data Source checkbox was already selected.
                // If so, we'll only include those and filter out the rest

                // this next function should return an array of Source models
                var selectedSources = app.collection.sources.getSourcesWithSensors().where({checked: true});
                // if the array contains more than one checked item, we'll proceed...
                if ( (selectedSources instanceof Array) && (selectedSources.length > 0) ) {
                    // we'll use the 'map' function to create an array of just the source ID's
                    selectedSources = _.map(selectedSources, function(source) {
                        return source.get('id');
                    });
                    // now we will update the sensors variable (remember that sensors was the raw data from the API)
                    // we'll use filter to only include the sensors that have one of the checked Data Sources in properties.type.id
                    sensors = _.filter(sensors, function(sensor) {
                        return (selectedSources.indexOf(sensor.properties.type.id) > -1);
                    });
                    // at this point, the sensors variable has been updated, removing any sensors where the Data Source was unchecked
                }

                // let's check to see if a parameter checkbox was already selected.
                // If so, we'll update our sensors variable that was returned from the API to only include those and filter out the rest

                // this next function should return an array of Parameter models
                var selectedParameters = app.collection.parameters.where({checked: true});
                // if the array contains more than one checked item, we'll proceed...
                if ( (selectedParameters instanceof Array) && (selectedParameters.length > 0) ) {
                    // we'll use 'map' to essentially reduce the parameter models to just the list of alternates
                    // and we wrap the map function in the 'flatten' function. This will give us one big array of all possible alternates
                    selectedParameters = _.flatten(_.map(selectedParameters, function(parameter) {
                        return parameter.get('alternates');
                    }));
                    // now we will update the sensors variable (remember that sensors was the raw data from the API)
                    // we'll use filter to only include the sensors that have one of the checked Parameters
                    sensors = _.filter(sensors, function(sensor) {
                        // since sensor.parameters is an array, and selectedParameters is an array, we'll use the "intersection" function.
                        // This will check if this sensor.parameters contains any of the parameters in selectedParameters,
                        // and if true, that sensor will remain in the "sensors" variable. All others will be removed.
                        return _.intersection(sensor.parameters, selectedParameters).length > 0;
                    });
                }

                // any sensor still in the sensors variable is going to be a "checked sensor"
                // we'll grab the sensor model for each, then assign the proper attributes
                // make sure that each sensor is both selectable and areaSelected
                _.each(sensors, function(sensor) {
                    var sensorModel = app.collection.sensors.findWhere({id: sensor.id});
                    sensorModel.set({
                        areaSelected: true,
                        selectable: true,
                        areaChecked: true
                    });
                });
                
                // we check if a callback function was passed so that we can perform some action after this complete
                // see SearchView.js
                if (typeof(callback) == 'function') {
                    callback(sensors);
                } else {
                    return sensors;
                }
                
            }
        });
        
    },

    groupByHUCcodeSourcesSensors: function(options) {
      var hucKey;
      var results = new Backbone.Collection;
      results.comparator = 'code';

      if (options.hucCode && ($.inArray(options.hucCode, [2,4,6,8]) > -1)) {
        hucKey = "huc" + options.hucCode;
      } else {
        console.warn("app.collection.sensors.groupByHUCcodeSourcesSensors(hucCode): hucCode argument is required.");
        return;
      }

      if (hucKey) {
        results['title'] = hucKey;
        // this.each refers to the collection. this.each will iterate all sensors
        this.each(function(sensor) {
          var huc = sensor.get('huc');
          if (huc[hucKey]) {
              var hucModel = results.add({id: huc[hucKey]['code'], code: huc[hucKey]['code'], title: huc[hucKey]['name']});
              hucModel.set('checked', true);
              if (! hucModel.get('sources')) {
                hucModel.set('sources', new Backbone.Collection);
              }
              var source = hucModel.get('sources').add(sensor.get('type'));
              if (! source.get('sensors')) {
                source.set('sensors', new Backbone.Collection);
              }
              source.get('sensors').add(sensor);
          }
        });
      }

      if (options.includedCodes) {
          return results.filter(function(sensor) {
              return _.contains(options.includedCodes, sensor.get('code'));
          });
      } else {
          return results;
      }

    }
});
// currently using this in the SearchView.js, but may further explore collections with other models
var Datapoints = Backbone.Collection.extend({
	url: app.config.clowderUrl + "/api/geostreams/datapoints"
});

app.datapoints = new Datapoints();
app.collection.Parameters = Backbone.Collection.extend({
    model: app.model.Parameter,

    // the comparator function is used to sort the collection
    // if you define a comparator function in app.config.parameters.comparator,
    // we will use that, otherwise it will sort by title
    comparator: function(model) {
        if (app.config.parameters && app.config.parameters.comparator &&
            _.isFunction(app.config.parameters.comparator)) {
            return app.config.parameters.comparator(model);
        } else {
            return model.get('title');
        }
    },

    // a url function is not defined since we are not using the api
    // url: app.config.clowderUrl + "/api/geostream/parameters"

    // convenience method to get only the visible parameters (for exploreView, searchView, etc)
    getVisible: function(view) {
        var options = {};
        view = view || 'exploreView';
        options[view] = 'visible';
        return this.where(options);
    },

    getChecked: function() {
        return this.where({checked: true});
    },

    uncheckAll: function() {
        this.each(function(model) {
            model.set("checked", false);
        });
    },
    groupByParametersSourcesSensors: function(collection) {
        // clone the entire parameters collection (or a passed collection)
        // into a new Backbone Collection called "response"
        // filter the collection only for visible parameters
        // we'll make modifications and return them, but won't alter the original collection
        collection = collection || this;
        var response = new Backbone.Collection(collection.clone().where({exploreView: 'visible'}));

        // for each parameter in the collection
        response.each(function(parameter) {
            // each parameter will look through it's sensors and keep a list of their sources
            // the "HucItems" collection uses a Collection -> Item -> Sensors structure that is
            // reusable here
            var sources = new app.collection.HucItems;

            // loop through each sensor for this parameter
            var theParameterSensors = [];
            if (parameter.get('sensors')) {

                var theSensors = app.collection.sensors;
                theSensors.filter(function(x){
                    x.attributes.parameters.forEach(function(y) {
                        if (parameter.id == y.id) {
                            theParameterSensors.push(x);
                        }
                    });
                    parameter.set({'sensors': theParameterSensors});
                });

            }
            if (parameter.get('sensors').length) {
                parameter.get('sensors').forEach(function (sensor) {

                    // if we call "add", it won't create a duplicate model for the source
                    var source = sources.add(sensor.get('type'));

                    // add this sensor to the source collection
                    source.get('sensors').add(sensor);

                });
            }
            // after looping through each parameter, add the sources collection as a property
            parameter.set('sources', sources);

        });

        return response;
    }
});
app.collection.Areas = Backbone.Collection.extend({
    model: app.model.Area,
    // a url function is not defined since we are not using the api
    // url: app.config.clowderUrl + "/api/geostream/areas"

    clearSelection: function() {
        _.each(this.models, function(area) {
            if (area.get('items')) {
                area.get('items').each(function(areaInGroup) {
                    areaInGroup.set({
                        selected: false,
                        checked: false
                    });
                });
            } else {
                area.set({
                    selected: false,
                    checked: false
                });
            }
        });
    },

    getChecked: function() {
        return this.where({checked: true});
    }

});


app.collection.HucItems = Backbone.Collection.extend({
    model: app.model.HucItem,
    // a url function is not defined since we are not using the api
    // url: app.config.clowderUrl + "/api/geostream/hucs"

    getChecked: function() {
        return this.where({checked: true});
    }

});


app.collection.Hucs = Backbone.Collection.extend({
    model: app.model.Huc,
    // a url function is not defined since we are not using the api
    // url: app.config.clowderUrl + "/api/geostream/hucs"

    getChecked: function() {
        return this.where({checked: true});
    }

});


app.collection.Regions = Backbone.Collection.extend({
  model: app.model.Region,
  // a url function is not defined since we are not using the api
  // url: app.config.clowderUrl + "/api/geostream/regions"

  getChecked: function() {
    return this.where({checked: true});
  },

  groupByRegionsParametersSensors: function() {
    var results = new Backbone.Collection;

    this.each(function(region) {
      var newRegion = results.add(region);
      var regionSensors = app.collection.sensors.where({region: region.id});
      _.each(regionSensors, function(sensor) {

        if (! newRegion.get('parameters')) {
          newRegion.set('parameters', new Backbone.Collection);
        }

        sensor.get('parameters').each(function(parameter){
          var newParameter = parameter.clone();
          newParameter.unset('sensors');

          var parameterSensors = parameter.get('sensors');
          parameterSensors = parameterSensors.where({region: region.id});

          newParameter.set('sensors', parameterSensors);
          newRegion.get('parameters').add(newParameter);
        });

      });
    });
    return results;
  },

  groupByRegionsSourcesSensors: function() {
    var results = new Backbone.Collection;

    this.each(function(region) {
      var newRegion = region.clone();
      var newRegionModel = results.add(newRegion);
      newRegionModel.get('sources').each(function(source) {
        source.set('sensors', new Backbone.Collection);
        var sourceSensors = app.collection.sensors.filter(function(sensor) {
          return sensor.get('type').id == source.id && sensor.get('region') == region.id;
        });
        source.get('sensors').add(sourceSensors);
      });
    });
    return results;
  }

});


app.collection.Sources = Backbone.Collection.extend({
    model: app.model.Source,
    comparator: 'id',
    // a url function is not defined since we are not using the api
    // url: app.config.clowderUrl + "/api/geostream/sources"

    getChecked: function() {
        return this.where({ checked: true });
    },

    // calling 'app.collection.sources.getColor(id)' will return a consistent color for each source
    getColor : function(source) {
        return source == "epa" ? "#0D71BA" :
            source == "usgs" ? "#39B54A" :
            source == "lec" ? "#4D6363" :
            source == "gsfmp" ? "#CC2128" :
            source == "iadn" ? "#F7941E" :
            "white";
    },

    // used by the Explore page. This goes through each source and returns the following:
    // Sources grouped by Areas, then Sensors. ie: [ epa: [ lake-superior: [ sensors: [ sensor1, sensor2...]]
    groupBySourcesAreasSensors: function(callback) {

        // we'll return an array of objects, one for each source
        var response = [];

        // "this" refers to the collection. this.each iterates over each source in app.collection.sources.
        this.each(function(source) {
            // build this source's object
            var sourceResponse = {};
            sourceResponse.id = source.get('id');
            sourceResponse.title = source.get('title');
            sourceResponse.prettyName = source.get('prettyName');
            sourceResponse.acronym = source.get('acronym');
            sourceResponse.description = source.get('description');
            sourceResponse.color = source.get('color');
            sourceResponse.areas = new Array(app.collection.sources.getSourcesWithSensors().length);

            // Filter the Sensors by Source
            var theSensors = app.collection.sensors.filter(function(x){return x.attributes.type.id == sourceResponse.id});
            theSensors.forEach(function(sensor) {
                var sensorRegion = sensor.get("region");
                if (sensorRegion) {
                    sensorRegion = sensorRegion.toLowerCase();
                    // get the Backbone model for this area
                    var areaModel = app.collection.areas.findWhere({ id: sensorRegion });
                    // if no Backbone model exists for this area (aka: the sensor's "region" property), create it
                    if (!areaModel) areaModel = app.collection.areas.add({id: sensorRegion, region: sensorRegion.toUpperCase(), title: sensorRegion.toUpperCase() });
                    var areaTitle = areaModel.get("title");
                    // foundArea must check to see if this area has already been found
                    // otherwise you'd get a huge array where each sensor recreated the area it was in
                    var foundArea = _.findWhere(sourceResponse.areas, { id: sensorRegion });

                    if (!foundArea) {
                        foundArea = { id: sensorRegion, title: areaTitle, sensors: [sensor.toJSON()]};
                        sourceResponse.areas.splice(areaModel.get('sortOrder'), 0, foundArea);
                    } else {
                        foundArea.sensors.push(sensor.toJSON());
                    }

                }

            });

            response.push(sourceResponse);
        });
        if (typeof callback == 'function') {
            callback(response);
        } else {
            return response;
        }
    },

    uncheckAll: function() {
        this.each(function(model) {
            model.set("checked", false);
        });
    },

    // get a unique list of all (non-ignored) parameters for the passed in source
    getParameters: function(source) {
        var parameters = [];
        if (source.get('sensors')) {
            source.get('sensors').each(function(sensor) {
                _.each(sensor.get('parameters'), function(parameter) {
                    if (parameter.get('id') != 'ignored') {
                        parameters.push(parameter);
                    }
                });
            });
        }
        return _.unique(parameters);
    },

    // maintain backwards compatibility of the original getAreas function
    getAreas: function(options) {
        return this.groupBySourcesAreasSensors(options);
    },

    getSourcesWithSensors: function(options) {
        var response = this.filter(function(source){
            return source.get('sensors') && source.get('sensors').length;
        });
        return new Backbone.Collection(response);
    }

});
app.collection.ParameterCategories = Backbone.Collection.extend({
    comparator: 'title',

    // a url function is not defined since we are not using the api
    // url: app.config.clowderUrl + "/api/geostream/parameterCategories"

    getChecked: function() {
        return this.where({checked: true});
    },

    uncheckAll: function() {
        this.each(function(model) {
            model.set("checked", false);
        });
    },

  groupByParameterCategoriesSourcesSensors: function() {
    // clone the entire parameters collection into a new Backbone Collection called "response"
    // filter the collection only for visible parameters
    // we'll make modifications and return them, but won't alter the original collection
    var response = this.clone();

    response.each(function(parameterCategory) {
      if (parameterCategory.get('parameters')) {
        parameterCategory.set('checked', true);
        var catResponse = app.collection.parameters.groupByParametersSourcesSensors(parameterCategory.get('parameters'));
        if (catResponse.get('sources') && catResponse.get('sources').length) {
          parameterCategory.set('parameters', catResponse);
        }

      }
    });

    response = response.filter(function(category) {
      return category.get('parameters') && category.get('parameters').find(function(param) {
          return param.get('sources') && param.get('sources').length && param.get('sources').find(function(source) {
              return source.get('sensors') && source.get('sensors').length;
            });
        });
    });

    return response;
  }
});
app.collection.Layers = Backbone.Collection.extend({
    model: app.model.Layer,
    // a url function is not defined since we are not using the api
    // url: app.clowderUrl + "/api/geostream/areas"


    clearSelection: function() {
        _.each(this.models, function(layer) {
            layer.set("checked", false);
        });
    }


});
app.view = app.view || {};

Backbone.View.prototype.close = function(){
  this.remove();
  this.unbind();
  if (this.onClose){
    this.onClose();
  }
}

function getBinningDesignators(value) {
    switch(value) {
        case "season":
            return {
                title: "Season",
                items: [
                    { id: 'spring', name: 'Spring' },
                    { id: 'summer', name: 'Summer' },
                    { id: 'fall', name: 'Fall' },
                    { id: 'winter', name: 'Winter' }
                 ]
            };
            break;
        case "semi":
        case "semi-year":
        case "semi-annually":
            return {
                title: "Season",
                items: [
                    { id: 'spring', name: 'Spring' },
                    { id: 'summer', name: 'Summer' }
                ]
            };
            break;
        case "monthly":
            return {
                title: "Month",
                items: [
                    {id: 'jan', name: 'January'},
                    {id: 'feb', name: 'February'},
                    {id: 'mar', name: 'March'},
                    {id: 'apr', name: 'April'},
                    {id: 'may', name: 'May'},
                    {id: 'jun', name: 'June'},
                    {id: 'jul', name: 'July'},
                    {id: 'aug', name: 'August'},
                    {id: 'sep', name: 'September'},
                    {id: 'oct', name: 'October'},
                    {id: 'nov', name: 'November'},
                    {id: 'dec', name: 'December'}
                ]
            };
            break;
        default:
            return {title: "Bin", items: []};
            break;
    }
}
app.view.AppView = Backbone.View.extend({
    el : "#appContainer",

    events : {
        "click #header-go-home" : "goHome"
    },

    initialize : function() {

        this.genericSpinner = {
            // spinner specific settings //
            lines: 12, // The number of lines to draw
            length: 60, // The length of each line
            width: 20, // The line thickness
            radius: 40, // The radius of the inner circle
            corners: 1, // Corner roundness (0..1)
            rotate: 0, // The rotation offset
            direction: 1, // 1: clockwise, -1: counterclockwise
            color: '#000000', // #rgb or #rrggbb or array of colors
            speed: 1.2, // Rounds per second
            trail: 80, // Afterglow percentage
            shadow: true, // Whether to render a shadow
            // spinner page settings //
            className: 'generic_spinner', // The CSS class to assign to the spinner
            zIndex: 2e9, // The z-index (defaults to 2000000000)
            top: '50%', // Top position relative to parent
            left: '50%', // Left position relative to parent
            position: 'fixed' // Element positioning
        };

        var self = this;
        this.router = app.router.router = new app.router.AppRouter();

        this.eventBus = app.event.eventBus = _({}).extend(Backbone.Events);

        // compile the map pin, used in HomeView and DetailView
        this.mapPin = _.template($('#map-pin-template').html())();

        this.geoStreamService = app.model.geoStreamService = new app.model.GeoStreamService();

        this.currentView = null;

        $('meta[name="keywords"]').attr('content', app.config.metadata.keywords);
        $('meta[name="description"]').attr('content', app.config.metadata.description);
        $('meta[property="og:title"]').attr('content', app.config.metadata.title);
        $('meta[property="og:description"]').attr('content', app.config.metadata.description);
        $('meta[property="og:url"]').attr('content', app.config.metadata.url);
        $('meta[name="twitter:title"]').attr('content', app.config.metadata.title);
        $('meta[name="twitter:description"]').attr('content', app.config.metadata.description);
        $('meta[name="twitter:url"]').attr('content', app.config.metadata.url);

        app.collection.areas = new app.collection.Areas();
        app.collection.regions = new app.collection.Regions();
        app.collection.hucs = new app.collection.Hucs();
        app.collection.sources = new app.collection.Sources();
        app.collection.parameterCategories = new app.collection.ParameterCategories();
        app.collection.parameters = new app.collection.Parameters();
        app.collection.sensors = new app.collection.Sensors();
        app.collection.layers = new app.collection.Layers();
        app.collection.glossary = new Backbone.Collection();

        if (app.config.areas) {
            if (app.config.areas.groups) {
                app.collection.areas = new app.collection.Areas();
                _.each(app.config.areas.groups, function(areaGroup) {
                    app.collection.areas.add(
                        {
                            id: areaGroup.title.toLowerCase().replace(/ /g, '-'),
                            title: areaGroup.title,
                            items: new app.collection.Areas(areaGroup.items)
                        }
                    );
                });
            } else if (app.config.areas.items) {
                _.each(app.config.areas.items, function(area) {
                    app.collection.areas.add(area);
                });
            }

        }

        if (app.config.parameters && app.config.parameters.items) {
            _.each(app.config.parameters.items, function(parameter) {
                app.collection.parameters.add(parameter, {merge: true});
            });
        }

        if (app.config.parameters.categories) {
            _.each(app.config.parameters.categories, function(category) {
                app.collection.parameterCategories.add(category);
            });
        }

        if (app.config.sources && app.config.sources.items) {
            _.each(app.config.sources.items, function(source) {
                app.collection.sources.add(source);
            });
        }

        if (app.config.sensors && app.config.sensors.items) {
            _.each(app.config.sensors.items, function(sensor) {
                app.collection.sensors.add(sensor);
            });
        }

        if (app.config.layers && app.config.layers.items) {
            _.each(app.config.layers.items, function(layer) {
                app.collection.layers.add(layer);
            });
        }

        if (app.config.glossary && app.config.glossary.items) {
            _.each(app.config.glossary.items, function(item){
                app.collection.glossary.add(item);
            });
        }

        // update all views when new data gets in
        app.collection.sensors.fetch({
            dataType: "jsonp",
            success: function(data) {
                app.event.eventBus.trigger("sensors:refresh");
                return data;
            }
        });

        // if a view in app.config contains an array of "children" pages, setup route listeners
        _.each(app.config.views, function(view) {
            if (view.children) {
                _.each(view.children, function(childView) {
                    if (childView.title && childView.link && childView.template) {
                        if (!childView.view) childView.view = 'PageView';
                        self.router.on("route:" + childView.title.toLowerCase(), (function(self) {
                            return function() {
                                document.title = app.config.title + " - " + childView.title;
                                $("ul.navbar > li").removeClass("active");
                                $("a[href='" + view.link + "']").parent().addClass("active");
                                if(!app.view[childView.title.toLowerCase() + "View"]) {
                                    app.view[childView.title.toLowerCase() + "View"] = new app.view[childView.view](childView);
                                }

                                self.setView(app.view[childView.title.toLowerCase() + "View"]);
                                self.render();
                            }
                        })(self));
                    }
                });
            }
        });

        if (app.config.views.about && app.config.views.about.title && app.config.views.about.link) {
            this.router.on("route:about", (function (self) {
                return function () {
                    document.title = app.config.title + " - " + app.config.views.about.title;
                    $("ul.navbar > li").removeClass("active");
                    $("a[href='" + app.config.views.about.link + "']").parent().addClass("active");
                    self.initializeAboutView();
                    self.render();
                };
            })(this));
        }

        if (app.config.views.welcome && app.config.views.welcome.title && app.config.views.welcome.link) {
            this.router.on("route:welcome", (function (self) {
                return function () {
                    document.title = app.config.title + " - " + app.config.views.welcome.title;
                    $("ul.navbar > li").removeClass("active");
                    $("a[href='" + app.config.views.welcome.link + "']").parent().addClass("active");
                    self.initializeWelcomeView();
                    self.render();
                };
            })(this));
        }

        if (app.config.views.compare && app.config.views.compare.title && app.config.views.compare.link) {
            this.router.on("route:compare", (function(self) {
                return function() {
                    document.title = app.config.title + " - " + app.config.views.compare.title;
                    $("ul.navbar > li").removeClass("active");
                    $("a[href='" + app.config.views.compare.link + "']").parent().addClass("active");
                    self.initializeCompareView();
                    self.render();
                };
            })(this));

            app.collection.parameterBins = new Backbone.Collection();

            // always sort parameter alphabetically in the dropdowns
            app.collection.compareSelectedParameters = new Backbone.Collection();
            app.collection.compareSelectedParameters.comparator = 'title';

            // sort dates by id so they are in order by time
            app.collection.compareSelectedDates = new Backbone.Collection();
            app.collection.compareSelectedDates.comparator = 'id';
        }


        // on route:home, set currentView to the HomeView
        this.router.on("route:home", (function(self) {
            return function() {
                document.title = app.config.title;
                $("ul.navbar > li").removeClass("active");
                $("a[href='" + (app.config.views.home.link || "#explore") + "']").parent().addClass("active");
                self.initializeHomeView();
                self.render();
            };
        })(this));

        this.router.on("route:populateDetailView", (function(self) {
            return function(location, startTime, endTime, parameters, season, depth, depthYear, depthSeason, biologyStartTime, biologyEndTime, biologyBin) {
                document.title = app.config.title + " - Details";
                self.initializeDetailView(location, startTime, endTime, parameters, season, depth, depthYear, depthSeason, biologyStartTime, biologyEndTime, biologyBin);
            };
        })(this));

        this.router.on("route:search", (function(self) {
            return function() {
                document.title = app.config.title + " - " + (app.config.views.search.title || "Search");
                $("ul.navbar > li").removeClass("active");
                $("a[href='" + (app.config.views.search.link || "#search") + "']").parent().addClass("active");
                self.initializeSearchView();
                self.render();
            };
        })(this));

        // catch event fired by explore-pill or map-pin clicks
        // use this to fire the detail page route.
        this.eventBus.bind("change:location", (function(self) {
            return function(location) {
                var loc = "location/" + location + "/";
                self.router.navigate("detail/" + loc, {trigger : true, replace : false});
            };
        })(this));

        this.footerView = new app.view.FooterView({config: app.config.views.footer});
        this.navBarView = new app.view.NavBarView().render();

        // get the config
        $.get(app.config.clowderUrl + "/api/geostreams/config", function(data) {
            _.extend(app.config, data);
        });

        Backbone.history.start();
    },

    render : function() {
        if(this.currentView) this.currentView.render();
    },


    goHome : function() {
        this.router.navigate("", {trigger : true, replace : true});
    },

    // handles changing the current view.
    setView : function(view) {
        // if the new view is not the current view.
        if(this.currentView !== view) {
            // detach (preserves event listeners) the current view from the DOM
            if(this.currentView) this.currentView.$el.detach();
            this.currentView = view;
            this.$("#content-container")
                .append(this.currentView.el); // append the new view.
        }
//      this.render();
    },

    initializeHomeView : function() {
        if(!app.view.homeView) {
            app.view.homeView = new app.view.HomeView({
                config: app.config.views.home
            });
        }

        this.setView(app.view.homeView);

        // Add spinner immediately
        this.$el.spinner = new Spinner(this.genericSpinner).spin(this.el);

        // Need a local copy of this to work properly
        var that = this;

        // Check the page readyState value every second until the if clause is true
        var checkPageState;
        checkPageState= setInterval(function() {
            if (document.readyState == 'complete') {

                // Remove the Spinner only when the page is finished rendering
                that.$el.spinner.stop();

                // Stop checking every second
                clearInterval(checkPageState);

            }
        }, 1000);
    },

    initializeDetailView : function() {
        // create the DetailView if is doesn't exist,
        // or if the existing detailView doesn't match the ID of the location we want
        if(!this.detailView || this.detailView.locationID != arguments[0]) {
            this.detailView = new app.view.DetailView({
                parentView : this,
                eventBus : this.eventBus,
                mapPin : this.mapPin,
                router : this.router,
                stationService : this.stationService,
                geostreamService : this.geoStreamService,
                arguments: arguments,
                attributes: app.config.views.detail.attributes,
                config: app.config.views.detail
            });
        }

        // call the function to set up the view.
        this.setView(this.detailView);
        this.detailView.initiate.apply(this.detailView, arguments);
    },

    initializeSearchView : function() {
        if (!app.view.searchView) {
            app.view.searchView = new app.view.SearchView({
                parentView : this,
                eventBus : this.eventBus,
                router : this.router,
                stationService : this.stationService,
                geostreamService : this.geoStreamService,
                attributes: app.config.views.search.attributes,
                config: app.config.views.search
            });
        }

        this.setView(app.view.searchView);
    },

    initializeAboutView : function() {
        if(!app.view.aboutView) {
            app.view.aboutView = new app.view.AboutView(app.config.views.about);
        }

        this.setView(app.view.aboutView);
    },

    initializeWelcomeView : function() {
        if(!app.view.welcomeView) {
            app.view.welcomeView = new app.view.WelcomeView(app.config.views.welcomeView);
        }

        this.setView(app.view.welcomeView);

        // Add spinner immediately
        this.$el.spinner = new Spinner(this.genericSpinner).spin(this.el);

        // Need a local copy of this to work properly
        var that = this;

        // Check the page readyState value every second until the if clause is true
        var checkPageState;
        checkPageState= setInterval(function() {
            if (document.readyState == 'complete') {

                // Remove the Spinner only when the page is finished rendering
                that.$el.spinner.stop();

                // Stop checking every second
                clearInterval(checkPageState);

            }
        }, 1000);
    },

    initializeCompareView : function() {
        if(!app.view.compareView) {
            app.view.compareView = new app.view.CompareView({
            });
        }

        this.setView(app.view.compareView);
    }
});
/* 
 * Handles the controls on the detail page.
 * Specifically covers the parameters checkboxes and the date slider.
 * Does not properly map to a single div within the DetailView because
 * of the way the tabs are setup in the page.
 */
app.view.ControlsView = Backbone.View.extend({
    attributes: function() {
        return {
            tagName: this.options.attributes.tagName,
            id: this.options.attributes.id
        }
    },

	events : {
		'change #times' : 'changeTime'
	},
	
	initialize : function(options) {
        this.config = options;
        _.bindAll(this, "renderSeasonSelect");
	    this.eventBus = options.eventBus || app.event.eventBus;
        this.eventBus.on("locationUpdated", this.location(arguments));
        this.eventBus.on("renderSeasonSelect", this.renderSeasonSelect);
	  
        this.parametersID = options.parametersID || null;
        this.binDropdown = options.binDropdown || null;
        this.sliderID = options.sliderID || null;
        this.dateRangeID = options.dateRangeID || null;
        this.yearFormID = options.yearFormID || null;
        this.yearSelectID = options.yearSelectID || null;
        this.name = options.name || null;

        this.render = _.bind(this.render, this);
	},
	
	/*
	 * This blog: http://ianstormtaylor.com/break-apart-your-backbonejs-render-methods/
	 * recommended making the render functions return 'this' to make them chainable.
	 * I only got around to applying it here.
	 */
	render : function() {
		this.renderParameters(this.parameters)
			.renderSlider(this.times)
			.renderSeasonSelect();
	},

	// uses a d3 paradigm to avoid reconstructing the slider every time a
	// dimension changes. Allows the same function to handle a change to the
	// dimensions and also create the slider from scratch if it hasn't been
	// initialized yet.
	renderSlider : function(times, location, updateUrl) {
        if (!this.sliderID) return;
		var $slider = d3.select("#" + this.sliderID);

		var data = { items: times, location: location, updateUrl: updateUrl};

        var instance = $slider.selectAll(".instance").data([data], function(d) { return d.location; });
        instance.enter().append("div").attr("class", "instance")
        .call((function (self) {
            return function(selection) {
                selection.each(function(d) {
                    $(this).slider({
                        range : true,
                        step : 1,
                        values : [0, d.items.length - 1],
                        min : 0,
                        max : d.items.length - 1,
                        change : function(event, ui) {
                            var updateUrl = false;
                            if(event.originalEvent){
                                updateUrl = true;
                            }
                            var values = ui.values;
                            self.changeTimes(values[0], values[1], updateUrl);
                        },
                        create : function(event, ui) {
                            self.changeTimes(0, d.items.length - 1, d.updateUrl);
                        },
                        slide: function(event, ui) {
                            var values = ui.values;
                            self.sliderTimes(self.times[values[0]], self.times[values[1]]);
                        }
                    });
                });
            };
        })(this));

        instance.exit().remove();
		
		return this;
	},
	
	// uses a d3 paradigm to avoid reconstructing the parameter checkboxes every
  // time a dimension changes. Allows the same function to handle a change to
  // the dimensions and also create the checkboxes from scratch if they
  // haven't been initialized yet.
	renderParameters : function(parameters, selected, location, colors) {
        if (!this.parametersID) return;
        var parametersID = this.parametersID;
        var $parameters = d3.selectAll("#" + parametersID);

        var data = parameters.map(function(d) {
            var parameterModel = app.collection.parameters.findWhere({id: d});
            if (parameterModel) {
                return { parameter: parameterModel, location: location, color: colors(d) };
            }
        });
        
        var controlGroup = $parameters.selectAll(".control-group").data(data, function(d) { return d.location + "." + d.parameter.id; });
        controlGroup.enter().append("div").attr("class", "control-group");
        controlGroup.exit().remove();

        var controls = controlGroup.selectAll(".controls").data(function(d) { return [d]; });
        controls.enter().append("div").attr("class", "controls");

        var input = controls.selectAll("input").data(function(d) { return [d]; });

        var controlLabel = controlGroup.selectAll(".control-label-parameters").data(function(d) { return [d]; });
        controlLabel.enter().append("label");
        controlLabel.attr("class", function(d) { return "control-label-parameters " + d.parameter.id; })
            .style("color", function(d) { return d.color; })
            .attr("for", function(d) { return d.parameter.id; }).html(function(d) {
              var parameterWithUnit = d.parameter.get('title');
              if (d.parameter.get('unit') && d.parameter.get('unit').length) {
                  parameterWithUnit += " <small>(";
                  parameterWithUnit += d.parameter.get('unit');
                  parameterWithUnit += ")</small>";
              }
              return parameterWithUnit;
          }); //TODO: hack to display Chl a with an italic a. fix this with parameter name mapping
            
        input.enter().append("input").attr("type", "checkbox").attr("name", "parameter")
            .property("checked", function(d, i) { if(selected) return selected.filter(function(e) { return e === d.parameter.id; } ).length === 0 ? null : true; })
            .attr("id", function(d) { return d.parameter.id; }).attr("value", function(d) { return d.parameter.get('title'); })
            .on("click", (function(self) {
                return function() {
                    self.changeParameters();
                }
            })(this));
		return this;
	},

    renderSeasonSelect : function() {
    if (!this.binDropdown) return;
    if (this.binDropdown && this.binDropdown.filterFunction && this.binDropdown.sensor) {
      if (!this.binDropdown.filterFunction(this.binDropdown.sensor)) {
        return;
      }
    }
    var data = getBinningDesignators(app.config.binning.time) || { title: "Season", items: [{ id: "spring", name: "Spring"}, { id: "summer", name: "Summer" }]};

      var seasonForm = d3.select("#" + this.binDropdown.seasonFormID);

	  var controlGroup = seasonForm.selectAll(".control-group").data([data.items]);
	  controlGroup.enter().append("div").attr("class", "control-group");
	  controlGroup.exit().remove();
	  
	  var label = controlGroup.selectAll(".control-label-season").data(function(d) { return [d]; });
	  label.enter().append("label").attr("class", "control-label-season").attr("for", this.binDropdown.seasonSelectID).html(data.title);
	  label.exit().remove();
	  
	  var controls = controlGroup.selectAll(".controls").data(function(d) { return [d]; });
	  controls.enter().append("div").attr("class", "controls");
	  controls.exit().remove();

	  var select = controls.selectAll("#" + this.binDropdown.seasonSelectID).data(function(d) { return [d]; });
	  select.enter().append("select").attr("id", this.binDropdown.seasonSelectID).attr("class", "input-season-select")
	    .on("change", (function(self) {
	      return function() {
	        self.changeSeason();
	      }
	    })(this));
	  select.exit().remove();
	  
	  var options = select.selectAll("option").data(function(d) { return d; });
	  options.enter().append("option").attr("value", function(d) { return d.id; }).html(function(d) { return d.name; })
	  options.exit().remove();
	  
	  return this;
	},

    renderYearSelect : function(times, that) {
        if (!that.yearFormID) return;
        // get the unique years. It gives us a list like: 1998, 1999, 2000...
        var data = _.uniq(_.map(times, function(time) {
                var year = new Date(time);
                return year.getUTCFullYear();
        }));
        var yearForm = d3.select("#" + this.yearFormID);

        var controlGroup = yearForm.selectAll(".control-group").data([data]);
        controlGroup.enter().append("div").attr("class", "control-group");
        controlGroup.exit().remove();

        var label = controlGroup.selectAll(".control-label-year").data(function(d) { return [d]; });
        label.enter().append("label").attr("class", "control-label-year").attr("for", this.yearSelectID).html("Year");
        label.exit().remove();

        var controls = controlGroup.selectAll(".controls").data(function(d) { return [d]; });
        controls.enter().append("div").attr("class", "controls");
        controls.exit().remove();

        var select = controls.selectAll("#" + this.yearSelectID).data(function(d) { return [d]; });
        select.enter().append("select").attr("id", this.yearSelectID).attr("class", "input-year-select")
            .on("change", (function(self) {
                return function() {
                    self.changeYear();
                }
            })(this));
        select.exit().remove();

        var options = select.selectAll("option").data(function(d) { return d; }, function(d) { return d; });
        options.enter().append("option").attr("value", function(d) { return d; }).html(function(d) { return d; });
        options.exit().remove();
        options.order();

        this.year = $("#" + this.yearSelectID).val();

        return this;
    },

    setParameters : function(parameters, selected, colors) {
        this.parameters = parameters;
        var selectedFound = selected || this.getSelectedParameters();
        this.renderParameters(parameters, selectedFound, this.location(), colors);
    },

    setTimes : function(times, updateUrl) {
        this.startTime = times[0];
        this.endTime = times[times.length - 1];
        this.times = times;
        this.renderSlider(times, this.location(), updateUrl) ;
        this.renderSeasonSelect();
    },

    updateTimes: function(start, end){
	    this.startTime = start.toUTCString();
	    this.endTime = end.toUTCString();
	    var $slider = $("#" + this.sliderID + " .instance");
        $slider.slider( "option", 'values', [this.times.indexOf(this.startTime), this.times.indexOf(this.endTime)]);
    },

    setYears: function(years) {
        this.renderYearSelect(years, this);
    },

    // the location is used by the d3 selections to determine if the controls
    // need to be repopulated.
    location : function(location, season) {
        if(!location) return this.loc;

        this.loc = location + (season || "");
    },


    // sets the time range next to the slider.
    sliderTimes : function(start, end) {
      var prettyStart = new Date(start);
      prettyStart = (prettyStart.getUTCMonth() + 1) + "/" + prettyStart.getUTCDate() + "/" + prettyStart.getUTCFullYear();

      var prettyEnd = new Date(end);
      prettyEnd = (prettyEnd.getUTCMonth() + 1) + "/" + prettyEnd.getUTCDate() + "/" + prettyEnd.getUTCFullYear();

      $('#' + this.dateRangeID).html(prettyStart + ' - ' + prettyEnd);
    },

    // catches changes to the date slider and fires an event to change the
    // route.
    changeTimes : function(start, end, updateUrl) {
      this.startTime = this.times[start];
      this.endTime = this.times[end];

      this.sliderTimes(this.startTime, this.endTime);

      if(updateUrl) {
          if(this.name == 'biology') {
              app.event.eventBus.trigger("change:biologyDates", this.startTime, this.endTime);
          } else {
              app.event.eventBus.trigger("change:times", this.startTime, this.endTime);
          }
      }

    },

	getTimes: function() {
		return [new Date(this.startTime), new Date(this.endTime)];
	},

  // catches changes to the parameters and fires an event to change the route.
	changeParameters: function() {
		app.event.eventBus.trigger("change:parameters", this.getSelectedParameters());
	},

	changeSeason: function() {
	    if(this.name == 'biology') {
            app.event.eventBus.trigger("change:biologyBin", $("#" + this.binDropdown.seasonSelectID).val(), this.name);
        } else {
            app.event.eventBus.trigger("change:bin", $("#" + this.binDropdown.seasonSelectID).val(), this.name);
        }

	},

    changeYear: function() {
        this.year = $("#" + this.yearSelectID).val();
        app.event.eventBus.trigger("change:year", this.year);
    },

	  // used by the DetailView to determine which parameters are already selected.
	  // mostly useful when going back and forth between the same DetailView,
	  // so that the selected parameters aren't reset.
    getSelectedParameters : function() {
        var ret = [];
        
        $('.parameters input[type="checkbox"]:checked').each(function() {
            ret.push(encodeURIComponent($(this).attr('id')));
        });
        
        return ret;
    }
});
app.view.DetailView = Backbone.View.extend({

    events : {
        "click #detail-go-back" : "goBack",
        "click #showDistribution" : "showDistribution",
        "click .download:not(.disabled)": app.utils.interceptDownload
    },

    initialize: function(options) {
        var that = this;
        if(app.collection.sensors.length > 0) {
            that.initializeAfterFetch(options);
        } else {
            app.collection.sensors.fetch({
                success: function(collection) {
                    that.initializeAfterFetch(options);
                }
            });

        }

    },

    getInfoIconSettings: function() {
        return (
            '<svg xmlns="http://www.w3.org/2000/svg" version="1.1" height="18" width="18">' +
                '<circle cx="9" cy="9" r="8" fill="url(#g1)" stroke="#939598" stroke-width="2px" />' +
                '<text x="7" y="14" fill="darkblue" style="font : bold 14px Arial sans-serif;">i</text>' +
                '<defs>' +
                    '<linearGradient id="g1" gradientUnits="userSpaceOnUse" ' +
                                    'x1="-3%" y1="3.41%" x2="103%" y2="96.59%">' +
                        '<stop offset=".15" stop-color="#fff" />' +
                        '<stop offset=".39" stop-color="#fafafa" />' +
                        '<stop offset=".67" stop-color="#ececed" />' +
                        '<stop offset=".97" stop-color="#d4d6d6" />' +
                        '<stop offset="1" stop-color="#d1d3d4" />' +
                    '</linearGradient>' +
                '</defs>' +
            '</svg>'
        );
    },

    initializeAfterFetch : function(options) {
        var self = this;
        this.config = options.config;
        _.bindAll(this, 'render', 'setTabTitles', 'updateDownloadLink');
        this.timeSeriesModal = _.template($("#time-series-modal-template").html())();
        this.boxWhiskerModal = _.template($("#box-whisker-modal-template").html())();
        this.dataAggregateModal = _.template($("#data-aggregate-modal-template").html())();
        this.sourceModals = _.template($("#source-modals-template").html())();
        this.stationService = options.stationService;
        this.geostreamService = options.geostreamService;
        this.router = options.router;
        this.eventBus = options.eventBus;
        this.mapPin = options.mapPin;
        // locationID is used in AppView when rendering the DetailView.
        // if the locationID doesn't match the DetailView we are rendering,
        // we'll initialize a new DetailView. Resolves GEOD-309
        this.locationID = options.arguments[0] || null;

        // this.binningTime will be something like 'smart', 'semi', 'yearly', etc. It is how the datapoints
        // are averaged before being drawn in the graphs.
        // Projects can setup a filterFunction that accepts the current sensor (the Backbone Sensor Model)
        // as an argument. From there, you can return 'smart', 'semi', or some other binning value
        // based on a condition. For example: return (sensor.get('type')['id'] == 'epa') ? 'semi' : 'smart';
        // would return 'semi' as the binningTime for epa sensors, all others would use 'smart' binning
        this.binningTime = "";

        this.sensor = app.collection.sensors.findWhere({name: this.locationID});
        if (app.config.binning.filterFunction) {

            this.binningTime = app.config.binning.filterFunction(this.sensor);
        } else if (app.config.binning.time) {
            this.binningTime = app.config.binning.time;
        } else {
            this.binningTime = 'smart';
        }

        this.startTime = null;
        this.endTime = null;
        this.parameters = null;

        this.$el.append(_.template($(this.config.templateSelector).html())());
        var that = this;
        this.miniMap = []
        this.config.miniMap.forEach(function(map) {
            map.config["parentView"] = that;
            map.config["mapPin"] = that;
            that.miniMap.push(new app.view.MiniMapView(map.config));
            that.$(map.id).append(that.miniMap[that.miniMap.length-1].el);
        });

        if (this.config.controlsViews && this.config.controlsViews.length) {
            _.each(this.config.controlsViews, function(controlView) {
                controlView['parentView'] = self;
                controlView['eventBus'] = self.eventBus;
                if (controlView.config.binDropdown && controlView.config.binDropdown.filterFunction) {
                    controlView.config.binDropdown.sensor = app.collection.sensors.findWhere({name: self.locationID});
                }
                self[controlView.name] = new app.view.ControlsView(controlView.config);
            })
        }

//        if (this.config.timeSeriesViews && this.config.timeSeriesViews.length) {
//            _.each(this.config.timeSeriesViews, function(timeSeriesView) {
//                timeSeriesView['parentView'] = self;
//                timeSeriesView['eventBus'] = self.eventBus;
//                self[timeSeriesView.name] = new app.view.TimeSeriesView(timeSeriesView.config);
//            });
//        }

        this.aggregateTimeSeriesView = new app.view.TimeSeriesView({
            parentView : this,
        });

        this.$("#aggregate-timeseries").append(this.aggregateTimeSeriesView.el);

        this.limnologyView = new app.view.LimnologyView({
            parentView : this,
        });

        this.$("#limnology-chart").append(this.limnologyView.el);

        this.limnologyView.changeDepth((function(self) {
            return function(i) {
                self.eventBus.trigger("change:depth", i);
            };
        })(this));

        this.timeSeriesView = new app.view.TimeSeriesView({
            parentView : this,
        });
        this.stackedBarViews  = [];

        this.stackedLineViews = [];

        //this.$("#timeseries-at-depth").append(this.timeSeriesView.el);

        this.$('a[data-toggle="tab"]').on("shown", function(self) {
            return function(event) {
                self.render();
            };
        }(this));

        this.$el.append(this.timeSeriesModal);
        this.$el.append(this.boxWhiskerModal);
        this.$el.append(this.dataAggregateModal);

        // setup a loading spinner we can turn on and off
        var spinner = app.config.views.detail.spinnerOptions || {
                lines: 13 // The number of lines to draw
                , length: 28 // The length of each line
                , width: 14 // The line thickness
                , radius: 42 // The radius of the inner circle
                , scale: 1 // Scales overall size of the spinner
                , corners: 1 // Corner roundness (0..1)
                , color: '#000' // #rgb or #rrggbb or array of colors
                , opacity: 0.25 // Opacity of the lines
                , rotate: 0 // The rotation offset
                , direction: 1 // 1: clockwise, -1: counterclockwise
                , speed: 1 // Rounds per second
                , trail: 60 // Afterglow percentage
                , fps: 20 // Frames per second when using setTimeout() as a fallback for CSS
                , zIndex: 2e9 // The z-index (defaults to 2000000000)
                , className: 'spinner' // The CSS class to assign to the spinner
                , top: '50%' // Top position relative to parent
                , left: '50%' // Left position relative to parent
                , shadow: false // Whether to render a shadow
                , hwaccel: false // Whether to use hardware acceleration
                , position: 'absolute'
            };
        this.spinner = new Spinner(spinner);

        // the spinner isn't appended to the DOM until the first time it gets called.
        app.event.eventBus.on("detail:spinner:start", function() {
            self.$("#detail-panel").append(self.spinner.spin().el);
        });
        // use the eventBus to stop the spinner from anywhere
        app.event.eventBus.on("detail:spinner:stop", function() {
            self.spinner.stop();
        });



        // This function will return a url string in the order required by the view.
        // ie: location/time/parameters/depth/bin
        // Each individual change event listener in this view only needs to check to see if
        // its attribute (like parameter, bin, depth or time) is already set -
        // if so, it will replace it with the new values.
        // Otherwise it will append the attributes to the end of the url
        // The urlReOrder function does the job of making sure the order
        // matches what is required by this view.
        this.urlReOrder = function(newUrl) {
            // find "detail/location/ID/
            var locationRegex = /detail\/location\/.+?\//g;
            var locationFragment = locationRegex.exec(newUrl) || "";

            // find "startTime/DATE/endTime/DATE/
            var timeRegex = /startTime\/.*\/endTime\/.*?\//g;
            var timeFragment = timeRegex.exec(newUrl) || "";

            // find "parameters/PARAMETERS
            var parametersRegex = /parameters\/.*?\//g;
            var parametersFragment = parametersRegex.exec(newUrl) || "";

            // find "bin/SEASON"
            var binIDs = _.map(getBinningDesignators(self.binningTime).items, function(bin) { return bin.id} );
            var binRegexString = binIDs.join("|");
            var binRegex = new RegExp("bin\/(" + binRegexString + ")\/");
            var binFragment = binRegex.exec(newUrl);
            // since we are using parenthesis to match spring or summer, regex returns
            // an array of matches. We only need the first one, which is the entire url, so we use [0] below.
            binFragment = (binFragment ? binFragment[0] : "");

            // find "depth/NUMBER"
            var depthRegex = /depth\/\d+\//g;
            var depthFragment = depthRegex.exec(newUrl) || "";

            // find "depthYear/YEAR"
            var depthYearRegex = /depthYear\/.*?\//g;
            var depthYearFragment = depthYearRegex.exec(newUrl) || "";
            depthYearFragment = (depthYearFragment ? depthYearFragment[0] : "");

            // find "depthBin/SEASON"
            var depthBinRegex = new RegExp("depthBin\/(" + binRegexString + ")\/");
            var depthBinFragment = depthBinRegex.exec(newUrl);
            // since we are using parenthesis to match spring or summer, regex returns
            // an array of matches. We only need the first one, which is the entire url, so we use [0] below.
            depthBinFragment = (depthBinFragment ? depthBinFragment[0] : "");

            // find "biologyStartTime/DATE/biologyEndTime/DATE/
            var biologyDateRegex = /biologyStartTime\/.*\/biologyEndTime\/.*?\//g;
            var biologyFragment = biologyDateRegex.exec(newUrl) || "";


            // find "biologyBin/SEASON"
            var biologyBinRegex = /biologyBin\/.*?\//g;
            var biologyBinFragment = biologyBinRegex.exec(newUrl) || "";
            biologyBinFragment = (biologyBinFragment ? biologyBinFragment[0] : "");

            return locationFragment + timeFragment + parametersFragment + binFragment + depthFragment + depthYearFragment + depthBinFragment + biologyFragment + biologyBinFragment;

        };

        // fire the route for a change to the selected time range in the detail view - times tab
        this.eventBus.bind("change:times", (function(self) {
            return function(start, end) {
                var timeFragment = "startTime/" + encodeURIComponent(start) + "/endTime/" + encodeURIComponent(end) + "/";
                var newUrl = Backbone.history.fragment;

                if(Backbone.history.fragment.indexOf("startTime") >= 0) {
                    newUrl = newUrl.replace(/startTime\/[^\/]*\/endTime\/[^\/]*\//, timeFragment);
                } else {
                    newUrl = newUrl + timeFragment;
                }
                self.router.navigate(self.urlReOrder(newUrl), {trigger : true, replace : true});
            };
        })(this));

        // fire the route for a change to the selected time range in the detail view - biology tab
        this.eventBus.bind("change:biologyDates", (function(self) {
            return function(start, end) {
                var timeFragment = "biologyStartTime/" + encodeURIComponent(start) + "/biologyEndTime/" + encodeURIComponent(end) + "/";
                var newUrl = Backbone.history.fragment;

                if(Backbone.history.fragment.indexOf("biologyStartTime") >= 0) {
                    newUrl = newUrl.replace(/biologyStartTime\/[^\/]*\/biologyEndTime\/[^\/]*\//, timeFragment);
                } else {
                    newUrl = newUrl + timeFragment;
                }
                self.router.navigate(self.urlReOrder(newUrl), {trigger : true, replace : true});
            };
        })(this));

        // fire the route for a change to the selected parameters in the detail view
        this.eventBus.bind("change:parameters", (function(self) {
            return function(parameters) {
                self.parameters = parameters;
                var parametersFragment =  parameters.length === 0 ? "" : "parameters/" + parameters.join() + "/";
                var newUrl = Backbone.history.fragment;

                if(Backbone.history.fragment.indexOf("parameters") >= 0) {
                    newUrl = newUrl.replace(/parameters\/.*?\//g, parametersFragment);
                } else {
                    newUrl = Backbone.history.fragment + parametersFragment;
                }
                self.router.navigate(self.urlReOrder(newUrl), {trigger : true, replace : true});
            };
        })(this));

        // fires the route for a change to the bin value
        this.eventBus.bind("change:bin", (function(self) {
            return function(bin, view) {
                if (view == "depth") {
                    self.eventBus.trigger("change:depthBin", bin, view);
                    return;
                } else if (view == "biology") {
                    self.eventBus.trigger("change:biologyBin", bin, view);
                    return;
                }
                var binFragment = "";
                var newUrl = Backbone.history.fragment;
                var binIDs = _.map(getBinningDesignators(self.binningTime).items, function(bin) { return bin.id} );
                var binRegex = binIDs.join("|");
                var binRegexObject = new RegExp("bin\/("+ binRegex +")?\/");

                if (bin != "") {
                    binFragment = "bin/" + bin + "/";
                }

                if (Backbone.history.fragment.indexOf("bin") >= 0) {
                    newUrl = newUrl.replace(binRegexObject, binFragment);
                } else {
                    newUrl = newUrl + binFragment;
                }

                self.router.navigate(self.urlReOrder(newUrl), { trigger : true, replace : true });

            };
        })(this));

        // fires the route for a change to the depth value in the limnology chart.
        this.eventBus.bind("change:depth", (function(self) {
            return function(depth) {
                var depthFragment = "depth/" + depth + "/";
                var newUrl = Backbone.history.fragment;

                if(Backbone.history.fragment.indexOf("depth") >= 0) {
                    newUrl = newUrl.replace(/depth\/[^\/]*\/?/, depthFragment);
                } else {
                    newUrl = newUrl + depthFragment;
                }
                self.router.navigate(self.urlReOrder(newUrl), {trigger : true, replace : true});
            };
        })(this));

        // fires the route for a change to the depth value in the limnology chart.
        this.eventBus.bind("change:year", (function(self) {
            return function(depthYear) {
                var depthYearFragment = "depthYear/" + depthYear + "/";
                var newUrl = Backbone.history.fragment;

                if(Backbone.history.fragment.indexOf("depthYear") >= 0) {
                    newUrl = newUrl.replace(/depthYear\/[^\/]*\/?/, depthYearFragment);
                } else {
                    newUrl = newUrl + depthYearFragment;
                }
                self.router.navigate(self.urlReOrder(newUrl), {trigger : true, replace : true});
            };
        })(this));

        // fires the route for a change to the bin value
        this.eventBus.bind("change:depthBin", (function(self) {
            return function(depthBin) {
                var depthBinFragment = "";
                var newUrl = Backbone.history.fragment;
                var binIDs = _.map(getBinningDesignators(self.binningTime).items, function(bin) { return bin.id} );
                var binRegex = binIDs.join("|");

                if (depthBin != "") {
                    depthBinFragment = "depthBin/" + depthBin + "/";
                }

                if (Backbone.history.fragment.indexOf("depthBin") >= 0) {
                    newUrl = newUrl.replace(new RegExp("depthBin\\/("+ binRegex +")?\\/"), depthBinFragment);
                } else {
                    newUrl = newUrl + depthBinFragment;
                }

                self.router.navigate(self.urlReOrder(newUrl), { trigger : true, replace : true });

            };
        })(this));

        // fires the route for a change to the bin value
        this.eventBus.bind("change:biologyBin", (function(self) {
            return function(biologyBin) {
                var biologyBinFragment = "";
                var newUrl = Backbone.history.fragment;
                var binIDs = _.map(getBinningDesignators(self.binningTime).items, function(bin) { return bin.id} );
                var binRegex = binIDs.join("|");

                if (biologyBin != "") {
                    biologyBinFragment = "biologyBin/" + biologyBin + "/";
                }

                if (Backbone.history.fragment.indexOf("biologyBin") >= 0) {
                    newUrl = newUrl.replace(new RegExp("biologyBin\\/("+ binRegex +")?\\/"), biologyBinFragment);
                } else {
                    newUrl = newUrl + biologyBinFragment;
                }

                self.router.navigate(self.urlReOrder(newUrl), { trigger : true, replace : true });

            };
        })(this));



    },

    initiate : function(location, startTime, endTime, parameters, bin, depth, depthYear, depthBin,  biologyStartTime, biologyEndTime, biologyBin) {
        this.locationID = location;
        this.sensor = app.collection.sensors.findWhere({name: this.locationID});

        app.collection.sensors.getStationByName(location, (function(self) {
            return function(station) {
                self.eventBus.trigger("locationUpdated", location, bin);
                self.miniMap.forEach(function(map) {
                    map.setGeoJsonFeature([ station ]);
                    map.render();
                });

                self.go(station, startTime, endTime, parameters, bin, depth, depthYear, depthBin, biologyStartTime, biologyEndTime, biologyBin);
            };
        })(this));
    },

    go : function(station, startTime, endTime, parameters, bin, depth, depthYear, depthBin,  biologyStartTime, biologyEndTime, biologyBin) {
        var self = this;
        self.sensorID = station.id;
        // If start / end time are null, we need to initialize the range slider with time data
        if(startTime == null && endTime == null || !sessionStorage.getItem(station.id)) {

            // Generate the dates for the sensor min / max
            var sensor = app.collection.sensors.findWhere({id: station.id});
            var dates = getDates(new Date(sensor.get('min_start_time')), new Date(sensor.get('max_end_time')));

            // save dates in session so we don't regenerate for the same sensor
            sessionStorage.setItem(station.id, JSON.stringify(dates));

            // here is how I understand the following:
            // the setTimes function sets the times for the controlView, which calls renderSlider,
            // which calls changeTimes, which triggers a "change:times" event which re-renders this view
            // Update 03/2017: If it is the first time you call this when coming from the explore page you want to update
            // the url so the points are filtered by the dates. Otherwise, if there are dates specified, you don't want to
            // update the already set start and end date in the url.
            var updateTimesInUrl = true;
            if(startTime != null && endTime != null) {
                updateTimesInUrl = false;
            }
            self.controlsView.setTimes(dates, updateTimesInUrl);


            // the return in the following case is heavily dependent on the setTimes function called above.
            // The first time running the DetailView go function, you won't have a startTime or endTime
            // so this returns and waits for setTimes to finish. See notes just above...
            if(updateTimesInUrl) {
                return;
            }

        }

        if(biologyStartTime == null && biologyEndTime == null && self.controlsBiologyView != null) {
            biologyStartTime = startTime;
            biologyEndTime = endTime;

        }

        var time = self.binningTime;
        if (_.isUndefined(self.binningTime)) {
            time = 'none';
        }

        var info_icon_appearance = self.getInfoIconSettings();

        // If self.binningTime is 'smart', then determine aggregation by temporal range
        if (self.binningTime == 'smart') {
            time = getAggregationBin(new Date(startTime), new Date(endTime));
            $("#bin-duration").text("Averaged by " + time).append('' +
                ' <a href="#data-aggregate-modal" role="button" ' +
                'data-toggle="modal" class="info-icon-spacing">' +
                info_icon_appearance  +
                '</a>');
        }

        // Use default depth if no depth value provided
        var sensorDepth = depth;
        if(sensorDepth === undefined || sensorDepth === null) {
            sensorDepth = app.config.binning.depth;
        }

        // Fetch and Display sensor data for a particular time bin
        this.geostreamService.getBinnedData(time, sensorDepth, station.id, true, function(data) {

            self.startTime = startTime = new Date(startTime);
            self.endTime = endTime = new Date(endTime);
            biologyStartTime = new Date(biologyStartTime);
            biologyEndTime = new Date(biologyEndTime);

            // If a list of selected parameters are provided, then only display selected
            parameters = parameters == null ? [] : parameters.split(",");

            // Retrieve generated dates for this station
            var dates = JSON.parse(sessionStorage.getItem(station.id));
            self.controlsView.setTimes(dates, false);
            if(startTime != null && endTime != null) {
                self.controlsView.updateTimes(startTime, endTime);
            }

            // Used to filter data outside of the range of start / end time
            var filterDatesByRange = function(x, startTime, endTime) {
                var date = new Date(x.date);
                var datapoint = {year: date.getUTCFullYear(), month: date.getUTCMonth(), day: date.getUTCDate(), hour: date.getUTCHours(), minute: date.getUTCMinutes() };

                if (startTime.getUTCFullYear() > datapoint.year) {
                    return false;
                }
                if (startTime.getUTCFullYear() == datapoint.year) {
                    if (!_.isUndefined(datapoint.month)) {
                        if (startTime.getUTCMonth() > datapoint.month) {
                            return false;
                        }
                        if (startTime.getUTCMonth() == datapoint.month) {
                            if (!_.isUndefined(datapoint.day)) {
                                if (startTime.getUTCDate() > datapoint.day) {
                                    return false
                                }
                                // I'm leaving this here in case we decide to support filter down to hour/minutes
                                // Since the slider does not filter to a smaller unit than days, I'm removing this unnecessary check
                                // The slider would need be smarter to change it's increment when the date range is small enough
                                // in order to support showing hourly / minute data
                                // if (startTime.getUTCDate() == datapoint.day) {
                                //   if (!_.isUndefined(datapoint.hour)) {
                                //     if (startTime.getUTCHours() > datapoint.hour) {
                                //       return false;
                                //     }
                                //     if (startTime.getUTCHours() == datapoint.hour) {
                                //       if (!_.isUndefined(datapoint.minute) && startTime.getUTCMinutes() > datapoint.minute) {
                                //         return false;
                                //       }
                                //     }
                                //   }
                                // }
                            }
                        }
                    }
                }
                if (endTime.getUTCFullYear() < datapoint.year) {
                    return false;
                }
                if (endTime.getUTCFullYear() == datapoint.year) {
                    if (!_.isUndefined(datapoint.month)) {
                        if (endTime.getUTCMonth() < datapoint.month) {
                            return false;
                        }
                        if (endTime.getUTCMonth() == datapoint.month) {
                            if (!_.isUndefined(datapoint.day)) {
                                if (endTime.getUTCDate() < datapoint.day) {
                                    return false
                                }
                                // I'm leaving this here in case we decide to support filter down to hour/minutes
                                // Since the slider does not filter to a smaller unit than days, I'm removing this unnecessary check
                                // The slider would need be smarter to change it's increment when the date range is small enough
                                // in order to support showing hourly / minute data
                                // if (endTime.getUTCDate() == datapoint.day) {
                                //   if (!_.isUndefined(datapoint.hour)) {
                                //     if (endTime.getUTCHours() < datapoint.hour) {
                                //       return false;
                                //     }
                                //     if (endTime.getUTCHours() == datapoint.hour) {
                                //       if (!_.isUndefined(datapoint.minute) && endTime.getUTCMinutes() < datapoint.minute) {
                                //         return false;
                                //       }
                                //     }
                                //   }
                                // }
                            }
                        }
                    }
                }
                return true;
            };

            // time series parameters found in binned data
            var time_series_params = [];

            // depth parameters found in binned data
            var depth_profile_params = [];

            // biology parameters found in binned data
            var biology_params = [];

            // Dissolved oxygen parameters found in binned data
            var stacked_line_params = [];

            // We need both keys and ids to support parameters that were not found, but added at runtime
            // The returned data from the API matches the key, but the app.collection.parameters are stored by id and needed later
            var timeSeriesMap = {};
            var depthKeyMap = {};
            var biologyMap = {};
            var stackedLineMap = {};

            // properties in the binned data
            var properties = data[0].properties;

            // Break out depth and time data and store display name for graphs
            for(var key in properties) {
                var parr = app.collection.parameters.filter(function(x) {
                    return (x.id == key) || x.get("alternates").indexOf(key) >= 0;
                });

                if (parr[0] && parr[0]['id'] ) {
                    if(parr[0].id.indexOf('ignored') == -1 && parr[0].get('detailView') === 'time') {
                        timeSeriesMap[parr[0].id] = key;
                        time_series_params.push(parr[0].id);
                    } else if (parr[0].id.indexOf('ignored') == -1 && parr[0].get('detailView') === 'depth') {
                        depthKeyMap[parr[0].id] = key;
                        depth_profile_params.push(parr[0].id);
                    } else if (parr[0].id.indexOf('ignored') == -1 && parr[0].get('detailView') === 'biology') {
                        biologyMap[parr[0].id] = key;
                        biology_params.push(parr[0].id);
                    }  else if (parr[0].id.indexOf('ignored') == -1 && parr[0].get('detailView') === 'stacked_line') {
                        stackedLineMap[parr[0].id] = key;
	                    stacked_line_params.push(parr[0].id);
                    }

                }
            }

            if (time_series_params.length > 0) {
                $("#time-tab").toggle(true);

                time_series_params.sort();

                // For time series data, if applicable, use bin selected in UI or default value if none specified
                // This is used when selecting a subset of the semi bin (e.g spring or summer data)
                bin = bin || $('#bin-select-time').val();
                if (bin === undefined) {
                    if ( self.binningTime && !(self.binningTime == 'none' || self.binningTime == 'smart')) {
                        if(!_.isEmpty(getBinningDesignators(self.binningTime).items)) {
                            bin = getBinningDesignators(self.binningTime).items[0]['id'];
                        }
                    }
                }

                // Use a unique color for each parameter
                var time_series_params_colors = Object.keys(properties).length > 10 ? d3.scale.category20().domain(time_series_params) : d3.scale.category10().domain(time_series_params);

                // keep a list of data sources, pointing back to clowder dataset
                var sources = [];

                var alldata = [], summerEPdata = [];
                var foundEP = false;
                // iterate over each of the unique parameters
                for (var i = 0; i < time_series_params.length; i++) {
                    // 'parameters' is what gets passed in via the URL (ie: the Backbone router gives us these)
                    // if there are parameters in the URL, but the param we are on isn't one of them, skip it
                    if (!_.isEmpty(parameters) && (_.indexOf(parameters, time_series_params[i]) < 0)) {
                        continue;
                    }

                    var propKey = timeSeriesMap[time_series_params[i]];

                    var valuesAll = [], valuesEPsummer = [];

                    // Gather date ranges to set time series X range
                    var dateRange = [];

                    // Since server side binning separates by depth code, we use this to see if a bin has been processed
                    var currentBin = null;
                    _.each(properties[propKey], function(entry) {

                        // Skip entry if we have already processed it
                        if(currentBin != entry.label) {
                            currentBin = entry.label;

                            // Check if data is in date range and matches the selected bin (e.g. summer/spring), if applicable
                            if((_.isUndefined(bin) || (entry.label.indexOf(bin) != -1)) && filterDatesByRange(entry, startTime, endTime)) {
                                // Server side binning separates by depth code so we must locate all values for a bin
                                var obj = _.select(properties[propKey], function(item) {
                                    return item.label === entry.label;
                                });

                                var countAll = 0, countEPdata = 0;
                                var sumAll = 0, sumEPdata = 0;
                                var rawAll = [], rawEP = [];
                                var entryDate;

                                _.each(obj, function(x) {
                                    var depthCode = x.depth_code;

                                    // Added this because client side filtering mapped these depth codes to EP
                                    if(["SRF", "LEP", "TRM"].indexOf(depthCode) > -1) {
                                        depthCode = "EP";
                                    }

                                    if(x.label.indexOf("summer") != -1 && !_.isUndefined(depthCode) && (depthCode == "EP")) {
                                        foundEP = true;
                                    }

                                    if(x.label.indexOf("summer") == -1 || _.isUndefined(depthCode) || (depthCode == "EP") || (depthCode == "NA")) {
                                        countEPdata += x.count;
                                        sumEPdata += x.average * x.count;
                                        rawEP.push(x.average * x.count);
                                    }

                                    countAll += x.count;
                                    sumAll += x.average * x.count;
                                    rawAll.push(x.average);
                                    entryDate = x.date;
                                    _.each(x.sources, function(source) {
                                        sources.push(source);
                                    });
                                });

                                valuesAll.push({ x: entryDate, y: sumAll / countAll, label: currentBin, raw: _.flatten(rawAll) });
                                dateRange.push(entryDate);

                                if(countEPdata != 0) {
                                    valuesEPsummer.push({ x: entryDate, y: sumEPdata / countEPdata, label: currentBin, raw: _.flatten(rawEP) })
                                }
                            }
                        }
                    });

                    var yrange = d3.extent(valuesAll.concat(valuesEPsummer), function(d) { return d.y; });
                    if (app.config.graphs.timeSeries.showUnits.showZero != false && yrange[0] > 0) {
                        yrange[0] = 0;
                    }

                    var xrange = [new Date(dateRange[0]), new Date(dateRange[dateRange.length-1])];
                    var xExtent = app.config.views.detail.timeSeriesFullExtent ? self.controlsView.getTimes() : xrange;

                    if(valuesAll.length > 0) {
                        alldata.push({
                            key: i.toString(),
                            values: [ {
                                color: time_series_params_colors(time_series_params[i]),
                                key: time_series_params[i],
                                xextent: xExtent,
                                xrange: xrange,
                                yrange: yrange,
                                values: valuesAll,
                                sources: _.uniq(_.flatten(sources)),
                                unit: (app.collection.parameters.findWhere({id: time_series_params[i]})).get("unit")
                            }]
                        });
                    }

                    if(!_.isEmpty(valuesEPsummer)) {
                        summerEPdata.push({
                            key: i.toString(),
                            values: [ {
                                color: time_series_params_colors(time_series_params[i]),
                                key: time_series_params[i],
	                            xextent: xExtent,
                                xrange: xrange,
                                yrange: yrange,
                                values: valuesEPsummer,
                                unit: (app.collection.parameters.findWhere({id: time_series_params[i]})).get("unit")
                            }]
                        })
                    }
                }
                if((bin == "spring") || !foundEP) {
                    self.aggregateTimeSeriesView.setData(alldata, undefined);
                } else {
                    self.aggregateTimeSeriesView.setData(alldata, summerEPdata);
                }
                self.controlsView.setParameters(time_series_params, _.isEmpty(parameters) ? time_series_params : parameters, time_series_params_colors);
            } else {
                $("#time-tab").toggle(false);
            }

            // Setup depth profile tab
            var dates = JSON.parse(sessionStorage.getItem(station.id));
            self.controlsDepthView.setTimes(dates, false);

            if (depth_profile_params.length > 0) {
                $("#depth-tab").toggle(true);

                // If depth year is set in the UI, use that as the depth year filter
                var depthDateFilter = null;
                if(depthYear) {
                    depthDateFilter = new Date(depthYear, 0, 1);
                }

                depth_profile_params.sort();

                // Used to filter depth data outside of the selected year
                var filterByYear = function(x, depthDateYear) {
                    var date = new Date(x.date);
                    var datapoint = {year: date.getUTCFullYear() };

                    if(depthDateYear.getUTCFullYear() == datapoint.year) {
                        return true;
                    } else {
                        return false;
                    }
                };

                // For depth data, if applicable, use bin selected in UI or default value if none specified
                // This is used when selecting a subset of the semi bin (e.g spring or summer data)
                depthBin = depthBin || $('#bin-select-depth').val();
                if (depthBin === undefined) {
                    if (self.binningTime && !(self.binningTime == 'none' || self.binningTime == 'smart')) {
                        if(!_.isEmpty(getBinningDesignators(self.binningTime).items)) {
                            depthBin = getBinningDesignators(self.binningTime).items[0]['id'];
                        }
                    }
                }

                // Use d3 to create colors for each of the unique parameters
                var depth_profile_params_colors = depth_profile_params.length > 10 ? d3.scale.category20().domain(depth_profile_params) : d3.scale.category10().domain(depth_profile_params);

                // List of available years for depth data
                var depthDates = [];

                // We're creating a display_data object for this date
                var display_data = [{
                    key: new Date(startTime),
                    values: []
                }];

                for(var i = 0; i < depth_profile_params.length; i++) {
                    // 'parameters' is what gets passed in via the URL (ie: the Backbone router gives us these)
                    // if there are parameters in the URL, but the param we are on isn't one of them, skip it
                    if (!_.isEmpty(parameters) && (_.indexOf(parameters, depth_profile_params[i]) < 0)) {
                        continue;
                    }

                    // keep a list of data sources, pointing back to clowder dataset
                    var sources = [];

                    var property = depth_profile_params[i];
                    var propKey = depthKeyMap[property];

                    var valuesAll = [];
                    _.each(properties[propKey], function(x) {
                        // Check if data matches the selected bin (e.g. summer/spring), if applicable
                        if(_.isUndefined(depthBin) || x.label.indexOf(depthBin) > -1) {

                            // If no depth year is set in the UI, then use the first date as the filter
                            if(depthDateFilter == null) {
                                depthDateFilter = new Date(x.date);
                            }

                            // Create a list of unique dates to build the depth date year selection widget
                            if(_.isEmpty(_.where(depthDates, x.date))) {
                                depthDates.push(x.date);
                            }

                            // Only add values for the selected depth year
                            if(filterByYear(x, depthDateFilter)) {
                                for(var j = 0; j < x.sources.length; j++) {
                                    if(_.isEmpty(_.where(sources, x.sources[j]))) {
                                        sources.push(x.sources[j]);
                                    }
                                }
                                valuesAll.push({
                                    x: x.average,
                                    x0: 0,
                                    x1: 0,
                                    y: x.depth
                                });
                            }
                        }
                    });

                    // Only add parameter if it has values for the depth year
                    if(valuesAll.length > 0) {
                        display_data[0].values.push({
                            color: depth_profile_params_colors(property),
                            key: property,
                            sources: sources,
                            values: valuesAll,
                            unit: (app.collection.parameters.findWhere({id: property})).get("unit")
                        });
                    }
                }

                depthDates.sort();

                self.controlsDepthView.setYears(depthDates);
                self.limnologyView.setData(display_data);
                self.controlsDepthView.setParameters(depth_profile_params, _.isEmpty(parameters) ? depth_profile_params : parameters, depth_profile_params_colors);
            } else {
                $("#depth-tab").toggle(false);
            }

            if (biology_params.length > 0) {
                $("#biology-tab").toggle(true);
                $('#biology-graphs').html("");
                var biology_params_colors = [];

                biologyBin = biologyBin || $('#bin-select-biology').val();

                if(biologyBin === undefined) {
                    if( self.binningTime && !(self.binningTime == 'none' || self.binningTime == 'smart')) {
                        if(!_.isEmpty(getBinningDesignators(self.binningTime).items)) {
                            biologyBin = getBinningDesignators(self.binningTime).items[0]['id'];
                        }
                    }
                }
                self.stackedBarViews = [];
                // Iterate over all of the unique parameters
                for(var i =0; i < biology_params.length; i++) {
                    if (!_.isEmpty(parameters) && _.indexOf(parameters, biology_params[i]) < 0) {
                        continue;
                    }
                    var stackedGraphData = [];
                    var propKey = biologyMap[biology_params[i]];
                    var biologyDates = [];

                    _.each(properties[propKey], function (entry) {
                        biologyDates.push(new Date(entry.date));
                        //Check if data is in date range and is part of the season selected on the bin
                        if((_.isUndefined(biologyBin) || (entry.label.indexOf(biologyBin) != -1)) && filterDatesByRange(entry, biologyStartTime, biologyEndTime)) {
                            for (var k = 0; k < entry.array.length; k++) {
                                var biology_row = JSON.parse(entry.array[k]);
                                biology_row["date"] = entry.date;
                                stackedGraphData.push(biology_row);
                            }
                        }

                    });

                    //If the start and end date are the ones in the 'times' we want to update them to be a 'smaller' range
                    // of dates just to cover the range of the biology dates.
                    var biologyMinDate = new Date(Math.min.apply(null, biologyDates));
                    var biologyMaxDate =  new Date(Math.max.apply(null, biologyDates));

                    //This should only be called the first time. That is why the comparison between the biology and the biology start times.
                    if(biologyStartTime.getTime() == startTime.getTime() && biologyMinDate.getTime() > biologyStartTime.getTime()) {
                        biologyStartTime = biologyMinDate;
                        if(biologyEndTime.getTime() == endTime.getTime() && biologyMaxDate.getTime() < biologyEndTime.getTime()) {
                            biologyEndTime = biologyMaxDate;
                        }
                        var dates = getDates(biologyStartTime, biologyEndTime);
                        self.controlsBiologyView.setTimes(dates, false);
                    } else if (biologyStartTime.getTime() >= biologyMinDate.getTime() || biologyEndTime.getTime() <= biologyMaxDate.getTime()) {
                        //This should be called when you refresh the details page.
                        var dates = getDates(biologyMinDate, biologyMaxDate);
                        self.controlsBiologyView.setTimes(dates, false);
                        self.controlsBiologyView.updateTimes(biologyStartTime, biologyEndTime);
                    }

                    var parameterProps = app.collection.parameters.findWhere({id: propKey});
                    var newGraphId = "stacked-biology-graph" + propKey;
                    $("#"+newGraphId).remove();
                    $('#biology-graphs').append('<svg id="' + newGraphId + '" width="960" height="500"></svg>');
                    var biologyZ = d3.scale.category10();
                    var scaleNames = [];

                    if(parameterProps.get("scaleColors") != undefined && parameterProps.get("scaleNames") != undefined) {
                        scaleNames = parameterProps.get("scaleNames");
                        biologyZ = d3.scale.ordinal().domain(scaleNames).range(parameterProps.get("scaleColors"));

                    }
                    var areaView = new app.view.StackedBarView({
                        id: newGraphId,
                        data: stackedGraphData,
                        scale: d3.time.years,
                        title: parameterProps.get("title") + " (" + parameterProps.get("unit") + ")",
                        scaleNames: scaleNames,
                        unit: parameterProps.get("unit"),
                        z: biologyZ
                    });
                    self.stackedBarViews.push(areaView);

                    biology_params_colors.push("#000000");
                }
                var biology_colors = d3.scale.ordinal().range(biology_params_colors);
                self.controlsBiologyView.setParameters(biology_params, _.isEmpty(parameters) ? biology_params : parameters, biology_colors);
            } else {
                $("#biology-tab").toggle(false);
            }

            if (stacked_line_params.length > 0) {
                $("#stacked-line-tab").toggle(true);
                $("#stacked-line-graphs").html("");
                self.stackedLineViews = [];

                for(var k =0; k < stacked_line_params.length; k++) {
	                if (!_.isEmpty(parameters) && _.indexOf(parameters, stacked_line_params[k]) < 0) {
		                continue;
	                }
	                var propKey = stacked_line_params[k];
	                var stackedLineGraphData = []; // properties[propKey];

	                var parameterProps  = app.collection.parameters.findWhere({id: propKey});
	                var newGraphId = "stacked-line-graph-"+ propKey;
                    $("#"+newGraphId).remove();
	                $("#stacked-line-graphs").append('<svg id="' + newGraphId + '" width="960" height="500"></svg>');

	                _.each(properties[propKey], function(entry) {
                        var stackedLineRow = {};
                        stackedLineRow["date"] = entry.date.substring(0,10);
                        stackedLineRow["values"] = JSON.parse(entry.array);
                        stackedLineGraphData.push(stackedLineRow);

                    });
                    var stackedLineView = new app.view.StackedLineView({
                        id: newGraphId,
                        data: stackedLineGraphData,
                        scale: d3.time.years,
                        title: parameterProps.get("title") + " (" + parameterProps.get("unit") + ")",
                        unit: parameterProps.get("unit"),
                        color: d3.scale.category10()
                    });
                    self.stackedLineViews.push(stackedLineView);
                }

	            stacked_line_params.sort();
            } else {
                $("#stacked-line-tab").toggle(false);
            }

            // render everything
            if (self.sensor) self.updateDownloadLink();
            self.render();

        });
    },

    render : function() {
        var sourceColor;
        app.event.eventBus.trigger("detail:spinner:start");
        if (!this.sensor) {
            this.sensor = app.collection.sensors.findWhere({name: this.locationID});
        }
        this.sourceColor = app.collection.sensors.findWhere({name: this.locationID}).attributes.color;
        this.source = app.collection.sources.findWhere({id: this.sensor.get('type').id});

        $("#detail-view-pin").html(this.mapPin);
        $("#detail-view-pin > svg > g > ellipse").attr("style", "fill: " + this.sourceColor);
        $("#detail-view-source").html(this.source.attributes.prettyName);
        $("#detail-view-source").attr("title", this.source.attributes.prettyName);

        var header_text = (this.sensor && this.sensor.get('popupContent')) || this.locationID;
        if (this.sensor) {
            if (this.sensor.get('online_status') && this.sensor.get('online_status')=="online"){
                header_text +=  " [" + this.sensor.get('online_status').charAt(0).toUpperCase() + this.sensor.get('online_status').slice(1) +  "]";
                $("#detail-view-pin > svg > g > path").attr("style", "stroke: green; stroke-width: 2");
            }
            $("#detail-view-header").html(header_text);
            $("#detail-view-header").attr("title", header_text);
        }

        $("#time-series.tab-pane").each((function(self) {
            return function() {
                self.aggregateTimeSeriesView.render();
            };
        })(this));
        $("#depth-profile.tab-pane").each((function(self) {
            return function() {
                self.limnologyView.render();
                self.timeSeriesView.render();

                self.stackedBarViews.forEach(function(view) {
                    view.render();
                });
            };
        })(this));
        $('#stacked-line-profile.tab-pane').each((function(self){
            return function() {
	            self.stackedLineViews.forEach(function(view) {
	                view.render();
                });
            };
        })(this));
        // using true here makes the map pan into position each time. With false, it stays in place
        this.miniMap.forEach(function(mapElement){
            mapElement.map.invalidateSize(false);
        });
        this.eventBus.trigger("renderBinSelect");
        this.setTabTitles();
        this.$el.append(this.sourceModals);

        var info_icon_appearance = this.getInfoIconSettings();

        if (this.source) {
            if (this.source.get('calculation') && this.source.get('calculation')['time-series']) {
                var timeTitle = $("#time-series-title");
                timeTitle.find('a').remove();
                timeTitle.find('i').remove();

                timeTitle.append('' +
                    '<a href="#' + this.source.id + '-time-series-modal" role="button" ' +
                    'data-toggle="modal" class="info-icon-spacing">' +
                    info_icon_appearance +
                    '</a>');
            }
            if (this.source.get('calculation') && this.source.get('calculation')['depth-profile']) {
                var depthTitle = $("#depth-profile-title");
                depthTitle.find('a').remove();
                depthTitle.find('i').remove();
                depthTitle.append('<a href="#' + this.source.id + '-depth-profile-modal" role="button" ' +
                    'data-toggle="modal"><i class="icon-info-sign bw-info"></i></a>');
            }

            $("#detail-source-icon").html(
                '<a href="#about-' + this.source.id + '-modal" role="button" data-toggle="modal">' +
                    info_icon_appearance  +
                '</a>'
            );
	        $("#detail-biology-source-icon").html(
		        '<a href="#about-' + this.source.id + '-modal" role="button" data-toggle="modal">' +
		        info_icon_appearance  +
		        '</a>'
	        );
        }
        app.event.eventBus.trigger("detail:spinner:stop");

    },

    showDistribution : function() {
        this.$("#showDistribution").button("toggle");
        this.aggregateTimeSeriesView.render();
        this.timeSeriesView.render();
    },

    goBack : function(event) {
        if (this.sensor && this.sensor.id) {
            this.sensorID = this.sensor.id;
        }
        sessionStorage.removeItem(this.sensorID);
        this.router.navigate("#explore", {
            trigger : true,
            replace : false
        });
    },

    setTabTitles: function() {
        var source = app.collection.sources.findWhere({id: this.sensor.attributes.type.id});
        if (source && source.get('tabs')) {
            if (source.get('tabs').timeseries) {
                $('#times-tab').find('a').text(source.get('tabs').timeseries);
            }
            if (source.get('tabs').depthprofile) {
                $('#depth-tab').find('a').text(source.get('tabs').depthprofile);
            }
            if (source.get('tabs').biology) {
                $('#biology-tab').find('a').text(source.get('tabs').biology);
            }
            if (source.get('tabs').stackedLine) {
                $('#staccke-line-tab').find('a').text(source.get('tabs').stackedLine);
            }
        }
    },

    updateDownloadLink : function() {
        var format = "YYYY-MM-DD HH:mm:ss";
        var params = {};

        params['sensor_id'] = [
            this.sensor.get('id')
        ].toString();

        if (this.startTime) params["since"] = moment(this.startTime).format(format);
        if (this.endTime) params["until"] = moment(this.endTime).format(format);

        if (this.parameters) params["attributes"] = this.parameters;

        // create the url
        var url = app.datapoints.url + '?' + $.param(params, true);

        // update the DOM
        var downloadAsJsonButton = $("#detail-download-json");
        downloadAsJsonButton.attr('href', url + "&format=json");

        var downloadAsCsvButton = $("#detail-download-csv");
        downloadAsCsvButton.attr('href', url + "&format=csv");

    }
});

// Determine the bin based on the temporal range of the start / end date
function getAggregationBin(startTime, endTime) {
    var time = endTime.getUTCFullYear() - startTime.getUTCFullYear();
    var timeBin = 'day';

    // This should probably be configurable by project to select the binning
    // that makes the most sense for the expected temporal range of the data
    if(time > 100) {
        timeBin = 'decade';
    } else if(time > 50) {
        timeBin = 'lustrum';
    } else if(time > 25) {
        timeBin = 'year';
    } else if(time > 10) {
        timeBin = 'season';
    } else if(time > 1) {
        timeBin = 'month';
    } else {
        // Convert to days
        time = (endTime - startTime) / 86400000;
        if(time > 14) {
            timeBin = 'day';
        } else if(time > 3) {
            timeBin = 'hour';
        } else {
            timeBin = 'minute';
        }
    }
    return timeBin;
}

// Generate dates between start/end times with 1 day time steps
function getDates(startTime, endTime) {
    var dates = [];
    var currentDate = startTime;
    while(currentDate <= endTime) {
        dates.push(new Date(currentDate).toUTCString());
        currentDate = new Date(currentDate);
        currentDate.setDate(currentDate.getDate() + 1);
    }
    dates.push(new Date(endTime).toUTCString());

    return dates;
}

app.view.SelectItemView = Backbone.View.extend({
    tagName: "option",
    attributes: function() {
        return {
            value: this.model.id || ''
        }
    },

    initialize : function(options) {
        this.model = options.model;
        this.selected = options.selected;
        var content = this.model.get('name') || this.model.get('title') || '';
        this.template = _.template(content.replace(/<(?:.|\n)*?>/gm, ''));
        this.render();
    },

    render : function() {
        this.$el.empty();
        this.$el.append(this.template());
        if (this.selected) this.$el.attr('selected', 'selected');
        return this;
    }
});

app.view.SelectCollectionView = Backbone.View.extend({
    tagName : "select",
    className : "dropdown",

    events: {
        'change': 'changeSelect'
    },

    initialize: function(options) {
        this.trigger = options.trigger;
        if (options.placeholder) this.placeholder = new Backbone.Model({name: options.placeholder});

        _.bindAll(this, "render", "changeSelect", "append", "setSelected");
        this.collection.on('all', this.render);

        this.render();
    },

    render : function() {
        var self = this;
        this.$el.empty();

        // if there are models in this collection, we will populate them as select options
        if (this.collection.length) {
            // self.placeholder allows you to insert a default field like "None" or "Select a Value"
            if (self.placeholder) {
              self.$el.append(new app.view.SelectItemView({ model: self.placeholder, selected: true }).render().el);
            }

            // add each of the options to the select element and mark the selected item
            this.collection.each(function(item) {
                var selected = false;
                if (self.selectedItem && (item.id == self.selectedItem)) {
                  selected = true;
                }
                self.$el.append(new app.view.SelectItemView({ model: item, selected: selected }).render().el);
            });
        }
        return this;
    },

    append: function(item) {
        this.$el.append(new app.view.SelectItemView({model: item}).render().el);
    },

    changeSelect: function(event) {
        var self = this;
        self.setSelected(event.target.value);
        event.cancleBubble = true;
        if (this.trigger) {
            var parameterElement = $('.parameter-trend-active');
            if (parameterElement) {
              var parameterID = parameterElement.data('id');
              if (parameterID) {
                app.event.eventBus.trigger(this.trigger + ":" + parameterID, event.target.value);
              }
            }

        }
    },

    setSelected: function(value) {
        this.selectedItem = value;
    }


});

app.view.ExploreSourcesView = Backbone.View.extend({

    initialize : function(options) {

        var self = this;
        this.config = options.config;
        _.bindAll(this, "render");

        this.template = _.template($(this.config.templateID).html());

        this.modals = _.template($(this.config.modalsID).html())();

        this.sources = app.collection.sources.getSourcesWithSensors().toJSON();

        this.listenTo(app.collection.sources, 'change', this.render);

        // This event is fired when someone opens/closes an accordion
        // in the ExploreView.
        app.event.eventBus.on("change:sources", function(sources, updateGeoJsonFeature) {
            if ($('#sources').find('.data-nav-header-active').length) {
                if (!sources) {
                    sources = [];
                    $('input[name="source"]:checked').each(function() {
                        sources.push($(this).attr("value"));
                    });
                }

                var sensorModels = app.collection.sensors.filter(function(sensor) {
                    return _.contains(sources, sensor.get('type').id);
                });
                var sensors = _.map(sensorModels, function(sensor) {
                    return sensor.toJSON();
                });

                if(updateGeoJsonFeature) {
                    app.view.exploreMapView.setGeoJsonFeature(sensors, self.config.map);
                    app.view.exploreMapView.render(self.config);
                }

            }
        });

    },

    render : function() {

        var self = this;

        if (self.sources.length > 0) {
            var checkedSources = [];

            if ($('input[name="source"]').length == 0) {
                checkedSources = app.collection.sources.getSourcesWithSensors().map(
                    function(source) { return source.get('id'); }
                );
            } else {
                $('input[name="source"]:checked').each(function() {
                    checkedSources.push($(this).attr("value"));
                });
            }

            this.$el.empty();

            app.collection.sources.getAreas().forEach(function(source) {
                // if sources.json hasn't been filled out, the default will be an
                // empty template that we should not render
                if (source.id) {
                    if (_.contains(checkedSources, source.id)) source.checked = "checked";
                    self.$el.append(self.template({ model: source }));
                }
            });

        } else {
            // display placeholder text if no sources are found
            this.$el.html(app.config.sources.noneFound);
        }

        this.$el.append(this.modals);

        var $explorePills = this.$(this.config.pills.className);
        // enable bootstrap tooltips
        // $explorePills.tooltip(this.config.pills.tooltipOptions);

        // fires events to highlight or unhighlight map pins
        $explorePills.hover(function(self) {
            return function(event) {
                app.event.eventBus.trigger("marker:mouseover",
                    event.currentTarget.value.replace(/\W/g, "-"));
            };
        }(this), function(self) {
            return function(event) {
                app.event.eventBus.trigger("marker:mouseout",
                    event.currentTarget.value.replace(/\W/g, "-"));
            };
        }(this));

        $explorePills.on("click", (function(self) {
            return function(event) {
                app.event.eventBus.trigger("marker:click",
                    event.currentTarget.value.replace(/\W/g, "-"));
            };
        })(this));

        var $collapse = this.$(this.config.collapse.className);

        $collapse.on("hide", (function(self) {
            return function(event) {
                if(event.target !== this) return;
                $(this).parent().find(self.config.collapse.inputSelector).prop("checked", false);

                var sources = [];
                $(self.config.collapse.inputSelector + ':checked').each(function() {
                    sources.push($(this).attr("value"));
                });
                app.event.eventBus.trigger("change:sources", sources, true);
            };
        })(this));

        $collapse.on("show", (function(self) {
            return function(event) {
                if(event.target !== this) return;
                $(this).parent().find(self.config.collapse.inputSelector).prop("checked", true);

                var sources = [];
                $(self.config.collapse.inputSelector + ':checked').each(function() {
                    sources.push($(this).attr("value"));
                });
                app.event.eventBus.trigger("change:sources", sources, true);
            };
        })(this));

        self.$('.showHideExplore').on('click', this.toggleButton);

        return this;

    },

    toggleButton: function() {

        if(this.innerHTML.indexOf("more") >=0) {
            this.innerHTML = "Show less";
        } else {
            this.innerHTML = "Show more (" + this.dataset.sensors + " more sensors)";
        }

   }

});
app.view.ExploreByCustomView = Backbone.View.extend({
    initialize: function (options) {
        var self = this;
        this.config = options.config;
        this.elementID = options.elementID;
        _.bindAll(this, "render");

        this.template = _.template($(this.config.templateID).html());

        if (_.isArray(this.config.modalsID)) {
            this.modals = _.map(this.config.modalsID, function (template) {
                return _.template($(template).html())();
            });
        } else if (this.config.modalsID) {
            // setting showModal to true will enable the info icon in the explore_custom template
            this.showModal = true;
            this.modals = _.template($(this.config.modalsID).html())();
        }


        this.firstGroup = this.config.groupingOrder[0];
        this.lastGroup = this.config.groupingOrder[this.config.groupingOrder.length - 1];
        this.primaryCollection = this.config.primaryCollection || this.firstGroup;
        this.config.collectionGroupingFunctionArguments = this.config.collectionGroupingFunctionArguments || '';

        app.event.eventBus.on("change:" + self.elementID, function (models) {
            if (models && models.length) {
                if ($('#' + self.elementID).find('.data-nav-header-active').length) {
                    var filteredModels = app.collection[self.lastGroup].filter(function (model) {
                        var huc;
                        switch (self.firstGroup) {
                            case "categories":
                                var cats = new Backbone.Collection;
                                model.get('parameters').each(function (p) {
                                    _.each(p.get('categories'), function (cat) {
                                        cats.add(cat)
                                    })
                                });
                                return _.intersection(cats.pluck('id'), models).length;
                                break;
                            case "parameterCategories":
                                return _.intersection(model.get('parameters').pluck('id'), models).length;
                                break;
                            case "parameters":
                                return _.intersection(model.get(self.firstGroup).pluck('id'), models).length;
                                break;
                            case "sources":
                                return _.contains(models, model.get('type')['id']);
                                break;
                            case "regions":
                            case "areas":
                                return _.contains(models, model.get('region'));
                                break;
                            case "river-reaches":
                                return _.contains(models, model.get('river-reaches'));
                                break;
                            case "huc2":
                                huc = model.get('huc');
                                if (huc && huc['huc2'] && _.contains(models, huc['huc2']['code'])) {
                                    return model;
                                } else {
                                    return false;
                                }
                            case "huc4":
                                huc = model.get('huc');
                                if (huc && huc['huc4'] && _.contains(models, huc['huc4']['code'])) {
                                    return model;
                                } else {
                                    return false;
                                }
                            case "huc6":
                                huc = model.get('huc');
                                if (huc && huc['huc6'] && _.contains(models, huc['huc6']['code'])) {
                                    return model;
                                } else {
                                    return false;
                                }
                            case "huc8":
                                huc = model.get('huc');
                                if (huc && huc['huc8'] && _.contains(models, huc['huc8']['code'])) {
                                    return model;
                                } else {
                                    return false;
                                }
                            default:
                                return false;
                        }

                    });
                    if (filteredModels && filteredModels.length) {
                        var filteredModelsToJSON = _.map(filteredModels, function (model) {
                            return model.toJSON();
                        });
                        app.view.exploreMapView.setGeoJsonFeature(filteredModelsToJSON, self.config.map);
                        app.view.exploreMapView.render(self.config);
                    }

                }
            }
        });

    },

    render: function () {
        var self = this;

        if (app.collection[self.primaryCollection] && app.collection[self.primaryCollection].length > 0) {
            var checkedCollectionItems = new Backbone.Collection;

            if ($('input[name="collection"]').length == 0) {
                app.collection[self.primaryCollection].each(function (model) {
                    checkedCollectionItems.add({id: model.get('id')});
                });
            } else {
                $('input[name="collection"]:checked').each(function () {
                    checkedCollectionItems.add({id: $(this).attr("value")});
                });
            }

            this.$el.empty();

            app.collection[self.primaryCollection][self.config.collectionGroupingFunction](self.config.collectionGroupingFunctionArguments).forEach(function (model) {
                // if models.json hasn't been filled out, the default will be an empty template that we shouldn't render
                if (model.id) {
                    if (checkedCollectionItems.contains(model.id)) model.set('checked', "checked");
                    self.$el.append(self.template({
                        model: model.toJSON(),
                        grouping: self.config.groupingOrder,
                        showModal: (self.showModal ? self.showModal : false)
                    }));
                }
            });
            app.event.eventBus.trigger("change:" + self.elementID, checkedCollectionItems.pluck('id'));

        } else {
            // display placeholder text if no sources are found
            if (app.config[self.primaryCollection]) {
                this.$el.html(app.config[self.primaryCollection].noneFound);
            } else {
                this.$el.html("No models found.");
            }

        }

        if (this.modals) {
            this.$el.append(this.modals);
        }


        var $explorePills = this.$(".explore-pill");
//       $explorePills.tooltip(this.config.pills.tooltipOptions); // enable bootstrap tooltips

        // fires events to highlight or unhighlight map pins
        $explorePills.hover(function (self) {
            return function (event) {
                app.event.eventBus.trigger("marker:mouseover", event.currentTarget.value.replace(/\W/g, "-"));
            };
        }(this), function (self) {
            return function (event) {
                app.event.eventBus.trigger("marker:mouseout", event.currentTarget.value.replace(/\W/g, "-"));
            };
        }(this));

        $explorePills.on("click", (function (self) {
            return function (event) {
                app.event.eventBus.trigger("marker:click", event.currentTarget.value.replace(/\W/g, "-"));
            };
        })(this));

        /* Keep the checkboxes in sync with the collapsed accordion
         * checked - accordion open
         * unchecked - accordion closed
         * not happy with this solution.
         * In the future the EPA wants the checkboxes to be displayed,
         * the checkboxes should determine which pins are on the map.
         * TODO : see GEO-258
         */
        var $collapse = this.$('.collapse');

        $collapse.on("hide", (function (self) {
            return function (event) {
                if (event.target !== this) return;
                $(this).parent().find('input[name="collection"]').prop("checked", false);

                var collectionItems = new Backbone.Collection;
                self.$('input[name="collection"]:checked').each(function () {
                    collectionItems.add({id: $(this).attr("value")});
                });
                app.event.eventBus.trigger("change:" + self.elementID, collectionItems.pluck('id'));
            };
        })(this));

        $collapse.on("show", (function (self) {
            return function (event) {
                if (event.target !== this) return;
                $(this).parent().find('input[name="collection"]').prop("checked", true);

                var collectionItems = new Backbone.Collection;
                self.$('input[name="collection"]:checked').each(function () {
                    collectionItems.add({id: $(this).attr("value")});
                });
                app.event.eventBus.trigger("change:" + self.elementID, collectionItems.pluck('id'));
            };
        })(this));

        self.$('.showHideExplore').on('click', this.toggleButton);
        return this;
    },

    toggleButton: function () {
        if (this.innerHTML.indexOf("more") >= 0) {
            this.innerHTML = "Show less";
        } else {
            this.innerHTML = "Show more (" + this.dataset.sensors + " more sensors)";
        }

    }
});
app.view.ExploreAccordionItemView = Backbone.View.extend({
    className: 'accordion-group',

    initialize: function (options) {
        var self = this;
        _.bindAll(this, "render", "renderAccordionContents");
        // these items are used to construct the template
        this.template = _.template($(options.template).html());

        // these items are passed into the template
        this.options = options;
        this.id = options.attributes.id;
        this.title = options.title;
        this.defaultOpen = options.defaultOpen;
        this.showStationLegend = options.showStationLegend;
        this.showHeader = (_.isBoolean(options.showHeader)) ? options.showHeader : true;
        this.subviewID = (options.contents && options.contents.containerID) ? options.contents.containerID : "";

        this.render();
        if (this.showHeader) {
            this.$el.on('show', function (event) {
                if (event.target.id === 'accordion-nav-collapse-' + self.id) {
                    $(this).find('.accordion-heading.data-nav-header').addClass('data-nav-header-active');
                }
            });
            this.$el.on('hide', function (event) {
                if (event.target.id === 'accordion-nav-collapse-' + self.id) {
                    $(this).find('.accordion-heading.data-nav-header').removeClass('data-nav-header-active');
                }
            });
        }
    },

    render: function () {
        var self = this;
        // all model properties are directly available to this.template (defined above)
        this.$el.empty();
        this.$el.html(this.template({
            id: this.id,
            title: this.title,
            defaultOpen: this.defaultOpen,
            showStationLegend: this.showStationLegend,
            subviewID: this.subviewID,
            showHeader: this.showHeader
        }));

        this.renderAccordionContents();
        this.$('#accordion-nav-collapse-' + this.id).on('shown', function () {
            if (self.id === "sources") {
                app.event.eventBus.trigger("change:" + self.id, false, true);
            } else {
                app.event.eventBus.trigger("change:" + self.id);
            }
        });

        if (this.defaultOpen) {
            app.event.eventBus.trigger('change:' + this.id);
        }

        return this;
    },

    renderAccordionContents: function () {
        var config = this.options.contents;
        var elementID = this.options.attributes.id;

        if (config && config.viewType && config.viewConfig && app.view[config.viewType]) {
            var view = new app.view[config.viewType]({config: config.viewConfig, elementID: elementID});
            this.$("#" + config.containerID).append(view.render().el);
        }

        return this;
    }
});
app.view.ExploreTrendsCollectionView = Backbone.View.extend({
//    tagName : "div",
//    className : "accordian",

    initialize: function(options) {
        this.config = options.config;
        this.selected = this.config.selected;
        this.collection = [];
        this.render();
    },

    render : function() {
        var self = this;

        if (app.collection.parameters.length > 0) {

            this.$el.empty();
            this.collection.forEach(function(subView) {
              subView.remove();
              subView.unbind();
              if (subView.close) subView.close();
            });

            //Removed the depth parameters until Medici can handle depth parameter calculations
            app.collection.parameters.where({exploreView: "visible", detailView: "time"}).forEach(function(parameter) {
              if (self.config.filterFunction && typeof self.config.filterFunction == 'function') {
                if (!self.config.filterFunction(parameter)) {
                  return;
                }
              }
              // don't show parameters in the list if they don't have any sensors
                if (parameter.id && parameter.get('sensors') && parameter.get('sensors').length) {
                    var selected = false;
                    if (self.selected && (parameter.get('title') == self.selected)) {
                      selected = true;
                    }
                    var itemView = new app.view.ExploreTrendsItemView({ model: parameter, selected: selected });
                    self.collection.push(itemView);
                    self.$el.append(itemView.render().el);
                }
            });
        } else {
            // display placeholder text if no sources are found
            this.$el.html(app.config.parameters.noneFound);
        }
        return this;
    }


});

app.view.ExploreTrendsItemView = Backbone.View.extend({

    attributes: function() {
        return {
          class: 'btn parameter-trend',
          'data-id': this.model.id,
          'data-title': this.model.get('title').replace(/<(.*?)>/g, '')
        }
    },
    events: {
        'click': 'clickItem'
    },

    initialize : function(options) {
        var self = this;
        _.bindAll(this, "redrawMapArrows");
        this.model = options.model;
        this.selected = options.selected;
        this.geostreamService = app.model.geoStreamService;
        this.template = _.template(this.model.get('title').replace(/<(.*?)>/g, ''));

        app.event.eventBus.on("trends:redrawMapArrows:" + self.model.get('id'), function(season) {
            var activeParameterModel = self.getActiveParameterModel();
            if (activeParameterModel.get('id') == self.model.get('id')) {
              self.redrawMapArrows(null, season);
            }

        });

        app.event.eventBus.on('trends:changeLegendTitle', function(parameter) {
            if (parameter) {
              self.getActiveParameterModel(parameter, function(parameterModel) {
                  self.changeLegendTitle(parameterModel.get('title'));
              });
            }
        });

        this.render();
    },

    close: function() {
        app.event.eventBus.off('trends:redrawMapArrows');
    },

    render : function() {
        var sensors = this.model.get('sensors');
        if (sensors.length > 0) {
            this.$el.empty();
            this.$el.html(this.template);
        }
//        if (this.selected) {
//            this.$el.trigger('click');
//        }
        return this;
    },

    clickItem: function(event) {
        event.cancelBubble = true;
        var self = this;
        if (event) {
          $('.parameter-trend').removeClass("parameter-trend-active");
          $(event.target).addClass('parameter-trend-active');
          var parameterID = $(event.target).data('id');
          if (self.model.get('id') != parameterID) return;

          var parameterModel = app.collection.parameters.findWhere({id: parameterID});

          self.redrawMapArrows(parameterID);

          app.event.eventBus.trigger("change:trends", self.model.get('sensors'), parameterModel.get('title'));
        }
    },

    redrawMapArrows : function(parameter, season) {
        app.event.eventBus.trigger("explore:map:spinner:start");

        var self = this;

        parameter = self.getActiveParameterModel(parameter);

        if (self.model.get('id') != parameter.get('id')) {
            return;
        }

        self.changeLegendTitle(parameter.get('title'));

        season = self.getActiveSeason(season);

        var attr_str = $.param({attributes: this.model.get('id') }, true);
        this.geostreamService.getTrend(attr_str, season, function(sensors, parameterTitle){
            app.event.eventBus.trigger("change:trends", sensors, parameterTitle);
            app.event.eventBus.trigger("explore:map:spinner:stop");
        });



    },

    getActiveParameterModel: function(parameterID, callback) {
      var parameterElement = $('.parameter-trend-active');
      if (parameterElement) {
        parameterID = parameterID || ((parameterElement.data('id')) ? parameterElement.data('id') : null);
        if (parameterID) {
          var parameter = app.collection.parameters.findWhere({id: parameterID});
        }
      }
      if (typeof callback == 'function') {
          callback(parameter);
      } else {
          return parameter;
      }

    },

    getActiveSeason: function(season) {
        // TODO this should not be using app.config.binning.time since the dropdown isn't being used to 'bin'
        // like the TODO in the ExploreMapView, this should probably be moved to app.config.views.trendMap or similar
        season = season || $('#trends-map-season-control-div').find('select').val() ||
            ((getBinningDesignators(app.config.binning.time).items && getBinningDesignators(app.config.binning.time).items[0]) ? getBinningDesignators(app.config.binning.time).items[0].id : 'spring');
        return season;
    },

    changeLegendTitle: function(title) {
      if (title) {
        $("#trends-map-legend-title-info").html(title);
      }
    }


});

app.view.LayerItemView = Backbone.View.extend({
    tagName: 'div',
    className: "toggleLayerView",
    attributes: function() {
        return {
            id : "layer-" + this.model.get("id")
        }
    },

    events: {
        //"click input": 'check',
        "click .layer-radio": 'check',
        "change .layer-opacity" : "updateOpacity"
    },


    initialize: function(options) {
        // the whole model gets passed to this template as "model", so we can directly use any values we need like "model.id"
        this.template = _.template($('#explore-layers-template').html());

        this.layers = options.layers;
//    	layers: 'sf:USRB_30m_DEM',
//        format: 'image/png',
//        transparent: true,
//        opacity: 0.75
//
        this.shapeOptions = {
            stroke: true,
            color: '#f06eaa',
            weight: 4,
            opacity: 0.5,
            fill: true,
            fillColor: null, //same as color by default
            fillOpacity: 0.2,
            clickable: true
        };

        var self = this;
        return this;
    },

    render: function() {
        // all model properties are directly available to this.template (defined above)
        var self = this;
        this.$el.empty();

        this.$el.html(
            this.template({
                checked: (this.model.get('properties').checked) ? "checked" : '',
                group: this.model.get('group') || '',
                showLayerLegend: this.model.get('properties').showLayerLegend,
                layerLegendStartOpen: this.model.get('properties'). layerLegendStartOpen,
                layerLegendTitle: this.model.get('properties').layerLegendTitle,
                layerLegendContents: this.model.get('properties').layerLegendContents,
                id: this.model.get('properties').id,
                title: this.model.get('properties').title,
                opacity: this.model.get('properties').opacity || 0.5,
                legend: this.model.get('properties').legend
            })
        );
        return this;
    },

    // this function handles the action of actually clicking the div for this Area - like "checking" an input box
    check: function(event) {
        var self = this;

        var checkedState = this.model.get("checked");
        this.model.set("checked", !(checkedState));
        app.event.eventBus.trigger("change:layer", this.model);
    },

    updateOpacity: function(event) {
        this.model.get("properties").opacity = $(event.currentTarget).val();
        app.event.eventBus.trigger("change:layer:opacity", this.model);
    },

    selectLayer : function(event) {
        //var self = this;
        //var geometry = self.model.get("geometry");
        //var points = geometry.coordinates[0].map(function(x) { return L.latLng(x[1], x[0]); });
        //app.event.eventBus.trigger("map:polygon:select", {'points': points});
        //var polygon = L.polygon(points, self.shapeOptions);
        //self.model.set("polygon", polygon);
        //polygon.addTo(app.view.advancedSearchView.drawnItems);
    }
});

app.view.LayersGroupView = Backbone.View.extend({
    initialize: function(options) {
        this.layers = options.layers;
        this.group = options.group;

        this.template = _.template($('#explore-layers-groups-template').html());
        this.$el.empty();
        this.$el.html(
            this.template({
                group: this.group
            })
        );

        return this;
    },

    render: function() {
        var self = this;

        var group_id = this.group.replace(/ /g, '-');

        var templateClass = ("#explore-layers-group-" + group_id);
        _.each(this.layers, function(model) {
            self.append(templateClass, model);
        });

        return this;
    },

    append: function(templateClass, model) {
        this.$(templateClass).append(new app.view.LayerItemView({model: model}).render().el);
    }
});

app.view.LayersCollectionView = Backbone.View.extend({
    initialize: function() {
        this.collection = app.collection.layers;
        this.listenTo(this.collection, 'add', this.append, this);
        this.listenTo(this.collection, 'remove', this.render, this);
    },


    render: function() {
        var self = this;
        this.$el.empty();


        // append each of the areas to the view (like adding "li" elements to a "ul")...
        _.each(this.collection.models, function(model) {
            self.append(model);
        });


        return this;
    },


    append: function(model) {
        this.$el.append(new app.view.LayerItemView({model: model}).render().el);
    }
});
app.view.LayersCollectionGroupView = Backbone.View.extend({

    initialize: function() {
        this.collection = app.collection.layers;
        this.listenTo(this.collection, 'add', this.append, this);
        this.listenTo(this.collection, 'remove', this.render, this);
        this.layerGroupsToCreate = [];
        this.layersWithGroups = [];

        var self = this;

        // Get all Groups for Layers
        _.each(this.collection.models, function(model) {
            if (model.get('group')) {
                var layerIndex =  self.layerGroupsToCreate.indexOf(model.get('group'));
                if (layerIndex === -1) {
                    self.layerGroupsToCreate.push(model.get('group'));
                }
                self.layersWithGroups.push(model);
            }
        });
    },

    render: function() {
        var self = this;
        this.$el.empty();

        // Layers with Groups
        _.each(self.layerGroupsToCreate, function(group) {

            var layers = [];

            _.each(self.layersWithGroups, function(layer) {
                if (layer.get('group') === group) {
                    layers.push(layer);
                }
            });

            self.group(group, layers);

        });

        // Layers without Groups
        _.each(this.collection.models, function(model) {

            if (!model.get('group')) {
                self.append(model);
            }

        });

        return this;
    },

    group: function(group, layers) {
        // Pass the Groups with Layers Only, and only those Layers with Groups
        this.$el.append(new app.view.LayersGroupView({group: group, layers: layers}).render().el);
    },

    append: function(model) {
        // Non-Group Layers
        this.$el.append(new app.view.LayerItemView({model: model}).render().el);
    }

});

// this "view" controls how we list one specific Area on the Search page (like "Lake Michigan")
// each Area is a model, and this view ties that model to the UI
// AreaListView (below) does the job of inserting all of these views
// think of AreaView as an "li" and AreaListView is the "ul"
if (app.config.useV3 == false) {
    app.view.AreaItemView = Backbone.View.extend({
        tagName: 'div',
        className: "areaView",
        attributes: function () {
            return {
                id: "area-" + this.model.get("id")
            }
        },

        events: {
            "click": 'check'
        },

        initialize: function () {

            // the whole model gets passed to this template as "model", so we can directly use any values we need like "model.id"
            this.template = _.template($('#search-areas-template').html());

            this.shapeOptions = app.config.views.search.shapeOptions || {
                    stroke: true,
                    color: (app.config.areas && app.config.areas.shapeOptions && app.config.areas.shapeOptions.color) ? app.config.areas.shapeOptions.color : '#7eb4fc',
                    weight: 4,
                    opacity: 0.5,
                    fill: true,
                    fillColor: null, //same as color by default
                    fillOpacity: 0.2,
                    clickable: true
                };

            this.model.on("change", this.render, this);

            return this;
        },

        render: function () {
            // all model properties are directly available to this.template (defined above)
            var self = this;
            // if the area does not contain a geometry property, it won't be able to draw a region, so don't show it as an option to check
            if (this.model.get('geometry')) {
                this.$el.empty();
                this.$el.html(this.template({area: this.model.toJSON()}));
            }


            return this;
        },

        // this function handles the action of actually clicking the div for this Area - like "checking" an input box
        check: function (event) {

            // since we are using a radio input, "uncheck" all other areas
            app.collection.areas.clearSelection();
            this.model.set({
                checked: true,
                selected: true
            });
            this.selectLake(event);

            // always update the buttons when something gets checked
            app.view.searchView.updateDownloadLink();
        },

        selectLake: function (event) {
            var self = this;
            var geometry = self.model.get("geometry");
            if (!geometry) {
                return;
            }
            var points = geometry.coordinates[0].map(function (x) {
                return L.latLng(x[1], x[0]);
            });
            app.event.eventBus.trigger("map:polygon:select", {'points': points});
            var polygon = L.polygon(points, self.shapeOptions);
            self.model.set("polygon", polygon);
            polygon.addTo(app.view.searchView.drawnItems);

            // zoom into area if zoomInOnClick is set to true
            if (app.config.areas.zoomInOnClick) {
                app.view.searchView.map.fitBounds(points, {padding: [50, 50]});
            }

        }
    });
}

if (app.config.useV3 == false) {
    app.view.AreasCollectionView = Backbone.View.extend({

        initialize: function(options) {
            if (this.collection) {
                this.listenTo(this.collection, 'add', this.append, this);
                this.listenTo(this.collection, 'remove', this.render, this);
            }
            this.collapsibleGroupTemplate = _.template($('#search-area-group-template').html());
        },

        render: function() {
            var self = this;
            // if there are areas, create a list of them.
            // if there is only one area, make sure it isn't an empty one from the default app.config.areas.items
            if (this.collection && this.collection.length > 0) {
                this.$el.empty();

                // append each of the areas to the view (like adding "li" elements to a "ul")...
                this.collection.each(function(model) {

                    // if this view was created from a app.config.areas.groups, it will contain items
                    // and should nest them in a collapsible element
                    if (model.get('items') && model.get('items').length > 0) {
                        self.$el.append('<div class="accordion searchAreaGroup" id="searchAreaGroup-' + model.get('title').replace(/ /g, '-') + '">');

                        // create an array of the item views so that we can attach them to the DOM
                        // if you try to attach them inside of the template function, their events won't fire
                        var itemElements = model.get('items').map(function(item) {
                            return new app.view.AreaItemView({model: item});
                        });

                        self.$('#searchAreaGroup-' + model.get('title').replace(/\W/g, '-')).append(self.collapsibleGroupTemplate({
                            title: model.get('title'),
                            parent: "searchAreaGroup-" + model.get('title').replace(/\W/g, '-')
                        }));

                        // now attach items to the dom, preserving events
                        _.each(itemElements, function(item) {
                            self.$('#accordion-inner-' + model.get('title').toLowerCase().replace(/ /g, '-')).append(item.render().el);
                        });

                        self.$el.append('</div>');
                    } else {
                        self.append(model);
                    }
                });

            } else {
                // if this is seen, you need to define a FeatureCollection in /assets/layers/layers.json
                this.$el.html("<p>" + app.config.areas.noneFound + "</p>");
            }

            return this;
        },

        append: function(model) {
            this.$el.append(new app.view.AreaItemView({model: model}).render().el);
            return this;
        }
    });
}
// this "view" controls how we list one specific Parameter on the Search page (like "Chlorophyl a")
// each Parameter is a model, and this view ties that model to the UI
// ParameterListView (below) does the job of inserting all of these views
if (app.config.useV3 == false) {
    app.view.ParameterItemView = Backbone.View.extend({
        events: {
            "click :checkbox": 'check'
        },

        render: function () {
            // all model properties are directly available to this.template (defined above)
            this.$el.empty();
            this.$el.html(this.template({
                model: this.model.toJSON(),
                view: this.view
            }));

            return this;
        },

        initialize: function (options) {
            // the whole model gets passed to this template, so we can directly use any values we need
            this.template = _.template($('#search-parameters-template').html());

            this.view = options.view;

            _.bindAll(this, "check");
            this.model.on("change", this.render, this);

            return this;
        },

        check: function () {
            app.event.eventBus.trigger("check:parameter", {view: this.view, model: this.model});
        }

    });
}
if (app.config.useV3 == false) {
    app.view.ParametersCollectionView = Backbone.View.extend({

        initialize: function(options) {
            this.config = options;
            this.view = options.view;
            this.listenTo(this.collection, 'add', this.append, this);
            this.listenTo(this.collection, 'remove', this.render, this);

        },

        append: function(model) {
            if (model.id) {
                this.tagName = (this.config.itemOptions && this.config.itemOptions.tagName) ? this.config.itemOptions.tagName : "div";
                this.className = (this.config.itemOptions && this.config.itemOptions.className) ? this.config.itemOptions.className : "parameters";
                this.$el.append(new app.view.ParameterItemView({
                    model: model,
                    view: this.view,
                    tagName: this.tagName,
                    className: this.className
                }).render().el);
            }
        },

        render: function() {
            var self = this;
            // if there are parameters, create a list of them.
            // if there is only one parameter, make sure it isn't an empty one from the default parameters.json
            if (this.collection.length > 1 || (this.collection.length == 1 && this.collection.models[0].id)) {
                this.$el.empty();

                // append each of the parameter Views
                _.each(this.collection.models, function(model) {
                    if (model.get(self.view) != "ignore") {
                        self.append(model);
                    }
                });
            } else {
                // if you are seeing this error, you need to specify the parameters you want to shown in /public/data/parameters.json
                this.$el.html(app.config.parameters.noneFound)
            }
            return this;
        }

    });
}
// this "view" controls how we list one specific Data Source on the Search page (like "WATER QUALITY (epa)")
// each Data Source is a model, and this view ties that model to the UI
// SourcesView (below) does the job of inserting all of these views
// think of SourceView as an "li" and SourcesView is the "ul"
if (app.config.useV3 == false) {
    app.view.SourceItemView = Backbone.View.extend({
        tagName: 'tr',
        className: "sourceView",

        events: {
            // TODO: Need to create a progress bar icon on the map.
            "click [type='checkbox']": 'check'
        },

        initialize: function() {
            var self = this;
            // the whole model gets passed to this template as "model", so we can directly use any values we need like "model.id"
            this.template = _.template($('#search-sources-template').html());
            this.eventBus = app.event.eventBus;

            // TODO: add a loading gif on the map
            this.eventBus.on("map:loading:start", function() {
                $('#loading').removeClass("hidden");
            });

            // TODO: add a loading gif on the map
            this.eventBus.on("map:loading:stop", function() {
                $('#loading').addClass("hidden");
            });

            this.model.on("change", this.render, this);

            return this;
        },

        render: function() {
            // all model properties are directly available to this.template (defined above)
            var self = this;

            this.$el.empty();
            this.$el.html( this.template( { model: this.model.toJSON() } ) );

            return this;
        },

        // updates the colored badge with the number of sensors selected on the map
        renderSelectedCount: function() {
            // in the template above, we're assigning the colored badge an "id": sensors-epa-selected
            var id = "#sensors-" + this.model.get('id') + "-selected";
            // the selected() function returns the number of sensors whose selected attribute is true
            $(id).html(this.model.getSelectedSensors().length);
        },

        // this function handles the action of actually clicking the div for this Data Source - like "checking" an input box
        check: function() {
            var self = this;

            self.eventBus.trigger("map:loading:start");

            // when you "check" the Data Source, it will highlight only the sensors that are "selectable".
            // if the user hasn't drawn a region on the map, everything is selectable. But if the user has drawn a region,
            // only those sensors will be selectable
            var allSelectableSensors = app.collection.sensors.getSelectableSensors();
            var sourceSelectableSensors = self.model.getSelectableSensors();
            var areasChecked = app.collection.areas.getChecked();

            // we have to do this next step because if you look at the sensor.js file, each sensor's isChecked function
            // listens to areaChecked, sourceChecked or parameterChecked to determine if it should show its color marker
            // I'd rather not do this, because after you uncheck a source, the sensor's areaChecked doesn't go back to true
            // and the sensor won't draw its color marker again, even though the area has selected it, but I don't see any other way around this yet.
            if (areasChecked) {
                _.each(allSelectableSensors, function(sensor) {
                    sensor.set("areaChecked", false);
                });
            }

            // the model is currently checked...
            if (this.model.get("checked")) {
                // "uncheck" it...
                this.model.set("checked", false);

                // ...then deselect each of its sensors...
                _.each(sourceSelectableSensors, function(sensor) {
                    sensor.set("sourceChecked", false);
                });

            } else { // this model was not "checked"
                // "check" it...
                this.model.set("checked", true);

                // ...select each of its "selectable" sensors
                _.each(sourceSelectableSensors, function(sensor) {
                    sensor.set("sourceChecked", true);
                });

            }
            // always update the buttons when something gets checked
            app.view.searchView.updateDownloadLink();

            self.eventBus.trigger("map:loading:stop");


        }
    });
}

if (app.config.useV3 == false) {
    app.view.SourcesCollectionView = Backbone.View.extend({
        el: "table#searchSourcesView",
        className: 'lite-1',

        initialize: function() {
            this.listenTo(this.collection, 'add', this.append, this);
            this.listenTo(this.collection, 'remove', this.render, this);
        },

        render: function() {
            var self = this;
            // if there are sources, create a list of them.
            // if there is only one source, make sure it isn't an empty one from the default sources.json
            if (this.collection.length > 1 || (this.collection.length == 1 && this.collection.models[0].id)) {
                this.$el.empty();

                // append each of the sources to the view (like adding "li" elements to a "ul")...
                _.each(this.collection.models, function(model) {
                    self.append(model);
                });
            } else {
                this.$el.html(app.config.sources.noneFound);
            }

            return this;
        },

        append: function(model) {
            if (model.id){
                this.$el.append(new app.view.SourceItemView({model: model}).render().el);
            }

        }
    });
}

// this view is an experiment to see if we can bind a sensor directly to the map 
// so that it shows up when the model property "selected" changes
// UPDATE: this is working - it is being called at the bottom of SearchView
if (app.config.useV3 == false) {
    app.view.SensorItemView = Backbone.View.extend({

        initialize: function() {

            // get the color value from this sensor's Data Source
            // stored in app.config.sources - each source has a color
            var sourceModel = app.collection.sources.findWhere({id: this.model.get('type').id });
            var sourceColor = (sourceModel && sourceModel.get('color')) ? sourceModel.get('color') : 'blue';
            var geojsonMarkerOptions = {
                radius: 2,
                fillColor: sourceColor,
                color: sourceColor,
                weight: 5,
                opacity: 1,
                fillOpacity: 1
            };

            var geoJsonLayer = L.geoJson(this.model.toJSON(), {
                onEachFeature: function (feature, layer) {
                    layer.bindPopup("<div class='search-popup'>"+feature.name+'</div>');
                },
                pointToLayer: function (feature, latlng) {
                    return L.circleMarker(latlng, geojsonMarkerOptions);
                }
            });

            // we set this marker on the model so that we can use Leaflet's "bringToFront" function when switching between Pages
            this.model.set("marker", geoJsonLayer);
            this.listenTo(this.model, "change:selected", this.render, this);
            this.listenTo(this.model, "change:checked", this.render, this);
        },

        render: function() {
            if (this.model.get("checked")) {
                this.model.get("marker").addTo(app.view.searchView.map);
            } else {
                app.view.searchView.map.removeLayer(this.model.get("marker"));
            }

        }
    });
}
// this View gets created by SearchView.renderSensorViews as a "container" for the Sensor views
// it prevents them from getting recreated when switching between Explore and Search
app.view.SensorsCollectionView = Backbone.View.extend({
    render: function() {
        var selected = this.collection.selected();
        _.each(selected, function(sensor) {
            // the marker gets added to the sensor model when the sensor view is created in sensor.js
            // bringToFront() is a Leaflet function
            sensor.get("marker").bringToFront();
        });
    }
});
app.view.HomeView = Backbone.View.extend({
    id: 'home-view',

  initialize: function(options) {
    var self = this;
    this.mapPin = options.mapPin;
    this.router = options.router;
    this.stationService = options.stationService;
    this.config = options.config;
    this.explore = options.config.explore;
    this.accordionItems = [];
    this.mapConfigs = {};
    this.selection = {};

    this.render = _.bind(this.render, this);
    this.$el.append(_.template($("#explore-template").html())());

    if (this.explore.accordion && this.explore.accordion.items && this.explore.accordion.items.length) {
        _.each(this.explore.accordion.items, function(accordionItem) {
            // inherit options from the accordion if not specified
            accordionItem.template = '#explore-accordion-item-template';
            accordionItem.attributes.class = 'accordion-group';

            var accordionItemView = new app.view.ExploreAccordionItemView(accordionItem);

            self.$('#explore-accordion-container').append(accordionItemView.el);
            self.accordionItems.push(accordionItemView);
            if (accordionItem.contents && accordionItem.contents.viewConfig && accordionItem.contents.viewConfig.map) {
                self.mapConfigs[accordionItem.attributes.id] = accordionItem.contents.viewConfig.map;
                self.mapConfigs[accordionItem.attributes.id].title = accordionItem.attributes.id;
                self.selection[accordionItem.attributes.id] = {};
            }
        });
    }

    app.view.exploreMapView = new app.view.ExploreMapView({
      parentView : this,
      geoJsonFeature : [],
      mapOptions: this.explore.map
    });

    this.$("#explore-map-view").append(app.view.exploreMapView.el);

    if(self.mapConfigs['trends'] && self.mapConfigs['trends'].aboutTrendData) {
      this.$el.append(_.template($(self.mapConfigs['trends'].aboutTrendData).html())());
    }

    // catch event fired by explore-pill or map-pin clicks
    // use this to fire the detail page route.
    app.event.eventBus.bind("sensors:refresh", (function(self) {
        return function() {
            _.each(self.accordionItems, function(accordionItem) {
                accordionItem.render();
            });
          self.render();
        };
    })(this));

    app.event.eventBus.on("change:trends", function(sensors, parameterTitle) {
        if ($('#trends').find('.data-nav-header-active').length) {
          if (sensors) {
            sensors = _.isArray(sensors) ? new Backbone.Collection(sensors) : sensors;
            self.selection.trends.sensors = sensors;
          } else  {
            sensors = self.selection.trends.sensors || app.collection.sensors;
          }

          if (parameterTitle) {
            self.selection.trends.parameter = parameterTitle;
          }

          var parameterModel = app.collection.parameters.findWhere({title: self.selection.trends.parameter});
          if (parameterModel) {
            app.event.eventBus.trigger('trends:changeLegendTitle', parameterModel.get('id'));
          }

          app.view.exploreMapView.setGeoJsonFeature(sensors.toJSON(), self.mapConfigs['trends'], self.selection.trends.parameter);
          app.view.exploreMapView.render(self.mapConfigs['trends']);

        }
    });

  },

  render: function(options) {
      var checkedSources = [];
      var sensors, visibleSensors;

      $('input[name="source"]:checked').each(function() {
        checkedSources.push($(this).attr("value"));
      });

      if (checkedSources.length) {
        sensors = app.collection.sensors.filter(function(sensor) {
          return _.contains(checkedSources, sensor.get('type').id);
        });
      } else {
        sensors = app.collection.sensors.models;
      }

      visibleSensors = _.map(sensors, function(sensor) {
        return sensor.toJSON();
      });
      app.view.exploreMapView.setGeoJsonFeature(visibleSensors, options);
      var self = this;
      _.each(app.config.sources.items, function(source) {
          self.$('#about-' + source.id + '-modal').draggable({handle: '.modal-header'});
      });

  }
});

app.view.LimnologyView = Backbone.View
    .extend({
      tagName : 'div',
      id : 'main-graphs',
      className : 'v11n-title',

      initialize : function() {
        this.render = _.bind(this.render, this);

        this.data = [];

        // d3.dispatch handles events passed from d3 charts
        this.dispatch = d3.dispatch("changeDepth");

        // initialize depth app.graphs with height 400, width 300
        this.depthGraph = app.graphs.depth().height(300).width(800)
          .dispatch(this.dispatch);

        // this almost certainly belongs in the detail_view.scala.html
        this.$el.append('<h4 id="depth-profile-title">Depth Profile</h4><div id="timestamp"></div>');
      },

      render : function() {
          var container = d3.select(this.el);
          var graph = container.selectAll(".depth-chart").data(this.data),
          graphEnter = graph.enter().append("div").attr("class", "depth-chart"),
          graphExit = graph.exit().remove();          
          graph.call(this.depthGraph);
      },

      setData : function(data) {
        if (data && data.length > 0) {
          if (data[0].values.length > 0) {
            $("#timestamp").empty();
            this.data = data;
          } else {
            $("#timestamp").text("no data found for " + data[0].key.toUTCString());
            this.data = [];
          }
        } else {
          $("#timestamp").text("no data found");
          this.data = [];
        }
      },

      changeDepth : function(callback) {
        this.dispatch.on("changeDepth", callback);
      }
    });
app.view.MapView = Backbone.View.extend({
      tagName : "div",
      className : "leaflet_map",

      initialize : function(options) {
        this.mapOptions = options.mapOptions;
        this.eventBus = options.eventBus;
        this.mapPin = $('#map-pin-template').html();
        this.mapArrowUp = _.template($('#map-arrow-up-template').html())();
        this.mapArrowDown = _.template($('#map-arrow-down-template').html())();
        this.mapLocation = _.template($('#map-location-template').html())();
        this.mapArrowUpThresh = _.template($('#map-arrow-up-thresh-template').html())();
        this.mapArrowDownThresh = _.template($('#map-arrow-down-thresh-template').html())();


        this.render = _.bind(this.render, this);

        this.map = L.map(this.el, this.mapOptions);

        var mapTileSource = this.mapOptions.mapTileSource || '//{s}.tile.osm.org/{z}/{x}/{y}.png';
        var mapAttribution = this.mapOptions.mapAttribution || '&copy; <a href="http://osm.org/copyright">OpenStreetMap</a> contributors';
        this.tileLayer = L
            .tileLayer(mapTileSource,
                {
                  attribution: mapAttribution,
                  maxZoom: Math.min(16, app.config.views.home.explore.map.maxZoom),
                  maxBounds: app.config.views.home.explore.map.maxBounds,
                  minZoom: app.config.views.home.explore.map.minZoom
                });

        this.tileLayer.addTo(this.map); //check if this is necessary.

        this.setGeoJsonFeature(options.geoJsonFeature);
        return this;
      },

      render : function() {
        this.map.invalidateSize(true);

        if (this.geoJsonLayer) {
            if(app.config.views.home.explore.map.cluster){
                this.markerClusterLayer.clearLayers();
                var markerCluster = L.markerClusterGroup({
                    maxClusterRadius: app.config.views.home.explore.map.clusterSize
                });
                markerCluster.addLayer(this.geoJsonLayer);
                this.map.addLayer(markerCluster);
                this.markerClusterLayer = markerCluster;
            } else {
                this.geoJsonLayer.addTo(this.map);
            }

        }

        return this;
      },

      setGeoJsonFeature : function(geoJsonFeature, parameter) {
        if (this.markerClusterLayer) this.markerClusterLayer.clearLayers();
        if (this.geoJsonLayer) this.map.removeLayer(this.geoJsonLayer);

        if (geoJsonFeature instanceof app.view.SensorsCollectionView) {
            this.geoJsonFeature = geoJsonFeature.toJSON();
        } else {
            this.geoJsonFeature = geoJsonFeature;
        }

        this.markerClusterLayer = L.geoJson(this.geoJsonFeature, this.geoJsonOptions(this, parameter));
        this.geoJsonLayer = L.geoJson(this.geoJsonFeature, this.geoJsonOptions(this, parameter));

      },

      getSourcePin: function(sensor) {
        var source, sourceColor;
        if (sensor && sensor.properties && sensor.properties.type && sensor.properties.type.id) {
          source = app.collection.sources.findWhere({id: sensor.properties.type.id});
          if (source) {
            sourceColor = source.getColor();
            if (sensor.properties.online_status  == "online"){
              return _.template(this.mapPin)({fill: sourceColor, stroke: "green"});
            } else {
              return _.template(this.mapPin)({fill: sourceColor});
            }
          }
        }

        return _.template(this.mapPin)();
      }
});

app.view.ExploreMapView = app.view.MapView.extend({
    tagName : "div",
    className : "leaflet_map",

    markers : {},

    events: {
        'click .btn': 'viewDetails'
    },


    // limits the
    latlngFormat : d3.format(".4f"),

    // this function gets called in MapView.js (we're using MapView.js like a super-class)
    // It is used with: L.geoJson(geoJsonFeature, geoJsonOptions)
    // So this sets the options for the Map, specifically
    geoJsonOptions : function(self, parameter) {
        var pin = {};
        return {
            pointToLayer: function(feature, latlng) {
                pin.template = self.pinTemplate || self.getSourcePin(feature);
                if (self.pinFunction) {
                    pin = self.pinFunction(feature, parameter);
                } else {
                    pin.template = self.getSourcePin(feature);
                }
                var marker = L.marker(latlng, {
                    icon : L.divIcon({
                        html : '<div id="' + ("station-" + feature.properties.name.replace(/\W/g, "-")) + '" class="' + feature.properties.type.id + '">' + pin.template + '</div>'
                    }),
                    title : feature.properties.name,
                    riseOnHover : true
                });

                self.markers["station-" + feature.properties.name.replace(/\W/g, "-")] = marker;
                return marker;
            },
            onEachFeature: function(feature, layer) {

                if (feature.properties && feature.properties.popupContent) {
                    var popupContent = self.populatePopupTemplate(feature, pin.threshold, 'fetching...', ['fetching...']);

                    var sname = "station-" + feature.properties.name.replace(/\W/g, "-")
                    if ((!(sname in self.markers)) && self.map) {
                        var poly = L.polygon(feature.geometry.coordinates, {
                            title : feature.properties.name,
                            riseOnHover : true
                        });
                        self.markers[sname] = poly;
                        poly.addTo(self.map);
                    }

                    layer.bindPopup(popupContent, { autoPanPadding: [30, 30], offset: new L.Point(160, -50)});
                    layer.on("mouseover", function(event) {
                        if(event.originalEvent.target.className.baseVal === "mouseCapture") {
                            self.hoverMarker("station-" + feature.properties.name.replace(/\W/g, "-"));
                        }
                    });
                    layer.on("mouseout", function(event) {
                        if(event.originalEvent.target.className.baseVal === "mouseCapture") {
                            self.unhoverMarker("station-" + feature.properties.name.replace(/\W/g, "-"));
                        }
                    });
                    layer.on("click", function(event) {
                        self.highlightMarker("station-" + feature.properties.name.replace(/\W/g, "-"), self.title);
                    });
                }
            }
        };
    },

    initialize : function(options) {
        var self = this;
        this.options = options;
        this.popupTemplate = _.template($("#map-popup-template").html());

        this.geoStreamService = app.model.geoStreamService = new app.model.GeoStreamService();

        // essesntially super.initialize(options)
        app.view.MapView.prototype.initialize.apply(this, arguments);

        app.event.eventBus.bind("marker:mouseover", (function(self) {
            return function(markerId) {
                markerId = "station-" + markerId;
                self.hoverMarker(markerId);
            };
        })(this));

        app.event.eventBus.bind("marker:mouseout", (function(self) {
            return function(markerId) {
                markerId = "station-" + markerId;
                self.unhoverMarker(markerId);
            };
        })(this));

        app.event.eventBus.bind("marker:click", (function(self) {
            return function(markerId) {
                markerId = "station-" + markerId;

                self.highlightMarker(markerId);
            };
        })(this));

        app.event.eventBus.bind("marker:unclick", (function(self) {
            return function(markerId) {
                markerId = "station-" + markerId;
                self.unhighlightMarker(markerId);
            };
        })(this));


        app.event.eventBus.bind("station:update", (function(self) {
          return function(sensor) {
            var popupContent = self.populatePopupTemplate(sensor);

            self.markers["station-" + sensor.get('name').replace(/\W/g, "-")].setPopupContent(popupContent, { offset: new L.Point(160, -50)});
          };
        })(this));

        $(window).on('scroll', this, function(event) {
            event.data.resizemap(false);
        });

        $(window).on('resize', this, function(event) {
            event.data.resizemap(false);
        });

        app.event.eventBus.bind("change:region", (function(self) {
            return function(regionName) {
                ;
            };
        })(this));

        app.event.eventBus.on("change:layer", (function(self) {
            return function(layerModel) {
                if (layerModel.get("checked")) {
                    layerModel.get("layer").addTo(self.map);
                    if (app.config.layers && app.config.layers.zoomInOnClick) {
                        self.map.panTo(new L.LatLng(layerModel.get("geometry").coordinates[1], layerModel.get("geometry").coordinates[0]));
                    }

                } else {
                    self.map.removeLayer(layerModel.get("layer"));
                }
            }
        })(this));

        app.event.eventBus.on("change:layer:opacity", (function(self) {
            return function(layerModel) {
                var opacity = layerModel.get("properties").opacity;
                layerModel.get("layer").setOpacity(opacity);
            }
        })(this));
    },

    resizemap : function(callrender) {
        // constants, only need to be done once
        var exploreMapViewEl = $("#explore-map-view");
        var header = $("#header");
        if (!this.extra) {
            this.extra = exploreMapViewEl.outerHeight(false) - exploreMapViewEl.height();
            this.padding = 10;
            this.marginheader = parseInt(header.css('margin-bottom'), 10);
        }

        // compute map size and offset
        var windowHeight = $(window).height();
        if (header && header.length) {
            var headerBounds = header[0].getBoundingClientRect();
            var offset = headerBounds.height + this.padding + Math.max(this.marginheader, -headerBounds.height - headerBounds.top);
            var footerTop = $(".footer")[0].getBoundingClientRect().top;
            var height = windowHeight - this.marginheader - 2*this.padding - this.extra - Math.max(0, windowHeight - footerTop) - Math.max(0, headerBounds.bottom);

            // set the map size
            exploreMapViewEl.offset({ top: offset });
            exploreMapViewEl.height(height);

            // render map
            this.map.invalidateSize(true);
            if (callrender) {
                app.view.MapView.prototype.render.apply(this, arguments);
            }
        }

    },

    render : function() {
        if(!this.geoJsonFeature.length) return; //if geoJsonLayer doesn't exist, don't render this view
        // this.geoJsonLayer gets set in MapView.setGeoJsonFeature
        this.map.fitBounds(new L.featureGroup([this.geoJsonLayer]).getBounds().pad(0.05),
            {maxZoom: ((this.options && this.options.map && this.options.map.maxZoom) ? this.options.map.maxZoom : 16)});
        this.resizemap(true);

        if (app.collection.layers && app.collection.layers.length > 0) {
            app.collection.layers.forEach((function(self) {
                return function(layer) {
                    if (!layer.get("initialized")) {
                        layer.set("initialized", true);
                        if (layer.get("checked")) layer.get("layer").addTo(self.map);
                    }
                }
            })(this));
        }
    },
    setGeoJsonFeature : function(geoJsonFeature, options, parameter) {
        if (this.pinFunction) delete this.pinFunction;
        if (options) {
            if (options.popupTemplate) this.popupTemplate = _.template($(options.popupTemplate).html());
            if (options.pinTemplate) this.pinTemplate = _.template($(options.pinTemplate).html())();
            if (options.pinFunction && (typeof this[options.pinFunction] == 'function')) this.pinFunction = this[options.pinFunction];
            if (options.title) this.title = options.title;
            if (options.legend) {
                var legend = _.template($(options.legend).html());
                //Add legend if it doesn't already exist
                if($('#trends-map-legend').length == 0) {
                    $("#explore-map-view").prepend(legend);

                    // TODO this needs to be changed to something more specific
                    // what this is doing is using app.config.binning.time to determine
                    // what to put in the dropdown on the trends map
                    // instead, this should go in something like app.config.views.trendsMap
                    // since it isn't really binning
                    if (app.config.binning.time && app.config.binning.time != 'none') {
                        //Add season drop down
                        if (!app.view.trendsSelectSeason) {
                          app.view.trendsSelectSeason = new app.view.SelectCollectionView({
                            collection: new Backbone.Collection(getBinningDesignators(app.config.binning.time).items),
                            trigger: "trends:redrawMapArrows"
                          });
                        }
                        $("#trends-map-season-control-div").append(app.view.trendsSelectSeason.el);
                        $("select").addClass("trends-map-season-control");
                    }
                    //Add SVG icons.
                    var trendUp = _.template($("#map-arrow-up-template").html());
                    var trendDown = _.template($("#map-arrow-down-template").html());
                    var trendLocation = _.template($("#map-location-template").html());
                    var trendUpThresh = _.template($("#map-arrow-up-thresh-template").html());
                    var trendDownThresh = _.template($("#map-arrow-down-thresh-template").html());

                    $("#trends-map-legend-up-icon").append(trendUp);
                    $("#trends-map-legend-down-icon").append(trendDown);
                    $("#trends-map-legend-no-trend-icon").append(trendLocation);
                    $("#trends-map-legend-threshold-icon-up").append(trendUpThresh);
                    $("#trends-map-legend-threshold-icon-down").append(trendDownThresh);
                }
            } else {
                $('#map-legend').remove();
            }

        }
        if(geoJsonFeature && geoJsonFeature.length) {
            app.view.MapView.prototype.setGeoJsonFeature.call(this, geoJsonFeature, parameter);
        }

        if(this.geoJsonFeature && this.geoJsonFeature.length) {
            this.render();
        }
    },

    hoverMarker: function(markerId) {
        d3.selectAll('#' + markerId + ' svg').attr("class", "map-pin-hover");
    },

    highlightMarker : function(markerId, title) {


        if(app.config.views.home.explore.map.cluster){
            //This addresses when a marker is in a cluster the map zooms so the marker shows up and then opens the popup.
            var m = this.markers[markerId];
            this.markerClusterLayer.zoomToShowLayer(m, function() {
                m.openPopup();
            });
        } else {
            this.markers[markerId].openPopup();
        }


        if (title && (title == 'trends')) {
            this.panToMarker(markerId);
            return;
        }

        if( app.collection.sensors.findWhere({ name: this.markers[markerId].options.title }).get("parameters").length == 0 ) {
            $("#detail-button").text("No Data Available")
                .attr('disabled', true)
                .bind('click', false);
        }
        else {
            // disable button for now
            $("#detail-button").text("Loading Data ...")
                .attr('disabled', true)
                .bind('click', false);

            // this.binningTime will be something like 'smart', 'semi', 'yearly', etc. It is how the datapoints
            // are averaged before being drawn in the graphs.
            // Projects can setup a filterFunction that accepts the current sensor (the Backbone Sensor Model)
            // as an argument. From there, you can return 'smart', 'semi', or some other binning value
            // based on a condition. For example: return (sensor.get('type')['id'] == 'epa') ? 'semi' : 'smart';
            // would return 'semi' as the binningTime for epa sensors, all others would use 'smart' binning
            this.binningTime = "";
            if (app.config.binning.filterFunction) {
              this.binningTime = app.config.binning.filterFunction(app.collection.sensors.findWhere({name: markerId.slice(8).replace(/-/g, " ")}));
            } else if (app.config.binning.time) {
              this.binningTime = app.config.binning.time;
            } else {
              this.binningTime = 'smart';
            }

            // this.binningTime is 'smart', then determine aggregation by temporal range
            if (this.binningTime == 'smart') {
                this.binningTime = getAggregationBin(
                  new Date(this.markers[markerId].feature.properties.min_start_time),
                  new Date(this.markers[markerId].feature.properties.max_end_time)
                );
            }

            // Use default depth if no depth value provided
            var sensorDepth = app.config.binning.depth || 1;

            // fetch data, when fetched enable button
            this.geoStreamService.getBinnedData(this.binningTime, sensorDepth, this.markers[markerId].feature.id, true, function(data) {
                $("#detail-button").text("View Data")
                    .attr('disabled', false)
                    .unbind('click');
            });
        }
    },

    unhoverMarker: function(markerId) {
        d3.selectAll('#' + markerId + ' svg').attr("class", "map-pin");
    },

    unhighlightMarker : function(markerId) {
        this.markers[markerId].closePopup();
    },

    panToMarker : function(markerId) {
        // The 280 size is the size of the popup width.
        var targetPoint = this.map.project(this.markers[markerId].getLatLng(), this.map.getZoom()).add([280 / 2, 0]);
        var targetLatLng = this.map.unproject(targetPoint, this.map.getZoom());
        this.map.panTo(targetLatLng);
    },

    populatePopupTemplate : function(sensor, threshold) {
        if (sensor instanceof app.model.Sensor) {
            sensor = sensor.toJSON();
        }
        return this.popupTemplate({model: sensor, threshold: threshold, sourceColor: sensor.properties.color});
    },

    viewDetails: function(e) {
        e.preventDefault();
        var location = $(e.currentTarget).data('location');
        app.event.eventBus.trigger("change:location", location);
    },

    getTrendsPin: function(sensor, parameter) {
        var areaModel, parameterModel, thresholdParameter;
        _.each(app.config.areas.items, function(area) {
            if (area.properties.id == "er") return;
            if (gju.pointInPolygon(sensor.geometry, area.geometry)) {
              areaModel = app.collection.areas.findWhere({id: area.properties.id});
            }
        });

        if (parameter) {
            parameterModel = app.collection.parameters.findWhere({title: parameter});
            if (parameterModel) {
              thresholdParameter = parameterModel.get('trendThreshold');
            } else {
              thresholdParameter = undefined;
            }
        }

        if (sensor && sensor.properties && sensor.properties.trends && sensor.properties.trends[parameter]
            && sensor.properties.trends[parameter].percentage_change) {
            if (+sensor.properties.trends[parameter].percentage_change > 0 && areaModel) {
                if (+sensor.properties.trends[parameter].last_average > areaModel.get('threshold')[parameter] && thresholdParameter) {
                    return {template: this.mapArrowUpThresh, threshold: areaModel.get('threshold')[parameter]};
                } else {
                    return {template: this.mapArrowUp, threshold: areaModel.get('threshold')[parameter]};
                }
            } else if (+sensor.properties.trends[parameter].percentage_change < 0 && areaModel) {
                if (+sensor.properties.trends[parameter].last_average > areaModel.get('threshold')[parameter] && thresholdParameter) { 
                    return {template: this.mapArrowDownThresh, threshold: areaModel.get('threshold')[parameter]};
                } else {
                    return {template: this.mapArrowDown, threshold: areaModel.get('threshold')[parameter]};
                }
            } else if (+sensor.properties.trends[parameter].percentage_change > 0 && !areaModel) {
                return {template: this.mapArrowUp};
            } else if (+sensor.properties.trends[parameter].percentage_change < 0 && !areaModel) {
                return {template: this.mapArrowDown};
            } else {
                return {template: this.mapLocation};
            }
        } else { 
            return {template: this.mapLocation};
        }
    }

});

app.view.MiniMapView = app.view.MapView.extend({
    geoJsonOptions : function(self) {
        return {
            pointToLayer : function(feature, latlng) {
                var marker = L.marker(latlng, {
                    icon : L.divIcon({
                        html : '<div id="' + feature.properties.name + '" class="' + feature.properties.type.id + '">' + self.getSourcePin(feature) + '</div>'
                    })
                });
                
                return marker;
            },
            onEachFeature : function(feature, layer) {
                
            }
        };
    },
    
    initialize : function(options) {
        app.view.MapView.prototype.initialize.apply(this, arguments);
        return this;
    },
    
    render : function() {
        app.view.MapView.prototype.render.apply(this, arguments);
        return this;
    },
    
    setGeoJsonFeature : function(geoJsonFeature) {
        if(geoJsonFeature.length === 0) return;
      
        app.view.MapView.prototype.setGeoJsonFeature.apply(this, arguments);
        
        var coordinates = geoJsonFeature[0].geometry.coordinates;
        this.map.setView(new L.LatLng(coordinates[1], coordinates[0]), 5);
        this.map.invalidateSize();
    }
});
app.view.TimeSeriesView = Backbone.View.extend({
	tagName : 'div',
	id : 'time',
    
    defaultOptions : {
        height : 250,
        width : 525,
        boxWhiskerMarginH : 60,
        boxWhiskerMarginV : 60
    },
	
	initialize : function(options) {
		this.render = _.bind(this.render, this);
        
        this.data = [];
        this.showEP = true;
        this.haveEP = true;
        
        this.chartOptions = _.extend({}, this.defaultOptions, options.chartOptions);
        
		this.timeSeries = app.graphs.timeSeries().height(this.chartOptions.height).width(this.chartOptions.width).max(15).padding("30 10 50 60");
		this.boxWhisker = app.graphs.boxwhisker().height(this.chartOptions.height).width(175).margin(this.chartOptions.boxWhiskerMarginH, this.chartOptions.boxWhiskerMarginV);
		this.$el.append('<h4 id="time-series-title" class="span9">Time Series</h4>' +
            '<h4 id="box-whisker-title" class="span3">Box and Whisker' +
                '<a href="#bw-modal" role="button" data-toggle="modal" class="info-icon-spacing">' +
                    '<svg xmlns="http://www.w3.org/2000/svg" version="1.1" height="18" width="18">' +
                        '<circle cx="9" cy="9" r="8" fill="url(#g1)" stroke="#939598" stroke-width="2px" />' +
                        '<text x="7" y="14" fill="darkblue" style="font : bold 14px Arial sans-serif;">i</text>' +
                        '<defs>' +
                            '<linearGradient id="g1" gradientUnits="userSpaceOnUse" ' +
                                            'x1="-3%" y1="3.41%" x2="103%" y2="96.59%">' +
                                '<stop offset=".15" stop-color="#fff" />' +
                                '<stop offset=".39" stop-color="#fafafa" />' +
                                '<stop offset=".67" stop-color="#ececed" />' +
                                '<stop offset=".97" stop-color="#d4d6d6" />' +
                                '<stop offset="1" stop-color="#d1d3d4" />' +
                            '</linearGradient>' +
                        '</defs>' +
                    '</svg>' +
                '</a>' +
            '</h4>');
	},
	
	render : function() {
        if(!this.data || !this.data.length) {
            this.$el.html('<div class="no-data">No data for the selected attributes.</div>');
        } else {
            this.$('.no-data').remove();
            this.$('.graphs-mouseOverVerticalLine').remove();
            var that = this;
            var data = that.showEP ? that.summerData : that.data;
            
            var container = d3.select(this.el).selectAll(".time-series-distribution-charts").data(data, function(d) { return d.key; });
            container.enter().append("div").attr("class", "time-series-distribution-charts row-fluid");
            container.exit().remove();
            
            var mytimeseries = container.selectAll(".time-series-chart").data(function(d) { return that.showEP ? [d.ep] : [d.all]; }, function(d) { if(d) {return d.key;} });
            mytimeseries.enter().append("div").attr("class", "time-series-chart span9");
            mytimeseries.exit().remove();
            mytimeseries.call(this.timeSeries);

            // comment this line off to disable switching between all data and filtered data
            mytimeseries.on("click", function(e) { that.showEP = !that.showEP; that.render(); });
            
            var boxWhiskerChart = container.selectAll(".box-whisker-chart").data(function(d) { return that.showEP ? [d.ep] : [d.all]; }, function(d) { return d.key; });
            boxWhiskerChart.enter().append("div").attr("class", "box-whisker-chart span3");
            boxWhiskerChart.exit().remove();
            boxWhiskerChart.call(this.boxWhisker);
        }
        return this;
	},

    setData : function(alldata, summerEPdata) {
        _.each(alldata, function(d) {
            var parameter = app.collection.parameters.findWhere({id: d.values[0].key});
            if (parameter) {
                d.values[0].title = parameter.get("title").replace(/<(.*?)>/g, '');
            } else {
                d.values[0].title = d.values[0].key;
            }

            if(app.config.graphs.timeSeries.showUnits.graphTitle && d.values[0].unit)
                d.values[0].title += " (" + d.values[0].unit + ")";
        });
        if (_.isUndefined(summerEPdata)) {
            summerEPdata = alldata;
            this.haveEP = false;
            this.showEP = false;
        } else {
            if (!this.haveEP) {
                this.showEP = true;
            }
            this.haveEP = true;
            _.each(summerEPdata, function(d) {
                var parameter = app.collection.parameters.findWhere({id: d.values[0].key});
                if (parameter) {
                    d.values[0].title = parameter.get("title").replace(/<(.*?)>/g, '');
                } else {
                    d.values[0].title = d.values[0].key;
                }

                if(app.config.graphs.timeSeries.showUnits.graphTitle && d.values[0].unit)
                    d.values[0].title = d.values[0].title + " (" + d.values[0].unit + ")"

                d.values[0].title += " (summer displays epilimnion data only)";
            });
        }
        this.data = [];
        this.summerData = [];
        for (var i = 0; i < alldata.length; i++) {
            this.data.push({key: i, all: alldata[i]});
            if(summerEPdata[i] !== undefined) {
	            this.summerData.push({key: i, ep: summerEPdata[i]});
            }

        }
        this.render();
    }
});

if (app.config.useV3 == false) {
    app.view.SearchView = Backbone.View.extend({

        attributes: function() {
            return {
                tagName: this.options.attributes.tagName,
                className: this.options.attributes.className,
                id: this.options.attributes.id
            }
        },

        events : {
            "click #download-permalink:not(.disabled)" : "setPermalink",
            "click #download-visualize:not(.disabled)" : "visualizeResults",
            "click .download:not(.disabled)": app.utils.interceptDownload
        },

        initialize : function(options) {
            var self = this;
            this.config = options.config;
            this.resultsVisualized = false;
            _.bindAll(this, "setPermalink", "updateDownloadLink", "updateDatesFromData");
            this.$el.append(_.template($(this.config.templateSelector).html())());

            this.eventBus = app.event.eventBus;
            this.stationService = options.stationService;

            this.dateRangeSelector = this.config.dateRangeSelector;
            this.layers = this.config.layers;
            this.downloadButtons = this.config.downloadButtons;

            this.geojsonMarkerOptions = {
                fillColor: this.config.map.markerFillColor,
                color: this.config.map.markerColor,
                radius: this.config.map.radius,
                weight: this.config.map.weight,
                opacity: this.config.map.opacity,
                fillOpacity: this.config.map.fillOpacity
            };

            // container for currently selected sensors
            this.sensorsfound = L.layerGroup();
            this.sensors = {};
            this.coordinates = {};

            // the format specified in app.config.views.search.dateRangeSelector.format
            // is passed to startDate and endDate, however, if you change the format
            // in `this.startDate = moment('2010....'`  <-- here then this.format can't be changed
            this.format = this.dateRangeSelector.format;
            this.startDate = moment('1970-10-01 1:00:00', this.format);
            this.endDate = moment();
            this.dataStartDate = this.startDate;
            this.dataEndDate = this.endDate;

            // map
            this.map = L.map(this.$(this.config.map.id)[0], {
                maxBounds: this.config.map.maxBounds,
                minZoom: this.config.map.minZoom,
                maxZoom: this.config.map.maxZoom,
                scrollWheelZoom: this.config.map.scrollWheelZoom
            });

            // Initialise the FeatureGroup to store editable layers
            this.drawnItems = new L.FeatureGroup();
            this.map.addLayer(this.drawnItems);

            // tileLayer
            var mapTileSource = this.config.map.mapTileSource || '//{s}.tile.osm.org/{z}/{x}/{y}.png';
            var mapAttribution = this.config.map.mapAttribution || '&copy; <a href="http://osm.org/copyright">OpenStreetMap</a> contributors';
            var tileLayer = L.tileLayer(mapTileSource,
                {
                    attribution: mapAttribution,
                    maxZoom: Math.min(16, this.config.map.maxZoom)
                });
            tileLayer.addTo(this.map);

            // controls
            var baseMaps = {
                "BaseMap": tileLayer
            };

            this.mapControls = L.control.layers(baseMaps);
            this.mapControls.addTo(self.map);

            this.resultsView = new app.view.ResultsView({geostreamService: options.geostreamService});

            app.event.eventBus.bind("sensors:refresh", (function(self) {
                return function() {
                    app.view.sensors = new app.view.SensorsCollectionView({collection: app.collection.sensors});
                    self.render();
                };
            })(this));

            this.eventBus.on("dates:new", this.updateDatesFromData);

            this.eventBus.on("sensors:selected", (function(self) {
                return function(sensors) {
                    self.sensors = sensors;
                };
            })(this));

            this.eventBus.on("map:circle:select", (function(self) {
                return function(coordinates) {
                    self.coordinates = coordinates;
                    self.clearLayers();
                    var params = {geocode: coordinates.latlng.lat + ',' + coordinates.latlng.lng + ',' + (coordinates.radius/1000)};
                    app.collection.sensors.select(params, function() {
                        if (app.config.areas.zoomInOnClick) {
                            app.event.eventBus.trigger("centerMap");
                        }
                        self.updateDownloadLink();
                    });
                };
            })(this));

            this.eventBus.on("map:polygon:select", (function(self) {
                return function(coordinates) {
                    self.coordinates = coordinates;
                    self.clearLayers();
                    var params = {geocode: coordinates.points.map(function(x) { return x.lat + "," + x.lng; }).join()};
                    app.collection.sensors.select(params, function() {
                        if (app.config.areas.zoomInOnClick) {
                            app.event.eventBus.trigger("centerMap");
                        }
                        self.updateDownloadLink();
                    });
                };
            })(this));

            this.eventBus.on("check:parameter", function(options) {
                if (options.view && options.view == "searchView"){
                    self.checkParameter(options);
                }
            });

            this.shapeOptions = this.config.shapeOptions;
            this.drawOptions = {
                position: this.config.drawOptions.position,
                draw: {
                    polyline: this.config.drawOptions.draw.polyline,
                    polygon: {
                        shapeOptions: this.shapeOptions
                    },
                    circle: {
                        shapeOptions: this.shapeOptions
                    },
                    rectangle: {
                        shapeOptions: this.shapeOptions
                    },
                    marker: this.config.drawOptions.draw.marker
                },
                edit: {
                    featureGroup: this.drawnItems, //REQUIRED!!
                    remove: this.config.drawOptions.edit.remove
                }
            };

            // Initialise the draw control and pass it the FeatureGroup of editable layers
            var drawControl = new L.Control.Draw(this.drawOptions);
            this.map.addControl(drawControl);

            this.resetMapButtonOptions = {
                text: this.config.resetMapButtonOptions.text,
                iconUrl: this.config.resetMapButtonOptions.iconUrl,
                glyph: this.config.resetMapButtonOptions.glyph,
                onClick: function() {
                    $('input[name=areaRadios]').prop("checked", false);
                    self.resetSearch();
                    self.resultsView.render();
                },  // callback function
                hideText: this.config.resetMapButtonOptions.hideText,
                maxWidth: this.config.resetMapButtonOptions.maxWidth,
                doToggle: this.config.resetMapButtonOptions.doToggle,
                toggleStatus: this.config.resetMapButtonOptions.toggleStatus
            };

            var resetMapButton = new L.Control.Button(this.resetMapButtonOptions);
            resetMapButton.addTo(this.map);

            var colors = d3.scale.category10();
            // basins
            // originally from http://www.great-lakes.net/gis/data/refdata.html
            // TODO: extract this - check for options.layers && options.layers.items

            if (app.config.basins && app.config.basins.items && app.config.basins.items.length > 0) {
                self.basins = L.geoJson(app.config.basins.items,{
                    style: function(feature) {
                        switch (feature.properties.LAKEBASIN) {
                            case 'Erie': return {color: colors(0),"weight": 1, "opacity": 1};
                            case 'Huron': return {color: colors(1),"weight": 1, "opacity": 1};
                            case 'Michigan': return {color: colors(2),"weight": 1, "opacity": 1};
                            case 'Ontario': return {color: colors(3),"weight": 1, "opacity": 1};
                            case 'St. Lawrence': return {color: colors(4),"weight": 1, "opacity": 1};
                            case 'Superior': return {color: colors(5),"weight": 1, "opacity": 1};
                            default: return {color: colors(Math.round(Math.random()*10)),"weight": 1};
                        }
                    },
                    onEachFeature: function (feature, layer) {
                        layer.bindPopup("<div class='" + self.layers.bindPopupClass + "'>"+feature.properties.LAKEBASIN+'</div>');
                    }
                });
                self.mapControls.addOverlay(self.basins, app.config.basins.title);
            }
        },

        render : function() {
            var self = this;

            var map = this.map; // FIXME
            this.sensorsfound.addTo(map);
            this.map.on('draw:created', function (e) {
                var type = e.layerType,
                    layer = e.layer;

                if ((type === 'polygon') || (type === 'rectangle')) {
                    self.eventBus.trigger("map:polygon:select", {'points': layer.getLatLngs()});
                }

                if (type === 'circle') {
                    self.eventBus.trigger("map:circle:select", {'latlng': layer.getLatLng(), 'radius': layer.getRadius()});
                }

                $('input[name=areaRadios]').prop("checked", false);
                // Do whatever else you need to. (save to db, add to map etc)
                e.layer.addTo(self.drawnItems);
            });

            map.on('draw:edited', function () {
                // Update db to save latest changes.
            });

            map.on('draw:deleted', function () {
                self.clearMarkers();
                self.resetButtons();
            });

            app.collection.sensors.each(function(sensor) {

                var tempGeojsonMarkerOptions = JSON.parse(JSON.stringify(self.geojsonMarkerOptions));
                if (sensor.get('online_status') && sensor.get('online_status')=="online"){
                    tempGeojsonMarkerOptions["color"] = "green";
                }

                var geoJsonLayer = L.geoJson(sensor.toJSON(), {
                    onEachFeature: function (feature, layer) {
                        layer.bindPopup("<div class='" + self.layers.bindPopupClass + "'>"+feature.name+'</div>');
                    },
                    pointToLayer: function (feature, latlng) {
                        return L.circleMarker(latlng, tempGeojsonMarkerOptions);
                    }
                }).addTo(self.map);
            });

            this.eventBus.on("centerMap", function() {
                app.event.eventBus.trigger("zoomOut");
                self.map.fitBounds(self.drawnItems.getBounds(), {
                    padding: [ 5, 5 ]
                });
            });

            this.eventBus.on("zoomOut", function() {
                self.map.fitBounds(self.map.maxBounds);
            });

            // draw default map
            if (app.config.views.search.map.maxBounds === undefined) {
                var bbox = new L.LatLngBounds();
                app.collection.sensors.each(function(sensor) {
                    bbox.extend([sensor.attributes.geometry.coordinates[1],
                        sensor.attributes.geometry.coordinates[0]]);
                });
                if (bbox.isValid()) {
                    app.config.views.search.map.maxBounds = bbox.pad(0.05);
                }
            } else {
                this.map.fitBounds(app.config.views.search.map.maxBounds);
            }

            this.map.invalidateSize();

            this.renderDates();
            this.renderAreas();
            this.renderSources();
            this.renderParameters();
            this.renderSensorViews();

        },

        // this is registered in the events up in the top of SearchView
        updateDownloadLink : function() {
            // if the user has drawn a area, this.coordinates will be populated
            // if the user hasn't drawn a area, we shouldn't filter by geocode
            var params = {};
            if (this.coordinates) {
                if (this.coordinates.radius) {
                    // the user drew a circle
                    var latlng = this.coordinates.latlng;
                    var radius = this.coordinates.radius;
                    params["geocode"] = latlng.lat + ',' + latlng.lng + ',' + (radius/1000);

                } else if (this.coordinates.points) {
                    // the user drew a polygon
                    params["geocode"] = this.coordinates.points.map(function(x) {
                        return x.lat + "," + x.lng;
                    }).join();
                }
            }

            // query the sources collection for models that have been checked, and return the id's in an array
            var sources = _.pluck(_.where(app.collection.sources.toJSON(), {checked: true}), 'id');
            if (sources) params["sources"] = sources;

            // get selected attributes from the collection
            var parameters = _.flatten(_.pluck(_.where(app.collection.parameters.toJSON(), {checked: true}), 'title'));
            if (parameters) {
                params["attributes"] = parameters;
                parameters = _.flatten(_.pluck(_.where(app.collection.parameters.toJSON(), {checked: true}), 'id'));
                if (parameters) params["attributes"] = _.union(params["attributes"], parameters);
            }

            var sites_value = document.getElementById("sites_value");

            // verify sites exist to download
            if ( (sites_value === null) || !(sites_value.innerHTML > 0) ) {
                this.resetButtons();
            }
            else {

                if (this.dataStartDate) params["since"] = this.dataStartDate.format(this.format);
                if (this.dataEndDate) params["until"] = this.dataEndDate.format(this.format);

                // create the url
                var url = app.datapoints.url + '?' + $.param(params, true);

                // update the DOM
                $(this.downloadButtons.json.id).attr('href', url + "&format=json");
                $(this.downloadButtons.json.id).removeClass('disabled');
                $(this.downloadButtons.csv.id).attr('href', url + "&format=csv");
                $(this.downloadButtons.csv.id).removeClass('disabled');
                $(this.downloadButtons.permalink.downloadID).removeClass('disabled');
                $(this.downloadButtons.visualize.id).removeClass('disabled');
            }
        },

        setPermalink : function() {
            var template = $(this.downloadButtons.permalink.template).html();
            $(this.downloadButtons.permalink.id).html(_.template(template,{url: $(this.downloadButtons.json.id).attr('href')}));
            $(this.downloadButtons.permalink.modalID).modal('show');
        },

        // this function is the equivalent to this.sensorsfound.clearLayers(), but works with app.collection.sensors
        // would be nice to keep track of the layer id and related sensors when drawing a area on the map
        // so that clearing a drawn area would only deselect the sensors underneath it. for now, we're clearing all selected sensors
        clearMarkers: function() {
            app.collection.sensors.uncheckAll();
        },

        /**
         * Clear all drawn layers. Similar to just calling this.drawnItems.clearLayers() in initialize or render,
         * but as a function, it can be called by the ResetMapButton
         */
        clearLayers: function() {
            // create a reference to this view as 'self'
            var self = app.view.searchView;
            if (self.drawnItems && (typeof self.drawnItems.clearLayers == 'function')) {
                self.drawnItems.clearLayers();
            }
            // clearMarkers unchecks all sensors, sources, parameters and areas
            self.clearMarkers();
        },

        resetSearch: function() {
            this.clearLayers();
            app.collection.sources.uncheckAll();
            app.collection.parameters.uncheckAll();
            this.coordinates = {};
            app.event.eventBus.trigger("zoomOut");
            this.resetButtons();
        },

        resetButtons: function() {
            $(this.downloadButtons.csv.id).removeAttr('href');
            $(this.downloadButtons.csv.id).addClass('disabled');
            $(this.downloadButtons.json.id).removeAttr('href');
            $(this.downloadButtons.json.id).addClass('disabled');
            $(this.downloadButtons.permalink.downloadID).addClass('disabled');
            $(this.downloadButtons.visualize.id).addClass('disabled');
        },

        renderAreas : function() {
            if (!app.view.areasView) {
                app.view.areasView = new app.view.AreasCollectionView({el: "div#areasView", collection: app.collection.areas});
            }
            app.view.areasView.render();
        },

        renderSources : function() {
            if (!app.view.sourcesView) {
                if (app.collection.sources.getSourcesWithSensors().length > 0) {
                    app.view.sourcesView = new app.view.SourcesCollectionView({
                        collection: app.collection.sources.getSourcesWithSensors()
                    });
                    app.view.sourcesView.render();
                }
            } else {
                app.view.sourcesView.render();
            }
        },

        renderParameters: function() {
            if (!this.parametersView) {
                this.parametersView = new app.view.ParametersCollectionView({
                    el: "table#parametersView",
                    collection: app.collection.parameters,
                    view: "searchView",
                    itemOptions: {
                        tagName: 'tr',
                        className: 'parameterView'
                    }
                });
            }
            this.parametersView.render();

        },

        checkParameter: function(options) {
            this.model = options.model;

            if (app.collection.sensors) var allSelectableSensors = app.collection.sensors.getSelectableSensors();
            var parameterSelectableSensors = this.model.getSelectableSensors();
            if (app.collection.areas) var areasChecked = app.collection.areas.getChecked();

            // we have to do this next step because if you look at the sensor.js file, each sensor's isChecked function
            // listens to areaChecked, sourceChecked or parameterChecked to determine if it should show its color marker
            // I'd rather not do this, because after you uncheck a parameter, the sensor's areaChecked doesn't go back to true
            // and the sensor won't draw its color marker again, even though the area has selected it, but I don't see any other way around this yet.
            if (areasChecked) {
                _.each(allSelectableSensors, function(sensor) {
                    sensor.set("areaChecked", false);
                });
            }

            if (this.model.get("checked")) {
                this.model.set("checked", false);
                // before you set parameterChecked to false, you need to see if all other parameters for this sensor are unchecked.
                _.each(parameterSelectableSensors, function(sensor) {
                    sensor.set("parameterChecked", false);
                });


            } else {
                this.model.set("checked", true);
                _.each(parameterSelectableSensors, function(sensor) {
                    sensor.set("parameterChecked", true);
                });
            }
            // always update the buttons when something gets checked
            this.updateDownloadLink();

        },

        updateDatesFromData: function(dateRange) {
            if (this.isRendering) return;
            this.isRendering = true;
            this.dataStartDate = dateRange.min;
            this.dataEndDate = dateRange.max;
            this.renderDates();
            this.isRendering = false;
        },

        renderDates: function() {
            var self = this;
            var sliderRange = $( self.dateRangeSelector.id );
            sliderRange.slider(
                {
                    range: true,
                    min: self.startDate.year(),
                    max: self.endDate.year(),
                    values: [self.dataStartDate.year(), self.dataEndDate.year()],
                    slide: function(event, ui)
                    {
                        $("#searchtimeSelectedStart").text(ui.values[ 0 ]);
                        $("#searchtimeSelectedStop").text(ui.values[ 1 ]);
                        self.dataStartDate = moment(ui.values[0].toString() + '-01-01 1:00:00', self.format);
                        self.dataEndDate = moment(ui.values[1].toString() + '-12-31 23:59:59', self.format);
                        self.resultsView.renderDates(self.dataStartDate, self.dataEndDate);
                        self.updateDownloadLink();
                    }
                });

            this.$("#searchtimeStart").text(self.startDate.year());
            this.$("#searchtimeStop").text(self.endDate.year());
            this.$("#searchtimeSelectedStart").text(sliderRange.slider("values", 0));
            this.$("#searchtimeSelectedStop").text(sliderRange.slider("values", 1));

        },

        renderSensorViews: function() {
            if (!app.view.sensors) {
                app.view.sensors = new app.view.SensorsCollectionView({collection: app.collection.sensors});
            }
            app.view.sensors.render();
        },

        visualizeResults: function() {
            this.resultsVisualized = !this.resultsVisualized;
            if (this.resultsVisualized) {
                this.resultsView.visualizeResults();
            } else {
                this.resultsView.hideVisualizations();
            }
        }

    });
}

// experimental view on the Search page. Shows the results of selected sensors and their properties (date, attributes, etc.)
if (app.config.useV3 == false) {
    app.view.ResultsView = Backbone.View.extend({

        id: "searchResults",

        initialize: function(options) {
            this.geostreamService = options.geostreamService;
            this.listenTo(app.collection.sensors, 'change:checked', this.render);
            this.listenTo(app.collection.parameters, 'change:checked', this.render);
            this.listenTo(app.collection.parameters, 'change:sourceChecked', this.render);
            this.listenTo(app.collection.sources, 'change:checked', this.render);

            // load sparkline visualization
            this.sparkline = app.graphs.sparkline().height('100px').width('175px');

            this.timeSeries = app.graphs.timeSeries().height('100px').width('175px').max(15).padding("30 10 50 60");

            this.parametersSelected = [];

            this.checked = {sensors: [], parameters: [], bounds: []};
        },

        render: function() {
            if (this.isRendering) return;
            this.isRendering = true;

            var checked = {sensors: [], parameters: [], bounds: []};

            checked.sensors = app.collection.sensors.getChecked();
            checked.sensors.forEach(function(sensor) {
                if (sensor.get('max_end_time')) {
                    var max = moment(sensor.get('max_end_time'));
                    if (checked.max === undefined || max > checked.max) checked.max = max;
                }
                if (sensor.get('min_start_time')) {
                    var min = moment(sensor.get('min_start_time'));
                    if (checked.min === undefined || min < checked.min) checked.min = min;
                }
            });

            var sourcesChecked = app.collection.sources.where({checked: true});
            var sourcesSelected = sourcesChecked;
            if ( (sourcesSelected && sourcesSelected.length == 0)) {
                sourcesSelected = app.collection.sources.where({selected: true});
            }

            var parametersChecked = app.collection.parameters.where({checked: true});
            parametersChecked = _.filter(parametersChecked, function(parameter) {
                return parameter.get("searchView") != "ignore";
            });
            var parametersSelected = parametersChecked;
            if ( parametersChecked && parametersChecked.length > 0) {
                parametersSelected = parametersChecked;
            } else if ( (parametersChecked && parametersChecked.length == 0) && (sourcesChecked && sourcesChecked.length == 0)) {
                parametersSelected = app.collection.parameters.where({selected: true});
            } else {
                parametersSelected = app.collection.parameters.where({sourceChecked: true});
            }
            parametersSelected = _.filter(parametersSelected, function(parameter) {
                return parameter.get("searchView") != "ignore";
            });

            var searchResultsDiv = $('#searchResults');

            // Verify that results will return with a Download action
            if (parametersChecked.length >= 1 && sourcesChecked.length >= 1) {
                var number_params_sources = 0;
                _.each(parametersChecked, function(parameter) {
                    _.each(sourcesChecked, function(sources) {
                        if (sources.get("ParametersIDs") && sources.get("ParametersIDs").includes(parameter.get("id"))) {
                            number_params_sources += 1;
                        }
                    });
                });

            }

            var number_of_sites = 0;

            var sites_count_search_from_sources = app.config.views.search.sites_count_search_from_sources;

            if (
                ((sites_count_search_from_sources == true) &&
                    ( (number_params_sources > 0) ||
                    (parametersChecked.length == 0 && sourcesChecked.length >= 1) ||
                    (parametersChecked.length >= 1 && sourcesChecked.length == 0) )
                ) ||
                (sites_count_search_from_sources == false)
            ){
                var theSensors = app.collection.sensors;

                if (parametersChecked.length >= 1 && sourcesChecked.length == 0) {
                    var theParameterSensors = [];
                        theSensors.filter(function(x){
                            parametersChecked.forEach(function(parameter) {
                            x.attributes.parameters.forEach(function(y) {
                                if ((parameter.id == y.id) && (theParameterSensors.indexOf(x.id) < 0)){
                                    theParameterSensors.push(x.id);
                                }
                            });
                        });
                    });
                    number_of_sites = theParameterSensors.length;

                } else if (parametersChecked.length >= 1 && sourcesChecked.length >= 1) {
                    var theSelectedSensors = [];
                    theSensors.filter(function(x){
                        sourcesChecked.forEach(function(source) {
                            if ((source.id == x.attributes.type.id) && (theSelectedSensors.indexOf(x.id) < 0)){
                                parametersChecked.forEach(function(parameter) {
                                    x.attributes.parameters.forEach(function(y) {
                                        if ((parameter.id == y.id) && (theSelectedSensors.indexOf(x.id) < 0)){
                                            theSelectedSensors.push(x.id);
                                        }
                                    });
                                });
                            }
                        });
                    });
                    number_of_sites = theSelectedSensors.length;

                } else {
                    number_of_sites = (
                        (app.collection.sensors.getChecked().length ||
                        app.collection.sensors.where({selected: true}).length)
                    );
                }
            }

            var help_text = '';
            if (number_of_sites == 0) {
                help_text = 'Please update your search criteria to enable downloads.'
            }

            searchResultsDiv.html(
                '<div class="row search-results-row">' +
                '<strong>' + help_text + '</strong><br/>' +
                '<strong>Sites:</strong> ' +
                '<span id="sites_value">' + number_of_sites+ '</span>' +
                '</div>'
            );

            if (sourcesSelected.length >= 1) {
                searchResultsDiv.append('<div class="row search-results-row"><strong>' + app.config.sources.title + ':</strong>');
                _.each(sourcesSelected, function(source, index) {
                    searchResultsDiv.append('<div class="badge">' + source.id + '</div>');
                    if (index <= (sourcesSelected.length - 2)) { searchResultsDiv.append(' OR '); }
                });
                searchResultsDiv.append('</div>');
            }

            // show Measurements if we have some
            if (parametersSelected.length >= 1) {
                searchResultsDiv.append('<div class="row search-results-row"><strong>' + app.config.parameters.title + ':<strong> </div>').html();
                _.each(parametersSelected, function(parameter, index) {
                    searchResultsDiv.append('<div class="badge">' + parameter.get("title") + '</div>').html();
                    if (index <= (parametersSelected.length - 2)) { searchResultsDiv.append(' OR '); }
                });
            }

            this.parametersSelected = parametersSelected;
            this.checked = checked;

            // show date range if it exists
            if (checked.min && checked.max) {
                this.renderDates(checked.min, checked.max);
                app.event.eventBus.trigger('dates:new', {min: checked.min, max: checked.max});
            }

            this.hideVisualizations();

            this.isRendering = false;

        },

        renderDates: function(start, end) {
            var searchResultsDates = $('#search-results-dates');
            if (searchResultsDates.length) {
                searchResultsDates.html('<strong>Dates Selected:</strong> ' + start.format('MMMM Do YYYY') + ' - ' + end.format('MMMM Do YYYY')).html();
            } else {
                $('#searchResults').append('<div id="search-results-dates" class="row search-results-row"><strong>Dates Available:</strong> ' + start.format('MMMM Do YYYY') + ' - ' + end.format('MMMM Do YYYY') + '</div>').html();
            }

        },

        visualizeResults: function() {

            var that = this;

            // draw graphs for each parameter and station
            if(this.parametersSelected.length > 0) {

                var searchResultsVisualizationsDiv = $('#searchResultsVisualizations');

                searchResultsVisualizationsDiv.append('<div><h3>Visualizations</h3></div>').html();

                _.each(this.parametersSelected, function(parameter) {


                    searchResultsVisualizationsDiv.append('<div><strong>' + parameter.get("title") + '</strong></div>').html();

                    // place divs on the page for all the sensors/parameters matches
                    _.each(that.checked.sensors, function(sensor) {
                        if (_.contains(parameter.get("sensors").pluck("id"), sensor.get("id"))) {
                            searchResultsVisualizationsDiv.append('<div class="searchResultStation">' +
                                '<div id="vis_' + sensor.get("id") +  '_' + parameter.get("id") + '"></div>').html();
                        }
                    });

                    // retrieve data by sensor and draw graphs
                    _.each(that.checked.sensors, function(sensor) {

                        // this.binningTime will be something like 'smart', 'semi', 'yearly', etc. It is how the datapoints
                        // are averaged before being drawn in the graphs.
                        // Projects can setup a filterFunction that accepts the current sensor (the Backbone Sensor Model)
                        // as an argument. From there, you can return 'smart', 'semi', or some other binning value
                        // based on a condition. For example: return (sensor.get('type')['id'] == 'epa') ? 'semi' : 'smart';
                        // would return 'semi' as the binningTime for epa sensors, all others would use 'smart' binning
                        var time = "";
                        if (app.config.binning.filterFunction) {
                            time = app.config.binning.filterFunction(sensor);
                        } else if (app.config.binning.time) {
                            time = app.config.binning.time;
                        } else {
                            time = "smart";
                        }

                        // If app.config.binning.time is 'smart', then determine aggregation by temporal range
                        if (time == 'smart') {
                            time = getAggregationBin(new Date(sensor.get('min_start_time')), new Date(sensor.get('max_end_time')));
                        }

                        // bin depth in one by of 1000 meters below the surface and by year
                        that.geostreamService.getBinnedData(time, 1000, sensor.get("id"), false, function(data) {
                            // console.log("Got data for " + sensor.get("id"));
                            // console.log(data);

                            if (_.contains(parameter.get("sensors").pluck("id"), sensor.get("id"))) {

                                console.log("Drawning graph for sensor " + sensor.get("id") + " and parameter " + parameter.get("id"));


                                var dataBySensor;
                                dataBySensor = data[0].properties[parameter.get("id")];
                                if (!dataBySensor) {
                                    while(!dataBySensor) {
                                        _.each(parameter.get('alternates'), function(alternate) {
                                            dataBySensor = data[0].properties[alternate];
                                        });
                                    }
                                }

                                // order by year
                                // dataBySensor = _.sortBy(dataBySensor, function(obj) { return obj.year });

                                var margin = {top: 20, right: 5, bottom: 20, left: 30},
                                    width = 175 - margin.left - margin.right,
                                    height = 100 - margin.top - margin.bottom,
                                    chart_id = "#vis_" + sensor.get("id") +  '_' + parameter.get("id");

                                // X scale
                                var x = d3.time.scale()
                                    .domain(d3.extent(dataBySensor, function(d) { return new Date(d.date); }))
                                    .range([0, width]);

                                // Y scale
                                var y = d3.scale.linear()
                                    .domain(d3.extent(dataBySensor, function(d) { return d.average; }))
                                    .range([height, 0]);

                                // x-axis
                                var xAxis = d3.svg.axis()
                                    .scale(x)
                                    .ticks(3)
                                    .orient("bottom");

                                // y-axis
                                var yAxis = d3.svg.axis()
                                    .scale(y)
                                    .ticks(2)
                                    .orient("left");

                                // main svg element
                                var graph = d3.select(chart_id)
                                    .append("svg:svg").attr("width", width + margin.left + margin.right).attr("height", height + margin.top + margin.bottom)
                                    .append("g")
                                    .attr("transform", "translate(" + margin.left + "," + margin.top + ")");

                                // draw x-axis
                                graph.append("g")
                                    .attr("class", "x axis")
                                    .attr("transform", "translate(0," + height + ")")
                                    .call(xAxis);

                                // draw y-axis
                                graph.append("g")
                                    .attr("class", "y axis")
                                    .call(yAxis)

                                // line object
                                var line = d3.svg.line()
                                    .x(function(d) { return x(new Date(d.date)); })
                                    .y(function(d) { return y(d.average); });

                                // path with data
                                graph.append("svg:path")
                                    .datum(dataBySensor)
                                    .attr("d", line)
                                    .attr("class", "sparklines");

                                // link to detail page
                                graph
                                    .append("svg:a").attr("xlink:href", function(d){ return window.location.origin + window.location.pathname + "#detail/location/" + sensor.get("name") + "/"})
                                    .append("svg:text")
                                    .text(function(d) { return sensor.get("name"); })
                                    .attr("y", - 5)
                                    .attr("x", - margin.left)
                                    .attr("text-anchor", "start");

                            }

                        });

                    });
                });
            }
        },

        hideVisualizations: function() {
            $('#searchResultsVisualizations').empty();
        }

    });
}

app.view.AboutView = Backbone.View.extend({
    id: "about-view",
    tag: "div",
    
    template: _.template($('#about-template').html()),

    initialize: function(options) {
        return this;
    },
    
    render: function() {
        this.$el.empty();
        this.$el.html(this.template({sources: app.collection.sources.getSourcesWithSensors().toJSON()}));
        return this;
    }
});
app.view.CompareView = Backbone.View.extend({
    id: "compare-view",
    tag: "div",

    template: _.template($('#compare-template').html()),

    events: {
        'change #compareDropdownList': 'renderSubViews'
    },

    initialize: function() {
        var self = this;
        app.collection.compareSelectedSensors = app.collection.compareSelectedSensors || new Backbone.Collection();
        this.collection = app.collection.compareSelectedSensors;

        this.collection.on('add', this.renderComparisonDescription, this);
        this.collection.on('remove', this.renderComparisonDescription, this);

        _.bindAll(this, "render", "renderSubViews");
        // define the subviews that appear on the CompareView
        // this sets up the relationship between the views for the controls and the graphs
        // and hands off to renderSubViews (below)
        if (app.config.views && app.config.views.compare) {
            this.subViews = app.config.views.compare.subviews;
        }

        app.view.compareSidebarView = new app.view.CompareSidebarView({
            // here we are passing an array of strings to the CompareSidebarView so it can display
            // a dropdown of our choices
            el: "#compare-sidebar",
            items: _.map(this.subViews, function(subview) { return subview.label; }),
            views: this.subViews
        });

        app.event.eventBus.on("sensors:refresh", function(){
            self.render();
        });

        return this;
    },

    render: function() {
        this.$el.html(this.template());
        app.view.compareSidebarView.setElement(this.$('#compare-sidebar')).render();
        this.renderSubViews();

        return this;
    },

    renderSubViews: function(view) {
        // the first time renderSubViews is called, there will be no view (view is actually an "event" sent by Backbone)
        // when a user clicks the dropdown, renderSubViews will contain the freeform text of the desired view (ie: "Lakes")
        // we match the text from the dropdown back to the label in this.subViews and render the appropriate subviews
        if (view && view.currentTarget) {
            // in this case, a view was passed, so map the text to an item in this.subViews
            // TODO: Check this. The tabViews should show up here.
            view = _.findWhere(this.subViews, {label: view.currentTarget.value });
        } else {
            // in this case, no view was passed, so just get the first item in this.subViews with a label
            view = _.find(this.subViews, function(subview) { return subview.label; });
        }

        // the variable "view" should now contain a text label and the correct templates for the sidebar and the graph

        // we don't have to empty $el, since that is handled by CompareSidebarControlsView
        // the initialize method in CompareSidebarControlsView calls render for us
        // CompareSidebarControlsView will insert this into div#compare-sidebar-controls
        // TODO: Check the multiple subview rendering here.
        if (!app.view["compareSidebarControlsView" + view.label]) {
            app.view["compareSidebarControlsView" + view.label] = new app.view.CompareSidebarControlsView({
                template: view.sidebar.template,
                label: view.label,
                clickableDOMItems: view.sidebar.clickableDOMItems
            });
        }
        this.$('#compare-sidebar-controls').append(app.view["compareSidebarControlsView" + view.label].render().el);


        // we don't have to empty $el, since that is handled by CompareGraphView
        // the initialize method in CompareGraphView calls render for us
        // CompareGraphView will insert this into div#compare-graph
        if (!app.view["compareGraphView" + view.label]) {
            app.view["compareGraphView" + view.label] = new app.view.CompareGraphView({
                template: view.graph.template
            });
        }
        app.view["compareGraphView" + view.label].setElement(this.$('#compare-graph')).render();
    },

    renderComparisonDescription: function() {
        var selectedSensors = app.collection.compareSelectedSensors.map(function(sensor) {
            return sensor.get('name');
        });

        var comparisonDescription = $('#comparison-description');
        var comparisonDescriptionItems = $('#comparison-description-items');

         if (selectedSensors.length == 0 ) {
            comparisonDescription.text("Compare Parameters");
            comparisonDescriptionItems.text('');
         } else if (selectedSensors.length > 0) {
            comparisonDescriptionItems.text(selectedSensors.join(" & "));
         }

         if (selectedSensors.length == 1) {
            comparisonDescription.text("Showing Parameters for ");
         } else if (selectedSensors.length > 1) {
            comparisonDescription.text("Comparing Parameters at ");
         }

         if (selectedSensors.length > 3) {
            comparisonDescriptionItems.addClass("small");
         } else {
           comparisonDescriptionItems.removeClass("small");
         }
    }
});

app.view.CompareSidebarView = Backbone.View.extend({

    template: _.template($('#compare-sidebar-template').html()),

    initialize: function(options) {
        var self = this;
        _.bindAll(this, "render", "renderMap");
        this.items = options.items;
        this.miniMap = app.config.views.compare.miniMap;

        app.collection.compareSelectedSensors = app.collection.compareSelectedSensors || new Backbone.Collection();
        this.collection = app.collection.compareSelectedSensors;

        this.collection.on('add', this.renderMap, this);
        this.collection.on('remove', this.renderMap, this);

        app.collection.onHoverSensors = app.collection.onHoverSensors || new Backbone.Collection();
        this.onHoverSensors = app.collection.onHoverSensors;
        this.onHoverSensors.on('add', this.renderMap, this);
        this.onHoverSensors.on('remove', this.renderMap, this);

        return this;
    },

    render: function() {
        this.$el.html( this.template() );
        this.map = L.map(self.$('#compare-sidebar-minimap')[0], {
            zoomControl: false,
            attributionControl: false,
            dragging: true,
            touchZoom: false,
            scrollWheelZoom: true,
            doubleClickZoom: false,
            boxZoom: false
        }).setView(this.miniMap.config.mapOptions.center, this.miniMap.config.mapOptions.zoom);
        this.renderMap();
        return this;
    },

    renderMap: function() {
      var self = this;

      var activeSensors = (app.collection.compareSelectedSensors) ? app.collection.compareSelectedSensors.toJSON() : [];

      if (self.geoJsonLayer) {
        self.map.removeLayer(self.geoJsonLayer);
      }

      var onHoverSensors = (app.collection.onHoverSensors) ? app.collection.onHoverSensors.toJSON() : [];
      if(self.onHoverLayer) {
        self.map.removeLayer(self.onHoverLayer)
      }
      self.onHoverLayer = L.geoJson(onHoverSensors, {
        pointToLayer: function(feature, latlng) {
          var badge = L.divIcon({className: 'badger-outer', html: '<div class="badger-inner-hover"><p class="badger-map-badge"> * </p></div>'});
          return L.marker(latlng, {icon: badge})
        }
      });

      self.onHoverLayer.addTo(self.map);
      self.geoJsonLayer = L.geoJson(activeSensors, {
        pointToLayer: function (feature, latlng) {
          var badge = L.divIcon({className: 'badger-outer', html: '<div class="badger-inner"><p class="badger-map-badge">'+ ( _.indexOf(activeSensors, feature) + 1) + '</p></div>'});
          return L.marker(latlng, {icon: badge});
        }
      });

      self.geoJsonLayer.addTo(self.map);

      

      L.tileLayer(self.miniMap.config.mapOptions.mapTileSource, {
          maxZoom: 18
      }).addTo(self.map);

      self.$("#" + this.miniMap.id).html(self.map.el);

    }
});

app.view.CompareSidebarControlsView = Backbone.View.extend({
    el: "div#compare-sidebar-controls",
    // required arguments:
    // template: the script id for the template this will use
    // label: a string or variable that resolves to a string that is used to show in the dropdown
    // ie: new app.view.CompareByTypeView({
    //      template: "#compare-sidebar-lakes-template
    //      label: app.config.areas.title
    // });
    // this view should be called after it's parent is rendered

    initialize: function(options) {
        this.label = options.label;
        this.clickableDOMItems = options.clickableDOMItems;
        this.template = _.template($(options.template).html());
        // unbind click prevents the duplicate "clicking" of the event
        this.$el.unbind('click');

        app.collection.compareSelectedSensors = app.collection.compareSelectedSensors || new Backbone.Collection();
        this.collection = app.collection.compareSelectedSensors;

        app.collection.onHoverSensors = app.collection.onHoverSensors || new Backbone.Collection();
        
        // this.collection.on('add', this.render, this);
        // this.collection.on('remove', this.render, this);

        return this;
    },

    render: function() {
        this.$el.html(
            this.template({
                label: this.label,
                sources: app.collection.sources.getAreas(),
                activeItems: app.collection.compareSelectedSensors.pluck('id')
            })
        );

        // we could have setup the defaults "events" object, but this allows us to store in app.config what we want to be clickable
        this.$(this.clickableDOMItems).on('click', this.clickItem);
        this.$(this.clickableDOMItems).on('mouseenter', this.mouseEnter);
        this.$(this.clickableDOMItems).on('mouseleave', this.mouseLeave);
        this.$('.nodata-wrapper').tooltip({delay: { show: 150, hide: 100 }});
        this.$('.showHide').on('click', this.toggleButton);
        return this;

    },

    toggleButton: function() {
        if(this.innerHTML.indexOf("+") >=0) {
            this.innerHTML= "-";
        } else {
            this.innerHTML="+";
        }
    },
    mouseEnter: function() {
        event.preventDefault();
        
        //The id of the sensor
        var sensor = app.collection.sensors.findWhere({name: event.currentTarget.value});
        app.collection.onHoverSensors.add(sensor);
        var clickedOrder = _.indexOf(app.collection.onHoverSensors.models, sensor);
        if (clickedOrder > -1) {
          $(this).append('<div class="badger-outer-hover" id="Badger_'+clickedOrder+'"><div class="badger-inner-hover"><p class="badger-badge" id="Badger_'+clickedOrder+'_badge">*</p></div></div>');
        } 
    },
    mouseLeave: function() {
        event.preventDefault();
        $(this).find('.badger-outer-hover').remove();
        var sensor = app.collection.sensors.findWhere({name: event.currentTarget.value});
        app.collection.onHoverSensors.remove(sensor); 
    },

    clickItem: function(event) {
        // preventDefault ensures that the button doesn't do its default (ie: we control what it should do)
        event.preventDefault();

        // start the loading spinner
        app.event.eventBus.trigger("compare:spinner:start");

        // if the sensor is already active when this click begins, this is an "unclick"
        if ($(this).hasClass('active')) {
          $(this).find('.badger-outer').remove();
        }
        
        $(this).toggleClass('active');
        
        // the name of the sensor:
        // console.log(event.currentTarget.value);

        // the id of the sensor (in case we'd want to directly query the datapoints endpoint later:
        // console.log(event.currentTarget.getAttribute('data-id'));
        // or it's jQuery equivalent:
        // console.log($(event.target).data('id'));
        var sensor = app.collection.sensors.findWhere({name: event.currentTarget.value});

        if (sensor) {
            // if the clicked sensor was already selected, we will remove it immediately
            if (app.collection.compareSelectedSensors.contains(sensor)) {
                app.collection.compareSelectedSensors.remove(sensor);

                // now we need to determine if we can remove any of this sensor's parameters from the selection
                // other sensors might still be selected and they might offer the same parameters as this sensor
                // we will look at the remaining sensors to determine parameters that should still be selected
                var remainingParameters = [];
                app.collection.compareSelectedSensors.each(function(s){
                    s.get('parameters').each(function(p) {
                        remainingParameters.push(p.id);
                    });
                });

                remainingParameters = _.uniq(remainingParameters);

                // if there are no more parameters available,
                // then we can infer that all of the sensors have been unclicked, so reset everything
                if (!remainingParameters.length) {
                    app.collection.parameterBins.reset();
                    app.collection.compareSelectedDates.reset();
                    app.collection.compareSelectedSensors.reset();
                    app.collection.compareSelectedParameters.reset();
                } else {
                    // in this case, we have unchecked a sensor, but there are other parameters available,
                    // so we need to remove only the parameters that were unique to this sensor
                    sensor.get('parameters').each(function(parameter){
                        if (!_.contains(remainingParameters, parameter.get('id'))) {
                            app.collection.compareSelectedParameters.remove(parameter);
                        }
                    });
                }

                // stop the loading spinner
                app.event.eventBus.trigger("compare:spinner:stop");
                return this;
            } else {
                // the user is clicking this sensor for the first time
                app.collection.compareSelectedSensors.add(sensor);
            }
        }

        // this.binningTime will be something like 'smart', 'semi', 'yearly', etc. It is how the datapoints
        // are averaged before being drawn in the graphs.
        // Projects can setup a filterFunction that accepts the current sensor (the Backbone Sensor Model)
        // as an argument. From there, you can return 'smart', 'semi', or some other binning value
        // based on a condition. For example: return (sensor.get('type')['id'] == 'epa') ? 'semi' : 'smart';
        // would return 'semi' as the binningTime for epa sensors, all others would use 'smart' binning
        this.binningTime = "";
        if (app.config.binning.filterFunction) {
          this.binningTime = app.config.binning.filterFunction(app.collection.sensors.findWhere({name: event.currentTarget.value}));
        } else if (app.config.binning.time) {
          this.binningTime = app.config.binning.time;
        } else {
          this.binningTime = 'smart';
        }

        // If this.binningTime is 'smart', then determine aggregation by temporal range
        if (this.binningTime == 'smart') {
            // getAggregationBin is a global function in DetailView - that should probably change, but it works ok
            // this will just tell us if we should bin by year, month, day, etc.
            this.binningTime = getAggregationBin(new Date(sensor.get('min_start_time')),
              new Date(sensor.get('max_end_time')));
        }

        // request the bin endpoint
        app.model.geoStreamService.getBinnedData(this.binningTime, 1, sensor.get('id'), true, function(data) {
            // the bin endpoint will return a bunch of parameters in the "properties" object
            // each parameter uses an "id" format, but it can also be an original formatted string, as it came
            // from the source, so it might be one of the alternate spellings found in the "alternates"
            // array for a parameter
            _.each(data[0]["properties"], function(values, parameterID) {

                // if the parameter contains data...
                if (data[0].properties[parameterID].length) {

                    // search for this parameter against the collection by id
                    var parameterModel = app.collection.parameters.findWhere({id: parameterID});

                    // if it wasn't found by id, check to see if it matches one of the parameters alternate spellings
                    // if no match is found, we can go on to the next parameter
                    if (_.isUndefined(parameterModel)) {
                        parameterModel = app.collection.parameters.filter(function(p) {
                            return _.contains(p.get('alternates'), parameterID);
                        });
                        if (parameterModel.length) {
                            parameterModel = parameterModel[0];
                        } else {
                            console.log("CompareSidebarControlsView:clickItem - parameterModel was not found for " + parameterID);
                            return;
                        }
                    }

                    // make sure we aren't using a parameter that is either ignored or a depth-based parameter
                    if (parameterModel.get('detailView') == 'time') {
                        // adding to the compareSelectedParameters collection will trigger an event internally
                        // this collection is listened to by views and will take care of updating lists or graphs
                        // whenever this changes
                        app.collection.compareSelectedParameters.add(parameterModel.toJSON());
                    }

                    // a parameterBin is a way of organizing data so that we can compare multiple
                    // sensors for the same parameter. Since these are two separate calls to the API, we persist
                    // them in a collection. A parameterBin is organized like this:
                    //   id: the parameter id. If an alternate was used, we conform it to the ID above as parameterModel
                    //   dates: a Backbone Collection.
                    //         - each model in this collection uses the unique date string as it's ID
                    //         - each sensor that has a parameter on this date will be an attribute on this model.
                    //         - ie: sensor 1 will show up as an attribute: "1: 0.5", where 1 is the sensor's ID
                    //               and 0.5 is the value returned by the bin endpoint for this parameter and parameter

                    // the "add" method on a collection won't duplicate a unique ID.
                    var parameterBin = app.collection.parameterBins.add({id: parameterModel.get('id')});
                    // create the dates collection if it doesn't already exist
                    if (!parameterBin.get('dates')) {
                        parameterBin.set('dates', new Backbone.Collection());
                    }

                    // inside the response from the bin endpoint, this parameter will contain several values and metadata
                    // we are interested in the "date" and "average" values
                    for (var i = 0; i < data[0]["properties"][parameterID].length; i++) {
                        var parameterBinDatesCollection = parameterBin.get('dates');

                        // add this date as a unique ID in the parameterBin's "dates" collection
                        var selectedDate = parameterBinDatesCollection.add({
                            id: data[0]["properties"][parameterID][i]['date']
                        });

                        // also add this to the compareSelectedDates collection
                        // so that our slider range gets updated
                        app.collection.compareSelectedDates.add({id: data[0]["properties"][parameterID][i]['date']});

                        // set the value of the "average" on this date: {1: 0.5}
                        // any additional sensors with measurements of this parameter at this time will get added here
                        selectedDate.set(sensor.get('id'), data[0]["properties"][parameterID][i]['average']);
                    }

                }
            });
            // if we listen directly to the parameterBins collection, it would fire events too aggressively, even before
            // we are done assessing the data. Instead, wait to notify until after all of the parameters have been parsed above.
            app.event.eventBus.trigger("parameterBins:loaded");

            // stop the loading spinner
            app.event.eventBus.trigger("compare:spinner:stop");
        });

        var clickedOrder = _.indexOf(app.collection.compareSelectedSensors.models, sensor);
        if (clickedOrder > -1) {
          clickedOrder++;
          $(this).append('<div class="badger-outer" id="Badger_'+clickedOrder+'"><div class="badger-inner"><p class="badger-badge" id="Badger_'+clickedOrder+'_badge">'+clickedOrder+'</p></div></div>');
        }
    }
});

app.view.CompareGraphView = Backbone.View.extend({
    className: "compare-graph",
    // template must be passed in when creating an instance of this view
    // ie: new app.view.CompareGraphView({template: "#compare-graph-lakes-template});
    // this view should be called after it's parent is rendered

    initialize: function(options) {
        var self = this;
        this.template = _.template($(options.template).html());

        // this view will listen to the compareSelectedSensors collection
        app.collection.compareSelectedSensors = app.collection.compareSelectedSensors || new Backbone.Collection();
        this.collection = app.collection.compareSelectedSensors;

        // create the three parameter dropdowns
        // they will listen to changes in the compareSelectedParameters collection
        this.parameterDropdown1 = new app.view.SelectCollectionView({
          id: "compare-parameter-1",
          className: "span4 margin21R",
          trigger: "change:compare:parameter:1",
          collection: app.collection.compareSelectedParameters
        });
        this.parameterDropdown2 = new app.view.SelectCollectionView({
          id: "compare-parameter-2",
          className: "span4 margin21R",
          trigger: "change:compare:parameter:2",
          placeholder: "Additional Parameter",
          collection: app.collection.compareSelectedParameters
        });
        this.parameterDropdown3 = new app.view.SelectCollectionView({
          id: "compare-parameter-3",
          className: "span4",
          trigger: "change:compare:parameter:3",
          placeholder: "Additional Parameter",
          collection: app.collection.compareSelectedParameters
        });

        // when a parameter changes, update the corresponding graph
        app.event.eventBus.on('change:compare:parameter:1', function(data) {
          if (data && data.target && data.target.value) {
            self.graph1.update(data.target.value);
          }
        });
        app.event.eventBus.on('change:compare:parameter:2', function(data) {
            if (data && data.target && data.target.value) {
              self.graph2.update(data.target.value);
            }
        });
        app.event.eventBus.on('change:compare:parameter:3', function(data) {
            if (data && data.target && data.target.value) {
              self.graph3.update(data.target.value);
            }
        });

        // when a sensor is unclicked, update each graph based on the selected parameter
        this.collection.on('remove', function() {
          self.graph1.update($("#compare-parameter-1").val());
          self.graph2.update($("#compare-parameter-2").val());
          self.graph3.update($("#compare-parameter-3").val());
        }, this);


        // on a pristine page, if you click a sensor, this will load the graph for the first parameter
        app.event.eventBus.on("parameterBins:loaded", function() {
          self.graph1.update($("#compare-parameter-1").val());
          self.graph2.update($("#compare-parameter-2").val());
          self.graph3.update($("#compare-parameter-3").val());
        });

        // setup a loading spinner we can turn on and off
        var spinner = app.config.views.compare.spinnerOptions || {
            lines: 13 // The number of lines to draw
            , length: 28 // The length of each line
            , width: 14 // The line thickness
            , radius: 42 // The radius of the inner circle
            , scale: 1 // Scales overall size of the spinner
            , corners: 1 // Corner roundness (0..1)
            , color: '#000' // #rgb or #rrggbb or array of colors
            , opacity: 0.25 // Opacity of the lines
            , rotate: 0 // The rotation offset
            , direction: 1 // 1: clockwise, -1: counterclockwise
            , speed: 1 // Rounds per second
            , trail: 60 // Afterglow percentage
            , fps: 20 // Frames per second when using setTimeout() as a fallback for CSS
            , zIndex: 2e9 // The z-index (defaults to 2000000000)
            , className: 'spinner' // The CSS class to assign to the spinner
            , top: '60%' // Top position relative to parent
            , left: '57%' // Left position relative to parent
            , shadow: false // Whether to render a shadow
            , hwaccel: false // Whether to use hardware acceleration
            , position: 'absolute'
          };
        this.spinner = new Spinner(spinner);

        // the spinner isn't appended to the DOM until the first time it gets called.
        app.event.eventBus.on("compare:spinner:start", function() {
          $("#compare-parameter-graph-1").append(self.spinner.spin().el);
        });
        // use the eventBus to stop the spinner from anywhere
        app.event.eventBus.on("compare:spinner:stop", function() {
          self.spinner.stop();
        });

        // compareSelectedDates keeps track of the date range available to the graphs
        // whenever those dates change, update the date slider
        app.collection.compareSelectedDates.on('all', function() {
          var slider = $("#slider-range-stations");
          var minBounds = new Date(new Date().getFullYear() - 44, new Date().getMonth(), new Date().getDate());
          var minSelected = new Date(new Date().getFullYear() - 10, new Date().getMonth(), new Date().getDate());
          var last = new Date();
          // if compareSelectedDates has models, then we have data to look at and graph.
          // set the bounds to that of the compareSelectedDates collection and move the selection to the bounds too
          if (app.collection.compareSelectedDates.length) {
            minBounds = minSelected = new Date(app.collection.compareSelectedDates.first().get('id'));
            last = new Date(app.collection.compareSelectedDates.last().get('id'));
          }
          // bounds: the range of available dates
          slider.dateRangeSlider("bounds", minBounds, last);
          // values: the selected start and end times
          slider.dateRangeSlider("values", minSelected, last);

        });

        return this;
    },

    render: function() {
        this.$el.empty();
        this.$el.append( this.template() );
        if (app.config.views.compare.removeBinDropdown) {
            this.$('#compare-season-dropdown').remove();
        }
        this.renderParameterDropdowns(this);

        // create a dateRangeSlider, setting the bounds fairly large, and selecting the last year.
        // once data starts getting selected, this will get udpated
        var sliderRange = $( "#slider-range-stations" );
        sliderRange.dateRangeSlider({
          bounds: {
            min: new Date(new Date().getFullYear() - 44, new Date().getMonth(), new Date().getDate()),
            max: new Date()
          },
          defaultValues: {
            min: new Date(new Date().getFullYear() - 10, new Date().getMonth(), new Date().getDate()),
            max: new Date()
          },
          step: {
            days: 1
          },
          arrows: false
        });

        // when the slider range changes, data contains data.values.min and data.values.max
        sliderRange.on("userValuesChanged", function(e, data) {
          app.event.eventBus.trigger("compare:date:changed", data);
        });

        // create the three graphs in their empty state and render them
        this.graph1 = new app.view.CompareGraphMGTimeSeriesView({id: "#compare-parameter-graph-1"});
        this.graph2 = new app.view.CompareGraphMGTimeSeriesView({id: "#compare-parameter-graph-2"});
        this.graph3 = new app.view.CompareGraphMGTimeSeriesView({id: "#compare-parameter-graph-3"});
        this.graph1.render();
        this.graph2.render();
        this.graph3.render();

        return this;
    },

    renderParameterDropdowns: function(self) {
        var container = this.$('#availableParametersDefaultTitle');

        container.append(self.parameterDropdown1.render().el);
        container.append(self.parameterDropdown2.render().el);
        container.append(self.parameterDropdown3.render().el);

        self.$('#compare-parameter-1').on('change', function(event) {
          self.parameterDropdown1.setSelected(event.target.value);
          app.event.eventBus.trigger('change:compare:parameter:1', event);
        });
        self.$('#compare-parameter-2').on('change', function(event) {
          self.parameterDropdown2.setSelected(event.target.value);
          app.event.eventBus.trigger('change:compare:parameter:2', event);
        });
        self.$('#compare-parameter-3').on('change', function(event) {
          self.parameterDropdown3.setSelected(event.target.value);
          app.event.eventBus.trigger('change:compare:parameter:3', event);
        });
      return self;

    }
});

app.view.CompareGraphTimeSeriesView = Backbone.View.extend({
//    el: "div#compare-graph",
    // template must be passed in when creating an instance of this view
    // ie: new app.view.CompareGraphView({template: "#compare-graph-lakes-template});
    // this view should be called after it's parent is rendered

    initialize: function(options) {
        var self = this;
        this.template = _.template('<h4><%= name %></h4>');
        _.bindAll(this, "render", "setData", "getData");
        this.timeSeriesView = new app.view.TimeSeriesView({
            parentView : this
        });
        app.event.eventBus.on("compareGraph:data:" + this.model.id, this.setData);
        app.event.eventBus.on("change:compare:parameter", function(event) {
            self.setData(self.model, event);
        });

        $("#compare-hide-box-plot").on('click', function() {
            $(".box-whisker-chart").toggle();
        });

		    $("#compare-season-dropdown").on('change', function() {
			      self.setData(self.model, null, $(this).val());
		    });

        $("#slider-range-stations").on('slidechange', function() {
            self.setData(self.model);
        });


        return this;
    },

    render: function() {

        this.$el.empty();
        this.$el.html( this.template({name: this.model.get('popupContent') }) );
        this.getData(this.model);

        this.modals = _.template($("#box-whisker-modal-template").html())();

        this.$el.append(this.modals);
        return this;
    },

    append: function() {
      this.$el.append(this.timeSeriesView.render().el);

    },

    getData: function(model) {
        var self = this;
        if (!self.data) {
          app.model.geoStreamService.getByLocation(model.toJSON(), function(data) {
              self.data = data;
              app.event.eventBus.trigger("compareGraph:data:" + model.id, model);
          });
        } else {
          app.event.eventBus.trigger("compareGraph:data:" + model.id, model);
        }
    },

    setData: function(model, parameter, season) {
        season = season || $("#compare-season-dropdown").val();
        var self = this;
        if (model.id != this.model.id) return;
        if (!this.data) return;

        var parameter1 = $('.compare-parameter-1').val();
        var parameter2 = $('.compare-parameter-2').val();
		var parameter3 = $('.compare-parameter-3').val();

        var parameters = [];
        _.each([parameter1, parameter2, parameter3], function(parameter) {
          if (parameter && parameter.length) {
            parameters.push(parameter);
          }
        });

	    var startTime = new Date($("#slider-range-stations").slider('values')[0], null);
        var endTime = new Date($("#slider-range-stations").slider('values')[1], 11, 31, 23, 59, 59);

        // create time series values
        // 'data' is the original data from the API. Here we're just filtering it down to just the data with "detailView" of "time"
        var time_series_data = _.where(this.data, {detailView: "time"});
        if (time_series_data) {
            // convert data to following data structure. We need two
            // of these structures in summer, one with all values for
            // box-whisker plot and one with only EP for time series.
            // time-series data: [ {
            //   key: "0",
            //   values: [ {
            //     color: "red",
            //     key: "parameter name",
            //     sources: [ sources ],
            //     values: [ {
            //       x: date,
            //       y: value
            //     }]
            //   }]
            // }]

            // get a unique list of the parameters found in the time data
            var time_series_params = _.unique(_.pluck(_.where(time_series_data, {detailView: "time"}), "parameter"));

            // if a parameter is selected, but this sensor doesn't have that parameter, omit it
            _.each(parameters, function(parameter) {
                if (!_.contains(time_series_params, parameter)) {
                  parameters = _.without(parameters, parameter);
                }
            });

            var parameterTitles = app.collection.parameters.where({detailView: "time"});
            parameterTitles = _.pluck(parameterTitles, "id");
            // use d3 to create colors for each of the unique parameters
            var parameters_colors = parameterTitles.length > 10 ? d3.scale.category20().domain(parameterTitles) : d3.scale.category10().domain(parameterTitles);

            var time_season_data = this.data;
            if (!_.isUndefined(season) && (season != null) && (season != "all") && (season != "")) {
                time_season_data = _.where(this.data, {season: season});
            }

            // alldata is where we'll stuff all the time_series_data as we iterate through the time_series_params
            // summerepdata is where we'll add all spring data, but only summer ep data
            var alldata = [], summerEPdata = [];

            // sources is where a list of sources are kept, pointing back to clowder
            var sources = [];


          // if no start or endtime is given compute start/end time
            if (_.isUndefined(startTime) || _.isUndefined(endTime)) {
                _.each(this.data, function(d) {
                    if (_.isUndefined(startTime) || (startTime > d.date)) startTime = d.date;
                    if (_.isUndefined(endTime) || (endTime < d.date)) endTime = d.date;
                });
            }


            var filterWrongDates = function(dataset, startTime, endTime) {
                // filter any wrong date information
                if (!_.isUndefined(startTime) || !_.isUndefined(endTime)) {
                    dataset = _.filter(dataset, function (x) {
                        if (!_.isUndefined(startTime)) {
                            if (startTime.getUTCFullYear() > x.year) {
                                return false;
                            }
                            if (startTime.getUTCFullYear() == x.year) {
                                if (!_.isUndefined(x.month)) {
                                    if (startTime.getUTCMonth() > x.month) {
                                        return false;
                                    }
                                    if (startTime.getUTCMonth() == x.month) {
                                        if (!_.isUndefined(x.day)) {
                                            if (startTime.getUTCDate() > x.day) {
                                                return false
                                            }
                                            if (startTime.getUTCDate() == x.day) {
                                                if (!_.isUndefined(x.hour)) {
                                                    if (startTime.getUTCHours() > x.hour) {
                                                        return false;
                                                    }
                                                    if (startTime.getUTCHours() == x.hour) {
                                                        if (!_.isUndefined(x.minute) && startTime.getUTCMinutes() > x.minute) {
                                                            return false;
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        if (!_.isUndefined(endTime)) {
                            if (endTime.getUTCFullYear() < x.year) {
                                return false;
                            }
                            if (endTime.getUTCFullYear() == x.year) {
                                if (!_.isUndefined(x.month)) {
                                    if (endTime.getUTCMonth() < x.month) {
                                        return false;
                                    }
                                    if (endTime.getUTCMonth() == x.month) {
                                        if (!_.isUndefined(x.day)) {
                                            if (endTime.getUTCDate() < x.day) {
                                                return false
                                            }
                                            if (endTime.getUTCDate() == x.day) {
                                                if (!_.isUndefined(x.hour)) {
                                                    if (endTime.getUTCHours() < x.hour) {
                                                        return false;
                                                    }
                                                    if (endTime.getUTCHours() == x.hour) {
                                                        if (!_.isUndefined(x.minute) && endTime.getUTCMinutes() < x.minute) {
                                                            return false;
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        return true;
                    });
                }
                return [dataset, startTime, endTime];
            };

          var data_filtered_for_dates = filterWrongDates(time_season_data, startTime, endTime);
          startTime = data_filtered_for_dates[1];
          endTime = data_filtered_for_dates[2];
          data_filtered_for_dates = data_filtered_for_dates[0];

          // iterate over each of the unique parameters
            var foundEP = false;
            for (var i = 0; i < parameters.length; i++) {
                // 'parameters' is what gets passed in via the URL (ie: the Backbone router gives us these)
                // if there are parameters in the URL, but the param we are on isn't one of them, skip it
//                if (!_.isEmpty(parameters) && (_.contains(parameters, time_series_params[i]))) {
//                    parameters = [time_series_params[i]];
//                }

                // create a 'valuesAll' object. It will filter the time_series_data to get this one param we are on,
                // creating a sum and count of the number of datapoints so we can create an average. The same is done
                // for the 'valuesEPsummer' however in this case all values in the summer that have depthcode that is
                // NOT EP will be ignored.
                var valuesAll = [], valuesEPsummer = [];
                _.each(_.where(data_filtered_for_dates, {parameter: parameters[i]}), function (x) {
                    var countAll = 0, countEPdata = 0;
                    var sumAll = 0, sumEPdata = 0;
                    var rawAll = [], rawEP = [];
                    _.each(x.values, function (v) {
                        countAll += v.count;
                        sumAll += v.value;
                        rawAll.push(v.raw);
                        sources.push(v.source);
                        if (x.season == "summer" && !_.isUndefined(v.depthcode) && (v.depthcode == "EP")) foundEP = true;
                        if ((x.season != "summer") || _.isUndefined(v.depthcode) || (v.depthcode == "EP") || (v.depthcode == "NA")) {
                            countEPdata += v.count;
                            sumEPdata += v.value;
                            rawEP.push(v.raw);
                        }
                    });
                    valuesAll.push({ x: x.date, y: sumAll / countAll, label: x.label, raw: _.flatten(rawAll) });
                    if (countEPdata != 0) {
                        valuesEPsummer.push({ x: x.date, y: sumEPdata / countEPdata, label: x.label, raw: _.flatten(rawEP) });
                    }
                });
                var yrange = d3.extent(valuesAll.concat(valuesEPsummer).concat({ y: 0}), function (d) {
                    return d.y;
                });
                var xrange = [startTime, endTime];

                // push the computed values into alldata/summerepdata
                alldata.push({
                    key: i.toString(),
                    values: [
                        {
                            color: parameters_colors(parameters[i]),
                            key: parameters[i],
                            xrange: xrange,
                            yrange: yrange,
                            values: valuesAll,
                            sources: _.uniq(_.flatten(sources)),
                            unit: ((app.collection.parameters.findWhere({id: parameters[i]})) ? (app.collection.parameters.findWhere({id: parameters[i]})).get("unit") : 'unit')
                        }
                    ]
                });
                if (!_.isEmpty(valuesEPsummer)) {
                    summerEPdata.push({
                        key: i.toString(),
                        values: [
                            {
                                color: parameters_colors(parameters[i]),
                                key: parameters[i],
                                xrange: xrange,
                                yrange: yrange,
                                values: valuesEPsummer,
                                unit: ((app.collection.parameters.findWhere({id: parameters[i]})) ? (app.collection.parameters.findWhere({id: parameters[i]})).get("unit") : 'unit')
                            }
                        ]
                    });
                }
            }

            // sends the formatted data to generate the graphs
            if ((season == "spring") || !foundEP) {
                self.timeSeriesView.setData(alldata, undefined);
            } else {
                self.timeSeriesView.setData(alldata, summerEPdata);
            }


            self.append();
//                self.controlsView.setParameters(parameters, _.isEmpty(parameters) ? time_series_params : parameters, time_series_params_colors);

        }

    }
});

app.view.CompareGraphMGTimeSeriesView = Backbone.View.extend({
//    el: "div#compare-graph",
    // template must be passed in when creating an instance of this view
    // ie: new app.view.CompareGraphView({template: "#compare-graph-lakes-template});
    // this view should be called after it's parent is rendered

    initialize: function(options) {
        var self = this;
        // the MG graph below will reference the "result" and "legend" classes to insert the graph
        this.template = _.template('<div class="result"></div><div class="legend"></div>');
        _.bindAll(this, "render");

	    var timeFormats = [
		    [d3.time.format("%Y"), function() {
			    return true;
		    }],
		    [d3.time.format("%B"), function(d) {
			    return d.getMonth();
		    }],
		    [d3.time.format("%b %d"), function(d) {
			    return d.getDate() != 1;
		    }],
		    [d3.time.format("%a %d"), function(d) {
			    return d.getDay() && d.getDate() != 1;
		    }],
		    [d3.time.format("%I %p"), function(d) {
			    return d.getHours();
		    }],
		    [d3.time.format("%I:%M"), function(d) {
			    return d.getMinutes();
		    }],
		    [d3.time.format(":%S"), function(d) {
			    return d.getSeconds();
		    }],
		    [d3.time.format(".%L"), function(d) {
			    return d.getMilliseconds();
		    }]
	    ];
	    var timeFormatPicker = function (formats) {
		    return function (date) {
			    var i = formats.length - 1, f = formats[i];
			    while (!f[1](date)) {
				    f = formats[--i];
			    }
			    return f[0](date);
		    };
	    };

	    var xax_format = timeFormatPicker(timeFormats);

        // default options to show an intentionally empty graph.
        // This indicates to the user that no selection has been made.
        this.graphOptions = {
            title: 'Parameter',
            width: 900,
            height: 400,
            target: this.id,
            missing_text: "Click a " + ((app.config.sensors.title) ? app.config.sensors.title.toLowerCase().substring(0, app.config.sensors.title.length - 1) : "sensor") + " to generate a graph" ,
            y_label: "Average",
            y_extended_ticks: true,
            x_accessor: 'date',
            show_secondary_x_label:false,
	        xax_count: 10,
            xax_format: xax_format,
            legend_target: this.id + "-legend",
            chart_type: 'missing-data',
            aggregate_rollover: true
        };

        // this gets fired when the sliders are either moved by a user or reset by the app
        // if a user clicks on a sensor, then the dates get updated as we change the bounds
        app.event.eventBus.on("compare:date:changed", function(data) {
            // here we are calling update(parameterID, min, max)
            // since the eventBus doesn't pass in the value of this parameter, we need to get it from the DOM
            // this view's ID is "#compare-parameter-graph-1" (or 2 or 3), so we extract the number to create
            // "#compare-parameter-1" so we can get that value from the DOM
            self.update($("#compare-parameter-" + self.id.substr(self.id.length - 1, 1)).val(), data.values.min, data.values.max);
        });

        return this;
    },

    render: function() {
        // render is intentionally pretty sparse. This uses the MetricsGraphics library to call data_graphic.
        // all that does is inserts the graph into the DOM using the options we specified in the initialize function.
        // anytime we re-render this graph, it will default to this "no data" view.
        MG.data_graphic(this.graphOptions);

        return this;

    },

    update: function(parameterID, min, max) {
        // parameterID should be the ID of the parameterModel, but it can also be one of the alternate strings
        // min and max are date objects
        var self = this;
        var compareDateRangeSlider = $( "#slider-range-stations" );
        // if min or max isn't passed, get the values directly from the slider's built-in functions ("min", "max")
        min = min || compareDateRangeSlider.dateRangeSlider("min");
        max = max || compareDateRangeSlider.dateRangeSlider("max");

        // if we don't have a parameterID, then there is nothing to graph
        if (!parameterID) return self.render();

        // rendering begins here
        this.$el.html( this.template() );

        // find a model for this parameter. If ID doesn't find it, check the alternates, otherwise exit.
        var parameterModel = app.collection.parameters.findWhere({id: parameterID});
        if (_.isUndefined(parameterModel)) {
            parameterModel = app.collection.parameters.filter(function(p) {
                return _.contains(p.get('alternates'), parameterID);
            });
            if (parameterModel.length) {
                parameterModel = parameterModel[0];
            } else {
                return;
            }
        }

        // parameterBins are created when you click a sensor in CompareSidebarControlsView
        // each parameterBin uses the parameter ID as it's unique ID
        var parameterBin = app.collection.parameterBins.findWhere({id: parameterModel.get('id')});

        // we have the desired parameter, but not all sensors will have this parameter
        // if we don't filter out the sensors without the parameter, they will still be in the list
        // and they will show up in the legend, and may show the graph from the previously selected parameter
        var sensorsWithTheSelectedParameter = app.collection.compareSelectedSensors.filter(function(sensor) {
            return sensor.get('parameters').findWhere({id: parameterBin.get('id') });
        });

        // a parameterBin keeps track of all data for a parameter, from multiple sensors across a range of dates
        // we only want to graph sensors that are still selected, and have data for the selected parameter
        // so we will create an array of names and IDs that we can filter against below
        var selectedSensorNames = _.map(sensorsWithTheSelectedParameter, function(sensor) {
            return sensor.get('name');
        });

        var selectedSensorIDs = _.map(sensorsWithTheSelectedParameter, function(sensor) {
            return sensor.get('id');
        });

        // we will store the data from the parameterBin's dates collection in a local variable
        // these are all the values for each sensor, and we will format them before giving them to
        // MetricsGraphics to create the graph
        var data = [];
        if (!parameterBin || !parameterBin.get('dates')) {
            // if we never found a parameterBin for the parameter passed in, or it has no dates (ie: no data)
            // reset this graph and exit
            self.render();
            return this;
        }

        // the parameter will have a dates collection, but as the slider moves, we want to filter out those dates
        parameterBin.get('dates').each(function (bin) {
            var binDate = new Date(bin.get('id'));
            // only use dates if they are within the slider range
            if (min && max && ((min < binDate) && (binDate < max))) {

                app.collection.compareSelectedDates.add({id: bin.get('id')});

                var response = {"date": binDate};
                _.each(selectedSensorIDs, function(sensorID) {
                    if (bin.get(sensorID)) {
                        response[sensorID] = bin.get(sensorID);
                    }
                });
                if (response.date && (Object.keys(response).length > 1)) {
                    data.push(response);
                }
            }
        });

        var graphOptions = _.clone(self.graphOptions);

        if (data.length > 1) {
            graphOptions.title = parameterModel.get('title') + " " + parameterModel.get('unit');
            graphOptions.chart_type = app.config.graphs.compareGraphs.chartType;
            graphOptions.point_size = app.config.graphs.compareGraphs.pointSize;
            graphOptions.interpolate = 'linear';
            graphOptions.data = data;
            graphOptions.y_accessor = selectedSensorIDs;
            graphOptions.legend = selectedSensorNames;
            graphOptions.show_points_on_line_graphs = app.config.graphs.compareGraphs.showPointsOnLineGraphs;
            graphOptions.transition_on_update = !app.config.graphs.compareGraphs.showPointsOnLineGraphs;
        } else {
            // if no data, render this graph with the default options and exit
            // the default options are placeholder options that say 'no data'
            return self.render();
        }

        MG.data_graphic(graphOptions);

        var showPointsCheckbox = document.getElementById('show-points');

        if (showPointsCheckbox) {
            showPointsCheckbox.addEventListener('click', clickedShowPointsCheckbox);
            clickedShowPointsCheckbox();
        }

        function clickedShowPointsCheckbox(){
            var pointOnGraphs = document.getElementsByTagName('circle');
            if (showPointsCheckbox) {
                if (showPointsCheckbox.checked && pointOnGraphs) {
                    for (var i = 0; i < pointOnGraphs.length; i++) {
                        pointOnGraphs[i].style.visibility = "visible";
                    }
                }
                else if (pointOnGraphs) {
                    for (var j = 0; j < pointOnGraphs.length; j++) {
                        pointOnGraphs[j].style.visibility = "hidden";
                    }
                }
            }
        }

        return this;

    }

});

app.view.FooterView = Backbone.View.extend({
    el: "#footer-container",
    
    initialize: function(options) {
        this.template = _.template($('#footer-template').html());
        this.render();
        return this;
    },
    
    render: function() {
        this.$el.empty();
        this.$el.html(this.template);
        return this;
    }
});
app.view.NavBarView = Backbone.View.extend({
    el: "#navigation",

    initialize: function() {

        this.template = _.template("<li><a href='<%= link %>'><%= title %></a></li>");

        this.dropdownTemplate = _.template(
            "<li>" +
                "<div class='dropdown'>" +
                    "<a class='dropdown-toggle' data-toggle='dropdown' href='<%= link %>'><%= title %> " +
                        "<b class='caret'></b>" +
                    "</a>" +
                    "<ul class='dropdown-menu' role='menu' aria-labelledby='dLabel'>" +
                        "<% if(title_link == null) {%> "+
                            "<li><a href='<%= link %>'><%= title %></a></li>" +
                        "<%};%>" +
                        "<% _.each(children, function(item) { %>" +
                            "<li>" +
                                "<a href='<%= link %>/<%= item.link.replace('#', '') %>'>" +
                                    "<%= item.title.toUpperCase() %></a>" +
                            "</li>" +
                        "<% }); %>" +
                    "</ul>" +
                "</div>" +
            "</li>");
    },

    render: function() {
        var self = this;

        _.each(app.config.views, function(item) {
            if (item.title && item.link) {
                if (item.children) {
                    self.$el.append(self.dropdownTemplate({
                        title: item.title.toUpperCase(),
                        link: item.link,
                        children: item.children,
                        title_link: item.title_link
                    }));
                } else {
                    self.$el.append(self.template({
                        title: item.title.toUpperCase(),
                        link: item.link
                    }));
                }

            }
        });
    }

});
app.view.PageView = Backbone.View.extend({
    initialize: function(options) {
      this.config = options.config ? options.config : options;
      if (this.config.title) this.id = this.config.title.toLowerCase().replace(/ /g, '-') + "view";
      this.template = _.template($(this.config.template).html());
      return this;
    },
    
    render: function() {
        this.$el.empty();
        this.$el.html(this.template());
        return this;
    }
});
app.view.StackedBarView = Backbone.View.extend({
    tagName: 'div',
    id: 'biology-area',

    initialize: function(options) {
        this.render = _.bind(this.render, this);
        this.scale = options.scale;
        this.graphId = options.id;
        this.data = options.data;
        this.title = options.title;
        this.scaleNames = options.scaleNames;
        this.z = options.z;
        this.unit = options.unit;
    },

    render: function() {
        this.$el.empty();
        var margin = {top: 50, right: 30, bottom: 50, left: 70},
            width = 850 - margin.left - margin.right,
            height = 500 - margin.top - margin.bottom;

        var x = d3.scale.ordinal()
            .rangeRoundBands([0, width-160], .1);

        var y = d3.scale.linear()
            .rangeRound([height, 0]);

        var color = this.z;

        var xAxis = d3.svg.axis()
            .scale(x)
            .orient("bottom")
            .ticks(3)
            .tickFormat(d3.time.format("%Y"));

        var yAxis = d3.svg.axis()
            .scale(y)
            .orient("left");

        var varNames = this.scaleNames;

        var nestFunction = d3.nest().key(function(d) { return d.date;});

        var data = nestFunction.entries(this.data);

        data.forEach(function(d) {
            var y0 = 0;
            var length = d.values.length;

            d.mapping = varNames.map(function(name) {
                var temp = 0;
                d.values.forEach(function(k) {
                    temp +=k[name];
                });
                return {
                    name: name,
                    label: new Date(d.key),
                    y0: y0,
                    y1: y0 += temp/length
                };
            });
            d.date = new Date(d.key);
            d.total = d.mapping[d.mapping.length-1].y1;
        });

        x.domain(data.map(function(d) {return d.date;}));
        y.domain([0, d3.max(data, function(d) {return d.total; })]);

        var svgBiology = d3.select("#"+this.graphId)
            .attr("width", width + margin.left + margin.right)
            .attr("height", height + margin.top + margin.bottom)
            .attr("class", "graphs-biology")
            .append("g")
            .attr("transform", "translate(" + margin.left + "," + margin.top + ")");

        var selection = svgBiology.selectAll(".series")
            .data(data)
            .enter().append("g")
            .attr("class", "series, graphs-listener-biology")
            .attr("x", function(d) {return  x(d.date);})
            .attr("transform", function(d) {
                return "translate(" + x(d.date) + ",0)";
            })
            .style("stroke", "grey")
            .style("stroke-width", 0.2);

        selection.selectAll("rect")
            .data(function(d) {return d.mapping;})
            .enter().append("rect")
            .attr("width", x.rangeBand())
            .attr("y", function(d) {return y(d.y1);})
            .attr("x", function(d) {return  x(d.date);})
            .attr("height", function(d) {return y(d.y0) - y(d.y1);})
            .style("fill", function(d) {return color(d.name); })
            .style("stroke", "grey")
            .style("stroke-width", 0.2);

        // Adding Legend
        var legend = svgBiology.selectAll(".legend")
            .data(varNames.slice().reverse())
            .enter().append("g")
            .attr("class", "legend")
            .attr("transform", function (d, i) {
                return "translate(25, "  + (10 + i * 20) + ")";
            });

        legend.append("rect")
            .attr("x", width - 175)
            .attr("width", 10)
            .attr("height", 10)
            .style("fill", color);

        legend.append("text")
            .attr("x", width - 163)
            .attr("y", 6)
            .attr("dy", ".35em")
            .text(function (d) { return d; });

        svgBiology.append("g")
            .attr("class", "graphs-x-axis")
            .attr("transform", "translate(0," + height + ")")
            .call(xAxis)
            .append("text")
            .style("text-anchor", "end");

        svgBiology.append("g")
            .attr("class", "graphs-y-axis")
            .call(yAxis)
            .append("text")
            .attr("transform", "rotate(-90)")
            .attr("y", 6)
            .style("text-anchor", "end");

        // Adding Title
        svgBiology.append("text")
            .attr("x", 0)
            .attr("y", 0 - (margin.top / 2))
            .attr("text-anchor", "left")
            .style("font-size", "16px")
            .attr("class", "graphs-title")
            .text(this.title);

        //Adding labels
        var unit = this.unit;
        svgBiology.append("text")      // text label for the x axis
            .attr("x", width/2 - margin.left - 10)
            .attr("y",  height + 40)
            .attr("class", "graphs-xAxisLabel")
            .style("text-anchor", "middle")
            .text("Date");

        svgBiology.append("text")
            .attr("transform", "rotate(-90)")
            .attr("x", -height/2)
            .attr("y", -55)
            .attr("class", "graphs-yAxisLabel")
            .style("text-anchor", "middle")
            .text(unit);

        //Adding on Mouseover handle
        var popupBiologyDiv = d3.select('graphs-tooltip').data([0]);
        popupBiologyDiv.enter()
            .append("div")
            .attr("class", "graphs-tooltip")
            .style("opacity", "0");
        popupBiologyDiv.exit().remove();

        // TODO: Update when changed to seasons to be year - spring/summer
        var dateFormat = d3.time.format("%b-%y");

        svgBiology.selectAll(".graphs-listener-biology")
            .on("mouseover", function(d){

                var xPos = parseFloat(d3.select(this).attr("x"));
                var yPos = parseFloat(d3.select(this.children[this.children.length-1]).attr("y"));
                var valueToPrint = "<b>Date</b>: " + dateFormat(d.date) + "<br/>";
                for(x in d.mapping) {
                    var temp = d.mapping[x];
                    var curValue = Math.round(temp.y1 - temp.y0);
                    valueToPrint += "<b>" + temp.name + "</b>: " + curValue  + " (" + unit + ") Avg. <br/>";
                    d3.select(this.children[x]).style("stroke", "black").style("stroke-width", 2)
                }

                var topMargin = Math.min(yPos, 280);
                popupBiologyDiv.html(valueToPrint)
                    .style("left",this.parentElement.getBoundingClientRect().left + margin.left + xPos + "px")
                    .style("top", this.parentElement.getBoundingClientRect().top + margin.top + topMargin + "px");
                popupBiologyDiv.transition().style("opacity", 0.9);

            })
            .on("mouseout",function(d){
                for(x in d.mapping) {
                    d3.select(this.children[x]).style("stroke", "grey").style("stroke-width", 0.2);
                }
                popupBiologyDiv.transition().style("opacity", 0);

            });

    }
});
app.view.WelcomeView = Backbone.View.extend({
    id: "welcome-view",
    tag: "div",

    template: _.template($('#welcome-template').html()),

    initialize: function(options) {
        return this;
    },

    render: function() {

        this.$el.empty();

        this.$el.html(this.template(
            {sources: app.collection.sources.getSourcesWithSensors().toJSON()}
        ));

        return this;
    }

});
app.view.StackedLineView = Backbone.View.extend({
	tagName: 'div',
	id: 'stacked-line',

	initialize: function(options) {
		this.render = _.bind(this.render, this);
		this.data = options.data;
		this.color = options.color; 
		this.graphId = options.id;
		this.units = options.unit;
		this.title = options.title;
	},

	render: function() {
		this.$el.empty();
		var self = this;
		var data = this.data;
		var color = this.color;
		var margin = {top: 50, right: 200, bottom: 50, left: 200},
			width = 1170 - margin.left - margin.right,
			height = 500 - margin.top - margin.bottom;

		var parseDate = d3.time.format("%Y-%m-%d").parse;
		var x = d3.time.scale().range([0, width]);
		var y = d3.scale.linear()
			.range([height, 0]);

		var xAxis = d3.svg.axis()
			.scale(x)
			.orient("bottom");

		var yAxis = d3.svg.axis()
			.scale(y)
			.orient("left");

		var line = d3.svg.line()
			.defined(function(d) {return !isNaN(d.depth);})
			.interpolate("linear")
			.x(function(d) { return x(d.date); })
			.y(function(d) { return y(d.depth); });

		color.domain(d3.keys(this.data[0].values));

		data.forEach(function(d) {
			d.parsedDate = parseDate(d.date);
		});

		var svgMultiLineGraph = d3.select("#"+this.graphId)
			.attr("width", width + margin.left + margin.right)
			.attr("height", height + margin.top + margin.bottom)
			.attr("class", "graphs-dissolved-oxygen")
			.append("g")
			.attr("transform", "translate(" + margin.left + "," + margin.top + ")");

		var graphData  = this.color.domain().map(function(temperatureLevel) {
			return {
				name: temperatureLevel,
				values: data.map(function(d) {
					return{date: d.parsedDate, depth: +d.values[temperatureLevel]}
				})
			};
		});

		x.domain(d3.extent(data, function(d) {return d.parsedDate; }));

		y.domain([
			d3.max(graphData, function(c) { return d3.max(c.values, function(v) { return v.depth; }); }),
			d3.min(graphData, function(c) { return d3.min(c.values, function(v) { return v.depth; }); })

		]);

		// Adding axis and labels
		svgMultiLineGraph.append("g")
			.attr("class", "graphs-x-axis")
			.attr("transform", "translate(0," + height + ")")
			.call(xAxis).append("text");

		svgMultiLineGraph.append("g")
			.attr("class", "graphs-y-axis")
			.call(yAxis);

		// Adding Labels
		svgMultiLineGraph.append("text")
			.attr("x", width/2)
			.attr("y",  height + 40)
			.attr("class", "graphs-xAxisLabel")
			.style("text-anchor", "middle")
			.text("Date");

		svgMultiLineGraph.append("text")
			.attr("transform", "rotate(-90)")
			.attr("x", -height/2)
			.attr("y", -35)
			.attr("class", "graphs-yAxisLabel")
			.style("text-anchor", "middle")
			.text("Meters");

		// Adding lines
		var lines = svgMultiLineGraph.selectAll(".doline")
			.data(graphData)
			.enter().append("g")
			.attr("class", "doline, graphs-listener-multiline");

		lines.append("path")
			.attr("class", "graphs-line")
			.attr("d", function(d) {return line(d.values);})
			.style("stroke", function(d) {return color(d.name);});

		var circles = lines.append("g").selectAll("circle")
			.data(function(d){return d.values})
			.enter()
			.append("circle")
			.filter(function(d) {return !isNaN(d.depth)})
			.attr("r", 2)
			.attr("cx", function(d) {return x(d.date)})
			.attr("cy", function(d) {return y(d.depth)})
			.attr("fill", function(d) {return color(this.parentNode.__data__.name)})
			.attr("stroke", function(d) {return color(this.parentNode.__data__.name)});

		// Adding Title
		svgMultiLineGraph.append("text")
			.attr("x", 0)
			.attr("y", 0 - (margin.top / 2))
			.attr("text-anchor", "left")
			.style("font-size", "16px")
			.attr("class", "graphs-title")
			.text(this.title);

		// Adding Legend
		svgMultiLineGraph.append("g")
			.attr("transform", "translate(25,10)")
			.text("Temperature");
		var legend = svgMultiLineGraph.selectAll(".legend")
			.data(color.domain())
			.enter().append("g")
			.attr("class", "legend")
			.attr("transform", function (d,i) {
				return "translate( 25, " + (10 + i * 20) + ")";
			});

		// TODO: Try to add a title to the legend
		// legend.append("text")
		// 	.attr("x", width - 50)
		// 	.attr("y", 6)
		// 	.attr("dy", ".35em")
		// 	.text("Dissolved Oxygen Level");

		legend.append("rect")
			.attr("x", width - 20)
			.attr("width", 10)
			.attr("height", 10)
			.style("fill", color);

		legend.append("text")
			.attr("x", width + 5)
			.attr("y", 6)
			.attr("dy", ".35em")
			.text(function (d) { return d + " " + self.units; });


		// Adding on Mouseover handle
		// TODO: Update when changed to seasons to be year - spring/summer
		var dateFormat = d3.time.format("%b-%y");
		var popupMultiLineDiv = d3.select('graphs-tooltip').data([0]);
		popupMultiLineDiv.enter()
			.append("div")
			.attr("class", "graphs-tooltip")
			.style("opacity", "0");
		popupMultiLineDiv.exit()
			.remove();

		var g = svgMultiLineGraph.append("g");

		var points = lines.append("g").selectAll('.points')
			.data(function(d){return d.values})
			.enter()
			.append('g')
			.filter(function(d) {return !isNaN(d.depth)})
			.attr('class', 'points')
			.append('text');

		var focus = g.append('g')
			.attr('class', 'focus')
			.style('display', 'none');

		focus.append('line')
			.attr('class', 'x-hover-line hover-line')
			.attr('y1' , 0)
			.attr('y2', height);
		svgMultiLineGraph.append('rect')
			.attr("class", "overlay")
			.attr("width", width)
			.attr("height", height)
			.on("mouseover", mouseover)
			.on("mouseout", mouseout)
			.on("mousemove", mousemove);

		var timeScales = data.map(function(d) {return x(d.parsedDate)});

		function mouseover() {
			focus.style("display", null);
			d3.selectAll('.points text').style("display", null);
		}
		function mouseout() {
			focus.style("display", "none");
			popupMultiLineDiv.transition().style("opacity", 0);
			d3.selectAll('.points text').style("display", "none");
		}
		function mousemove() {
			var i = d3.bisect(timeScales, d3.mouse(this)[0], 1);
			var di = data[i-1];
			focus.attr("transform", "translate(" + x(di.parsedDate) + ",0)");
			var valueToPrint = "<b>Date</b>: " + dateFormat(di.parsedDate) + "</br>";
			for(var k in di.values) {
				if(di.values[k] != "NA") {
					valueToPrint += "<b>" + k + " (" + self.units + ") </b>: " + di.values[k] + " (meters) <br/>";
				}
			}
			popupMultiLineDiv.html(valueToPrint)
				.style("left", this.parentElement.getBoundingClientRect().left + x(di.parsedDate) + 60 + "px")
				.style("top", this.parentElement.getBoundingClientRect().top + margin.top + 0 + "px");
			popupMultiLineDiv.transition().style("opacity", 0.9);
		}

	}

});
(function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
(i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
})(window,document,'script','//www.google-analytics.com/analytics.js','ga');

ga('create', app.config.googleAnalytics, 'auto');

})(jQuery, Backbone, window);